/*------------------------------------------------------------------------*/
/* function to compute oxygen concentration from temperature and DPhase   */
/*------------------------------------------------------------------------*/
/**
   This function computes the oxygen concentration according to the matrix
   equation.

      \begin{verbatim}
             | D0 D1 D2 D3 D4 | | C00 C01 C02 C03 | | T0 |            
                                     | C10 C11 C12 C13 | | T1 |
      [O] =                          | C20 C21 C22 C23 | | T2 |
                                     | C30 C31 C32 C33 | | T3 |
                                     | C40 C41 C42 C43 |  
      \end{verbatim}

   where D is the compensated phase difference and T is the temperature (C).  
   The foil-matrix, C, contains calibration coefficients supplied by
   Aanderaa.

      \begin{verbatim}
      input:
         t............The water temperature (C) as reported by the optode.
         DPhase.......The compensated phase difference as reported by the
                        optode.
         foilmat......A 5x4 matrix containing the calibration coefficients
                        supplied by Aanderaa for each sensor.

      output:
         This function returns the oxygen concentration (uMolar).
      \end{verbatim}
*/
#include <stdio.h>
double OptodeCalibration(double t, double DPhase, double FoilMatrix[][4])
{
   double dswiftpoly(double x,const double coeff[],int n);
   int i;
   double o2;
   double coef[5];
   int debug=0;
      
   /* compute the temperature-dependent coefficients */
   for (i=0; i<5; i++) {
     coef[i] = dswiftpoly(t,FoilMatrix[i],3); 
   if (debug==1) printf ("OptodeCalibration: coef(i) %f\n", coef[i]);
   }

   /* compute the oxygen concentration */
   o2 = dswiftpoly(DPhase,coef,4);
   if (debug==1) printf ("OptodeCalibration: o2 = %f\n", o2);
   return o2;
}
