/*****************************************************************
  Revision Date: 27-Jun-01 (JZ) - changed compute_ (unsigned int -> int)
                 31-May-01 (JZ) - Changed 'BATTERY CPU' to 'BATTERY' 
                 09-Nov-00 (RS - Bitterman updates( diff1109.lis) 
                 31-Aug-00 (JZ) 
                 10-Aug-00 (RS - rename ARGOLIST_DIR)
                 27-Jul-00 (RS - eliminate redundant stm & add program name
                                 to stdout and trace/error output)
                 09-Jun-00 (RS - replace juldat with ymd2julday)
                 20-Jun-00 (RS - change LOG file name)
                 09-Jun-00

  cnt2phy_t1  
  ==========
   Converts the counts ARGOS data for temperature/salinity/pressure
   PALACE floats PALACE-T1 manufactured in 1997 by Webb Research Cooperation
******************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>

#define max_scans 2000  //Maximum scans for each profile
#define buff_size 256   //Maximum length of each scan read from *.str

char* strupr(char* ptr_to_string_to_be_upper_cased);

FILE *input_file, *out_file, *error_file, *trace_file, *meta_file, *list_file;

char PRGNAME[] ="cnt2phy_t1";
char buffer[512];

struct aoml1_cals { double R1;
                    double R2;
                    double R3;
                    double R4;
                    double T1;
                    double T2;
                    double T3;
                    double pslope;
                    double poff;
                    double vslope;
                    double voff;};

union u_cals { struct aoml1_cals s;
               double c[sizeof(struct aoml1_cals)/sizeof(double)];
             };

struct type2_meta_struct { int   AOML_number;
                           int   WMO_id;
                           int   ARGOS_id;
                           float cycle_time;
                           float start_time;
                           union u_cals cals; 
                          };

long ymd2julday(int, int, int);

/******************************************************************************
  Converts an ASCII string to uppercase
******************************************************************************/
char  *strupr(char* s) 
{
  char* p = s;
  while (*p)
  {
    if (islower((int)*p)) *p = toupper((int)*p);
    ++p;
  }
  return s;
}

/******************************************************************************
  Converts temperature counts to physical units 
******************************************************************************/
float compute_type2_temperature (struct type2_meta_struct *s, int t_counts)
{
  double t;

  if (t_counts == 0)
	  return (float) 0.0;
  t = t_counts/1000.0;
  t = s->cals.s.R1 + s->cals.s.R2*t + s->cals.s.R3*t*t + s->cals.s.R4*t*t*t;
  t *= 1000.0;

  if (t>0.0)
    t = log(t);
  else
    return ((float) 0.0);

  t = (1.0/(s->cals.s.T1 + s->cals.s.T2*t + s->cals.s.T3*t*t*t)) - 273.15;
  return((float) t);
}

/******************************************************************************
  Converts pressure counts to physical units 
******************************************************************************/
float compute_type2_pressure (struct type2_meta_struct *s, int p_counts)
{
  return((float) (0.6895*(s->cals.s.pslope * p_counts + s->cals.s.poff)));
}

/******************************************************************************
  Converts battery voltage counts to physical units 
******************************************************************************/
float compute_battery_voltage (struct type2_meta_struct *s, int v_counts)
{
  return((float) (s->cals.s.vslope * v_counts + s->cals.s.voff));
}

/******************************************************************************
  Computes days from 1997
******************************************************************************/
double dfr97 (const int yp, const int mp, const int dp, const int hr,
              const int minutes, const int secs, int *doy)
{
  int     yr, mo, dy, day_from_97;
  long    passed_jd, jd_xx0101, jd_970101;
  double  d_jd;

  yr = yp;
  mo = mp;
  dy = dp;

  if (yr <= 15)
    yr += 2000;
  else if (yr < 1900)
    yr += 1900;

  if ((yr<1997) || (yr>2015) || (mo<1) || (mo>12) || (dy<1) || (dy>31))
    return (0.0);

  passed_jd = ymd2julday( yr, mo, dy ) + 1;
  dy = 1;
  mo = 1;
  jd_xx0101 = ymd2julday( yr, mo, dy );
  yr = 1997;
  jd_970101 = ymd2julday( yr, mo, dy );
  day_from_97 = (int) (passed_jd - jd_970101);
  *doy = (int) (passed_jd - jd_xx0101);
  d_jd = day_from_97;
  d_jd += (hr+(minutes + secs/60.0)/60.0)/24.0;
  return (d_jd);
}

/******************************************************************************
  Converts data from counts to physical units 
******************************************************************************/
int convert_type2_data(struct type2_meta_struct *s, char *output_name)
{
  int   i1=0,i2=0,i3=0,i4=0,iflg=0,old_i3=1;
  float bp,p;
  char *ptr_phy_filename;

  if ((out_file = fopen(output_name,"w")) == NULL)
  {
    if (trace_file != NULL)
      fprintf (trace_file," CONVERT_TYPE2_DATA: error opening output file %s\n",
               output_name);

    if (error_file != NULL)
      fprintf (error_file," CONVERT_TYPE2_DATA: error opening output file %s\n",
               output_name);

    return 1;
  }

  if (trace_file != NULL)
    fprintf (trace_file," CONVERT_TYPE2_DATA: output file %s successfully opened",output_name);

  printf ("\nOutput file %s successfully opened",output_name);

  // Write Output Profile Physical Unit File Name to List File
  // Check if last occurence of '/' in buffer is NULL
  if ((ptr_phy_filename = strrchr(output_name, '/')) == NULL)
  {
    printf ("\nCan't get phy file basename %s", output_name);

    if (trace_file != NULL)
      fprintf (trace_file,
               "\n%s : Can't get phy file basename %s", PRGNAME, output_name);

    if (error_file != NULL)
      fprintf (error_file,
               "\n%s: Can't get phy file basename %s", PRGNAME, output_name);

    return 1;
  } // endif: ptr_phy_filename

  ptr_phy_filename++; // Increment file name pointer
  fprintf (list_file,"%s\n", ptr_phy_filename); // Write file name to list file

  // Reads a line from input_file into buff and check if NULL
  while (fgets(buffer,sizeof(buffer),input_file) != NULL)
  {
    // Check if pointer to 1st occurrence of DRIFT PRESSURE (ignoring case) in
    // buffer is not NULL
    if (strstr(strupr(buffer),"DRIFT PRESSURE") != NULL)
    {
      // Read chars from buffer and store them in i1. If none, set to zero
      //
      // Note: * means the field does not need to be included in list of
      // variable names to write to 
      if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)
        i1 = 0;

      sprintf (buffer,"DRIFT PRESSURE (DBAR)                   %-6.1f\n",
      bp=compute_type2_pressure(s,i1));
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(strupr(buffer),"SURFACE PRESSURE") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %d",&i1) != 1)
        i1 = 0;

      sprintf (buffer,"START SURFACE PRESSURE (DBAR)           %-6.1f\n",
      compute_type2_pressure(s,i1));
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(strupr(buffer),"DRIFT TEMPERATURE") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)
        i1 = 0;

      sprintf (buffer,"DRIFT TEMPERATURE (DEG C)               %-6.3f\n",
      compute_type2_temperature(s,i1));
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(strupr(buffer),"BATTERY") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %d",&i1) != 1)
        i1 = 0;

      sprintf (buffer,"BATTERY (VOLT)                          %-5.2f\n",
               compute_battery_voltage (s,i1));
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if ((strstr(strupr(buffer),"COLUMNS") != NULL) || (strstr(buffer,"COLUMNS")))
    {
      if (strstr(strupr(buffer),"COLUMNS") != NULL)
      {
        fprintf (out_file,"NUMBER OF COLUMNS                       3\n");
        fprintf (out_file,"1. COLUMN                               PRESSURE (DBAR)\n");
        fprintf (out_file,"2. COLUMN                               TEMPERATURE-90 (DEG C)\n");
        fprintf (out_file,"3. COLUMN                               CHECKSUM FLAG\n");
      }

      rewind(meta_file);

      while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
        if (strstr(buffer,"================") != NULL)
          break;

        while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
        {
          fwrite (strupr(buffer),1,strlen(buffer),out_file);

          if (strstr(buffer,"================") != NULL)
            break;
        }

        while (fgets(buffer,sizeof(buffer),input_file) != NULL)
          if (strstr(buffer,"LATITUDE") != NULL)
            break;

        fwrite (strupr(buffer),1,strlen(buffer),out_file);

        while (fgets(buffer,sizeof(buffer),input_file) != NULL)
        {
          fwrite (strupr(buffer),1,strlen(buffer),out_file);

          if (strstr(buffer,"================") != NULL)
            break;
        }

      iflg = 1;
      continue;
    }

    if (iflg == 1)
    {
      if (sscanf(buffer,"%d %d %d",&i1,&i2,&i3) == 3)
      {
		p = compute_type2_pressure(s,i1);
		if ((i3 > 0) && (old_i3 > 0))
			fprintf (out_file,"%6.1f   %6.3f   %1d\n",
                 (p+bp)/2.0, compute_type2_temperature(s,i2), i3);
		else if (i3 == 0)
			fprintf (out_file,"%6.1f   %6.3f   %1d\n",
                 (float) 0.0, (float) 0.0, 0);
//                 (float) 0.0, compute_type2_temperature(s,i2), i3);
		else if ((i3 > 0) && (old_i3 == 0))
		{
			i4 = ((i1+1) > 29) ? i1+2 : i1+1;
			fprintf (out_file,"%6.1f   %6.3f   %1d\n",
                 (p+compute_type2_pressure(s,i4))/2.0, compute_type2_temperature(s,i2), i3);
		}

        bp = p;
		old_i3 = i3;
      }
    }
    else
      fprintf (out_file,buffer);
  }

  if (trace_file != NULL)
    fprintf (trace_file," CONVERT_TYPE2_DATA: %s completed successfully\n",output_name);

  return 0;
}

/******************************************************************************
  Assumes str_name = *.cnt, returns *.meta in str_name
******************************************************************************/
int get_meta_file_name (char *str_name)
{
  char *ptr,tmp_buff[80];
  int  len;

  // Check if pointer to last occurrence of '_' is NULL
  if ((ptr = strrchr(str_name,'_')) == NULL)
    strcat(str_name,".meta");
  else
  {
    sprintf (tmp_buff,"%s\0", ptr);
    len = strlen(str_name) - strlen(tmp_buff);
    strncpy(tmp_buff, str_name, len);
    tmp_buff[len] = '\0';
    sprintf (str_name,"%s.meta",tmp_buff);
  }

  return 0;
}

/******************************************************************************
  Reads meta_file

  Calls: strupr
******************************************************************************/
int read_type2_meta_file (char *meta_name, struct type2_meta_struct *meta_struct)
{
  int i,j,syear,smonth,sday,shour,smin,x;
  char *ptr;
  char *cal_keys[] = {"R1=","R2=","R3=","R4=","T1=","T2=","T3=",
                      "PSLOPE=","POFF=","VSLOPE=","VOFF=","\0"};

  for (i=0; i<(sizeof(struct aoml1_cals)/sizeof(double)); i++)
    meta_struct->cals.c[i] = 0.0;

  meta_struct->cycle_time = (float) 0.0;
  meta_struct->start_time = (float) 0.0;

  printf("\nOpening meta file %s",meta_name);

  if ((meta_file = fopen(meta_name,"r")) == NULL)
  {
    printf ("\nError opening meta file %s",meta_name);

    if (error_file != NULL)
      fprintf (error_file," ERROR opening meta file %s\n",meta_name);

    if (trace_file != NULL)
      fprintf (trace_file," ERROR opening meta file %s\n",meta_name);

    return(1);
  }

  if (trace_file != NULL)
    fprintf (trace_file," Meta file %s successfully opened\n",meta_name);

  // Reads a line from meta_file into buffer and checks if NULL
  while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
  {
    // Convert buffer to upper case, return a pointer to the first
    // occurrence of INTERNAL ID NUMBER in buffer, and check if NULL
    if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) == 0)
        meta_struct->AOML_number = 0;

      continue;
    }

    if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) == 0)
        meta_struct->WMO_id = 0;

      continue;
    }

    if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) == 0)
        meta_struct->ARGOS_id = 0;

      continue;
    }

    if (strstr(strupr(buffer),"DELAY OF FIRST DOWN TIME") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&x) == 0)
        x = 0;

      meta_struct->start_time += (float) (x/24.0);

      continue;
    }

    if (strstr(strupr(buffer),"DOWN TIME") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&x) == 0)
        x = 0;

      meta_struct->cycle_time += x;

      continue;
    }

    if (strstr(strupr(buffer),"UP TIME") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&x) == 0)
        x = 0;

      meta_struct->cycle_time += (float) (x/24.0);
      meta_struct->start_time -= (float) (x/24.0);
      continue;
    }

    if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
      if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",&sday,&smonth,&syear,&shour,&smin) == 5)
        meta_struct->start_time += (float) (dfr97(syear,smonth,sday,shour,smin,0,&x) + 0.25);
      else
        meta_struct->start_time = 0;

      continue;
    }

    i = 0;

    while ((j = strlen(cal_keys[i])) > 1)
    {
      if ((ptr = strstr(buffer,cal_keys[i])) != NULL)
        meta_struct->cals.c[i] = atof(ptr+j);

      i++;
    }
  }

  return 0;
}

/******************************************************************************
  'main' function

  Calls: get_meta_file_name, read_type2_meta_file, convert_type2_data 
******************************************************************************/
int main (int argc, char *argv[])
{
  char     *ptr_path_meta, *ptr_path_cnt, *ptr_path_phy, 
           *ptr_path_log, *ptr_path_list,
           pathname_input_filename[161]="",
           pathname_trace[161]="", pathname_error[161]="";

  char     input_filename[80],output_directory[256], 
           msg[11]="", title[11]="", *ptr;

  struct   type2_meta_struct s;

  unsigned long t;
  int      ret_val = 0;

// Program Name
// ------------
  printf("Entering Program: %s\n", PRGNAME);
  strcpy(title, "Program calibrates counts files for PALACE_T1 floats");
  printf("%s\n", title);


// Input Arguments 
// ---------------
  if (argc < 6)
  {
    printf("Format: %s cnt_infile phy_outdir trace_outile error_outfile\n", PRGNAME);
    printf("\n");
    printf("cnt_infile:    counts input file name\n");
    printf("phy_outdir:    output directory for generated calibrated data file(s)\n");
    printf("trace_outile:  output directory for generated trace file\n");
    printf("error_outfile: output directory for generated error file\n");
    printf("list_outfile:  output directory for generated list file\n");
    return 1;
  }

// Input Counts File Name
// ---------------------
  sprintf (input_filename,"%s",argv[1]);
  printf("%s: input_filename - %s\n", PRGNAME, input_filename);

// Output directory path: defaults to PHYFILES_DIR
// -----------------------------------------------
  if ( (ptr_path_phy = getenv("PHYFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): PHYFILES_DIR <-- no such environment variable\n", PRGNAME);
    t = time(NULL); 
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf ("%s: Output Directory - %s\n", PRGNAME, argv[2]);
  sprintf (output_directory,"%s/",argv[2]);

// TRACE File
// ----------
  if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
  {
    printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val ;
  }

  sprintf (buffer,"%s",argv[3]);
  printf ("%s: Opening trace file %s\n", PRGNAME, buffer);

  if ((trace_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nERROR (%s): Opening trace file %s", PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  } 

  printf("%s: OK -> trace file opened", PRGNAME);

  if (trace_file != NULL)
  {
    t = time(NULL);
    fprintf (trace_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

// ERROR File
// ----------
  sprintf (buffer,"%s",argv[4]);
  printf ("%s: Opening error file %s\n", PRGNAME, buffer);

  if ((error_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
    if (trace_file != NULL)
      fprintf (trace_file," ERROR (%s): Opening file %s\n", PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));        
    ret_val = 1;
    return ret_val;
  }

  printf("%s: OK -> error file opened", PRGNAME);

  if (trace_file != NULL)
    fprintf (trace_file,"File %s successfully opened\n",buffer);

  if (error_file != NULL)
  {
    t = time(NULL);
    fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

// LIST File
// ---------
  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
    printf("\nERROR (%s): ARGOLIST_DIR <-- no such environment variable",
           PRGNAME);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  sprintf (buffer,"%s",argv[5]);
  printf ("%s: Opening list file %s", PRGNAME, buffer);

  if ((list_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nERROR (%s): Opening list file %s\n", PRGNAME, buffer);

    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): Error opening list file %s\n",
               PRGNAME, buffer);

    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): Error opening list file %s\n",
               PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf("%s: OK -> list file opened\n", PRGNAME);
  if (trace_file != NULL)
    fprintf (trace_file,"%s: List file %s successfully opened\n", 
             PRGNAME, buffer);

// Open Input Counts File
// ----------------------
  if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): CNTFILEDIR <-- no such environment variable\n", PRGNAME);

    if (trace_file != NULL)
      fprintf (trace_file,
               "ERROR (%s): CNTFILES_DIR <-- no such environment variable\n",
               PRGNAME);

    if (error_file != NULL)
      fprintf (error_file,
               "ERROR (%s): CNTFILES_DIR <-- no such environment variable\n",
               PRGNAME);

    if (trace_file != NULL)                  // Close Log files
      fclose (trace_file);

    if (error_file != NULL)
      fclose (error_file);

    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return(ret_val);
  }

  sprintf(pathname_input_filename, "%s/%s", ptr_path_cnt, input_filename);

  if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
  {
    printf ("\nERROR (%s): Opening input file %s", PRGNAME, pathname_input_filename);
    if (trace_file != NULL)
      fprintf (trace_file, "ERROR (%s): Opening input file %s\n", 
               PRGNAME, pathname_input_filename);
    else
      if (error_file != NULL)
        fprintf (error_file, "ERROR (%s): Opening input file %s \n", 
                 PRGNAME, pathname_input_filename);            

    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }
  else
  {
    printf ("%s: Input file %s successfully opened\n",
            PRGNAME, pathname_input_filename);
    if (trace_file != NULL)
       fprintf (trace_file,
                "%s: Input file %s successfully opened\n",
                PRGNAME, pathname_input_filename);
 
// META File
// ---------
    if (get_meta_file_name (input_filename) != 0)
    {
       printf ("ERROR (%s): Finding meta file name", PRGNAME);
       if (error_file != NULL)
         fprintf (error_file,
                  "\nERROR (%s): Finding meta file name - returned name %s\n",
                  PRGNAME, input_filename);
       if (trace_file != NULL)
         fprintf (trace_file,
                  "\nERROR (%s): Finding meta file name - returned name %s\n",
                  PRGNAME, input_filename);
       ret_val = 1;
    }
    else
    {                                             // Success: got meta filename
      if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
      {
         printf("%s: METAFILES_DIR <-- no such environment variable\n", PRGNAME);

         if (trace_file != NULL)
           fprintf (trace_file,
                    "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                    PRGNAME);

         if (error_file != NULL)
           fprintf (error_file,
                    "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                    PRGNAME);

        if (trace_file != NULL)                  // Close Log files
          fclose (trace_file);

        if (error_file != NULL)
          fclose (error_file);

        ret_val = 1;
      } // endif  ptr_path_meta

      sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

      if (read_type2_meta_file (pathname_input_filename, &s) != 0)
      {
        printf ("\nERROR (%s): Reading meta file", PRGNAME);
        if (trace_file != NULL)
          fprintf (trace_file,"ERROR (%s): Reading meta file %s\n",
                   PRGNAME, pathname_input_filename);
        if (error_file != NULL)
          fprintf (error_file, "ERROR (%s): Reading meta file %s\n",
                   PRGNAME, pathname_input_filename);
        ret_val = 1;
      }
      else
      {
        if (trace_file != NULL)
          fprintf (trace_file, "%s: Meta file %s successfully read\n", 
                   PRGNAME, pathname_input_filename);
          printf ("\n%s: Meta file %s successfully read\n", 
                  PRGNAME, pathname_input_filename);

// Convert Counts to Physical Units
// --------------------------------
        sprintf (input_filename,"%s%s",output_directory,argv[1]); // !!using input storage

        if ((ptr = strrchr(input_filename,'.')) == NULL)
          strcat (input_filename,".phy");
        else
          sprintf(ptr,".phy");

        printf ("%s: Output Path & File Name - %s", PRGNAME, input_filename);

        if (convert_type2_data(&s, input_filename) == 0)
        {
          printf ("\n%s: CONVERT_TYPE3_DATA  successful\n", PRGNAME);
          if (trace_file != NULL)
            fprintf (trace_file, "%s: CONVERT_TYPE3_DATA successful\n", PRGNAME);

        }
        else
        { 
          printf ("\nERROR (%s): CONVERT_TYPE3_DATA\n");
          if (trace_file != NULL)
            fprintf (trace_file, "ERROR (%s): CONVERT_TYPE3_DATA\n",
                     PRGNAME);
          if (error_file != NULL)
            fprintf (error_file, "ERROR (%s): CONVERT_TYPE3_DATA\n",
                     PRGNAME);
          ret_val = 1;
        } // endif: convert_type2_data
      } // endif: read_type2_meta_file
    } // endif: get_meta_file_name 
  } // endif: input_file

// Write finish message to trace & error files
// -------------------------------------------
  t = time(NULL);
  if ( ret_val == 0)
    strcpy(msg, "finished");
  else
    strcpy(msg, "terminated");

  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));

  if (trace_file != NULL)
   fprintf (trace_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

  if (error_file != NULL)
   fprintf (error_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

// Close all files
// ---------------
  if (input_file != NULL)
    fclose (input_file);

  if (trace_file != NULL)
    fclose (trace_file);

  if (error_file != NULL)
    fclose (error_file);

  if (list_file != NULL)
    fclose (list_file);

  return ret_val;
}
