/*****************************************************************
  Revision Date: 30-Dec-09 (ER - change function get_meta_file_name to adapt to
                                 variable length transmission ids.
  				 09-May-03 (CS - corrected salinity calculation by
                                 replacing 7.0262 with 7.0261. Since the impact
                                 on the salinity is about -1e-4 we do not need
                                 to reprocess any data)
                 26-Dec-00 (CS - added equation 2 & 3 to conductivy calibration)
                 19-Dec-00 (CS - correction of conductivity estimation)
                 29-Nov-00 (RS - format correction (BATTERY)
                 21-Nov-00 (DB - Change to handle two cases of temerature eq.)
                 09-Nov-00 (RS - Bitterman updates( diff1109.lis)
                 31-Aug-00 (JZ)
                 10-Aug-00 (RS - rename ARGOLIST_DIR)
                 01-Aug-00 (RS - eliminate redundant stm & add program name
                                 to stdout and trace/error output)
                 07-07-00 (JZ)
  cnt2phy_ts1
  ===========
  Converts count to physical units for Type I floats

******************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>

#define max_scans 2000  //Maximum scans for each profile
#define buff_size 256  //Maximum length of each scan read from *.str

char* strupr(char* ptr_to_string_to_be_upper_cased); /* function prototype */

FILE *input_file, *out_file, *error_file, *trace_file, *meta_file, *list_file;

char PRGNAME[] ="cnt2phy_ts1";
char buffer[512];

struct sio_cals { double pgain;
                  double poff;
                  double pmp_gain;
                  double pmp_off;
                  double cpugain;
                  double cpuoff;
                  double at;
                  double bt;
                  double ct;
                  double h1;
                  double h2;
                  double h3;
                  double blok;
                  double av1;
                  double av2;
                  double pb1;
                  double pb2;
                  double cdiv;
                  double asal;
                  double bsal;
                  double csal;
                  double coff;
                  double cgain;
                  double ctref;
                };

union u_cals { struct sio_cals s;
               double c[sizeof(struct sio_cals)/sizeof(double)];
             };

struct type1_meta_struct { int AOML_number;
                           int WMO_id;
                           int ARGOS_id;
                           float cycle_time;
                           float start_time;
                           union u_cals cals;
                         };


/******************************************************************************
  Computes conductivity for SIO salinity floats from raw
  temperature and salinity counts
******************************************************************************/
float compute_sio_conductivity (struct type1_meta_struct *s,
                                float s_counts, float t_counts, float t,
                                float p)
{
  float c, cc;

// correction for conductivity if s_counts is to large
// recommendation by Jeff Sherman: subtract 1024 from s_counts if cc>38

  cc=32.0 + 4.0 * s_counts * ((float)(s->cals.s.cdiv));

  if (cc>38)
    {
    s_counts=s_counts-1024.0;
    }

  c = 32.0 + 4.0 * s_counts * ((float)(s->cals.s.cdiv))
              + ((float)(s->cals.s.asal))
              + ((float)(s->cals.s.bsal)) * t_counts
              + ((float)(s->cals.s.csal)) * t_counts * t_counts;

// calibration equations 2 and 3 (for conductivity)
// for insitu calibration (if a CTD station is available):
  c = s->cals.s.coff + s->cals.s.cgain *c;

// correction due to tmeperature and pressure effects on the conductivity cell
// geometry (recommended by FSI):
  c = c*( 1 + 7.5e-6*(s->cals.s.ctref-t) + 1.5e-8*p);

  return(c);
}


/******************************************************************************
  Routine for computing temperature for SIO floats
******************************************************************************/
float compute_sio_temp (struct type1_meta_struct *s, float t_counts)
{
#define R120 2.094395102
  float t,aa,aa3,bb;

  t = ((float)(s->cals.s.at)) * (((float)(s->cals.s.bt)) + t_counts)/
                       (((float)(s->cals.s.ct)) + t_counts);
  if ((fabs(s->cals.s.h1) < 1.0) &&
          (fabs(s->cals.s.h2) < 1.0) && (fabs(s->cals.s.h3) < 1.0))
  {
   t *= 1000.0;
   if (t > 0.0)
     t = log(t);
   else
     return((float) 0.0);
   t = (1.0/(((float)(s->cals.s.h1)) + ((float)(s->cals.s.h2)) * t + ((float)(s->cals.s.h3)) * t * t * t)) - 273.15;
   return((float) t);
  }
  if (t < 0.0)
   return ((float) 0.0);
  if ((s->cals.s.h2 * s->cals.s.h3) == 0.0)
  {
   t = (log(t) - ((float)(s->cals.s.h1))) / ((float)(s->cals.s.h2));
   if (t != 0.0)
    return (1.0/t - 273.15);
   else
    return ((float) 0.0);
  }
  else
  {
   aa = (float)(s->cals.s.h2 / (3.0 * s->cals.s.h2 * s->cals.s.h3));
   aa3 = aa*aa*aa;
   bb = 0.5*((float)(s->cals.s.h1)-log(t))/((float)(s->cals.s.h2 * s->cals.s.h3));
   t = bb*bb + aa3;
   if ((t < 0.0) && (aa < 0.0))
   {
    bb = -bb/sqrt(-aa3);
    bb = acos(bb)/3.0;
	aa = 2.0 * sqrt(-aa);
	t = aa * cos(bb + 2.0*R120);
    if (t != 0.0)
     return (1.0/t - 273.15);
    else
     return ((float) 0.0);
   }
  }
  return ((float) 0.0);
}


/******************************************************************************

******************************************************************************/
float salinity (float c, float t, float p)
{
  double t2,t3,t4,r,r1,r3,r4,r5,rp,rt,ft,d,ds,s;

  if ((c>65.0) || (c<1.0) || (t>45.0) || (t<-3.0) || (p>6000.0) || (p<-20.0))
    return (0.0);

  t2 = t * t;
  t3 = t2 * t;
  t4 = t3 * t;
  r  = c/42.914;
  rt = .6766097 + .0200564 * t + .0001104259 * t2
       - .00000069698 * t3 + 1.0031E-09 * t4;
  d  = 1 + .03426 * t + .0004464 * t2 + (.4215 - .003107 * t) * r;
  rp = 1 + p * (.0000207 - 6.37E-10 * p + 3.989E-15 * p * p) / d;
  rt = r / rp / rt;
  ft = (t - 15) / (1 + .0162 * (t - 15));
  r1 = sqrt(rt);
  r3 = rt * r1;
  r4 = rt * rt;
  r5 = r4 * r1;
  ds = ft * (.0005 - .0056 * r1 - .0066 * rt -
             .0375 * r3 + .0636 * r4 - .0144 * r5);
  s = .008 - .1692 * r1 + 25.3851 * rt + 14.0941 * r3
      - 7.0261 * r4 + 2.7081 * r5 + ds;
  return ((float)s);
}


/******************************************************************************
  Converts an ASCII string to uppercase
******************************************************************************/
char *strupr(char* s)
{
  char* p = s;

  while (*p) {
    if (islower((int)*p))
      *p = toupper((int)*p);

    ++p;
  }

  return s;
}


/******************************************************************************
  Converts data from counts to physical units
******************************************************************************/
int convert_sio_ctd_data(struct type1_meta_struct *s, char *output_name)
{
  int i1=0,i2=0,i3=0,i4=0,ip[128],iflg[128];
  float a1,a2,fp[128],it[128],ic[128],opb2;
  char *ptr_phy_filename;

  if ((out_file = fopen(output_name,"w")) == NULL)
  {
    if (trace_file != NULL)
      fprintf (trace_file," CONVERT_TYPE2_DATA: error opening output file %s\n",
               output_name);

    if (error_file != NULL)
      fprintf (error_file," CONVERT_TYPE2_DATA: error opening output file %s\n",
               output_name);

    return 1;
  }

  if (trace_file != NULL)
    fprintf (trace_file," CONVERT_TYPE2_DATA: output file %s successfully opened",output_name);

  printf ("\nOutput file %s successfully opened",output_name);


  // Write Output Profile Physical Unit File Name to List File
  // Check if last occurence of '/' in buffer is NULL
  if ((ptr_phy_filename = strrchr(output_name, '/')) == NULL)
  {
    printf ("\nCan't get phy file basename %s", output_name);

    if (trace_file != NULL)
      fprintf (trace_file,
               "\n%s : Can't get phy file basename %s", PRGNAME, output_name);

    if (error_file != NULL)
      fprintf (error_file,
               "\n%s: Can't get phy file basename %s", PRGNAME, output_name);

    return 1;
  } // endif: ptr_phy_filename

  ptr_phy_filename++; // Increment file name pointer
  fprintf (list_file,"%s\n", ptr_phy_filename); // Write file name to list file


  // Reads a line from input_file into buff and check if NULL
  while (fgets(buffer,sizeof(buffer),input_file) != NULL)
  {
    // Check if pointer to 1st occurrence of INTERNAL ID NUMBER (ignoring case) in
    // buffer is not NULL
    if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
    {
      fprintf(out_file,buffer);
      continue;
    }

    if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
    {
      fprintf(out_file,buffer);
      continue;
    }

    if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
    {
      fprintf(out_file,buffer);
      continue;
    }

    if (strstr(strupr(buffer),"PROFILE NUMBER") != NULL)
    {
      fprintf(out_file,buffer);
      continue;
    }

    if (strstr(strupr(buffer),"PROFILE TERMINATION FLAG") != NULL)
    {
      fprintf(out_file,buffer);
      continue;
    }

    if (strstr(strupr(buffer),"DRIFT TEMPERATURE") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d %d",&i1,&i2) != 2)
        return(1);

      sprintf (buffer,"DRIFT TEMPERATURE (DEG C)               %-7.3f %-7.3f\n",
      compute_sio_temp(s, (float)i1),compute_sio_temp(s, (float)i2));
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(strupr(buffer),"DRIFT PRESSURE") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d %d",&i1,&i2) != 2)
        return(1);

//CS      a1 = (float) (s->cals.s.pgain*i1 + s->cals.s.poff);
//CS      a2 = (float) (s->cals.s.pgain*i2 + s->cals.s.poff);

      a1 = (float) (s->cals.s.pgain*(float)i1 + s->cals.s.poff);
      a2 = (float) (s->cals.s.pgain*(float)i2 + s->cals.s.poff);
      sprintf (buffer,"DRIFT PRESSURE (DBAR)                   %-+6.1f %-+6.1f\n",a1,a2);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(strupr(buffer),"SURFACE PRESSURE") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %d",&i1) != 1)
        return(1);

//CS      a1 = (float) (s->cals.s.pgain*i1 + s->cals.s.poff);
      a1 = (float) (s->cals.s.pgain*(float)i1 + s->cals.s.poff);
      sprintf (buffer,"START SURFACE PRESSURE (DBAR)           %-6.1f\n",a1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(strupr(buffer),"CPU") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)
        return(1);

      a1 = (float) (s->cals.s.cpugain*i1 + s->cals.s.cpuoff);
      sprintf (buffer,"BATTERY CPU (VOLT)                      %-5.2f\n",a1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(strupr(buffer),"PUMP") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)
        return(1);

      a2 = (float) (s->cals.s.pmp_gain*i1 + s->cals.s.pmp_off);
      sprintf (buffer,"BATTERY PUMP (VOLT)                     %-5.2f\n",a2);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(strupr(buffer),"COLUMNS") != NULL)
    {
      fprintf (out_file,"NUMBER OF COLUMNS                       5\n");
      fprintf (out_file,"1. COLUMN                               PRESSURE (DBAR)\n");
      fprintf (out_file,"2. COLUMN                               TEMPERATURE-90 (DEG C)\n");
      fprintf (out_file,"3. COLUMN                               CONDUCTIVITY\n");
      fprintf (out_file,"4. COLUMN                               SALINITY (PSU)\n");
      fprintf (out_file,"5. COLUMN                               CHECKSUM FLAG\n");
      rewind(meta_file);

      while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
        if (strstr(buffer,"================") != NULL)
          break;

      while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
      {
        fwrite (strupr(buffer),1,strlen(buffer),out_file);

        if (strstr(buffer,"================") != NULL)
          break;
      }

      while (fgets(buffer,sizeof(buffer),input_file) != NULL)
        if (strstr(strupr(buffer),"LATITUDE") != NULL)
          break;

      fwrite (strupr(buffer),1,strlen(buffer),out_file);

      while (fgets(buffer,sizeof(buffer),input_file) != NULL)
      {
        fwrite (strupr(buffer),1,strlen(buffer),out_file);

        if (strstr(buffer,"================") != NULL)
          break;
      }

      i1 = 0;

      while (fgets(buffer,sizeof(buffer),input_file) != NULL)
      {
        if (sscanf(buffer,"%d %f %f %d",&ip[i1],&it[i1],&ic[i1],&iflg[i1]) == 4)
        i1++;
      }

      fp[i1-1] = (float) (0.5 * s->cals.s.pgain * s->cals.s.blok);
      opb2 = (float)(s->cals.s.pb1 +((s->cals.s.pb2 - s->cals.s.pb1)/s->cals.s.av1));

      for (i2=i1-2; i2>=0; i2--)
      {
        i3 = i1-1-i2;

        if (i3 < s->cals.s.pb1)
          fp[i2] = (float)(fp[i2+1] + s->cals.s.pgain*s->cals.s.blok);
        else if (i3 == s->cals.s.pb1)
          fp[i2] = (float)(fp[i2+1] + (s->cals.s.pgain*s->cals.s.blok/2)
                    + (s->cals.s.av1*s->cals.s.pgain*s->cals.s.blok/2));
       	else if (i3 < opb2)
          fp[i2] = (float)(fp[i2+1] + s->cals.s.av1*s->cals.s.pgain*s->cals.s.blok);
        else if (i3 == opb2)
          fp[i2] = (float)(fp[i2+1] + (s->cals.s.av1*s->cals.s.pgain*s->cals.s.blok/2)
                           + (s->cals.s.av2*s->cals.s.pgain*s->cals.s.blok/2));
        else
          fp[i2] = (float)(fp[i2+1] + s->cals.s.av2*s->cals.s.pgain*s->cals.s.blok);
      }

      for (i2=0; i2<i1; i2++)
      {
        a1=compute_sio_temp(s,it[i2]);
	a2=compute_sio_conductivity(s,ic[i2],it[i2],a1,fp[i2]);
//CS	a2=compute_sio_conductivity(s,ic[i2],it[i2]);
        sprintf (buffer,"%6.1f   %6.3f   %6.3f   %6.3f   %d\n",
                 fp[i2],a1,a2,salinity(a2,a1,fp[i2]),iflg[i2]);
//CS        sprintf (buffer,"%8.3f   %6.3f   %6.3f   %6.3f   %d\n",
//CS                 fp[i2],a1,a2,salinity(a2,a1,fp[i2]),iflg[i2]);

        fwrite (buffer,1,strlen(buffer),out_file);
      }
    }
    else
      // Write anything else in the meta file to the .phy file
      fprintf (out_file,buffer);

    continue;
  }

  return 0;
}


/******************************************************************************
  Assumes str_name = *.cnt, returns *.meta in str_name
******************************************************************************/
int get_meta_file_name (char *str_name)
{
  char *ptr,tmp_buff[80];

  if ((ptr = strrchr(strupr(str_name),'_')) == NULL)
    strcat(str_name,".meta");
  else
  {
    *ptr = '\0';
    sprintf (tmp_buff,"%s", str_name );
//  sprintf (tmp_buff,"%s",ptr-10);
    sprintf (str_name,"%s.meta",tmp_buff);
  }

  return 0;
}


/******************************************************************************
  Reads meta_file
******************************************************************************/
int read_type1_meta_file (char *meta_name, struct type1_meta_struct *meta_struct)
{
  int i,j;
  char *ptr;
  char *cal_keys[] = {"PGAIN=", "POFF=", "PMP_GAIN=", "PMP_OFF=", "CPUGAIN=",
                      "CPUOFF=", "AT=", "BT=", "CT=", "H1=", "H2=", "H3=",
                      "BLOK=", "AV1=", "AV2=", "PB1=", "PB2=", "CDIV=", "ASAL=",
                      "BSAL=", "CSAL=", "COFF=", "CGAIN=", "CTREF=", "\0"};

  meta_struct->cycle_time = (float) 0.0;
  meta_struct->start_time = (float) 0.0;
  printf("\nOpening meta file %s",meta_name);

  if ((meta_file = fopen(meta_name,"r")) == NULL)
  {
    printf ("\nError opening meta file %s",meta_name);

    if (error_file != NULL)
      fprintf (error_file," ERROR opening meta file %s\n",meta_name);

    if (trace_file != NULL)
      fprintf (trace_file," ERROR opening meta file %s\n",meta_name);

    return(1);
  }

  if (meta_file != NULL)
    fprintf (trace_file," Meta file %s successfully opened\n",meta_name);

  while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
  {
    if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) != 1)
        meta_struct->AOML_number = 0;

      continue;
    }

    if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) != 1)
        meta_struct->WMO_id = 0;

      continue;
    }

    if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) != 1)
        meta_struct->ARGOS_id = 0;

      continue;
    }

    i = 0;

    while ((j = strlen(cal_keys[i])) > 1)
    {
      if ((ptr = strstr(buffer,cal_keys[i])) != NULL)
        meta_struct->cals.c[i] = atof(ptr+j);

      i++;
    }
  }

  return 0;
}


/******************************************************************************
  'main' function
******************************************************************************/
int main (int argc, char *argv[])
{
  char *ptr_path_meta, *ptr_path_cnt, *ptr_path_phy,
       *ptr_path_log, *ptr_path_list,
       pathname_input_filename[161]="",
       pathname_trace[161]="", pathname_error[161]="";

  char input_filename[80],output_directory[256],
       msg[11]="", title[11]="", *ptr;

  struct type1_meta_struct s;

  unsigned long t;
  int      ret_val = 0;

// Program Name
// ------------
  printf("Entering Program: %s\n", PRGNAME);
  strcpy(title, "Program calibrates  counts files for PALACE_TS1 floats");
  printf("%s\n", title);

// Input Paramenters
// -----------------
  if (argc < 6)
  {
    printf("Format: %s cnt_infile phy_outdir trace_outile error_outfile\n", PRGNAME);
    printf("\n");
    printf("cnt_infile:    counts input file name\n");
    printf("phy_outdir:    output directory for generated calibrated data file(s)\n");
    printf("trace_outile:  output directory for generated trace file\n");
    printf("error_outfile: output directory for generated error file\n");
    printf("list_outfile:  output directory for generated list file\n");
    return 1;
  }

// Input Counts File Name
// ----------------------
 printf ("%s: Input Hex File Name - %s\n", PRGNAME, argv[1]);
 sprintf (input_filename,"%s",argv[1]);

// Output directory path: defaults to PHYFILES_DIR
// -----------------------------------------------
  if ( (ptr_path_phy = getenv("PHYFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): PHYFILES_DIR <-- no such environment variable\n", PRGNAME);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf ("%s: Output Directory - %s\n", PRGNAME, argv[2]);
  sprintf (output_directory,"%s/",argv[2]);

// TRACE File
// ----------
  if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
  {
    printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val ;
  }

  printf ("%s: Opening trace file %s ", PRGNAME, buffer);
  sprintf (buffer,"%s",argv[3]);

  if ((trace_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nERROR (%s): Opening trace file %s", PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf("%s: OK -> trace file opened\n", PRGNAME);

  if (trace_file != NULL)
  {
    t = time(NULL);
    fprintf (trace_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

// ERROR File
// ----------
  sprintf (buffer,"%s",argv[4]);
  printf ("%s: Opening error file %s ", PRGNAME, buffer);

  if ((error_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
    if (trace_file != NULL)
      fprintf (trace_file," ERROR (%s): Opening file %s\n", PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf("%s: OK -> error file opened\n", PRGNAME);

  if (trace_file != NULL)
    fprintf (trace_file,"%s: File %s successfully opened\n", PRGNAME, buffer);
  if (error_file != NULL)
  {
    t = time(NULL);
    fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

// LIST File
// ---------
  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
    printf("\nERROR (%s): ARGOLIST_DIR <-- no such environment variable",
           PRGNAME);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  sprintf (buffer,"%s",argv[5]);
  printf ("%s: Opening list file %s ", PRGNAME, buffer);

  if ((list_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nERROR (%s): Opening list file %s\n", PRGNAME, buffer);

    if (trace_file != NULL)
     fprintf (trace_file,"ERROR (%s): Opening list file %s\n",
              PRGNAME, buffer);
    if (error_file != NULL)
     fprintf (error_file,"ERROR (%s): Opening list file %s\n",
              PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf("%s: OK -> list file opened\n", PRGNAME);
  if (trace_file != NULL)
    fprintf (trace_file,"%s: List file %s successfully opened\n",
             PRGNAME, buffer);

// Open Input Counts File
// ----------------------
  if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): CNTFILEDIR <-- no such environment variable\n",
           PRGNAME);
    if (trace_file != NULL)
      fprintf (trace_file,
               "ERROR (%s) : CNTFILES_DIR <-- no such environment variable\n",
               PRGNAME);
    if (error_file != NULL)
      fprintf (error_file,
               "ERROR (%s): CNTFILES_DIR <-- no such environment variable\n",
               PRGNAME);

    if (trace_file != NULL)                  // Close Log files
      fclose (trace_file);
    if (error_file != NULL)
      fclose (error_file);

    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return(ret_val);
  }

  sprintf(pathname_input_filename, "%s/%s", ptr_path_cnt, input_filename);

  if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
  {
    printf ("\nERROR (%s): Opening input file %s\n",
            PRGNAME, pathname_input_filename);

    if (trace_file != NULL)
      fprintf (trace_file, "ERROR (%s): Opening input file %s\n",
               PRGNAME, pathname_input_filename);
    if (error_file != NULL)
      fprintf (error_file, "ERROR (%s): Opening input file %s\n",
                PRGNAME, pathname_input_filename);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
  }
  else
  {
    printf ("%s: Input file %s successfully opened\n",
            PRGNAME, pathname_input_filename);
    if (trace_file != NULL)
      fprintf (trace_file,
              "%s: Input file %s successfully opened\n",
              PRGNAME, pathname_input_filename);

// META File
// ---------
    if (get_meta_file_name (input_filename) != 0)
    {
      printf ("ERROR (%s): Finding meta file name", PRGNAME);
      if (error_file != NULL)
      fprintf (error_file,
               "\nERROR (%s): Finding meta file name - returned name %s\n",
               PRGNAME, input_filename);
      if (trace_file != NULL)
        fprintf (trace_file,
                 "\nERROR (%s): Finding meta file name - returned name %s\n",
                 PRGNAME, input_filename);
      ret_val = 1;
    }
    else
    {                                             // Success: got meta filename
      if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
      {
        printf("%s: METAFILES_DIR <-- no such environment variable\n", PRGNAME);
        if (trace_file != NULL)
        fprintf (trace_file,
                 "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                 PRGNAME);

        if (error_file != NULL)
          fprintf (error_file,
                   "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                   PRGNAME);

        if (trace_file != NULL)                  // Close Log files
          fclose (trace_file);
        if (error_file != NULL)
          fclose (error_file);

        ret_val = 1;
      } // endif  ptr_path_meta

      sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

      if (read_type1_meta_file (pathname_input_filename, &s) != 0)
      {
        printf ("\nERROR (%s): Reading meta file", PRGNAME);
        if (trace_file != NULL)
          fprintf (trace_file,"ERROR (%s): Reading meta file %s\n",
                   PRGNAME, pathname_input_filename);
        if (error_file != NULL)
          fprintf (error_file, "ERROR (%s): Reading meta file %s\n",
                   PRGNAME, pathname_input_filename);
        ret_val = 1;
      }
      else
      {
        if (trace_file != NULL)
          fprintf (trace_file, "%s: Meta file %s successfully read\n",
                   PRGNAME, pathname_input_filename);
          printf ("\n%s: Meta file %s successfully read\n",
                  PRGNAME, pathname_input_filename);


// Convert Counts to Physical Units
// --------------------------------
        sprintf (input_filename,"%s%s",output_directory,argv[1]);

        if ((ptr = strrchr(input_filename,'.')) == NULL)
          strcat (input_filename,".phy");
        else
          sprintf(ptr,".phy");

        printf ("%s: Output Path & File Name - %s", PRGNAME,input_filename);

        if (convert_sio_ctd_data(&s, input_filename) == 0)
        {
          printf ("\n%s: CONVERT_TYPE4_DATA  successful\n", PRGNAME);
          if (trace_file != NULL)
            fprintf (trace_file, "%s: CONVERT_TYPE4_DATA successful\n", PRGNAME);
        }
        else
        {
          printf ("\nERROR (%s): CONVERT_TYPE4_DATA\n");
          if (trace_file != NULL)
            fprintf (trace_file, "ERROR (%s): CONVERT_TYPE4_DATA\n",
                     PRGNAME);
          if (error_file != NULL)
            fprintf (error_file, "ERROR (%s): CONVERT_TYPE4_DATA\n",
                     PRGNAME);
          ret_val = 1;
        } // endif: convert_sio_ctd_data
      } // endif: read_type1_meta_file
    } // endif: get_meta_file_name
  } // endif: input_file

// Write finish message to trace & error files
// -------------------------------------------
  t = time(NULL);
  if ( ret_val == 0)
    strcpy(msg, "finished");
  else
    strcpy(msg, "terminated");

  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));

  if (trace_file != NULL)
   fprintf (trace_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

  if (error_file != NULL)
   fprintf (error_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

// Close all files
// ---------------
  if (input_file != NULL)
    fclose (input_file);

  if (trace_file != NULL)
    fclose (trace_file);

  if (error_file != NULL)
    fclose (error_file);

  if (list_file != NULL)
    fclose (list_file);

  return ret_val;
}
