/******************************************************************************   
   07-Mar-2005   XXD-convert ITS-90 temperature to ITS-68 temperature to get compute salinity(FSI floats only)
   
   cnt2phy_ts14.cpp
   ============
   Converts the counts ARGOS data for temperature/salinity/pressure
   TS14 floats manufactured in 2002 by Webb Research 
   Cooperation and deployed by Steve Riser
   
   Author: Xiangdong Xia
   Date:   March 2003
   Note: adjust pressure to average neighbouring pressure values
   Revision: 08-07-2003 (XX-
   In the header part salinity needs to use physical pressure to compute,not count value.)
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <vector>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include <iostream>

#define max_scans 2000  //Maximum scans for each profile
#define buff_size 256   //Maximum length of each scan read from *.str
using namespace std;

char *strupr(char* ptr_to_string_to_be_upper_cased); /* function prototype */
int nbrk ( char *original, int *a, int n );

FILE *input_file, *out_file, *error_file, *trace_file, *list_file, *meta_file;

char PRGNAME[] ="cnt2phy_ts14";
char buffer[512];

struct aoml_cals {double pgain;
                  double poff;
                  double tgain;
                  double toff;
                  double cgain;
                  double coff;
                  double p1;
                  double p2;
                  double p3;
                  double dp1;
                  double dp2;
		  double dp3;
		  double cpugain;
		  double cpuoff;
		  double auxgain;
		  double auxoff;                   
                  };

union u_cals {struct aoml_cals s;
              double c[sizeof(struct aoml_cals)/sizeof(double)];
             };

struct ts14_meta_struct {
              int AOML_number;
              int WMO_id;
              int ARGOS_id;
              int wmo_instrument;
              int wmo_recorder;
              float cycle_time;
              int exp_no;
              char float_type[40];
              char PI[60];
              float launch_time;
              float start_cycle_time;
              char hex_id[20];//newly added hexadecimal ARGOS ID
	      
	      int year;
    	      int month;
   	      int day;
 	      int hour;
	      int minute;
	      float lat;
	      float latm;
	      float lon;
	      float lonm;  
			  
              union u_cals cals;
                         };

long ymd2julday(int, int, int);

/**********************************************************************************
                  Convert lower case characters to upper case 
***********************************************************************************/

char *strupr(char *input)          // changes characters to upper case
{
   int  i;

   for (i=0; input[i] != '\0'; i++)
     input[i] = toupper(input[i]);
   return input;
}

/******************************************************************************
  This function is used by FSI floats only.SBE floats report salinity directly.
  convert conductivity to salinity standard function,convert ITS-90 temperature to ITS-68 temperature
  to use the conductivity to salinity conversion equation.
******************************************************************************/
float salinity (float c, float t, float p)
{  
  double t2,t3,t4,r,r1,r3,r4,r5,rp,rt,ft,d,ds,s;
  double t68;
  
  //cout<<"t="<<t<<" p="<<p<<" c="<<c;
/*
  if ((c>65.0) || (c<0.0) || (t>45.0) || (t<-3.0) || (p>12000.0) || (p<-20.0))
    return (0.0);
*/  
  if ((c<0.0) || (t<-3.0) || (p<-20.0))
    return (0.0);
  t68=t*1.00024;  
  t2 = t68 * t68;
  t3 = t2 * t68;
  t4 = t3 * t68;
  r  = c/42.914;
  rt = .6766097 + .0200564 * t68 + .0001104259 * t2
       - .00000069698 * t3 + 1.0031E-09 * t4;
  d  = 1 + .03426 * t68 + .0004464 * t2 + (.4215 - .003107 * t68) * r;
  rp = 1 + p * (.0000207 - 6.37E-10 * p + 3.989E-15 * p * p) / d;
  rt = r / rp / rt;
  ft = (t68 - 15) / (1 + .0162 * (t68 - 15));
  r1 = sqrt(rt);
  r3 = rt * r1;
  r4 = rt * rt;
  r5 = r4 * r1;
  ds = ft * (.0005 - .0056 * r1 - .0066 * rt -
             .0375 * r3 + .0636 * r4 - .0144 * r5);
  s = .008 - .1692 * r1 + 25.3851 * rt + 14.0941 * r3
      - 7.0261 * r4 + 2.7081 * r5 + ds;
  //cout<<" s="<<s<<endl;    
  return ((float)s);
}


/**********************************************************************************
               Convert temperature count to physical units
***********************************************************************************/

float compute_temperature (struct ts14_meta_struct *s, int t_counts)
{
 return (t_counts*s->cals.s.tgain + s->cals.s.toff);	
}

/**********************************************************************************
                Convert pressure count to physical units(for drift data)
***********************************************************************************/


float compute_drift_pressure (struct ts14_meta_struct *s, int p_counts)
{
 float p;
 p=s->cals.s.pgain*p_counts+s->cals.s.poff;
 return(p);
}



/**********************************************************************************
     Convert pressure count to physical units(for profile data) based on bin number
***********************************************************************************/


float compute_pressure (struct ts14_meta_struct *s, int bin_num)
{
 int j1,j2,j3;
 float p;
 
 j1=(int)(s->cals.s.p1/s->cals.s.dp1);
 j2=(int)((s->cals.s.p2-s->cals.s.p1)/s->cals.s.dp2+j1);
 j3=(int)((s->cals.s.p3-s->cals.s.p2)/s->cals.s.dp3+j2);
 
 if((bin_num>=0) && (bin_num<=j1)) p=bin_num*s->cals.s.dp1;
 if((bin_num>j1) && (bin_num<=j2)) p=(bin_num-j1)*s->cals.s.dp2+s->cals.s.p1;
 if((bin_num>j2) && (bin_num<=j3)) p=(bin_num-j2)*s->cals.s.dp3+s->cals.s.p2; 
 
 return(p);
}

/******************************************************************************
  convert conductivity, specific for TS14 float
******************************************************************************/


float compute_conductivity (struct ts14_meta_struct *s, int s_counts, int t_counts)
{
 float t,c;
 t=compute_temperature(s,t_counts);
 c= s_counts*s->cals.s.cgain + s->cals.s.coff + t;//calibration equation 
 return(c);
}



/******************************************************************************
  convert conductivity to salinity specific for TS14 float based on pressure physical values
******************************************************************************/


float compute_salinity_p (struct ts14_meta_struct *s, int s_counts, int t_counts,float pressure)
{
 float t,c;
 float sa; 
 t=compute_temperature(s,t_counts);
 c= s_counts*s->cals.s.cgain + s->cals.s.coff + t;//calibration equation
  
 sa=salinity(c,t,pressure);
 
 return(sa);
}
/**********************************************************************************
                Convert CPU voltage count to physical units
***********************************************************************************/


float compute_cpu (struct ts14_meta_struct *s, int cpu_counts)
{
 float cpu;
 cpu=s->cals.s.cpugain*cpu_counts+s->cals.s.cpuoff;
 return(cpu);
}

/**********************************************************************************
                Convert AUX voltage count to physical units
***********************************************************************************/


float compute_aux (struct ts14_meta_struct *s, int aux_counts)
{
 float aux;
 aux=s->cals.s.auxgain*aux_counts+s->cals.s.auxoff;
 return(aux);
}


/***************************************************
Convert year,month,day to julian day
*****************************************************/

long ymd2julday(int year, int month, int day)
{
   int  ic, iy;
   long julday;

   if ( month > 2 )
     month = month - 3;
   else if ( month < 3 )
   {
     month = month + 9;
     year = year - 1;
   }

   ic = (int) floor((double)(year/100) );
   iy = year - 100*ic;

// printf("ic: %d iy: %d\n", ic, iy);
 
   julday =   (long) floor((double)(146097*ic)/4) 
             + (long) floor((double)(1461*iy)/4)
             + (long) floor((double)(153*month+2)/5) 
             + day + 1721119;
   julday = julday - 2440000;
   return julday;
}



/**********************************************************************************
                Compute julian date from 1997
***********************************************************************************/
double dfr97 (const int yp, const int mp, const int dp,
              const int hr, const int minutes, const int secs)
{
  int    yr, mo, dy, day_from_97;
  long   passed_jd, jd_xx0101, jd_970101;
  double d_jd;

  yr = yp;
  mo = mp;
  dy = dp;

  if (yr <= 15)
   yr += 2000;
  else if (yr < 1900)
   yr += 1900;

  if ((yr<1997) || (yr>2015) || (mo<1) || (mo>12) || (dy<1) || (dy>31))
   return (0.0);

  passed_jd = ymd2julday( yr, mo, dy ) + 1;     // (current day/current month/current
                                                //  year)
  dy = 1;
  mo = 1;
  jd_xx0101 = ymd2julday( yr, mo, dy );         // (1/1/current year)
  yr = 1997;
  jd_970101 = ymd2julday( yr, mo, dy );         // (1/1/1997)
  day_from_97 = (int) (passed_jd - jd_970101);  // Days from 1/1/1997 (ignoring hrs,
                                                // mins, secs)
  d_jd = day_from_97;
  d_jd += (hr+(minutes + secs/60.0)/60.0)/24.0; // Converts hrs, mins, secs to days
  return (d_jd);                                // and adds this to d_jd
}


/**********************************************************************************
                Convert Counts Data to Physical Data File(Key part)
		for drift pressure, use pgain,poff to compute physical pressure
		for drift temperature,use tgain,toff
		for profile pressure, use p1,p2,p3,dp1,dp2,dp3 to compute physical pressure
		for profile temperature, use tgain,toff
		for profile salinity and drift salinity, use standard conductivity conversion routine
		this routine requires conductivity counts,temperature counts,and pressure counts
***********************************************************************************/


int convert_data(struct ts14_meta_struct *s, char *output_name)
{
 int  i1=0,i2=0,i3=0,i4=0,iflg=0,breaks=0,j=0,jj=0;
 //new added variables in order to call the compute_salinity function since it need several input parameters
 int ip=0,it=0,ic=0;//initial pressure,temperature,conductivity 
 int fdp=0,fdt=0,fdc=0;//first half drift pressure,temperature,conductivity
 int sdp=0,sdt=0,sdc=0;//second half drift pressure,temperature,conductivity
 int dp[10],dt[10],ds[10];//store drift P,T,S values 
 int vcpu,vaux;//voltage cpu,aux counts
 
 char *ptr_phy_filename;
 char buffer1[512];//used for distinguish from buffer[512] in drift part
 vector<float> P_vector,PNew_vector,T_vector,C_vector,S_vector;
 vector<int> Flag_vector;
 vector<int> i1_vector,i2_vector,i3_vector,i4_vector;
 int flag_data=0;

 
 if ((out_file = fopen(output_name,"w")) == NULL)
 {
  if (trace_file != NULL)
   fprintf (trace_file," CONVERT_DATA: error opening output file %s\n",
    output_name);
  if (error_file != NULL)
   fprintf (error_file," CONVERT_DATA: error opening output file %s\n",
    output_name);
  return 1;
 }
 
 
 if (trace_file != NULL)
  fprintf (trace_file," CONVERT_DATA: output file %s successfully opened\n",output_name);

 printf ("\nOutput file %s successfully opened", output_name);

// Write Output Profile Physical Unit File Name to List File
// ---------------------------------------------------------
 if ( (ptr_phy_filename = strrchr(output_name, '/')) == NULL)
 {
  printf ("\nCan't get phy file basename %s", output_name);
  if (trace_file != NULL)
   fprintf (trace_file,
            "\n%s : Can't get phy file basename %s", PRGNAME, output_name);
   if (error_file != NULL)
    fprintf (error_file,
             "\n%s: Can't get phy file basename %s", PRGNAME, output_name);
   return 1;
 } // endif: ptr_phy_filename
 
 ptr_phy_filename++;
 fprintf (list_file,"%s\n", ptr_phy_filename);
  
 while (fgets(buffer,sizeof(buffer),input_file) != NULL)
 {
 
  if (strstr(strupr(buffer),"INITIAL PRESSURE (BAR)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&ip) != 1)
    ip = 0;
   sprintf (buffer, "INITIAL PRESSURE (DBAR)                 %-6.1f\n",
    (float)ip*10);   
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(strupr(buffer),"INITIAL TEMPERATURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&it) != 1)
    it = 0;
   sprintf (buffer, "INITIAL TEMPERATURE (DEG C)             %-6.3f\n",
    compute_temperature(s,it));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  
  if (strstr(buffer,"INITIAL CONDUCTIVITY") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&ic) != 1)
    ic = 0;
   
   sprintf (buffer,"INITIAL CONDUCTIVITY (MS/CM)            %-6.3f\n",
    compute_conductivity(s,ic,it));
   fwrite (buffer,1,strlen(buffer),out_file);
   
   sprintf (buffer,"INITIAL SALINITY (PSU)                  %-6.3f\n",
    compute_salinity_p(s,ic,it,ip*10));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
    
  if (strstr(strupr(buffer),"START OF DRIFT PRESSURE (COUNTS)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&fdp) != 1)
    fdp = 0;
   sprintf (buffer, "START OF DRIFT PRESSURE (DBAR)          %-6.1f\n",
    compute_drift_pressure(s,fdp));
   cout<<"DRIFT PRESSURE START="<<fdp<<" "<<compute_drift_pressure(s,fdp)<<endl; 
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  
  if (strstr(strupr(buffer),"START OF DRIFT TEMPERATURE (COUNTS)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&fdt) != 1)
    fdt = 0;
   sprintf (buffer, "START OF DRIFT TEMPERATURE (DEG C)      %-6.3f\n",
    compute_temperature(s,fdt));
   cout<<"DRIFT TEMPERATURE START="<<fdt<<" "<<compute_temperature(s,fdt)<<endl; 
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(buffer,"START OF DRIFT CONDUCTIVITY (COUNTS)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&fdc) != 1)
    fdc = 0;
    
    sprintf (buffer,"START OF DRIFT CONDUCTIVITY (MS/CM)     %-6.3f\n",
    compute_conductivity(s,fdc,fdt));
   cout<<"DRIFT CONDUCTIVITY START="<<fdc<<" "<<fdt<<" "<<compute_conductivity(s,fdc,fdt)<<endl; 
   fwrite (buffer,1,strlen(buffer),out_file);

   sprintf (buffer, "START OF DRIFT SALINITY (PSU)           %-6.3f\n",
    compute_salinity_p(s,fdc,fdt,compute_drift_pressure(s,fdp)));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
    
  if (strstr(strupr(buffer),"END OF DRIFT PRESSURE (COUNTS)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&sdp) != 1)
    sdp = 0;
   sprintf (buffer, "END OF DRIFT PRESSURE (DBAR)            %-6.1f\n",
    compute_drift_pressure(s,sdp));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  
  
  if (strstr(strupr(buffer),"END OF DRIFT TEMPERATURE (COUNTS)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&sdt) != 1)
    sdt = 0;
   sprintf (buffer, "END OF DRIFT TEMPERATUR (DEG C)         %-6.3f\n",
    compute_temperature(s,sdt));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
 
  if (strstr(buffer,"END OF DRIFT CONDUCTIVITY (COUNTS)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&sdc) != 1)
    sdc = 0;
    
   sprintf (buffer,"END OF DRIFT CONDUCTIVITY (MS/CM)       %-6.3f\n",
    compute_conductivity(s,sdc,sdt));
   fwrite (buffer,1,strlen(buffer),out_file);

   sprintf (buffer,"END OF DRIFT SALINITY (PSU)             %-6.3f\n",
    compute_salinity_p(s,sdc,sdt,compute_drift_pressure(s,sdp)));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  
  if (strstr(buffer,"BATTERY CPU") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %d",&vcpu) != 1)
    vcpu = 0;
   sprintf (buffer,"BATTERY OF CPU (VOLT)                   %-6.1f\n",
    compute_cpu(s,vcpu));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  
  if (strstr(buffer,"BATTERY OF AUX") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %d",&vaux) != 1)
    vaux = 0;
   sprintf (buffer,"BATTERY OF AUX (VOLT)                   %-6.1f\n",
    compute_aux(s,vaux));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  
  if (strstr(buffer,"DRIFT PRESSURE (COUNTS)") != NULL)
  {
   sprintf (buffer1,"DRIFT PRESSURE (DBAR)                   ");
   fwrite (buffer1,1,strlen(buffer1),out_file);
     
   breaks=nbrk(buffer,dp,10);
   
   sprintf(buffer1,"drift pressure breaks= %d",breaks);
   fwrite (buffer1,1,strlen(buffer1),trace_file);
   
   for(j=0;j<breaks;j++){
     if(dp[0]==99999){
       sprintf(buffer1,"%-6.1f ",99999.); 
       fwrite (buffer1,1,strlen(buffer1),out_file);    
       break;
     }

     sprintf(buffer1,"%-6.1f ",compute_drift_pressure(s,dp[j])); 
     fwrite (buffer1,1,strlen(buffer1),out_file);
   }
   sprintf(buffer1,"\n"); 
   fwrite (buffer1,1,strlen(buffer1),out_file);
   continue;
  }
  
   
  
  if (strstr(buffer,"DRIFT TEMPERATURE (COUNTS)") != NULL)
  {
   sprintf (buffer1,"DRIFT TEMPERATURE (DEG C)               ");
   fwrite (buffer1,1,strlen(buffer1),out_file);
     
   breaks=nbrk(buffer,dt,10);
   
   sprintf(buffer1,"drift temperature breaks= %d",breaks);
   fwrite (buffer1,1,strlen(buffer1),trace_file);
   

   
   for(j=0;j<breaks;j++){
     if(dt[0]==99999){
       sprintf(buffer1,"%-6.1f ",99999.); 
       fwrite (buffer1,1,strlen(buffer1),out_file);    
       break;
     }

     sprintf(buffer1,"%-6.3f ",compute_temperature(s,dt[j])); 
     fwrite (buffer1,1,strlen(buffer1),out_file);
   }
   sprintf(buffer1,"\n"); 
   fwrite (buffer1,1,strlen(buffer1),out_file);
   continue;
  }
  
  
  
  if (strstr(buffer,"DRIFT CONDUCTIVITY (COUNTS)") != NULL)
  {
     
   breaks=nbrk(buffer,ds,10);
   
   sprintf(buffer1,"drift salinity breaks= %d",breaks);
   fwrite (buffer1,1,strlen(buffer1),trace_file);

   sprintf (buffer1,"DRIFT CONDUCTIVITY (MS/CM)              ");
   fwrite (buffer1,1,strlen(buffer1),out_file);
   
   for(j=0;j<breaks;j++){
     if(ds[0]==99999){
       sprintf(buffer1,"%-6.1f ",99999.); 
       fwrite (buffer1,1,strlen(buffer1),out_file);    
       break;
     }

     sprintf(buffer1,"%-6.3f ",compute_conductivity(s,ds[j],dt[j])); 
     fwrite (buffer1,1,strlen(buffer1),out_file);
   }
   sprintf(buffer1,"\n"); 
   fwrite (buffer1,1,strlen(buffer1),out_file);
   
   
   sprintf (buffer1,"DRIFT SALINITY (PSU)                    ");
   fwrite (buffer1,1,strlen(buffer1),out_file);
   
   for(j=0;j<breaks;j++){
     if(ds[0]==99999){
       sprintf(buffer1,"%-6.1f ",99999.); 
       fwrite (buffer1,1,strlen(buffer1),out_file);    
       break;
     }

     sprintf(buffer1,"%-6.3f ",compute_salinity_p(s,ds[j],dt[j],compute_drift_pressure(s,dp[j]))); 
     fwrite (buffer1,1,strlen(buffer1),out_file);
   }
  
   
   sprintf(buffer1,"\n"); 
   fwrite (buffer1,1,strlen(buffer1),out_file);
   continue;
  }
  
  
  if ((strstr(buffer,"COLUMNS") != NULL) || (strstr(buffer,"COLUMNS")))
  {
   if (strstr(strupr(buffer),"COLUMNS") != NULL)
   {
    fprintf (out_file,"NUMBER OF COLUMNS                       5\n");
    fprintf (out_file,"1. COLUMN                               PRESSURE (DBAR)\n");
    fprintf (out_file,"2. COLUMN                               TEMPERATURE-90 (DEG C)\n");
    fprintf (out_file,"3. COLUMN                               CONDUCTIVITY \n");
    fprintf (out_file,"4. COLUMN                               SALINITY (PSU)\n");
    fprintf (out_file,"5. COLUMN                               CHECKSUM FLAG\n");
   }
   rewind(meta_file);
   while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
    if (strstr(buffer,"================") != NULL)
     break;
   while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
   {
    if (strstr(buffer,"================") != NULL)
	{
	 sprintf (buffer,"==========================================================\n");
     fwrite (buffer,1,strlen(buffer),out_file);
     break;
	}
    fwrite (strupr(buffer),1,strlen(buffer),out_file);
   }

   while (fgets(buffer,sizeof(buffer),input_file) != NULL)
    if (strstr(buffer,"LATITUDE") != NULL)
     break;
   fwrite (buffer,1,strlen(buffer),out_file);
   while (fgets(buffer,sizeof(buffer),input_file) != NULL)
   {
    fwrite (buffer,1,strlen(buffer),out_file);
    if (strstr(buffer,"================") != NULL)
     break;
   }
   iflg = 1;
   continue;
  }
  if (iflg == 1)
  {
  
   if (sscanf(buffer,"%d %d %d %d",&i1,&i2,&i3,&i4) == 4)
    {
      if (i1==0 & i2==0 & i3==0 & i4==0)  // no data
      { flag_data=0;//no data
        i1_vector.push_back(i1);i2_vector.push_back(i2);i3_vector.push_back(i3);i4_vector.push_back(i4);

/*      
        fprintf (out_file,"%6.1f   %6.3f   %6.3f   %1d\n",
             (float)i1,(float)i2,(float)i3,i4);
*/	 

	
	   
      }
      else
      {flag_data=1;//has data
       i1_vector.push_back(i1);i2_vector.push_back(i2);i3_vector.push_back(i3);i4_vector.push_back(i4);
       
     }
    }

    
  }
  else
   fprintf (out_file,buffer);
 }
    //get original pressure values(no averaging performed)
    if(i1_vector.size()!=0){ 
     for(jj=0;jj<i1_vector.size();jj++){
      P_vector.push_back(compute_pressure(s,i1_vector[jj]));
     } 
    }
 
   cout<<"P_vector.size()="<<P_vector.size()<<endl;
  
  if(P_vector.size()!=0){ 
   //adjust pressure values  
   //add the last pressure(largest pressue)
   PNew_vector.push_back((compute_pressure(s,P_vector.size()-1)+compute_pressure(s,P_vector.size()))/2.0);
   for(jj=0;jj<P_vector.size()-1;jj++){
    PNew_vector.push_back((P_vector[jj]+P_vector[jj+1])/2.0);//average neighbouring pressure values
   } 
   //get T,C,S,Flag vector
   for(jj=0;jj<PNew_vector.size();jj++){
    T_vector.push_back(compute_temperature(s,i2_vector[jj]));
    C_vector.push_back(compute_conductivity(s,i3_vector[jj],i2_vector[jj]));
    //get salinity based on pressure physical value,not bin number since pressure averaged should be used
    S_vector.push_back(compute_salinity_p(s,i3_vector[jj],i2_vector[jj],PNew_vector[jj]));
    

    //reset P,T,C,S=0 if in cnt file, T=C=0, they are missing messages. 
    if(i2_vector[jj]==0 && i3_vector[jj]==0){
      PNew_vector[jj]=0;
      T_vector[jj]=0;
      C_vector[jj]=0;
      S_vector[jj]=0;
      i4_vector[jj]=0;
    }
    Flag_vector.push_back(i4_vector[jj]);
   }
   
   cout<<"PNew_vector.size()="<<PNew_vector.size()<<" T_vector.size()="<<T_vector.size()<<endl;
  //print data to phy file
  if(flag_data=1){
  for(jj=0;jj<PNew_vector.size();jj++){
     fprintf (out_file,"%6.1f   %6.3f   %6.3f   %6.3f   %1d\n",PNew_vector[jj],T_vector[jj],C_vector[jj],S_vector[jj],Flag_vector[jj]);
  } 
  }
  }
 if (trace_file != NULL)
  fprintf (trace_file," CONVERT_DATA:%s completed successfully\n",output_name);
 return 0;

}

/**********************************************************************************
                Get Meta Data File Name
***********************************************************************************/


int get_meta_file_name (char *str_name)
{
 char *ptr,tmp_buff[80];
 int len;

// Assume str_name = *.hex, returns *.meta in str_name

 if ((ptr = strrchr(str_name,'_')) == NULL)    // get pointer to last '_'
  strcat(str_name,".meta");                    
 else
 {
  sprintf (tmp_buff,"%s\0", ptr);
  len = strlen(str_name) - strlen(tmp_buff);
  strncpy(tmp_buff, str_name, len);
  tmp_buff[len] = '\0';
  sprintf (str_name,"%s.meta",tmp_buff);
 }
 return 0;
}


/**********************************************************************************
                Read Meta Data File
***********************************************************************************/


int read_meta_file (char *meta_name, struct ts14_meta_struct *meta_struct)
{

  int i,j,syear,smonth,sday,shour,smin;
  float x,lat,latm,lon,lonm;
  int lyear,lmonth,lday,lhour,lmin;
  char *ptr;
  char buffer[512];

  char *cal_keys[] = {"PGAIN=", "POFF=","TGAIN=", "TOFF=","CGAIN=", "COFF=",
    "P1=", "P2=","P3=","DP1=", "DP2=","DP3=","CPUGAIN=","CPUOFF=","AUXGAIN=","AUXOFF=","\0"};

  meta_struct->cycle_time = (float) 0.0;
  meta_struct->start_cycle_time = (float) 0.0;

  printf("\nOpening meta file!!!!!! %s\n",meta_name);

  if ((meta_file = fopen(meta_name,"r")) == NULL)
  {
    printf ("Error opening meta file!!!!!! %s\n",meta_name);

    if (error_file != NULL)
      fprintf (error_file," ERROR opening meta file %s\n",meta_name);

    return(1);
  }

  if (trace_file != NULL)
    fprintf (meta_file," Meta file %s successfully opened\n",meta_name);

  // Reads a line from meta_file into buffer and checks if NULL
  while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
  {
    // Convert buffer to upper case, return a pointer to the first
    // occurrence of INTERNAL ID NUMBER in buffer, and check if NULL
    if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
    {
      // Read chars from buffer and store them in AOML_number inside
      // structure.
      //
      // Note: * means the field does not need to be included in vector of
      // variable names to write to
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) != 1)
        meta_struct->AOML_number = 0;
      continue;
    }

    if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) != 1)
        meta_struct->WMO_id = 0;
      continue;
    }

    if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) != 1)
        meta_struct->ARGOS_id = 0;
      continue;
    }

    if (strstr(strupr(buffer),"DELAY OF FIRST DOWN TIME") != NULL)
    {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %f",&x);
      meta_struct->start_cycle_time += (float) (x/24.0);
      continue;
    }

    if (strstr(strupr(buffer),"WMO INSTRUMENT TYPE") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_instrument) != 1)
      meta_struct->wmo_instrument = 0;
     continue;
    }

    if (strstr(strupr(buffer),"WMO RECORDER TYPE") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_recorder) != 1)
      meta_struct->wmo_recorder = 0;

	 continue;
    }

// DOWN TIME was changed to CYCLE TIME in meta files (CS)
    if (strstr(strupr(buffer),"CYCLE TIME") != NULL)
    {
      sscanf(buffer,"%*s %*s %*s %f",&x);
      meta_struct->cycle_time = x;
      continue;
    }

// UP TIME was changed to SURFACE TIME in meta files (CS)
    if (strstr(strupr(buffer),"SURFACE TIME") != NULL)
    {
      sscanf(buffer,"%*s %*s %*s %f",&x);
// (CS) now obsolete     meta_struct->cycle_time += (float) (x/24.0);
      meta_struct->start_cycle_time -= (float) (x/24.0);
      continue;
    }

    // start_cycle_time = FIRST DOWN TIME - SURFACE TIME + START TIME
    //            ^                 ^
    //            6 hours           1 day     dfr97 days
    if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",
             &sday,&smonth,&syear,&shour,&smin);

      meta_struct->start_cycle_time += (float) (dfr97(syear,smonth,sday,shour,smin,0));
      continue;
    }

    if (strstr(strupr(buffer),"PROGRAM NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->exp_no) != 1)
        meta_struct->exp_no = 0;

      continue;
    }
    //newly added hexadecimal ARGOS ID field 
    if (strstr(strupr(buffer),"HEXADECIMAL ARGOS ID") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %s",&meta_struct->hex_id) != 1)
      sprintf (meta_struct->hex_id,"\0");
     continue;
    }
     

    if ((strstr(strupr(buffer),"INSTRUMENT TYPE") != NULL) && (strstr(strupr(buffer),"WMO") == NULL))
    {
      if (sscanf(buffer,"%*s %*s %s",meta_struct->float_type) == 0)
      {
        sprintf (meta_struct->float_type,"\0");
        continue;
      }
    }

    if (strstr(strupr(buffer),"PI     ") != NULL)
    {
      if (sscanf(buffer,"%*s %s",meta_struct->PI) == 0)
      {
        sprintf (meta_struct->PI,"\0");
        continue;
      }

      if ((ptr = strstr(&buffer[3],meta_struct->PI)) != NULL)
        sprintf (meta_struct->PI,"%s",ptr);

      if ((ptr = strstr(meta_struct->PI,"\n")) != NULL)
        sprintf (ptr,"\0");

      continue;
    }

    if ((strstr(strupr(buffer),"LAUNCH TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",
           &lday,&lmonth,&lyear,&lhour,&lmin);
      meta_struct->launch_time = (float) (dfr97(lyear,lmonth,lday,lhour,lmin,0));

      meta_struct->year=lyear;
      meta_struct->month=lmonth;
      meta_struct->day=lday;
      meta_struct->hour=lhour;
      meta_struct->minute=lmin;
      continue;
    }

    if ((strstr(strupr(buffer),"LAUNCH POSITION") != NULL) && (strstr(strupr(buffer),"LONM") != NULL)) {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d %d %d %d",&lat,&latm,&lon,&lonm);
      meta_struct->lat=lat;
      meta_struct->latm=latm;
      meta_struct->lon=lon;
      meta_struct->lonm=lonm;
      continue;
    }


    i = 0;
    while ((j = strlen(cal_keys[i])) > 1)
    {
      // Get pointer to 1st occurence of cal_keys[i] in buffer
      if ((ptr = (char *)strstr(buffer,cal_keys[i])) != NULL)
        meta_struct->cals.c[i] = atof((const char *)(ptr+j));

      i++;
    }

  }
  

  return 0;
}

/****************************************************************************
  char buffer[]="ABC DEF GHJ 23 345 67 890   76567"; 

  Get number of words from original and convert words into numbers, 
  save it to array a
  
****************************************************************************/

int nbrk ( char *original, int *a, int n ){ 
                
                  int i = 0; 
                  char *temp, *p; 



                  /* Make a copy of original because strtok modifies
                its argument */ 
                  temp =(char*) malloc ( strlen ( original ) * sizeof
                (*temp) + 1 ); 

                  

                  strcpy ( temp, original ); 

                  /* Find first token */ 
                  p = strtok ( temp, " " ); 

                  /* Loop while there are more tokens AND places left
                in a */ 
                  while ( p != NULL && i < n ) { 
                    /* Assumes if the first cell is a digit then the
                whole token is a number */ 
                    if ( isdigit ( *p ) ) 
                      /* Convert the string to an int and assign it to
                a[i], increment i */ 
                      a[i++] = (int)strtol ( p, NULL, 10 ); 

                    /* Find the next token in the same string */ 
                    p = strtok ( NULL, " " ); 
                  } 

                  /* Always free your malloc'd memory */ 
                    delete temp; 

                  /* Return the elements filled in a */ 
                  return i; 
} 


int main (int argc, char *argv[])
{

 char     *ptr_path_meta, *ptr_path_cnt, *ptr_path_phy, 
          *ptr_path_log, *ptr_path_list,
          pathname_input_filename[161]="",
          pathname_trace[161]="", pathname_error[161]="";

 char     input_filename[80],output_directory[256], 
          msg[11]="", title[11]="", *ptr;
 struct   ts14_meta_struct s;
                                                    // long t;
 long t;
 int      ret_val = 0;

// Program Name
// ------------
  printf("Entering Program: %s\n", PRGNAME);
  strcpy(title, "Program calibrates counts files for TS14 float");
  printf("%s\n", title);

// Input Arguments 
// ---------------
 if (argc < 6)
 {
  printf("Format: %s cnt_infile phy_outdir trace_outile error_outfile\n", PRGNAME);
  printf("\n");
  printf("cnt_infile:    counts input file name\n");
  printf("phy_outdir:    output directory for generated calibrated data file(s)\n");
  printf("trace_outile:  output directory for generated trace file\n");
  printf("error_outfile: output directory for generated error file\n");
  printf("list_outfile:  output directory for generated list file\n");
  return 1;
 }

// Input Counts File Name 
// ----------------------
 printf ("%s: Input Hex File Name - %s\n", PRGNAME, argv[1]);
 sprintf (input_filename,"%s",argv[1]);

// Output directory path: defaults to PHYFILES_DIR
// -----------------------------------------------
 if ( (ptr_path_phy = getenv("PHYFILES_DIR")) == NULL)
 {
  printf("ERROR (%s): PHYFILES_DIR <-- no such environment variable\n", PRGNAME);
  t = time(NULL);
  printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
  ret_val = 1;
  return ret_val;
 }

 printf ("%s: Output Directory - %s\n", PRGNAME, argv[2]);
 sprintf (output_directory,"%s/",argv[2]);

// TRACE File
// ----------
 if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
 {
  printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
  t = time(NULL);
  printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
  ret_val = 1;
  return ret_val ;
 }

 printf ("%s: Opening trace file %s ", PRGNAME, buffer);
 sprintf (buffer,"%s",argv[3]);

 if ((trace_file = fopen(buffer,"a+")) == NULL)
 {
  printf ("\nERROR (%s): Opening trace file %s", PRGNAME, buffer);
  t = time(NULL);
  printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
  ret_val = 1;
  return ret_val;
 }
 printf("%s: OK -> trace file opened\n", PRGNAME);

 if (trace_file != NULL)
 {
  t = time(NULL);
  fprintf (trace_file,"=> %s: start %s", PRGNAME, ctime(&t));
 }

// ERROR File
// ----------
 sprintf (buffer,"%s",argv[4]);
 printf ("%s: Opening error file %s ", PRGNAME, buffer);

 if ((error_file = fopen(buffer,"a+")) == NULL)
 {
  printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
  if (trace_file != NULL)
   fprintf (trace_file," ERROR (%s): Opening file %s\n", PRGNAME, buffer);
  t = time(NULL);
  printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
  ret_val = 1;
  return ret_val;
 }
 printf("%s: OK -> error file opened\n", PRGNAME);

 if (trace_file != NULL)
   fprintf (trace_file,"%s: File %s successfully opened\n", PRGNAME, buffer);
 if (error_file != NULL)
 {
   t = time(NULL);
   fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

// LIST File
// ---------
  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
   printf("\nERROR (%s): ARGOLIST_DIR <-- no such environment variable",
          PRGNAME);
   t = time(NULL);
   printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 1;
   return ret_val;
  }

  sprintf (buffer,"%s",argv[5]);
  printf ("%s: Opening list file %s ", PRGNAME, buffer);

  if ((list_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nERROR (%s): Opening list file %s\n", PRGNAME, buffer);

    if (trace_file != NULL)
     fprintf (trace_file,"ERROR (%s): Opening list file %s\n",
              PRGNAME, buffer);
    if (error_file != NULL)
     fprintf (error_file,"ERROR (%s): Opening list file %s\n",
              PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }
  printf("%s: OK -> list file opened\n", PRGNAME);
  if (trace_file != NULL)
   fprintf (trace_file,"%s: List file %s successfully opened\n", 
            PRGNAME, buffer);

// Open Input Counts File
// ----------------------
 if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
 {
   printf("ERROR (%s): CNTFILEDIR <-- no such environment variable\n", PRGNAME);

  if (trace_file != NULL)
   fprintf (trace_file,
            "ERROR (%s) : CNTFILES_DIR <-- no such environment variable\n",
            PRGNAME);
  if (error_file != NULL)
   fprintf (error_file,
            "ERROR (%s): CNTFILES_DIR <-- no such environment variable\n",
            PRGNAME);

  if (trace_file != NULL)                  // Close Log files
    fclose (trace_file);
  if (error_file != NULL)
    fclose (error_file);

  t = time(NULL);
  printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
  ret_val = 1;
  return(ret_val);
 }

 sprintf(pathname_input_filename, "%s/%s", ptr_path_cnt, input_filename);

 if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
 {                                                 // Failure: could not open cnt file 
  printf ("\nERROR (%s): Opening input file %s", 
          PRGNAME, pathname_input_filename);

  if (trace_file != NULL)
    fprintf (trace_file, "ERROR (%s): Opening input file %s\n", 
             PRGNAME, pathname_input_filename);
   if (error_file != NULL)
    fprintf (error_file, "ERROR (%s): Opening input file %s\n", 
             PRGNAME, pathname_input_filename);
   t = time(NULL);
   printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 1;
   return ret_val;
 }
 else                                              // Success: cnt file opened
 {
  printf ("%s: Input file %s successfully opened\n", 
          PRGNAME, pathname_input_filename);
  if (trace_file != NULL)
   fprintf (trace_file,
            "%s: Input file %s successfully opened\n", 
            PRGNAME, pathname_input_filename);
  
// META File
// ---------
  if (get_meta_file_name (input_filename) != 0)
  {                                             // Failure: did not get meta filename     
   printf ("ERROR (%s): Finding meta file name", PRGNAME);
   if (error_file != NULL)
    fprintf (error_file,
             "\nERROR (%s): Finding meta file name - returned name %s\n",
             PRGNAME, input_filename);
   if (trace_file != NULL)
    fprintf (trace_file,
             "\nERROR (%s): Finding meta file name - returned name %s\n",
             PRGNAME, input_filename);
   ret_val = 1;
  }
  else
  {                                             // Success: got meta filename 
   if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
   {
    printf("%s: METAFILES_DIR <-- no such environment variable\n", PRGNAME); 
    if (trace_file != NULL) 
     fprintf (trace_file,
              "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
              PRGNAME);

    if (error_file != NULL)
     fprintf (error_file,
              "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
              PRGNAME);

    if (trace_file != NULL)                  // Close Log files
     fclose (trace_file);
    if (error_file != NULL)
     fclose (error_file);

    ret_val = 1;
   } // endif  ptr_path_meta

   sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

   if (read_meta_file (pathname_input_filename, &s) != 0)
   {                                         // Failure: couln't read meta_file
    printf ("\nERROR (%s): Reading meta file", PRGNAME);
    if (trace_file != NULL)
     fprintf (trace_file,"ERROR (%s): Reading meta file %s\n", 
              PRGNAME, pathname_input_filename);
    if (error_file != NULL)
     fprintf (error_file, "ERROR (%s): Reading meta file %s\n", 
              PRGNAME, pathname_input_filename);
    ret_val = 1;
   }
   else
   {
    if (trace_file != NULL)
     fprintf (trace_file, "%s: Meta file %s successfully read\n", 
              PRGNAME, pathname_input_filename);
    printf ("\n%s: Meta file %s successfully read\n", PRGNAME, pathname_input_filename);
 
// Convert Counts to Physical Units 
// --------------------------------
    sprintf (input_filename,"%s%s", output_directory, argv[1]); // !! using input storage
    
    if ( (ptr = strrchr(input_filename,'.')) == NULL)
     strcat (input_filename,".phy");
    else
     sprintf(ptr,".phy");                             // ???

     printf ("%s: Output Path & File Name - %s", PRGNAME,input_filename);
 
    if (convert_data(&s, input_filename) == 0)
    {
      printf ("\n%s: CONVERT_DATA  successful\n", PRGNAME);
      if (trace_file != NULL)
        fprintf (trace_file, "%s: CONVERT_DATA successful\n", PRGNAME);
    }
    else
    { 
      printf ("\nERROR (%s): CONVERT_DATA\n");
      if (trace_file != NULL)
        fprintf (trace_file, "ERROR (%s): CONVERT_DATA\n", 
                 PRGNAME);
      if (error_file != NULL)
        fprintf (error_file, "ERROR (%s): CONVERT_DATA\n", 
                 PRGNAME);
      ret_val = 1;
    } // endif: convert_data
   } // endif: read_meta_file
  } // endif: get_meta_file_name
 } // endif: input_file

// Write finish message to trace & error files
// -------------------------------------------
 t = time(NULL);
 if ( ret_val == 0)
   strcpy(msg, "finished");
 else
   strcpy(msg, "terminated");

 printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));

 if (trace_file != NULL)
  fprintf (trace_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

 if (error_file != NULL)
  fprintf (error_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

// Close all files
// ---------------
 if (input_file != NULL)
   fclose (input_file);

 if (trace_file != NULL)
   fclose (trace_file);

 if (error_file != NULL)
  fclose (error_file);

 if (list_file != NULL)
  fclose (list_file);

 return ret_val;
}
