/******************************************************************************
   Revision Date: 26-Sep-08 (LL)  - AIR PUMP TIME without 2*
   				  29-Dec-03 (XXD) - donot output arg
                  29-Dec-03 (XXD) - adapted from cnt2phy_ts19.
                  02-Dec-02 (CS)  - corrected output for cycles without data
                  07-Oct-02 (DB)  - original adapted from CNT2PHY_TS8

   cnt2phy_ts19.c
   ============
   Converts the counts ARGOS data for temperature/salinity/pressure
   University of Washington PALACE floats manufactured in 2000 by Webb Research
   Cooperation and deployed by Steve Riser
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>

#define max_scans 2000  //Maximum scans for each profile
#define buff_size 256   //Maximum length of each scan read from *.str

char *strupr(char* ptr_to_string_to_be_upper_cased); /* function prototype */

FILE *input_file, *out_file, *error_file, *trace_file, *list_file, *meta_file;

char PRGNAME[] ="CNT2PHY_TS19";
char buffer[1024];

struct aoml_ts19_cals {double TSLOPE;
                       double TOFF;
                       double PSLOPE;
                       double POFF;
                       double SSLOPE;
                       double SOFF;
                       double VSLOPE;
                       double VOFF;
                       double VACSLOPE;
                       double VACOFF;
                      };

union u_cals {struct aoml_ts19_cals s;
              double c[sizeof(struct aoml_ts19_cals)/sizeof(double)];
             };

struct type_ts19_meta_struct { int   AOML_number;
                               int   WMO_id;
                               int   ARGOS_id;
                               float cycle_time;
                               float start_time;
                               union u_cals cals;};

long ymd2julday(int, int, int);

char *strupr(char *input)          // changes characters to upper case
{
   int  i;

   for (i=0; input[i] != '\0'; i++)
     input[i] = toupper(input[i]);
   return input;
}

float compute_type_ts19_temperature (struct type_ts19_meta_struct *s, int t_counts)
{
  float t;
  //changed from F447 to EFFF
  t = (t_counts <= 0XEFFF) ? ((float) (s->cals.s.TSLOPE * t_counts + s->cals.s.TOFF)) :
           ((float) (-s->cals.s.TSLOPE * (0XFFFF - t_counts + 1) + s->cals.s.TOFF));
  return t;
}

float compute_type_ts19_pressure (struct type_ts19_meta_struct *s, int p_counts)
{
  float pres;
  pres = (p_counts <= 0X7FFF) ? ((float) (s->cals.s.PSLOPE * p_counts + s->cals.s.POFF)) :
           ((float) (-s->cals.s.PSLOPE * (0XFFFF - p_counts + 1) + s->cals.s.POFF));
  return pres;
}

float compute_type_ts19_salinity (struct type_ts19_meta_struct *s, int s_counts)
{
  float salt;
  //changed from F447 to EFFF
  salt = (s_counts <= 0XEFFF) ? ((float) (s->cals.s.SSLOPE * s_counts + s->cals.s.SOFF)) :
           ((float) (-s->cals.s.SSLOPE * (0XFFFF - s_counts + 1) + s->cals.s.SOFF));
  return salt;
}


float compute_battery_voltage (struct type_ts19_meta_struct *s, int v_counts)
{
 float b;
 b=(float) (s->cals.s.VSLOPE * v_counts + s->cals.s.VOFF);
 if (v_counts==0)
    b=0.;
 return(b);
}

float compute_vacuum_pressure (struct type_ts19_meta_struct *s, int vac_counts)
{
 float b;
 b=(float) (s->cals.s.VACSLOPE * vac_counts + s->cals.s.VACOFF);
 if (vac_counts==0)
    b=0.;
 return(b);
}

double dfr97 (const int yp, const int mp, const int dp,
     const int hr, const int minutes, const int secs, int *doy)
{
 int     yr, mo, dy, day_from_97;
 long    passed_jd, jd_xx0101, jd_970101;
 double  d_jd;

 yr = yp;
 mo = mp;
 dy = dp;
 if (yr <= 15)
  yr += 2000;
 else if (yr < 1900)
  yr += 1900;
 if ((yr<1997) || (yr>2015) || (mo<1) || (mo>12) || (dy<1) || (dy>31))
  return (0.0);
 passed_jd = ymd2julday( yr, mo, dy ) + 1;
 dy = 1;
 mo = 1;
 jd_xx0101 = ymd2julday( yr, mo, dy );
 yr = 1997;
 jd_970101 = ymd2julday( yr, mo, dy );
 day_from_97 = (int) (passed_jd - jd_970101);
 *doy = (int) (passed_jd - jd_xx0101);
 d_jd = day_from_97;
 d_jd += (hr+(minutes + secs/60.0)/60.0)/24.0;
 return (d_jd);
}

int convert_type_ts19_data(struct type_ts19_meta_struct *s, char *output_name)
{
 int  i1=0,i2=0,i3=0,i4=0,iflg=0;
 char *ptr_phy_filename;
 int dp=-1,dt=-1,ds=-1;

 if ((out_file = fopen(output_name,"w")) == NULL)
 {
  if (trace_file != NULL)
   fprintf (trace_file," CONVERT_TYPE_TS19_DATA: error opening output file %s\n",
    output_name);
  if (error_file != NULL)
   fprintf (error_file," CONVERT_TYPE_TS19_DATA: error opening output file %s\n",
    output_name);
  return 1;
 }
 if (trace_file != NULL)
  fprintf (trace_file," CONVERT_TYPE_TS19_DATA: output file %s successfully opened\n",output_name);

 printf ("\nOutput file %s successfully opened", output_name);

// Write Output Profile Physical Unit File Name to List File
// ---------------------------------------------------------
 if ( (ptr_phy_filename = strrchr(output_name, '/')) == NULL)
 {
  printf ("\nCan't get phy file basename %s", output_name);
  if (trace_file != NULL)
   fprintf (trace_file,
            "\n%s : Can't get phy file basename %s", PRGNAME, output_name);
   if (error_file != NULL)
    fprintf (error_file,
             "\n%s: Can't get phy file basename %s", PRGNAME, output_name);
   return 1;
 } // endif: ptr_phy_filename
 ptr_phy_filename++;
 fprintf (list_file,"%s\n", ptr_phy_filename);

 while (fgets(buffer,sizeof(buffer),input_file) != NULL)
 {

  if (strstr(strupr(buffer),"DRIFT PRESSURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&dp) != 1)
    dp = 0;
   sprintf (buffer, "DRIFT PRESSURE (DBAR)                   %-6.1f\n",
    compute_type_ts19_pressure(s,dp));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(buffer,"SURFACE PRESSURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"SURFACE PRESSURE (DBAR)                 %-6.1f\n",
    compute_type_ts19_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(buffer,"DRIFT TEMPERATURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&dt) != 1)
    dt = 0;
   sprintf (buffer,"DRIFT TEMPERATURE (DEG C)               %-6.3f\n",
    compute_type_ts19_temperature(s,dt));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(buffer,"DRIFT SALINITY") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&ds) != 1)
    ds = 0;
   sprintf (buffer,"DRIFT SALINITY (PSU)                    %-6.3f\n",
    compute_type_ts19_salinity(s,ds));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(buffer,"DRIFT BATTERY VOLTAGE (COUNTS)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"DRIFT BATTERY (VOLT)                    %-5.2f\n",
            compute_battery_voltage (s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"BATTERY VOLTAGE AIR PUMP ON (COUNTS)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"BATTERY AIR PUMP ON (VOLT)              %-5.2f\n",
            compute_battery_voltage (s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"BATTERY VOLTAGE SBE PUMP ON (COUNTS)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"BATTERY SBE PUMP ON (VOLT)              %-5.2f\n",
            compute_battery_voltage (s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(buffer,"BATTERY VOLTAGE (COUNTS)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)       // ????
    i1 = 0;
   sprintf (buffer,"BATTERY (VOLT)                          %-5.2f\n",
            compute_battery_voltage (s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"TARGET PRESSURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"TARGET PRESSURE (DBAR)                  %-6.1f\n",
    compute_type_ts19_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"CURRENT PRESSURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"CURRENT PRESSURE (DBAR)                 %-6.1f\n",
    compute_type_ts19_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"OK VACUUM") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"OK VACUUM (INCHES HG)                   %-4.1f\n",
    compute_vacuum_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"INTERNAL VACUUM") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"INTERNAL VACUUM (INCHES HG)             %-4.1f\n",
    compute_vacuum_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"AIR PUMP ON TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"AIR PUMP ON TIME (SECONDS)              %d\n",i1);
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if ((strstr(buffer,"COLUMNS") != NULL) || (strstr(buffer,"COLUMNS")))
  {
   if (strstr(strupr(buffer),"COLUMNS") != NULL)
   {
    fprintf (out_file,"NUMBER OF COLUMNS                       4\n");
    fprintf (out_file,"1. COLUMN                               PRESSURE (DBAR)\n");
    fprintf (out_file,"2. COLUMN                               TEMPERATURE-90 (DEG C)\n");
    fprintf (out_file,"3. COLUMN                               SALINITY (PSU)\n");
    fprintf (out_file,"4. COLUMN                               CHECKSUM FLAG\n");
   }
   rewind(meta_file);
   while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
    if (strstr(buffer,"================") != NULL)
     break;
   while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
   {
    if (strstr(buffer,"================") != NULL)
	{
	 sprintf (buffer,"==========================================================\n");
     fwrite (buffer,1,strlen(buffer),out_file);
     break;
	}
    fwrite (strupr(buffer),1,strlen(buffer),out_file);
   }

   while (fgets(buffer,sizeof(buffer),input_file) != NULL)
    if (strstr(buffer,"LATITUDE") != NULL)
     break;
   fwrite (buffer,1,strlen(buffer),out_file);
   while (fgets(buffer,sizeof(buffer),input_file) != NULL)
   {
    fwrite (buffer,1,strlen(buffer),out_file);
    if (strstr(buffer,"================") != NULL)
     break;
   }
   iflg = 1;
   continue;
  }
  if (iflg == 1)
  {
   if (sscanf(buffer,"%d %d %d %d",&i1,&i2,&i3,&i4) == 4)
    {
      if (i1==0 & i2==0 & i3==0 & i4==0)  // no data
      {
        fprintf (out_file,"%6.1f   %6.3f   %6.3f   %1d\n",
             (float)i1,(float)i2,(float)i3,i4);
      }
      else
      {
        fprintf (out_file,"%6.1f   %6.3f   %6.3f   %1d\n",
             compute_type_ts19_pressure(s,i1),
             compute_type_ts19_temperature(s,i2),
             compute_type_ts19_salinity(s,i3),
	     i4);
     }
    }
  }
  else
   fprintf (out_file,buffer);
 }
 if (trace_file != NULL)
  fprintf (trace_file," CONVERT_TYPE_TS19_DATA:%s completed successfully\n",output_name);
 return 0;

}

int get_meta_file_name (char *str_name)
{
 char *ptr,tmp_buff[80];
 int len;

// Assume str_name = *.hex, returns *.meta in str_name

 if ((ptr = strrchr(str_name,'_')) == NULL)    // get pointer to last '_'
  strcat(str_name,".meta");
 else
 {
  sprintf (tmp_buff,"%s\0", ptr);
  len = strlen(str_name) - strlen(tmp_buff);
  strncpy(tmp_buff, str_name, len);
  tmp_buff[len] = '\0';
  sprintf (str_name,"%s.meta",tmp_buff);
 }
 return 0;
}

int read_type_ts19_meta_file (char *meta_name, struct type_ts19_meta_struct *meta_struct)
{
 int i,j,syear,smonth,sday,shour,smin,x;
 char *ptr;
 char *cal_keys[] = {"TSLOPE=","TOFF=","PSLOPE=","POFF=","SSLOPE=","SOFF=",
                     "VSLOPE=","VOFF=",
                     "VACSLOPE=","VACOFF=","\0"};

 for (i=0; i<(sizeof(struct aoml_ts19_cals)/sizeof(double)); i++)
  meta_struct->cals.c[i] = 0.0;
 meta_struct->cycle_time = (float) 0.0;
 meta_struct->start_time = (float) 0.0;
 printf("Opening meta file %s",meta_name);
 if ((meta_file = fopen(meta_name,"r")) == NULL)
 {
  printf ("\nError opening meta file %s",meta_name);
  if (error_file != NULL)
   fprintf (error_file," ERROR opening meta file %s\n",meta_name);
  if (trace_file != NULL)
   fprintf (trace_file," ERROR opening meta file %s\n",meta_name);
  return(1);
 }
 if (trace_file != NULL)
  fprintf (trace_file," Meta file %s successfully opened\n",meta_name);

 while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
 {
  if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
  {
   if(sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) == 0)
    meta_struct->AOML_number = 0;
   continue;
  }
  if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) == 0)
    meta_struct->WMO_id = 0;
   continue;
  }
  if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) == 0)
    meta_struct->ARGOS_id = 0;
   continue;
  }
  if (strstr(strupr(buffer),"DELAY OF FIRST DOWN TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&x) == 0)
    x = 0;
   meta_struct->start_time += (float) (x/24.0);
   continue;
  }
  if (strstr(strupr(buffer),"DOWN TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&x) == 0)
    x = 0;
   meta_struct->cycle_time += x;
   continue;
  }
  if (strstr(strupr(buffer),"UP TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&x) == 0)
    x = 0;
   meta_struct->cycle_time += (float) (x/24.0);
   meta_struct->start_time -= (float) (x/24.0);
   continue;
  }
  if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",&sday,&smonth,&syear,&shour,&smin) == 5)
    meta_struct->start_time += (float) (dfr97(syear,smonth,sday,shour,smin,0,&x) + 0.25);
   else
    meta_struct->start_time = 0;
   continue;
  }
  i = 0;
  while ((j = strlen(cal_keys[i])) > 1)
  {
   if ((ptr = strstr(buffer,cal_keys[i])) != NULL)
    meta_struct->cals.c[i] = atof(ptr+j);
   i++;
  }
 }
 return 0;
}

int main (int argc, char *argv[])
{

 char     *ptr_path_meta, *ptr_path_cnt, *ptr_path_phy,
          *ptr_path_log, *ptr_path_list,
          pathname_input_filename[161]="",
          pathname_trace[161]="", pathname_error[161]="";

 char     input_filename[80],output_directory[256],
          msg[11]="", title[11]="", *ptr;
 struct   type_ts19_meta_struct s;
                                                    // long t;
 unsigned long t;
 int      ret_val = 0;

// Program Name
// ------------
  printf("Entering Program: %s\n", PRGNAME);
  strcpy(title, "Program calibrates counts files for APEX_TS19 float");
  printf("%s\n", title);

// Input Arguments
// ---------------
 if (argc < 6)
 {
  printf("Format: %s cnt_infile phy_outdir trace_outile error_outfile\n", PRGNAME);
  printf("\n");
  printf("cnt_infile:    counts input file name\n");
  printf("phy_outdir:    output directory for generated calibrated data file(s)\n");
  printf("trace_outile:  output directory for generated trace file\n");
  printf("error_outfile: output directory for generated error file\n");
  printf("list_outfile:  output directory for generated list file\n");
  return 1;
 }

// Input Counts File Name
// ----------------------
 printf ("%s: Input Hex File Name - %s\n", PRGNAME, argv[1]);
 sprintf (input_filename,"%s",argv[1]);

// Output directory path: defaults to PHYFILES_DIR
// -----------------------------------------------
 if ( (ptr_path_phy = getenv("PHYFILES_DIR")) == NULL)
 {
  printf("ERROR (%s): PHYFILES_DIR <-- no such environment variable\n", PRGNAME);
  t = time(NULL);
  printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
  ret_val = 1;
  return ret_val;
 }

 printf ("%s: Output Directory - %s\n", PRGNAME, argv[2]);
 sprintf (output_directory,"%s/",argv[2]);

// TRACE File
// ----------
 if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
 {
  printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
  t = time(NULL);
  printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
  ret_val = 1;
  return ret_val ;
 }

 printf ("%s: Opening trace file %s ", PRGNAME, buffer);
 sprintf (buffer,"%s",argv[3]);

 if ((trace_file = fopen(buffer,"a+")) == NULL)
 {
  printf ("\nERROR (%s): Opening trace file %s", PRGNAME, buffer);
  t = time(NULL);
  printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
  ret_val = 1;
  return ret_val;
 }
 printf("%s: OK -> trace file opened\n", PRGNAME);

 if (trace_file != NULL)
 {
  t = time(NULL);
  fprintf (trace_file,"=> %s: start %s", PRGNAME, ctime(&t));
 }

// ERROR File
// ----------
 sprintf (buffer,"%s",argv[4]);
 printf ("%s: Opening error file %s ", PRGNAME, buffer);

 if ((error_file = fopen(buffer,"a+")) == NULL)
 {
  printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
  if (trace_file != NULL)
   fprintf (trace_file," ERROR (%s): Opening file %s\n", PRGNAME, buffer);
  t = time(NULL);
  printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
  ret_val = 1;
  return ret_val;
 }
 printf("%s: OK -> error file opened\n", PRGNAME);

 if (trace_file != NULL)
   fprintf (trace_file,"%s: File %s successfully opened\n", PRGNAME, buffer);
 if (error_file != NULL)
 {
   t = time(NULL);
   fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

// LIST File
// ---------
  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
   printf("\nERROR (%s): ARGOLIST_DIR <-- no such environment variable",
          PRGNAME);
   t = time(NULL);
   printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 1;
   return ret_val;
  }

  sprintf (buffer,"%s",argv[5]);
  printf ("%s: Opening list file %s ", PRGNAME, buffer);

  if ((list_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nERROR (%s): Opening list file %s\n", PRGNAME, buffer);

    if (trace_file != NULL)
     fprintf (trace_file,"ERROR (%s): Opening list file %s\n",
              PRGNAME, buffer);
    if (error_file != NULL)
     fprintf (error_file,"ERROR (%s): Opening list file %s\n",
              PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }
  printf("%s: OK -> list file opened\n", PRGNAME);
  if (trace_file != NULL)
   fprintf (trace_file,"%s: List file %s successfully opened\n",
            PRGNAME, buffer);

// Open Input Counts File
// ----------------------
 if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
 {
   printf("ERROR (%s): CNTFILEDIR <-- no such environment variable\n", PRGNAME);

  if (trace_file != NULL)
   fprintf (trace_file,
            "ERROR (%s) : CNTFILES_DIR <-- no such environment variable\n",
            PRGNAME);
  if (error_file != NULL)
   fprintf (error_file,
            "ERROR (%s): CNTFILES_DIR <-- no such environment variable\n",
            PRGNAME);

  if (trace_file != NULL)                  // Close Log files
    fclose (trace_file);
  if (error_file != NULL)
    fclose (error_file);

  t = time(NULL);
  printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
  ret_val = 1;
  return(ret_val);
 }

 sprintf(pathname_input_filename, "%s/%s", ptr_path_cnt, input_filename);

 if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
 {                                                 // Failure: could not open cnt file
  printf ("\nERROR (%s): Opening input file %s",
          PRGNAME, pathname_input_filename);

  if (trace_file != NULL)
    fprintf (trace_file, "ERROR (%s): Opening input file %s\n",
             PRGNAME, pathname_input_filename);
   if (error_file != NULL)
    fprintf (error_file, "ERROR (%s): Opening input file %s\n",
             PRGNAME, pathname_input_filename);
   t = time(NULL);
   printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 1;
   return ret_val;
 }
 else                                              // Success: cnt file opened
 {
  printf ("%s: Input file %s successfully opened\n",
          PRGNAME, pathname_input_filename);
  if (trace_file != NULL)
   fprintf (trace_file,
            "%s: Input file %s successfully opened\n",
            PRGNAME, pathname_input_filename);

// META File
// ---------
  if (get_meta_file_name (input_filename) != 0)
  {                                             // Failure: did not get meta filename
   printf ("ERROR (%s): Finding meta file name", PRGNAME);
   if (error_file != NULL)
    fprintf (error_file,
             "\nERROR (%s): Finding meta file name - returned name %s\n",
             PRGNAME, input_filename);
   if (trace_file != NULL)
    fprintf (trace_file,
             "\nERROR (%s): Finding meta file name - returned name %s\n",
             PRGNAME, input_filename);
   ret_val = 1;
  }
  else
  {                                             // Success: got meta filename
   if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
   {
    printf("%s: METAFILES_DIR <-- no such environment variable\n", PRGNAME);
    if (trace_file != NULL)
     fprintf (trace_file,
              "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
              PRGNAME);

    if (error_file != NULL)
     fprintf (error_file,
              "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
              PRGNAME);

    if (trace_file != NULL)                  // Close Log files
     fclose (trace_file);
    if (error_file != NULL)
     fclose (error_file);

    ret_val = 1;
   } // endif  ptr_path_meta

   sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

   if (read_type_ts19_meta_file (pathname_input_filename, &s) != 0)
   {                                         // Failure: couln't read meta_file
    printf ("\nERROR (%s): Reading meta file", PRGNAME);
    if (trace_file != NULL)
     fprintf (trace_file,"ERROR (%s): Reading meta file %s\n",
              PRGNAME, pathname_input_filename);
    if (error_file != NULL)
     fprintf (error_file, "ERROR (%s): Reading meta file %s\n",
              PRGNAME, pathname_input_filename);
    ret_val = 1;
   }
   else
   {
    if (trace_file != NULL)
     fprintf (trace_file, "%s: Meta file %s successfully read\n",
              PRGNAME, pathname_input_filename);
    printf ("\n%s: Meta file %s successfully read\n", PRGNAME, pathname_input_filename);

// Convert Counts to Physical Units
// --------------------------------
    sprintf (input_filename,"%s%s", output_directory, argv[1]); // !! using input storage

    if ( (ptr = strrchr(input_filename,'.')) == NULL)
     strcat (input_filename,".phy");
    else
     sprintf(ptr,".phy");                             // ???

     printf ("%s: Output Path & File Name - %s", PRGNAME,input_filename);

    if (convert_type_ts19_data(&s, input_filename) == 0)
    {
      printf ("\n%s: CONVERT_TYPE_TS19_DATA  successful\n", PRGNAME);
      if (trace_file != NULL)
        fprintf (trace_file, "%s: CONVERT_TYPE_TS19_DATA successful\n", PRGNAME);
    }
    else
    {
      printf ("\nERROR (%s): CONVERT_TYPE_TS19_DATA\n");
      if (trace_file != NULL)
        fprintf (trace_file, "ERROR (%s): CONVERT_TYPE_TS19_DATA\n",
                 PRGNAME);
      if (error_file != NULL)
        fprintf (error_file, "ERROR (%s): CONVERT_TYPE_TS19_DATA\n",
                 PRGNAME);
      ret_val = 1;
    } // endif: convert_type_ts19_data
   } // endif: read_type_ts19_meta_file
  } // endif: get_meta_file_name
 } // endif: input_file

// Write finish message to trace & error files
// -------------------------------------------
 t = time(NULL);
 if ( ret_val == 0)
   strcpy(msg, "finished");
 else
   strcpy(msg, "terminated");

 printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));

 if (trace_file != NULL)
  fprintf (trace_file,"=> %s: %s %s\n", PRGNAME, msg, ctime(&t));

 if (error_file != NULL)
  fprintf (error_file,"=> %s: %s %s\n", PRGNAME, msg, ctime(&t));

// Close all files
// ---------------
 if (input_file != NULL)
   fclose (input_file);

 if (trace_file != NULL)
   fclose (trace_file);

 if (error_file != NULL)
  fclose (error_file);

 if (list_file != NULL)
  fclose (list_file);

 return ret_val;
}
