/******************************************************************************
   Revision Date: 17-May-2012 (CS - added pressure compensation to oxygen computation)
                  15-May-2012 (CS - corrected print statement for lines without data)
                  10-May-2012 (CS - changed oxygen computation to follow manual)
                  24-Oct-11 (ER  - Add columns BPHASE_DOXY and TEMP_DOXY)
                  13-May-11 (ER  - Convert T90 to T68 when calculating ptem)
                  25-Sep-08 (LL  - AIR PUMP TIME without 2*)
                  08-Mar-06 (XXD - adapted from cnt2phy_tso3)

   IMPORTANT: Use cmplnk_cnt2phy_oxsol.sh for compilation

   cnt2phy_tso7.c
   ============
   Converts the counts ARGOS data for temperature/salinity/pressure
   University of Washington PALACE floats manufactured in 2000 by Webb Research
   Cooperation and deployed by Steve Riser
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>

#define max_scans 2000  //Maximum scans for each profile
#define buff_size 256   //Maximum length of each scan read from *.str

char *strupr(char* ptr_to_string_to_be_upper_cased); /* function prototype */

//for use in computing oxygen
double FoilMatrix[5][4];

float PhaseCoef[4];
static float swiftpoly(float x,float coeff[],int n);
float OptodeDPhase(float BPhase, float *PhaseCoef);
double OptodeCalibration(double t, double DPhase, double matrix[][4]);

double ox_units(double o,double s,double t,double p);
double dswiftpoly(double x,const double coeff[],int n);

int debug=0;//1 debug, 0 no debug
FILE *input_file, *out_file, *error_file, *trace_file, *list_file, *meta_file;

char PRGNAME[] ="CNT2PHY_TSO7";
char buffer[1024];
int umol=1;//  for oxygen concentration: equal 1 = true -> use micromol/kg as unit
            //                           not 1 = false -> use ml/l as unit

struct aoml_tso7_cals {double TSLOPE;
                       double TOFF;
                       double PSLOPE;
                       double POFF;
                       double SSLOPE;
                       double SOFF;
                       double SOC;
                       double FOFFSET;
                       double VSLOPE;
                       double VOFF;
                       double VACSLOPE;
                       double VACOFF;
		       double C00;
		       double C01;
		       double C02;
		       double C03;
		       double C10;
		       double C11;
		       double C12;
		       double C13;
		       double C20;
		       double C21;
		       double C22;
		       double C23;
		       double C30;
		       double C31;
		       double C32;
		       double C33;
		       double C40;
		       double C41;
		       double C42;
		       double C43;
		       double B0;
		       double B1;
		       double B2;
		       double B3;
                      };

union u_cals {struct aoml_tso7_cals s;
              double c[sizeof(struct aoml_tso7_cals)/sizeof(double)];
             };

struct type_tso7_meta_struct { int   AOML_number;
                               int   WMO_id;
                               int   ARGOS_id;
                               float cycle_time;
                               float start_time;
			       double launch_lat;
                               union u_cals cals;};

long ymd2julday(int, int, int);

// changes characters to upper case
char *strupr(char *input)
{
   int  i;

   for (i=0; input[i] != '\0'; i++)
     input[i] = toupper(input[i]);
   return input;
}

float compute_temperature (struct type_tso7_meta_struct *s, int t_counts)
{
  float t;
  //changed from F447 to EFFF
  t = (t_counts <= 0XEFFF) ? ((float) (s->cals.s.TSLOPE * t_counts + s->cals.s.TOFF)) :
           ((float) (-s->cals.s.TSLOPE * (0XFFFF - t_counts + 1) + s->cals.s.TOFF));
  return t;
}

float compute_pressure (struct type_tso7_meta_struct *s, int p_counts)
{
  float pres;
  pres = (p_counts <= 0X7FFF) ? ((float) (s->cals.s.PSLOPE * p_counts + s->cals.s.POFF)) :
           ((float) (-s->cals.s.PSLOPE * (0XFFFF - p_counts + 1) + s->cals.s.POFF));
  return pres;
}

float compute_salinity (struct type_tso7_meta_struct *s, int s_counts)
{
  float salt;
  //changed from F447 to EFFF
  salt = (s_counts <= 0XEFFF) ? ((float) (s->cals.s.SSLOPE * s_counts + s->cals.s.SOFF)) :
           ((float) (-s->cals.s.SSLOPE * (0XFFFF - s_counts + 1) + s->cals.s.SOFF));
  return salt;
}

void initOxygenCoeff(struct type_tso7_meta_struct *s){
  int i,j;
  PhaseCoef[0]=(float)s->cals.s.B0;
  PhaseCoef[1]=(float)s->cals.s.B1;
  PhaseCoef[2]=(float)s->cals.s.B2;
  PhaseCoef[3]=(float)s->cals.s.B3;
  FoilMatrix[0][0]=(double)s->cals.s.C00;FoilMatrix[0][1]=(double)s->cals.s.C01;
  FoilMatrix[0][2]=(double)s->cals.s.C02;FoilMatrix[0][3]=(double)s->cals.s.C03;
  FoilMatrix[1][0]=(double)s->cals.s.C10;FoilMatrix[1][1]=(double)s->cals.s.C11;
  FoilMatrix[1][2]=(double)s->cals.s.C12;FoilMatrix[1][3]=(double)s->cals.s.C13;
  FoilMatrix[2][0]=(double)s->cals.s.C20;FoilMatrix[2][1]=(double)s->cals.s.C21;
  FoilMatrix[2][2]=(double)s->cals.s.C22;FoilMatrix[2][3]=(double)s->cals.s.C23;
  FoilMatrix[3][0]=(double)s->cals.s.C30;FoilMatrix[3][1]=(double)s->cals.s.C31;
  FoilMatrix[3][2]=(double)s->cals.s.C32;FoilMatrix[3][3]=(double)s->cals.s.C33;
  FoilMatrix[4][0]=(double)s->cals.s.C40;FoilMatrix[4][1]=(double)s->cals.s.C41;
  FoilMatrix[4][2]=(double)s->cals.s.C42;FoilMatrix[4][3]=(double)s->cals.s.C43;

 if(debug>0){
 for(i=0;i<5;i++){
   for(j=0;j<4;j++){
     printf("FoilMatrix[%d][%d]=%f \n",i,j,FoilMatrix[i][j]);
   }
 }
 printf("\n");

 for(i=0;i<4;i++){
   printf("PhaseCoef[%d]=%f \n",i,PhaseCoef[i]);
 }
 printf("\n");
 }
}


//use dana swift's new oxygen calibration equation to get oxygen
float compute_oxygen (struct type_tso7_meta_struct *s, int p_counts,int t_counts,int s_counts,int bphase_counts,int t2_counts)
{
  double bphase,dphase,t2,o2,OX,P,T,S,density,sigma,Ts;

  t2=(t2_counts/100.0-3.0);
  P = compute_pressure(s,p_counts);
  T = compute_temperature(s,t_counts);
  S = compute_salinity(s,s_counts);
  bphase=(bphase_counts/100.0+23.0);
//printf ("\np, t, s, t2, bphase = %f %f %f %f %f\n",P,T,S,t2,bphase);
  dphase=OptodeDPhase(bphase, PhaseCoef);
//printf ("dphase = %f\n",dphase);
// temperature compensation
  o2=OptodeCalibration(t2,dphase,FoilMatrix);
//printf ("o2 = %f\n",o2);
// scaled temperature
  Ts=log((298.15-t2)/(273.15+t2));
  double B0=-6.24097e-3,B1=-6.93498e-3,B2=-6.90358e-3,B3=-4.29155e-3,C0=-3.11680e-7;
  double B[]={B0,B1,B2,B3};
// salinity compensation
  o2 *= exp(S * dswiftpoly(Ts,B,3) + C0*S*S );
//printf ("o2 = %f\n",o2);
// pressure compensation
  o2 *= (1+0.032*P/1000.0);
//printf ("o2 = %f\n",o2);
  OX=ox_units(o2,S,T,P/10.0);
//printf ("OX = %f\n",OX);
  OX *= 1/44.6596;
//printf ("OX = %f\n",OX);
  if(isnan(OX)!=0) OX=-999.9;
  return OX;
}


float compute_battery_voltage (struct type_tso7_meta_struct *s, int v_counts)
{
 float b;
 b=(float) (s->cals.s.VSLOPE * v_counts + s->cals.s.VOFF);
 if (v_counts==0)
    b=0.;
 return(b);
}

float compute_vacuum_pressure (struct type_tso7_meta_struct *s, int vac_counts)
{
 float b;
 b=(float) (s->cals.s.VACSLOPE * vac_counts + s->cals.s.VACOFF);
 if (vac_counts==0)
 b=0.;
 return(b);
}

double dfr97 (const int yp, const int mp, const int dp,
     const int hr, const int minutes, const int secs, int *doy)
{
 int     yr, mo, dy, day_from_97;
 long    passed_jd, jd_xx0101, jd_970101;
 double  d_jd;

 yr = yp;
 mo = mp;
 dy = dp;
 if (yr <= 15)
  yr += 2000;
 else if (yr < 1900)
  yr += 1900;
 if ((yr<1997) || (yr>2015) || (mo<1) || (mo>12) || (dy<1) || (dy>31))
  return (0.0);
 passed_jd = ymd2julday( yr, mo, dy ) + 1;
 dy = 1;
 mo = 1;
 jd_xx0101 = ymd2julday( yr, mo, dy );
 yr = 1997;
 jd_970101 = ymd2julday( yr, mo, dy );
 day_from_97 = (int) (passed_jd - jd_970101);
 *doy = (int) (passed_jd - jd_xx0101);
 d_jd = day_from_97;
 d_jd += (hr+(minutes + secs/60.0)/60.0)/24.0;
 return (d_jd);
}

int convert_type_tso7_data(struct type_tso7_meta_struct *s, char *output_name)
{
 int  i1=0,i2=0,i3=0,i4=0,i5=0,i6=0,i7=0,i8=0,i9=0,iflg=0;
 char *ptr_phy_filename;
 int dp=-1,dt=-1,ds=-1;

 float dphase,t2;

 if ((out_file = fopen(output_name,"w")) == NULL)
 {
  if (trace_file != NULL)
   fprintf (trace_file," CONVERT_TYPE_TSO7_DATA: error opening output file %s\n",
    output_name);
  if (error_file != NULL)
   fprintf (error_file," CONVERT_TYPE_TSO7_DATA: error opening output file %s\n",
    output_name);
  return 1;
 }
 if (trace_file != NULL)
  fprintf (trace_file," CONVERT_TYPE_TSO7_DATA: output file %s successfully opened\n",output_name);

 printf ("\nOutput file %s successfully opened", output_name);

// Write Output Profile Physical Unit File Name to List File
// ---------------------------------------------------------
 if ( (ptr_phy_filename = strrchr(output_name, '/')) == NULL)
 {
  printf ("\nCan't get phy file basename %s", output_name);
  if (trace_file != NULL)
   fprintf (trace_file,
            "\n%s : Can't get phy file basename %s", PRGNAME, output_name);
   if (error_file != NULL)
    fprintf (error_file,
             "\n%s: Can't get phy file basename %s", PRGNAME, output_name);
   return 1;
 } // endif: ptr_phy_filename
 ptr_phy_filename++;
 fprintf (list_file,"%s\n", ptr_phy_filename);

 while (fgets(buffer,sizeof(buffer),input_file) != NULL)
 {
  if (strstr(strupr(buffer),"DRIFT PRESSURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&dp) != 1)
    dp = 0;
   sprintf (buffer, "DRIFT PRESSURE (DBAR)                   %-6.1f\n",
    compute_pressure(s,dp));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(buffer,"SURFACE PRESSURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"SURFACE PRESSURE (DBAR)                 %-6.1f\n",
    compute_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(buffer,"DRIFT TEMPERATURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&dt) != 1)
    dt = 0;
   sprintf (buffer,"DRIFT TEMPERATURE (DEG C)               %-6.3f\n",
    compute_temperature(s,dt));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(buffer,"DRIFT SALINITY") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&ds) != 1)
    ds = 0;
   sprintf (buffer,"DRIFT SALINITY (PSU)                    %-6.3f\n",
    compute_salinity(s,ds));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(buffer,"DRIFT OXYGEN") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d %d",&i1,&i2) != 2){
    i1 = 0;
    i2 = 0;
   }

   if(umol==1){
     sprintf (buffer,"DRIFT OXYGEN (MICROMOL/KG)              %-6.3f\n",
     compute_oxygen(s,dp,dt,ds,i1,i2));
   }else{
     sprintf (buffer,"DRIFT OXYGEN (ML/L)                     %-6.3f\n",
     compute_oxygen(s,dp,dt,ds,i1,i2));
   }

   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(buffer,"PROFILE PRESSURE (COUNTS)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"PROFILE PRESSURE (DBAR)                 %-6.1f\n",
    compute_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(buffer,"DRIFT BATTERY VOLTAGE (COUNTS)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"DRIFT BATTERY (VOLT)                    %-5.2f\n",
            compute_battery_voltage (s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"BATTERY VOLTAGE AIR PUMP ON (COUNTS)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"BATTERY AIR PUMP ON (VOLT)              %-5.2f\n",
            compute_battery_voltage (s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"BATTERY VOLTAGE SBE PUMP ON (COUNTS)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"BATTERY SBE PUMP ON (VOLT)              %-5.2f\n",
            compute_battery_voltage (s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"BATTERY VOLTAGE OPTICS ON (COUNTS)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"BATTERY VOLTAGE OPTICS ON (VOLT)        %-5.2f\n",
            compute_battery_voltage (s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(buffer,"BATTERY VOLTAGE (COUNTS)") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)       // ????
    i1 = 0;
   sprintf (buffer,"BATTERY (VOLT)                          %-5.2f\n",
            compute_battery_voltage (s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"TARGET PRESSURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"TARGET PRESSURE (DBAR)                  %-6.1f\n",
    compute_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"CURRENT PRESSURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"CURRENT PRESSURE (DBAR)                 %-6.1f\n",
    compute_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"OK VACUUM") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"OK VACUUM (INCHES HG)                   %-4.1f\n",
    compute_vacuum_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"INTERNAL VACUUM") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"INTERNAL VACUUM (INCHES HG)             %-4.1f\n",
    compute_vacuum_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"AIR PUMP ON TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"AIR PUMP ON TIME (SECONDS)              %d\n",i1);
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(buffer,"MEAN TEMPERATURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&dt) != 1)
    dt = 0;
   sprintf (buffer,"MEAN TEMPERATURE (DEG C)                %-6.3f\n",
    compute_temperature(s,dt));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(buffer,"STANDARD DEVIATION OF TEMP") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&dt) != 1)
    dt = 0;
   sprintf (buffer,"STANDARD DEVIATION OF TEMP (DEG C)      %-6.3f\n",
    compute_temperature(s,dt));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"MINIMUM TEMPERATURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&dt) != 1)
    dt = 0;
   sprintf (buffer,"MINIMUM TEMPERATURE (DEG C)             %-6.3f\n",
    compute_temperature(s,dt));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"MAXIMUM TEMPERATURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&dt) != 1)
    dt = 0;
   sprintf (buffer,"MAXIMUM TEMPERATURE (DEG C)             %-6.3f\n",
    compute_temperature(s,dt));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }

  if (strstr(buffer,"MEAN PRESSURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"MEAN PRESSURE (DBAR)                    %-6.1f\n",
    compute_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"STANDARD DEVIATION OF PRESSURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"STANDARD DEVIATION OF PRESSURE (DBAR)   %-6.1f\n",
    compute_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"PRESSURE WHEN MINIMUM TEMP") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"PRESSURE WHEN MINIMUM TEMP (DBAR)       %-6.1f\n",
    compute_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"PRESSURE WHEN MAXIMUM TEMP") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"PRESSURE WHEN MAXIMUM TEMP (DBAR)       %-6.1f\n",
    compute_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"MINIMUM PRESSURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"MINIMUM PRESSURE (DBAR)                 %-6.1f\n",
    compute_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }
  if (strstr(buffer,"MAXIMUM PRESSURE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)
    i1 = 0;
   sprintf (buffer,"MAXIMUM PRESSURE (DBAR)                 %-6.1f\n",
    compute_pressure(s,i1));
   fwrite (buffer,1,strlen(buffer),out_file);
   continue;
  }


  if ((strstr(buffer,"COLUMNS") != NULL) || (strstr(buffer,"COLUMNS")))
  {
   if (strstr(strupr(buffer),"COLUMNS") != NULL)
   {
    fprintf (out_file,"NUMBER OF COLUMNS                       7\n");
    fprintf (out_file,"1. COLUMN                               PRESSURE (DBAR)\n");
    fprintf (out_file,"2. COLUMN                               TEMPERATURE-90 (DEG C)\n");
    fprintf (out_file,"3. COLUMN                               SALINITY (PSU)\n");
    fprintf (out_file,"4. COLUMN                               TEMP_DOXY (DEG C)\n");
    fprintf (out_file,"5. COLUMN                               BPHASE_DOXY\n");

    if(umol==1){
      fprintf (out_file,"6. COLUMN                               DISSOLVED OXYGEN (MICROMOL/KG)\n");
    }else{
      fprintf (out_file,"6. COLUMN                               DISSOLVED OXYGEN (ML/L)\n");
    }

    fprintf (out_file,"7. COLUMN                               CHECKSUM FLAG\n");
   }
   rewind(meta_file);
   while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
    if (strstr(buffer,"================") != NULL)
     break;
   while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
   {
    if (strstr(buffer,"================") != NULL)
	{
	 sprintf (buffer,"==========================================================\n");
     fwrite (buffer,1,strlen(buffer),out_file);
     break;
	}
    fwrite (strupr(buffer),1,strlen(buffer),out_file);
   }

   while (fgets(buffer,sizeof(buffer),input_file) != NULL)
    if (strstr(buffer,"LATITUDE") != NULL)
     break;
   fwrite (buffer,1,strlen(buffer),out_file);
   while (fgets(buffer,sizeof(buffer),input_file) != NULL)
   {
    fwrite (buffer,1,strlen(buffer),out_file);
    if (strstr(buffer,"================") != NULL)
     break;
   }
   iflg = 1;
   continue;
  }
  if (iflg == 1)
  {

   if (sscanf(buffer,"%d %d %d %d %d %d",&i1,&i2,&i3,&i4,&i5,&i6) == 6)
    {
      if (i1==0 & i2==0 & i3==0 & i4==0 & i5==0 & i6==0)  // no data
      {
        fprintf (out_file,"%6.1f   %6.3f   %6.3f  %6.2f  %6.2f   %8.3f   %1d\n",
             0.0,0.0,0.0,0.0,0.0,0.0,0);
      }
      else
      {
          double bphase = (i4/100.0+23.0);       
          double t2 =(i5/100.0-3.0);
          fprintf (out_file,"%6.1f   %6.3f   %6.3f  %6.2f  %6.2f   %8.3f   %1d\n",
             ((compute_pressure(s,i1)>=0)?compute_pressure(s,i1):-999.0),
             compute_temperature(s,i2),
             compute_salinity(s,i3),
             t2, bphase, 
             ((compute_oxygen(s,i1,i2,i3,i4,i5)>=0)?compute_oxygen(s,i1,i2,i3,i4,i5):-999.0),
	     i6);
     }
    }
  }
  else
   fprintf (out_file,buffer);
 }
 if (trace_file != NULL)
  fprintf (trace_file," CONVERT_TYPE_TSO7_DATA:%s completed successfully\n",output_name);
 return 0;

}

int get_meta_file_name (char *str_name)
{
 char *ptr,tmp_buff[80];
 int len;

// Assume str_name = *.hex, returns *.meta in str_name

 if ((ptr = strrchr(str_name,'_')) == NULL)    // get pointer to last '_'
  strcat(str_name,".meta");
 else
 {
  sprintf (tmp_buff,"%s\0", ptr);
  len = strlen(str_name) - strlen(tmp_buff);
  strncpy(tmp_buff, str_name, len);
  tmp_buff[len] = '\0';
  sprintf (str_name,"%s.meta",tmp_buff);
 }
 return 0;
}

int read_type_tso7_meta_file (char *meta_name, struct type_tso7_meta_struct *meta_struct)
{
 int i,j,syear,smonth,sday,shour,smin,x;
 float lat,latm,lon,lonm;
 char *ptr;
 char *cal_keys[] =
 {"TSLOPE=","TOFF=","PSLOPE=","POFF=","SSLOPE=","SOFF=","SOC=","FOFFSET=","VSLOPE=","VOFF=","VACSLOPE=","VACOFF=",
                     "C00=","C01=","C02=","C03=","C10=","C11=","C12=","C13=","C20=","C21=","C22=","C23=",
		     "C30=","C31=","C32=","C33=","C40=","C41=","C42=","C43=","P0=","P1=","P2=","P3=","\0"};

 for (i=0; i<(sizeof(struct aoml_tso7_cals)/sizeof(double)); i++)
  meta_struct->cals.c[i] = 0.0;
 meta_struct->cycle_time = (float) 0.0;
 meta_struct->start_time = (float) 0.0;
 printf("Opening meta file %s",meta_name);
 if ((meta_file = fopen(meta_name,"r")) == NULL)
 {
  printf ("\nError opening meta file %s",meta_name);
  if (error_file != NULL)
   fprintf (error_file," ERROR opening meta file %s\n",meta_name);
  if (trace_file != NULL)
   fprintf (trace_file," ERROR opening meta file %s\n",meta_name);
  return(1);
 }
 if (trace_file != NULL)
  fprintf (trace_file," Meta file %s successfully opened\n",meta_name);

 while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
 {
  if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
  {
   if(sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) == 0)
    meta_struct->AOML_number = 0;
   continue;
  }
  if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) == 0)
    meta_struct->WMO_id = 0;
   continue;
  }
  if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) == 0)
    meta_struct->ARGOS_id = 0;
   continue;
  }
 if ((strstr(strupr(buffer),"LAUNCH POSITION") != NULL)) {

      if(sscanf(buffer,"%*s %*s %*s %*s %*s %*s %f %f %f %f",&lat,&latm,&lon,&lonm)==4){
      //printf("lat=%f,latm=%f,lon=%f,lonm=%f\n",lat,latm,lon,lonm);
      meta_struct->launch_lat =lat+latm/60.0;
      }
      continue;
  }

  if (strstr(strupr(buffer),"DELAY OF FIRST DOWN TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&x) == 0)
    x = 0;
   meta_struct->start_time += (float) (x/24.0);
   continue;
  }
  if (strstr(strupr(buffer),"DOWN TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&x) == 0)
    x = 0;
   meta_struct->cycle_time += x;
   continue;
  }
  if (strstr(strupr(buffer),"UP TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&x) == 0)
    x = 0;
   meta_struct->cycle_time += (float) (x/24.0);
   meta_struct->start_time -= (float) (x/24.0);
   continue;
  }
  if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",&sday,&smonth,&syear,&shour,&smin) == 5)
    meta_struct->start_time += (float) (dfr97(syear,smonth,sday,shour,smin,0,&x) + 0.25);
   else
    meta_struct->start_time = 0;
   continue;
  }

  i = 0;
  while ((j = strlen(cal_keys[i])) > 1)
  {
   if ((ptr = strstr(buffer,cal_keys[i])) != NULL){
    meta_struct->cals.c[i] = atof(ptr+j);
    //printf("meta_struct.c[%d]=%f\n",i,meta_struct->cals.c[i]);
   }
   i++;
  }
 }
initOxygenCoeff(meta_struct);

 return 0;
}

int main (int argc, char *argv[])
{

 char     *ptr_path_meta, *ptr_path_cnt, *ptr_path_phy,
          *ptr_path_log, *ptr_path_list,
          pathname_input_filename[161]="",
          pathname_trace[161]="", pathname_error[161]="";

 char     input_filename[80],output_directory[256],
          msg[11]="", title[11]="", *ptr;
 struct   type_tso7_meta_struct s;
                                                    // long t;
 unsigned long t;
 int      ret_val = 0;

// Program Name
// ------------
  printf("Entering Program: %s\n", PRGNAME);
  strcpy(title, "Program calibrates counts files for APEX_TSO7 float");
  printf("%s\n", title);

// Input Arguments
// ---------------
 if (argc < 6)
 {
  printf("Format: %s cnt_infile phy_outdir trace_outile error_outfile\n", PRGNAME);
  printf("\n");
  printf("cnt_infile:    counts input file name\n");
  printf("phy_outdir:    output directory for generated calibrated data file(s)\n");
  printf("trace_outile:  output directory for generated trace file\n");
  printf("error_outfile: output directory for generated error file\n");
  printf("list_outfile:  output directory for generated list file\n");
  return 1;
 }

// Input Counts File Name
// ----------------------
 printf ("%s: Input Hex File Name - %s\n", PRGNAME, argv[1]);
 sprintf (input_filename,"%s",argv[1]);

// Output directory path: defaults to PHYFILES_DIR
// -----------------------------------------------
 if ( (ptr_path_phy = getenv("PHYFILES_DIR")) == NULL)
 {
  printf("ERROR (%s): PHYFILES_DIR <-- no such environment variable\n", PRGNAME);
  t = time(NULL);
  printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
  ret_val = 1;
  return ret_val;
 }

 printf ("%s: Output Directory - %s\n", PRGNAME, argv[2]);
 sprintf (output_directory,"%s/",argv[2]);

// TRACE File
// ----------
 if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
 {
  printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
  t = time(NULL);
  printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
  ret_val = 1;
  return ret_val ;
 }

 printf ("%s: Opening trace file %s ", PRGNAME, buffer);
 sprintf (buffer,"%s",argv[3]);

 if ((trace_file = fopen(buffer,"a+")) == NULL)
 {
  printf ("\nERROR (%s): Opening trace file %s", PRGNAME, buffer);
  t = time(NULL);
  printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
  ret_val = 1;
  return ret_val;
 }
 printf("%s: OK -> trace file opened\n", PRGNAME);

 if (trace_file != NULL)
 {
  t = time(NULL);
  fprintf (trace_file,"=> %s: start %s", PRGNAME, ctime(&t));
 }

// ERROR File
// ----------
 sprintf (buffer,"%s",argv[4]);
 printf ("%s: Opening error file %s ", PRGNAME, buffer);

 if ((error_file = fopen(buffer,"a+")) == NULL)
 {
  printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
  if (trace_file != NULL)
   fprintf (trace_file," ERROR (%s): Opening file %s\n", PRGNAME, buffer);
  t = time(NULL);
  printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
  ret_val = 1;
  return ret_val;
 }
 printf("%s: OK -> error file opened\n", PRGNAME);

 if (trace_file != NULL)
   fprintf (trace_file,"%s: File %s successfully opened\n", PRGNAME, buffer);
 if (error_file != NULL)
 {
   t = time(NULL);
   fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

// LIST File
// ---------
  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
   printf("\nERROR (%s): ARGOLIST_DIR <-- no such environment variable",
          PRGNAME);
   t = time(NULL);
   printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 1;
   return ret_val;
  }

  sprintf (buffer,"%s",argv[5]);
  printf ("%s: Opening list file %s ", PRGNAME, buffer);

  if ((list_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nERROR (%s): Opening list file %s\n", PRGNAME, buffer);

    if (trace_file != NULL)
     fprintf (trace_file,"ERROR (%s): Opening list file %s\n",
              PRGNAME, buffer);
    if (error_file != NULL)
     fprintf (error_file,"ERROR (%s): Opening list file %s\n",
              PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }
  printf("%s: OK -> list file opened\n", PRGNAME);
  if (trace_file != NULL)
   fprintf (trace_file,"%s: List file %s successfully opened\n",
            PRGNAME, buffer);

// Open Input Counts File
// ----------------------
 if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
 {
   printf("ERROR (%s): CNTFILEDIR <-- no such environment variable\n", PRGNAME);

  if (trace_file != NULL)
   fprintf (trace_file,
            "ERROR (%s) : CNTFILES_DIR <-- no such environment variable\n",
            PRGNAME);
  if (error_file != NULL)
   fprintf (error_file,
            "ERROR (%s): CNTFILES_DIR <-- no such environment variable\n",
            PRGNAME);

  if (trace_file != NULL)                  // Close Log files
    fclose (trace_file);
  if (error_file != NULL)
    fclose (error_file);

  t = time(NULL);
  printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
  ret_val = 1;
  return(ret_val);
 }

 sprintf(pathname_input_filename, "%s/%s", ptr_path_cnt, input_filename);

 if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
 {                                                 // Failure: could not open cnt file
  printf ("\nERROR (%s): Opening input file %s",
          PRGNAME, pathname_input_filename);

  if (trace_file != NULL)
    fprintf (trace_file, "ERROR (%s): Opening input file %s\n",
             PRGNAME, pathname_input_filename);
   if (error_file != NULL)
    fprintf (error_file, "ERROR (%s): Opening input file %s\n",
             PRGNAME, pathname_input_filename);
   t = time(NULL);
   printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 1;
   return ret_val;
 }
 else                                              // Success: cnt file opened
 {
  printf ("%s: Input file %s successfully opened\n",
          PRGNAME, pathname_input_filename);
  if (trace_file != NULL)
   fprintf (trace_file,
            "%s: Input file %s successfully opened\n",
            PRGNAME, pathname_input_filename);

// META File
// ---------
  if (get_meta_file_name (input_filename) != 0)
  {                                             // Failure: did not get meta filename
   printf ("ERROR (%s): Finding meta file name", PRGNAME);
   if (error_file != NULL)
    fprintf (error_file,
             "\nERROR (%s): Finding meta file name - returned name %s\n",
             PRGNAME, input_filename);
   if (trace_file != NULL)
    fprintf (trace_file,
             "\nERROR (%s): Finding meta file name - returned name %s\n",
             PRGNAME, input_filename);
   ret_val = 1;
  }
  else
  {                                             // Success: got meta filename
   if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
   {
    printf("%s: METAFILES_DIR <-- no such environment variable\n", PRGNAME);
    if (trace_file != NULL)
     fprintf (trace_file,
              "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
              PRGNAME);

    if (error_file != NULL)
     fprintf (error_file,
              "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
              PRGNAME);

    if (trace_file != NULL)                  // Close Log files
     fclose (trace_file);
    if (error_file != NULL)
     fclose (error_file);

    ret_val = 1;
   } // endif  ptr_path_meta

   sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

   if (read_type_tso7_meta_file (pathname_input_filename, &s) != 0)
   {                                         // Failure: couln't read meta_file
    printf ("\nERROR (%s): Reading meta file", PRGNAME);
    if (trace_file != NULL)
     fprintf (trace_file,"ERROR (%s): Reading meta file %s\n",
              PRGNAME, pathname_input_filename);
    if (error_file != NULL)
     fprintf (error_file, "ERROR (%s): Reading meta file %s\n",
              PRGNAME, pathname_input_filename);
    ret_val = 1;
   }
   else
   {
    if (trace_file != NULL)
     fprintf (trace_file, "%s: Meta file %s successfully read\n",
              PRGNAME, pathname_input_filename);
    printf ("\n%s: Meta file %s successfully read\n", PRGNAME, pathname_input_filename);

// Convert Counts to Physical Units
// --------------------------------
    sprintf (input_filename,"%s%s", output_directory, argv[1]); // !! using input storage

    if ( (ptr = strrchr(input_filename,'.')) == NULL)
     strcat (input_filename,".phy");
    else
     sprintf(ptr,".phy");                             // ???

     printf ("%s: Output Path & File Name - %s", PRGNAME,input_filename);

    if (convert_type_tso7_data(&s, input_filename) == 0)
    {
      printf ("\n%s: CONVERT_TYPE_TSO7_DATA  successful\n", PRGNAME);
      if (trace_file != NULL)
        fprintf (trace_file, "%s: CONVERT_TYPE_TSO7_DATA successful\n", PRGNAME);
    }
    else
    {
      printf ("\nERROR (%s): CONVERT_TYPE_TSO7_DATA\n");
      if (trace_file != NULL)
        fprintf (trace_file, "ERROR (%s): CONVERT_TYPE_TSO7_DATA\n",
                 PRGNAME);
      if (error_file != NULL)
        fprintf (error_file, "ERROR (%s): CONVERT_TYPE_TSO7_DATA\n",
                 PRGNAME);
      ret_val = 1;
    } // endif: convert_type_tso7_data
   } // endif: read_type_tso7_meta_file
  } // endif: get_meta_file_name
 } // endif: input_file

// Write finish message to trace & error files
// -------------------------------------------
 t = time(NULL);
 if ( ret_val == 0)
   strcpy(msg, "finished");
 else
   strcpy(msg, "terminated");

 printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));

 if (trace_file != NULL)
  fprintf (trace_file,"=> %s: %s %s\n", PRGNAME, msg, ctime(&t));

 if (error_file != NULL)
  fprintf (error_file,"=> %s: %s %s\n", PRGNAME, msg, ctime(&t));

// Close all files
// ---------------
 if (input_file != NULL)
   fclose (input_file);

 if (trace_file != NULL)
   fclose (trace_file);

 if (error_file != NULL)
  fclose (error_file);

 if (list_file != NULL)
  fclose (list_file);

 return ret_val;
}
