/******************************************************************************
   Revision Date: 26-AUG-04 (CS&XX- correction to the change of 15-AUG-03, exit with 2 only 
                                    if all profiles are processed(nprofile=1),otherwise 
				    exit with 1(nprofile=0))
                  15-AUG-03 (XX&CS-add exit status 2 when frame missing problem happens 
                             in some but not all profile cycles)
                  17-Apr-02 (CS - modified write_sio_to_disk to compute profiles
                                  sequentially;
                                  added flag to write_sio_to_disk to be always
                                  called so that prof_no is computed;
                                  same as change in ts2 by DB)
                  20-NOV-01 (CS - write abs(s->WMO_id) not s->WMO_id)
                  06-NOV-01 (CS - modified reading of meta-file
                                  DOWN TIME replaced with CYCLE TIME
                                  UP TIME replaced with SURFACE TIME)
                  18-DEC-00 (CS - Make sure to use messages received only once) 
                  20-Nov-00 (DB - Change to uppercase in HEADER) 
                  09-Nov-00 (RS - Bitterman updates( diff1109.lis)
                  29-Aug-00 (JZ - rename ARGOLIST_DIR and ARGOSLOG_DIR)
                  07-14-00 (JZ) - Modified write_sio_to_disk function to 
                                  account for clock drift error 
                  07-07-00 (JZ) 
                  02-Feb-00 (RS - Fixed union and added "strupr" function)
				  01-Nov-00 (DB - fixed meta file read routine for DELAY OF FIRST DOWN TIME)

   hex2cnt_ts1.c
   =============
   Decoding of the hexadecimal ARGOS data for temperature/salinity/pressure
   PALACE floats PALACE_TS1 manufactured in 1997 by Webb Research Cooperation

Exit code
  nprofile=0(only process last profile) case:
        0--means success(one data scan profile will be ignored and return 0)
	1--means failure(e.g. frame missing problem)
  nprofile=1(process all profiles)      case:
        0--means success(all profiles are processed succefully,one data scan profile will be ignored)
	2--means not all of the profiles are successfully generated(one data scan profile will be ignored)
   
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>

#define max_scans 2000  //Maximum scans for each profile
#define buff_size 256  //Maximum length of each scan read from *.hex
#define max_positions 100 //Maximum number of positions per profile
#define frames_per_message 8
#define good_bad_position 43 //Location of status byte in *.hex records
#define start_of_hex_data 63 //Start position of data in *.hex records
int no_output_cycle_count=0;//added new
int frame_problem_count=0,total_cycles_count=0;

char* strupr(char* ptr_to_string_to_be_upper_cased); /* function prototype */

FILE *input_file, *out_file, *error_file, *trace_file, *list_file;

char PRGNAME[] ="hex2cnt_ts1";
char buffer[512];

// Calibration values for Scripps style salinity float
struct sio_cals { double pgain;
                  double poff;
                  double pmpgain;
                  double pmpoff;
                  double cpugain;
                  double cpuoff;
                  double at;
                  double bt;
                  double ct;
                  double h1;
                  double h2;
                  double h3;
                  double blok;
                  double av1;
                  double av2;
                  double pb1;
                  double pb2;
                  double cdiv;
                  double asal;
                  double bsal;
                  double csal;
                  double coff;
                  double cgain;
                  double ctref;
                };

union u_cals  { struct sio_cals s;
                double c[sizeof(struct sio_cals)/sizeof(double)];};

struct type1_meta_struct { int AOML_number;
                           int WMO_id;
                           int ARGOS_id;
                           int wmo_instrument;
                           int wmo_recorder;
                           float cycle_time;
                           int exp_no;
                           unsigned char float_type[40];
                           unsigned char PI[60];
                           float launch_time;
                           float ref_time;
                           union u_cals cals;
                         };

// First structure used processing SIO salinity floats.
// Contains position info, linear array of data bytes.
struct sio_ctd_out_array { int jday;
                           double position_jd[max_positions];
                           float latitude[max_positions];
                           float longitude[max_positions];
                           char date_time[max_positions][32];
                           int n_xmits[max_positions];
                           char sat_name[max_positions];
                           char loc_class[max_positions];
                         };

// The following two sub structures used in processing SIO 
// salinity floats.  These sub structures are part of the
// SIO_CTD_RAW structure described below
struct sio_ctd_data_frame { unsigned int ktmin;
                            unsigned int ktmax;
                            unsigned int ksmin;
                            unsigned int ksmax;
                            unsigned int tmsb2;
                            unsigned int tmsb1;
                            unsigned int tlsb[14];
                            unsigned int smsb1;
                            unsigned int slsb[14];
                          };

struct sio_ctd_diagnostics { unsigned int pcnts;
                             unsigned int t1;
                             unsigned int v0;
                             unsigned int v100[7];
                             unsigned int v50[7];
                             unsigned int cval[7];
                             unsigned int t2;
                           };

// Equivalence of above structure to array of integers to 
// simplify printing diagnostic values
union diags { struct sio_ctd_diagnostics sio;
              unsigned int ints[25];
            };

// Second structure used in decoding SIO salinity floats
// All parameters encoded in linear array x in SIO_CTD_OUT_ARRAY 
// structure described above are decoded and put into the 
// following structure
struct sio_ctd_raw { unsigned int pavg1;
                     unsigned int tavg2;
                     unsigned int vcpu;
                     unsigned int tavg1;
                     unsigned int pavg2;
                     unsigned int vpmp;
                     unsigned int spress;
                     unsigned int v50;
                     unsigned int v100;
                     unsigned int err;
                     unsigned int imin;
                     unsigned int bmax;
                     struct sio_ctd_data_frame data[4];
                     union diags diagnostics;
                   };

// Final array for decoding SIO salinity floats
/* (JZ)
struct temp_sal_counts { int depth[104];
                         int temp[104];
                         int salinity[104];
                       };
*/
struct temp_sal_counts { int depth[104];
                         float temp[104];
                         float salinity[104];
                       };

long ymd2julday (int, int, int);

/******************************************************************************
  Converts an ASCII string to uppercase
******************************************************************************/
char  *strupr(char* s)
{
  char* p = s;
  while (*p) {
    if (islower((int)*p)) *p = toupper((int)*p);
    ++p;
  }
  return s;
}

/******************************************************************************
  Computes days from 1997
******************************************************************************/
double dfr97 (const int yp, const int mp, const int dp,
              const int hr, const int minutes, const int secs)
{
  int    yr, mo, dy, day_from_97;
  long   passed_jd, jd_xx0101, jd_970101;
  double d_jd;

  yr = yp;
  mo = mp;
  dy = dp;

  if (yr <= 15)
   yr += 2000;
  else if (yr < 1900)
   yr += 1900;

  if ((yr<1997) || (yr>2015) || (mo<1) || (mo>12) || (dy<1) || (dy>31))
   return (0.0);

  passed_jd = ymd2julday( yr, mo, dy ) + 1;     // (current day/current month/current
                                                //  year)
  dy = 1;
  mo = 1;
  jd_xx0101 = ymd2julday( yr, mo, dy );         // (1/1/current year)
  yr = 1997;
  jd_970101 = ymd2julday( yr, mo, dy );         // (1/1/1997)
  day_from_97 = (int) (passed_jd - jd_970101);  // Days from 1/1/1997 (ignoring hrs,
                                                // mins, secs)
  d_jd = day_from_97;
  d_jd += (hr+(minutes + secs/60.0)/60.0)/24.0; // Converts hrs, mins, secs to days
  return (d_jd);                                // and adds this to d_jd
}

/******************************************************************************
  Computes final values for SIO salinity floats and fills temp_sal_counts 
  structure 
******************************************************************************/
int compute_sio_ctd_counts (struct type1_meta_struct *s, struct sio_ctd_raw *x, 
                            struct temp_sal_counts *ts)
{
// (JZ)int i,j,c0;
  int i,j;
  float c0;
  float lookup[16] = {-4.0,-2.5,-1.5,-1.0,-0.75,-0.5,-0.25,0.0,
                      0.25,0.5,0.75,1.0,1.5,2.5,4.0,6.25};
  float gain,offset;

  // Compute temperature counts - pretty complicated - see manual for details
  for (i=0; i<4; i++)
  {
    gain = lookup[x->data[i].ktmax] - lookup[x->data[i].ktmin]; // Compute gain
// (JZ)    offset = (float) lookup[x->data[i].ktmin] * 256; // Compute offset
    offset = (float) (lookup[x->data[i].ktmin] * 256.0); // Compute offset
// (JZ)    c0 = (int) (x->data[i].tmsb1 * 256 + x->data[i].tlsb[0]); // Compute counts for
    c0 = (float) (x->data[i].tmsb1 * 256 + x->data[i].tlsb[0]); // Compute counts for
                                                                // first bin
    for (j=0; j<13; j++)
    {
      ts->temp[14*i + j] = c0;
// (JZ)c0 += (int)(x->data[i].tlsb[j+1] * gain + offset); // Compute counts for
      c0 += (float)(x->data[i].tlsb[j+1]) * gain + offset; // Compute counts for
                                                          // remaining bins
    }

    ts->temp[14*i + j] = c0;
  }

  // Compute conductivity counts - see manual for details
  for (i=0; i<4; i++)
  {
    gain = lookup[x->data[i].ksmax] - lookup[x->data[i].ksmin]; // Compute gain
// (JZ)    offset = lookup[x->data[i].ksmin] * 64; // Compute offset
    offset = (float) (lookup[x->data[i].ksmin] * 64.0); // Compute offset 
// (JZ)    c0 = (int) (x->data[i].smsb1 * 64 + x->data[i].slsb[0]); // Compute counts for
    c0 = (float) (x->data[i].smsb1 * 64 + x->data[i].slsb[0]); // Compute counts for
                                                               // first bin

    for (j=0; j<13; j++)
    {
      ts->salinity[14*i + j] = c0;
// (JZ)c0 += (int)(x->data[i].slsb[j+1] * gain + offset); // Compute counts for
      c0 += (float)(x->data[i].slsb[j+1]) * gain + offset; // Compute counts for
                                                           // remaining bins
    }

    ts->salinity[14*i + j] = c0;
  }

  // Depth - see manual
  for (i=0; i<56; i++)
    ts->depth[i] = i;

  if (trace_file != NULL)
    fprintf(trace_file," COMPUTE_SIO_CTD_COUNTS successful\n");

  return(0);
}

/******************************************************************************
  Decodes raw data for SIO salinity floats and fills intermediate
  structure sio_ctd_raw 
******************************************************************************/
int decode_sio_ctd_data (unsigned char *x, struct sio_ctd_raw *y)
{
  unsigned char *ptr;
  unsigned char *ptr2;
  unsigned int v5c,tmp[7];
  int i,j,k;

  ptr = x;
  ptr2 = ptr + 1;
  sscanf ( (const char *) ptr2,"%3x%2x%2x",&y->pavg1,&y->tavg2,&y->vcpu); // Copies from  
                                                                          // location ptr 
                                                                          // into structure
  ptr += 64;
  sscanf ( (const char *) ptr,"%*1x%3x%2x%2x%",&y->tavg1,&y->pavg2,&y->vpmp);
  ptr += 64;
  sscanf ( (const char *) ptr,"%*1x%3x%1x%3x%",&y->spress,&v5c,&y->v100);
  ptr += 64;
  sscanf ( (const char *) ptr,"%*1x%1x%2x%2x%2x",&y->err,&y->imin,&y->bmax,&y->v50);
  y->v50 = 256*v5c + y->v50;
  y->tavg2 += (y->tavg1 & 0x0F00);
  y->pavg2 += (y->pavg1 & 0x0F00);

  if (y->pavg2 > y->pavg1 + 128)
     y->pavg2=y->pavg2-256;
  else if (y->pavg1 > y->pavg2 + 128)
     y->pavg2=y->pavg2+256;

  if (y->tavg2 > y->tavg1 + 128)
     y->tavg2=y->tavg2-256;
  else if (y->tavg1 > y->tavg2 + 128)
     y->tavg2=y->tavg2+256;

  for (k=0; k<4; k++)
  {
    ptr = x + 64*k + 8;

    sscanf ((const char *) ptr,"%1x%1x%1x%1x%1x%1x",
            &y->data[k].ktmin,&y->data[k].ktmax,
            &y->data[k].ksmin,&y->data[k].ksmax,
            &y->data[k].tmsb2,&y->data[k].tmsb1);

    ptr += 6;

    sscanf ((const char *) ptr,"%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x",
            &y->data[k].tlsb[0],&y->data[k].tlsb[1],
            &y->data[k].tlsb[2],&y->data[k].tlsb[3],
            &y->data[k].tlsb[4],&y->data[k].tlsb[5],
            &y->data[k].tlsb[6],&y->data[k].tlsb[7],
            &y->data[k].tlsb[8],&y->data[k].tlsb[9],
            &y->data[k].tlsb[10],&y->data[k].tlsb[11],
            &y->data[k].tlsb[12],&y->data[k].tlsb[13]);

    ptr += 28;

    sscanf ((const char *) ptr,"%1x%3x%3x%3x%3x%3x%3x%3x",&y->data[k].smsb1,
            &tmp[0],&tmp[1],&tmp[2],&tmp[3],&tmp[4],
            &tmp[5],&tmp[6]);

    j = 0;

    for (i=0; i<7; i++)
    {
      y->data[k].slsb[j++] = (tmp[i] & 0x0FC0) >> 6;
      y->data[k].slsb[j++] = tmp[i] & 0x003F;
    }
  }

  // Decode diagnostic values if present.  I don't know what to do with 
  // them!!! Maybe they shoudl be written to the output???
  if (strlen( (const char *) x) > 256)
  {
    ptr = x + 256;

    sscanf((const char *) ptr,"%*1x%3x%4x%4x%4x%4x%4x",
           &y->diagnostics.sio.pcnts,&y->diagnostics.sio.t1,
           &y->diagnostics.sio.v100[0],&y->diagnostics.sio.v50[0],
           &y->diagnostics.sio.v0,&y->diagnostics.sio.cval[0]);

    ptr = x + 256 + 24;

    for (i=1; i<7; i++)
    {
      sscanf ( (const char *) ptr,"%2x%2x%2x",
      &y->diagnostics.sio.v100[i],&y->diagnostics.sio.v50[i],
      &y->diagnostics.sio.cval[i]);
      y->diagnostics.sio.v100[i] += y->diagnostics.sio.v100[0] & 0xFF00;
      y->diagnostics.sio.v50[i] += y->diagnostics.sio.v50[0] & 0xFF00;
      y->diagnostics.sio.cval[i] += y->diagnostics.sio.cval[0] & 0xFF00;
      ptr += 6;
    }

    sscanf ( (const char *) ptr,"%4x",&y->diagnostics.sio.t2);
  }
  else
    for (i=0; i<25; i++)
      y->diagnostics.ints[i] = 0;

  if (trace_file != NULL)
    fprintf(trace_file," DECODE_SIO_CTD_DATA successful %d\n",j);

  return(0);
}

/******************************************************************************
  SIO salinity floats have no check sum, so must determine the most
  likely scan.  Each message consists of four scans with first hex
  character giving scan number (0 - 3).  There is also a less frequent
  diagnostic scan which has the first hex character 'F'.  This routine
  compares the hex strings of each of the scan types and finds the most
  common scan of each type.  It then composes a linear array of these
  most likely scans ond returns them in array x.  Returns 0 if successful,
  1 if error
******************************************************************************/
int compose_array (unsigned char *x, unsigned char **hex_data, int n_scans)
{ 
  int i=0,j,k,l;
  int most_likely[5]={-1,-1,-1,-1,-1},matches[5]={0,0,0,0,0};

  total_cycles_count++;//added new
  
  for (j=0; j<5; j++)   // Zero returned array
  {
    l = 64*j;

    for (k=0; k<64; k++) // added to zero scan
      x[l+k] = '\0';   //Append NULL character
  }

  if (n_scans < 5)   // Need at leat 4 scans to form a message ,Note: n_scan is alway larger than the actual number
                     // of scans by 1 in DB's code.
  {
    x[0] = '\0';   //Error - return 0 length array

    if (error_file != NULL)
      fprintf(error_file," ERROR: COMPOSE_ARRAY to few scans to form complete message, n_scans = %d\n",n_scans);

    if (trace_file != NULL)
      fprintf(trace_file," ERROR: COMPOSE_ARRAY to few scans to form complete message, n_scans = %d\n",n_scans);

    frame_problem_count++; //added new

    return(1);
  }       // end if (n_scans < 5)

  for (i=0; i<n_scans-1; i++)
  {
    // Do include scans that have position data - hex string is all '0'
    if (strstr( (const char *) hex_data[i],
        "0000000000000000000000000000000000000000000000000000000000000000") != NULL)
      continue;

    // Check if first hex character is in range 0-3 or F */
    if ((sscanf( (const char *) hex_data[i],"%1x",&k) != 1) || (((k < 0) || (k > 3)) 
        && (k != 15)))
      continue;

    k = (k == 15) ? 4 : k; //First character = F -> scan type 4
    l = 0;

    for (j=i+1; j<n_scans; j++)
      if (strcmp( (const char *) hex_data[i], (const char *) hex_data[j]) == 0) // Compare
                                                                                // scans
        l++;   //Increment count if scans match

    if (l >= matches[k]) //Does this scan have the most matches?
    {
      most_likely[k] = i; //If yes, store this scan number
      matches[k] = l;
    }
    if ((l==0) && (matches[k]==0)) // Make sure to use messages received only once
    {
      matches[k]=-1;
    }
  }   // end for (i=0; i<n_scans-1; i++)

  for (j=0; j<5; j++)
  {
    // If any data type scans missing, return error.  If diagnostic
    // scan missing, no problem
    if (matches[j] == 0)
    {
      if (j >= 4)
        break;  // No diagnostic message -> no problem

      x[0] = '\0';

      if (error_file != NULL)
        fprintf(error_file," ERROR: COMPOSE_ARRAY frame missing, scan %d\n",j);

      if (trace_file != NULL)
        fprintf(trace_file," ERROR: COMPOSE_ARRAY frame missing, scan %d\n",j);

      frame_problem_count++; //added new

      return(1); //Error return
    }   // end if (matches[j] == 0)

    // Put most likely scans in array in structure
    l = 64*j;

    for (i=0; i<64; i++)
      x[l+i] = hex_data[most_likely[j]][i];

    x[l+i] = '\0'; // Append NULL character
  }   // end for (j=0; j<5; j++)
   
  if (trace_file != NULL)
    fprintf(trace_file," COMPOSE_ARRAY successful\n");

  return(0);
}

/******************************************************************************
  Writes output data to disk for floats.
******************************************************************************/
int write_sio_to_disk(char *dir_name, struct type1_meta_struct *s, 
                      struct sio_ctd_out_array *out_array, 
                      unsigned char **hex_data, int j, int write_flg)
{
  struct sio_ctd_raw z;
  struct temp_sal_counts ts;
  static int prof_no=0,old_day=0;
  int i,n_bins;
  int compose_array_error;
  unsigned char data_array[1024];
  char *ptr_cnt_filename;

  //Reject if only one data scan received
  if (j <= 1)
  {
    if (error_file != NULL)
      fprintf(error_file," ERROR: WRITE_SIO_TO_DISK, single data scan - no output file written\n");

    if (trace_file != NULL)
      fprintf(trace_file," ERROR: WRITE_SIO_TO_DISK, single data scan - no output file written\n");

    return(2);
  }

  // Decode profile
  // Write hex_data to data_array
  if ((compose_array_error = compose_array (data_array,hex_data,j)) != 0 )
  {
    if (error_file != NULL)
      fprintf(error_file," ERROR: WRITE_SIO_TO_DISK, compose array error - no output file written\n");

    if (trace_file != NULL)
      fprintf(trace_file," ERROR: WRITE_SIO_TO_DISK, compose array error - no output file written\n");

    return(1);
  }

  // Raw data is now stored in structures DATA_ARRAY[].
  // Now compute output values from DATA_ARRAY and write to disk
  //
  // WARNING:
  // If the numerator increases by < cycle_time (11 days), the new prof_no will
  // be the same as, and hence overwrite, the old one. So the new cnt file will
  // be written over the old one. This happens occasionally due to a clock drift
  // error in the float. For this reason the number is rounded (by adding .5 and
  // then truncating to an integer). This takes care of clock drift in ref_time.
  //
  // Note: jday is an int and ref_time and cycle_time are float
  //

//  if ((prof_no = (int) ((out_array->jday - s->ref_time)/s->cycle_time + .5)) < 0)
//    return (1);

  if ((out_array->jday - s->ref_time) < 0)
    return (1);
  if (old_day == 0)
  {
   prof_no = (int) ((out_array->jday - s->ref_time)/s->cycle_time + .5);
   old_day = out_array->jday;
  }
//  else if ((out_array->jday - old_day) < (int) (s->cycle_time - 1.0))
  else if ((out_array->jday - old_day) < 1)
   return 1;
  else
  {
   prof_no = prof_no + (int) ((out_array->jday - old_day)/s->cycle_time + .5);
   old_day = out_array->jday;
//   prof_no++;
  }
  if (!write_flg)
	  return 0;

  sprintf (buffer,"%s%04d_%05d_%03d.cnt",dir_name,s->AOML_number,s->ARGOS_id,prof_no);
  printf ("Opening output file %s\n",buffer);

  if ((out_file = fopen(buffer,"w+")) == NULL) // used to be fopen(buffer,"a+")
  {
    if (error_file != NULL)
      fprintf(error_file," ERROR: WRITE_SIO_TO_DISK, error opening output file %s\n",buffer);

    if (trace_file != NULL)
      fprintf(trace_file," ERROR: WRITE_SIO_TO_DISK, error opening output file %s\n",buffer);

    return(1);
  }

  if (trace_file != NULL)
    fprintf(trace_file," WRITE_SIO_TO_DISK: output file %s successfully opened\n",buffer);

  // Write Output Profile Count File Name to List File

  // Check if last occurence of '/' in buffer is NULL
  if ( (ptr_cnt_filename = strrchr(buffer, '/')) == NULL)
  {
    printf ("\nCan't get cnt file basename %s", buffer);

    if (trace_file != NULL)
      fprintf (trace_file, "\n%s : Can't get cnt file basename %s", PRGNAME, buffer);

    if (error_file != NULL)
      fprintf (error_file, "\n%s: Can't get cnt file basename %s", PRGNAME, buffer);

    return 1;
  } // endif: ptr_cnt_filename

  ptr_cnt_filename++;
  fprintf (list_file,"%s\n", ptr_cnt_filename); // Count File Name

  decode_sio_ctd_data (data_array,&z); // Decode data from data_array and write into z
  compute_sio_ctd_counts(s,&z,&ts); // Compute counts from z and write into ts

  // Write header info to buffer 
  sprintf (buffer,"INTERNAL ID NUMBER                      %04d\n"
                  "WMO ID NUMBER                           %d\n"
                  "TRANSMISSION ID NUMBER                  %05d\n"
                  "PROFILE NUMBER                          %d\n"
                  "INSTRUMENT TYPE                         %s\n",
            s->AOML_number,abs(s->WMO_id),s->ARGOS_id,prof_no,s->float_type);

  // Write buffer to out_file
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,"WMO INSTRUMENT TYPE (TABLE 1770)        %d\n"
                  "WMO RECORDER TYPE (TABLE 4770)          %d\n"
                  "ARGOS PROGRAM NUMBER                    %d\n"
                  "PI                                      %s\n",
           s->wmo_instrument, s->wmo_recorder, s->exp_no, s->PI);

  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,"DRIFT TEMPERATURE (COUNTS)              %-5d %-5d\n"
                  "DRIFT PRESSURE (COUNTS)                 %-5d %-5d\n"
                  "START SURFACE PRESSURE (COUNTS)         %d\n"
                  "BATTERY CPU (COUNTS)                    %d\n"
                  "BATTERY PUMP (COUNTS)                   %d\n",
           z.tavg1,z.tavg2,z.pavg1,z.pavg2,z.spress,z.vcpu,z.vpmp);

  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,"PROFILE TERMINATION FLAG                %d\n"
                  "NUMBER OF COLUMNS                       4\n"
                  "1. COLUMN                               PRESSURE (COUNTS)\n"
                  "2. COLUMN                               TEMPERATURE (COUNTS)\n"
                  "3. COLUMN                               CONDUCTIVITY (COUNTS)\n"
                  "4. COLUMN                               CHECKSUM FLAG\n",
           compose_array_error);

  fwrite (strupr(buffer),1,strlen(buffer),out_file);
  sprintf (buffer,"==========================================================\n");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  i = 0;
  sprintf (buffer,"   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS\n");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  while (out_array->position_jd[i] > 0.0)
  {
    //           "----LLL.LLL---LLLL.LLL-YYYY/MM/DD-HH:MM:SS---XXX---S-----C"
    sprintf (buffer,"    %+7.3f   %+8.3f %s   %03d   %1c     %1c\n",
             out_array->latitude[i], out_array->longitude[i],out_array->date_time[i],
             out_array->n_xmits[i], out_array->sat_name[i],out_array->loc_class[i]);

    fwrite (strupr(buffer),1,strlen(buffer),out_file);

    if (++i >= max_positions)
      exit (1);
  }

  fprintf (out_file,"==========================================================");

  // Write depth, temperature, and salinity to disk
  if (compose_array_error == 0)
  {
    n_bins = (z.bmax > 56) ? 55 : z.bmax-1;

    for (i=n_bins; i>=0; i--)
    {
// (JZ)      sprintf (buffer,"\n %5d %5d %5d 1",
// (JZ)      ts.depth[i],ts.temp[i],ts.salinity[i]);
      sprintf (buffer,"\n %5d %9.3f %9.3f 0",
                         ts.depth[i],ts.temp[i],ts.salinity[i]);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
    }
    fprintf(out_file,"\n");
  }
  else
  {
    if (trace_file != NULL)
      fprintf (trace_file," ERROR: missing scans - no T,C,D counts written\n");

    if (error_file != NULL)
      fprintf (error_file," ERROR: missing scans - no T,C,D counts written\n");
  }

//  fwrite("\n\n",1,strlen("\n\n"),out_file); // Append 2 <LF>
         fclose (out_file);                        // Close output file 

  if (trace_file != NULL)
    fprintf (trace_file," WRITE_SIO_TO_DISK successful\n");

  return(0);
}

/******************************************************************************
  Main routine for decoding SIO salinity float data.  Reads input
  file and gets data scans for float number = id and profile on
  julian day = jday.  If jday=0, return all profiles.  Returns 0
  if successful, 1 if error
******************************************************************************/
int get_sio_ctd_data (int flg, struct type1_meta_struct *s, char *out_dir)
{
  struct sio_ctd_out_array out_array;
  struct sio_ctd_raw;
  struct temp_sal;

  unsigned char **hex_data,l_c,sat;
  int n_pos=0,n_prof=0,jd_old=0,jd,n_l;
  int j=0,i,iy,imo,ida,ih,im,is,g;
  int write_sio_to_disk_error;

  long int dummy;

  float lat,lon;

  // Allocate needed memory 
  if ((hex_data = (void *) malloc (max_scans*sizeof(void *))) == NULL)
  {
    printf ("\nError allocating raw out_array");
    return(1);
  }

  for (i=0; i<max_scans; i++)
    if ((hex_data[i] = (unsigned char *) malloc (buff_size)) == NULL)
    {
      printf ("\nError allocating raw out_array");
      return(1);
    }

  for (i=1; i<max_positions; i++)
    out_array.position_jd[i] = -1.0;

  // Read in the data
  j = 0;

  // Reads a line from input_file into buffer and check if NULL
  // (Data accumulates in buffer until a new profile is read)
  while (fgets(buffer,sizeof(buffer),input_file) != NULL)
  {
    // Read chars from buffer, store them in variables and check if
    // exactly 15 were read
    if(sscanf(buffer,"%d %d %d %d %d %d %d %f %f %d %d %c %c %ld %64s",
       &i,&iy,&imo,&ida,&ih,&im,&is,&lat,&lon,&g,&n_l,&sat,&l_c,&dummy,hex_data[j]) == 15)
    {
      // Check if data collected after launch time
      if (dfr97(iy,imo,ida,ih,im,is) < s->launch_time)
        continue;

      // Check if new profile (since data collected every 10 days) and
      // not 1st profile
      if ((abs(jd_old - (jd = (int) dfr97 (iy,imo,ida,0,0,0))) > 1) && (j>0))
      {
        // Check if you want to include all profiles (or just the last one). If so,
        // then read/write profile each time. Otherwise, do not read/write profile.
        // The last profile will be written when the loop exits.
//        if (flg > 0)
         write_sio_to_disk_error=write_sio_to_disk(out_dir,s,&out_array,hex_data,j,flg);
	  //if processing all profiles, and for some reason it can not write output file,count it
	 if(flg==1 && write_sio_to_disk_error==1) no_output_cycle_count++;//added new 
     
        strcpy((char *) hex_data[0],(const char *) hex_data[j]); // write  hex_data[j] to
                                                                 // hex_data[0], since 
                                                                 // new profile
        j = 1; // Reset to 1 in preparation for next profile (>1 day later)

        for (i=1; i<max_positions; i++)
          out_array.position_jd[i] = -1.0;

        n_pos = 0;
      }

      jd_old = jd; // set to days currently read

      if (n_pos == 0)
        out_array.jday = jd;

      out_array.sat_name[n_pos] = sat;
      out_array.loc_class[n_pos] = l_c;
      out_array.position_jd[n_pos] = dfr97 (iy,imo,ida,ih,im,is);
      out_array.latitude[n_pos] = lat;
      out_array.longitude[n_pos] = lon;

      sprintf(out_array.date_time[n_pos],
              "%4d/%02d/%02d %2d:%02d:%02d",iy,imo,ida,ih,im,is);

      out_array.n_xmits[n_pos] = (int) floor( (float)(n_l-1)/8.);

      if ((lat == (float) 0.0) && (lon == (float) 0.0))
      {
        out_array.latitude[n_pos] = (float) 99.999;
        out_array.longitude[n_pos] = (float) 999.999;
        out_array.n_xmits[n_pos] = 0;

        // Increment j and check if maximum scans exceeded
        if (++j >= max_scans) // hex_data[0],...,hex_data[max_scans-1]
          exit(1);

        n_pos =  (n_pos == 0) ? 1 : n_pos;
      }
      else
      {
        if (++n_pos >= max_positions)
          exit(1);
      }
    } // if
  } // while

  // Read/write last profile
  write_sio_to_disk_error=write_sio_to_disk(out_dir,s,&out_array,hex_data,j,1);
  //if processing all profiles, and for some reason it can not write output file,count it
  if(flg==1 && write_sio_to_disk_error==1) no_output_cycle_count++;//added new 
  
  free (hex_data);
  
  //if processing last profile only and it can not output,return 1
  if(flg==0 && write_sio_to_disk_error==1)   return(1);

  //if processing all profiles, not all profiles are generated to output directory,then return 2
  if(flg==1 && no_output_cycle_count>0 && no_output_cycle_count<total_cycles_count) return(2);

  if (trace_file != NULL)
    fprintf (trace_file," GET_SIO_CTD_DATA successful\n");

  return(0);
}

/******************************************************************************
  Assumes str_name = *.hex, returns *.meta in str_name
******************************************************************************/
int get_meta_file_name (char *str_name)
{
 char *ptr;

 // Check if pointer to 1st occurrence of .hex in str_name is NULL
  if ((ptr = strstr(str_name,".hex"))!= NULL)
    sprintf (ptr,".meta");
  else
    strcat(str_name,".meta");

  return 0;
}

/******************************************************************************
  Reads meta_file
******************************************************************************/
int read_type1_meta_file (char *meta_name, struct type1_meta_struct *meta_struct)
{
  FILE *meta_file;
// (JZ)   int i,j,syear,smonth,sday,shour,smin,x;
  int i,j,syear,smonth,sday,shour,smin;
  float x;
  int lyear,lmonth,lday,lhour,lmin;
  unsigned char *ptr;

  char *cal_keys[] = {"PGAIN=", "POFF=", "PMP_GAIN=", "PMP_OFF=", "CPUGAIN=", 
                      "CPUOFF=", "AT=", "BT=", "CT=", "H1=", "H2=", "H3=", "BLOK=", 
                      "AV1=", "AV2=", "PB1=", "PB2=", "CDIV=", "ASAL=", "BSAL=", 
                      "CSAL=", "COFF=", "CGAIN=", "CTREF=", "\0"};

  meta_struct->cycle_time = (float) 0.0;
  meta_struct->ref_time = (float) 0.0;

  printf("\nOpening meta file %s\n",meta_name);

  if ((meta_file = fopen(meta_name,"r")) == NULL)
  {
    printf ("Error opening meta file %s\n",meta_name);

    if (error_file != NULL)
      fprintf (error_file," ERROR opening meta file %s\n",meta_name);

    return(1);
  }

  if (trace_file != NULL)
    fprintf (meta_file," Meta file %s successfully opened\n",meta_name);

  // Reads a line from meta_file into buffer and checks if NULL
  while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
  {
    // Convert buffer to upper case, return a pointer to the first
    // occurrence of INTERNAL ID NUMBER in buffer, and check if NULL
    if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
    {
      // Read chars from buffer and store them in AOML_number inside
      // structure.
      //
      // Note: * means the field does not need to be included in list of
      // variable names to write to
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) != 1)
        meta_struct->AOML_number = 0;
      continue;
    }

    if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) != 1)
        meta_struct->WMO_id = 0;
      continue;
    }

    if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) != 1)
        meta_struct->ARGOS_id = 0;
      continue;
    }

    if (strstr(strupr(buffer),"DELAY OF FIRST DOWN TIME") != NULL)
    {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %f",&x);
      meta_struct->ref_time += (float) (x/24.0);
      continue;
    }

    if (strstr(strupr(buffer),"WMO INSTRUMENT TYPE") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_instrument) != 1)
      meta_struct->wmo_instrument = 0;
     continue;
    }
  
    if (strstr(strupr(buffer),"WMO RECORDER TYPE") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_recorder) != 1)
      meta_struct->wmo_recorder = 0;
   
	 continue;
    }

// DOWN TIME was changed to CYCLE TIME in meta files (CS)
    if (strstr(strupr(buffer),"CYCLE TIME") != NULL)
    {
      sscanf(buffer,"%*s %*s %*s %f",&x);
      meta_struct->cycle_time = x;
      continue;
    }

// UP TIME was changed to SURFACE TIME in meta files (CS)
    if (strstr(strupr(buffer),"SURFACE TIME") != NULL)
    {
      sscanf(buffer,"%*s %*s %*s %f",&x);
// (CS) now obsolete     meta_struct->cycle_time += (float) (x/24.0);
      meta_struct->ref_time -= (float) (x/24.0);
      continue;
    }

    // ref_time = FIRST DOWN TIME - SURFACE TIME + START TIME
    //            ^                 ^
    //            6 hours           1 day     dfr97 days   
    if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",
             &sday,&smonth,&syear,&shour,&smin);

      meta_struct->ref_time += (float) (dfr97(syear,smonth,sday,shour,smin,0));
      continue;
    }

    if (strstr(strupr(buffer),"PROGRAM NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->exp_no) != 1)
        meta_struct->exp_no = 0;

      continue;
    }

    if ((strstr(strupr(buffer),"INSTRUMENT TYPE") != NULL) && (strstr(strupr(buffer),"WMO") == NULL))
    {
      if (sscanf(buffer,"%*s %*s %s",meta_struct->float_type) == 0)
      {
        sprintf (meta_struct->float_type,"\0");
        continue;
      }
    }

    if (strstr(strupr(buffer),"PI     ") != NULL)
    {
      if (sscanf(buffer,"%*s %s",meta_struct->PI) == 0)
      {
        sprintf (meta_struct->PI,"\0");
        continue;
      }

      if ((ptr = strstr(&buffer[3],meta_struct->PI)) != NULL)
        sprintf (meta_struct->PI,"%s",ptr);

      if ((ptr = strstr(meta_struct->PI,"\n")) != NULL)
        sprintf (ptr,"\0");

      continue;
    }

    if ((strstr(strupr(buffer),"LAUNCH TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) { 
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",
           &lday,&lmonth,&lyear,&lhour,&lmin);
      meta_struct->launch_time = (float) (dfr97(lyear,lmonth,lday,lhour,lmin,0));
      continue;
    }

    i = 0;

    while ((j = strlen(cal_keys[i])) > 1)
    {
      // Get pointer to 1st occurence of cal_keys[i] in buffer
      if ((ptr = (unsigned char *)strstr(buffer,cal_keys[i])) != NULL)
        meta_struct->cals.c[i] = atof((const char *)(ptr+j));

      i++;
    }
  }
  return 0;
}

/******************************************************************************
  'main' function
******************************************************************************/
int main (int argc, char *argv[])
{
  char     *ptr_path_meta, *ptr_path_hex, *ptr_path_cnt,
           *ptr_path_log, *ptr_path_list,
           pathname_input_filename[161]="",
           pathname_trace[161]="", pathname_error[161]="";

  char     input_filename[80],output_directory[256];
  enum     {LAST=0,ALL} get_which_profile;
  struct   type1_meta_struct s = {0,0,0,0,0,(float)0.0,0,"UNKNOWN","UNKNOWN",(float)0.0,(float)0.0};
  unsigned long t;
  int      ret_val = 0;

  // Program Name 
  if (argc < 7)
  {
    printf("Program generates temperature, salinity and pressure counts for PALACE_TS1 Floats\n");
    printf("Format: %s hex_infile nprofile cnt_outdir trace_outile error_outfile\n",
           PRGNAME);
    printf("\n");
    printf("hex_infile:    hexadecimal input file name\n");
    printf("nprofile:      1=get all profiles (default); 0=get last profile\n");
    printf("cnt_outdir:    output directory for generated count data file(s)\n");
    printf("trace_outile:  output directory for generated trace file\n");
    printf("error_outfile: output directory for generated error file\n");
    printf("list_outfile:  output directory for generated list file\n");
    return 1;
  }

  sprintf (input_filename,"%s",argv[1]);

  // Number of profiles to process: 0, get last profile or 1, get all profiles
  if (atoi(argv[2]) == 1)
    get_which_profile = ALL;
  else
    get_which_profile = LAST;

  // Output directory path for count file(s)
  // ---------------------------------------
  if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): CNTFILES_DIR <-- no such environment variable\n", PRGNAME);
    ret_val = 1;
    return(ret_val);
  }

  sprintf (output_directory,"%s/",argv[3]);

  // TRACE File
  // ----------
  if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
  {
    printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
    ret_val = 1;
    return(ret_val);
  }

  sprintf (buffer,"%s",argv[4]);
  printf ("\nOpening trace file %s",buffer);

  if ((trace_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nError opening trace file %s",buffer);
    ret_val = 1;
    return ret_val;
  }

  printf("\nOK -> trace file opened\n");

  if (trace_file != NULL)
  {
    t = time(NULL);
    fprintf (trace_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

  // ERROR File
  // ----------
  sprintf (buffer,"%s",argv[5]);

  printf ("\nOpening error file %s",buffer);

  if ((error_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("Error opening error file %s\n",buffer);
    if (trace_file != NULL)
      fprintf (trace_file," ERROR (%s): error opening file %s\n", PRGNAME, buffer);
    ret_val = 1;
    return ret_val;
  }

  printf("\nOK -> error file opened\n");

  if (trace_file != NULL)
    fprintf (trace_file," Error file %s successfully opened\n",buffer);

  if (error_file != NULL)
  {
    t = time(NULL);
    fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

  // LIST File
  // ---------
  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
    printf("\nERROR (%s): ARGOLIST_DIR <-- no such environment variable",
           PRGNAME);
    ret_val = 1;
    return ret_val;
  }

  sprintf (buffer,"%s",argv[6]);

  printf ("\nOpening list file %s",buffer);

  if ((list_file = fopen(buffer,"w")) == NULL)
  {
    printf ("\nError opening list file %s\n",buffer);

    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): Error opening list file %s\n",
               PRGNAME, buffer);

    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): Error opening list file %s\n",
              PRGNAME, buffer);

    ret_val =1;
    return ret_val;
  }

  printf("\nOK -> list file opened\n");

  if (trace_file != NULL)
    fprintf (trace_file," List file %s successfully opened\n", buffer);

  // INPUT Hex File
  // --------------
  if ( (ptr_path_hex = getenv("HEXFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): HEXFILEDIR <-- no such environment variable\n", PRGNAME);

    if (trace_file != NULL)
      fprintf (trace_file," ERROR: HEXFILES_DIR <-- no such environment variable\n");

    if (error_file != NULL)
      fprintf (error_file," ERROR: HEXFILES_DIR <-- no such environment variable\n");

    if (trace_file != NULL)                  // Close Log files
      fclose (trace_file);

    if (error_file != NULL)
      fclose (error_file);

    ret_val = 1;
    return(ret_val);
  }

  sprintf(pathname_input_filename, "%s/%s", ptr_path_hex, input_filename);

  if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
  {                                           // Fail to open Hex file
    printf ("\nError opening input file %s", pathname_input_filename);

    if (trace_file != NULL)
      fprintf (trace_file," ERROR: input file %s not opened\n", pathname_input_filename);

    if (error_file != NULL)
      fprintf (error_file," ERROR: input file %s not opened\n", pathname_input_filename);

    ret_val = 1;
  }
  else                                        // Success: Hex file opened
  {
    printf ("\nInput file %s successfully opened", pathname_input_filename);

    if (trace_file != NULL)
      fprintf (trace_file," Input file %s successfully opened\n", pathname_input_filename);

    // META File
    // ---------
    if (get_meta_file_name (input_filename) != 0)
    {                                              // Failure: couldn't get meta filename
      printf ("\nError finding meta file name");

      if (error_file != NULL)
        fprintf (error_file, " ERROR: meta file name construction - returned name %s\n",
                 input_filename);

      if (trace_file != NULL)
        fprintf (trace_file, " ERROR: meta file name construction - returned name %s\n",
                 input_filename);

      ret_val = 1;
    }
    else
    {                                              // Success: got meta filename
      if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
      {
        printf("METAFILES_DIR <-- no such environment variable\n");

        if (trace_file != NULL)
          fprintf (trace_file," ERROR: METAFILES_DIR <-- no such environment variable\n");

        if (error_file != NULL)
          fprintf (error_file," ERROR: METAFILES_DIR <-- no such environment variable\n");

        if (trace_file != NULL)                  // Close Log files
          fclose (trace_file);

        if (error_file != NULL)
          fclose (error_file);
 
        ret_val = 1;
        return(ret_val);
      } // endif: ptr_path_meta

      sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

      if (read_type1_meta_file (pathname_input_filename, &s) != 0)
      {                                         // Failure: couldn't read meta_file
        printf ("\nError reading meta file");

        if (error_file != NULL)
          fprintf (error_file," ERROR reading meta file %s\n", pathname_input_filename);

        if (error_file != NULL)
          fprintf (error_file," ERROR reading meta file %s\n", pathname_input_filename);

        ret_val = 1;
      }
      else                                      // Success: read meta_file
      {
        if (trace_file != NULL)
          fprintf (trace_file," Meta file %s successfully read\n", pathname_input_filename);

        // Make profile(s) selection
        // -------------------------
        ret_val=get_sio_ctd_data(get_which_profile,&s,output_directory);
/*        
	if (get_sio_ctd_data(get_which_profile,&s,output_directory) != 0)
          ret_val = 1;
*/
      } // endif: read_type1_meta_file
    } // endif: get_meta_file_name
  } // endif: input_file (open hex file)
  
  printf("HEX filename=%s\n",argv[1]);
  printf("total cycles for this hex file=%d\n",total_cycles_count);
  
  if(get_which_profile==1){
    printf("frame missing problem counts=%d\n",frame_problem_count); 
    printf("number of profiles which failed to produce cnt file=%d\n",no_output_cycle_count);
  }
  printf("return status=%d\n",ret_val);
  
/*  
  if((frame_problem_count>0) && (frame_problem_count<total_cycles_count)){
    ret_val=2;
  }
*/  

  // Write finish message to trace & error files
  // -------------------------------------------
  t = time(NULL);
  if (trace_file != NULL)
    fprintf (trace_file,"=> %s: finished %s\n\n", PRGNAME, ctime(&t));

// Close data files
// ----------------
  if (input_file != NULL)
    fclose (input_file);

  if (trace_file != NULL)
    fclose (trace_file);

  if (error_file != NULL)
    fclose (error_file);

  if (list_file != NULL)
   fclose (list_file);

  return(ret_val);
}
