/******************************************************************************
   Revision Date: 21-JUN-04 (XXD&CS - do not allow negative profile numbers
                            (bug fix); do not try to write a data file if
                            no lines are available (j=n_scans=0), in
                            get_type15_data)
                  07-OCT-03 (XX-correct profile_no if it is not
                            continuous,follow ts3 practice)
                  28-May-03 (XX)
 

   hex2cnt_ts15.c 
   =============
   Decoding of the hexadecimal ARGOS data for temperature/salinity/pressure
   APEX floats APEX_TS15 manufactured in 2000 by Webb Research Cooperation
   and deployed by the University of Washington
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>

#define max_scans        2000    //Maximum scans for each profile
#define buff_size         256    //Maximum length of each scan read from *.str
#define max_positions     100    //Maximum number of positions per profile
#define min_frame_no        1    //Minimun frame number
#define frames_per_message 20    //Number of frames per message
#define good_bad_position  43    //Location of status byte in *.str records
#define start_of_hex_data  63    //Start position of data in *.str records
#define GOOD 1
#define BAD 4
#define NEVER_RECEIVED 0

int actual_frames_per_message;

FILE *input_file, *out_file, *error_file, *trace_file, *list_file, *meta_file;
char   PRGNAME[] ="hex2cnt_ts15";
char   buffer[512];

char *strupr(char* ptr_to_string_to_be_upper_cased); 

long ymd2julday(int, int, int);
char *julday2ymd(long);
char *timefrac2hms(double);

struct aoml4_cals { double TSLOPE;
                    double TOFF;
                    double PSLOPE;
                    double POFF;
                    double SSLOPE;
                    double SOFF;
                    double VSLOPE;
                    double VOFF;
                    double VACSLOPE;
                    double VACOFF;
                  };

union u_cals { struct aoml4_cals s;
               double c[sizeof(struct aoml4_cals)/sizeof(double)];
             };

struct type15_meta_struct { int AOML_number;
                           int WMO_id;
                           int ARGOS_id;
                           int wmo_instrument;
                           int wmo_recorder;
                           int exp_no;
			   unsigned char float_type[40];
                           unsigned char PI[60];
                           float ARGOS_rep_rate;
                           float cycle_time;		//Down time + surface time
                           float start_time;		//Instrument reset
			   float start_of_cycles;	//Start_time+delay_of_start+down_time
                           union u_cals cals;
                         };

/*      Raw data structure used in decoding Temperature/Pressure (T/P)
 float data.  Part of RAW_DATA_RECORD described below */

struct test_fmt { unsigned char msg_blk_no;                  //3
                  unsigned char controller_sn[2];            //4&5
                  unsigned char profile_no;                  //6
                  unsigned char msg_type_flag;               //7 
                  unsigned char current_pressure[2];         //8&9
                  unsigned char battery_volts;               //10
                  unsigned char internal_vacuum;             //11
                  
		  unsigned char zeros;                       //12       different from TS3
                  
		  unsigned char float_status;                //13  
                  unsigned char hours_since_start;           //14
                  unsigned char minutes_since_start;         //15
                  unsigned char seconds_since_start;         //16 
                  unsigned char ARGOS_rep_counts;            //17
                  unsigned char up_time_intervals;           //18  
                  unsigned char down_time_intervals[2];      //19&20
                  unsigned char trip_interval_time;          //21		  
		  
		  unsigned char deep_pump_duration;          //22       different from TS3
		  unsigned char first_surface_pump;          //23       different from TS3
		  unsigned char second_surface_pump;         //24       different from TS3
		  unsigned char ascend_time;                 //25       different from TS3
		  unsigned char always_zero[2];		     //26&27	different from TS3	  
                  
		  unsigned char software_rev_month;          //28
                  unsigned char software_rev_day;            //29
                  unsigned char software_rev_year;           //30
		  
		  unsigned char always_31;                   //31       different from TS3
		  unsigned char always_32;                   //32       different from TS3   
                };

struct data_fmt { unsigned char msg_blk_no;                  //3
                  unsigned char controller_sn[2];            //4&5
                  unsigned char profile_no;                  //6
                  unsigned char profile_length;              //7  
                  unsigned char termination_flag;            //8
                  unsigned char bottom_temperature[2];       //9&10  
                  unsigned char bottom_salinity[2];          //11&12
                  unsigned char bottom_pressure[2];          //13&14
                  unsigned char battery_volts;               //15 
                  unsigned char surface_pressure[2];         //16&17 
                  unsigned char internal_vacuum;             //18
                  unsigned char T_S_P[5*frames_per_message][6];
                };

/* Equivalence of structure with linear array.  Linear array
 contains raw data byte minus first two bytes (check sum and
 frame number) of each frame.  Read T/P data format description */

union data_size { unsigned char data_bytes[frames_per_message][30];
                  struct data_fmt d_fmt;
                  struct test_fmt t_fmt;
                };

/* Intermediate structure containing T/P raw data and positions */

struct raw_data_record { int    jday;
                         double jd[max_positions];
                         float  latitude[max_positions];
                         float  longitude[max_positions];
                         char   date_time[max_positions][32];
                         int    n_xmits[max_positions];
                         char   sat_name[max_positions];
                         char   loc_class[max_positions];
                         union  data_size data;
                       };

/* Final structure containig T/P data and positions */

struct output_record { int    float_id;
                       int    profile_no;
                       int    jd;
                       int    status[frames_per_message];
                       int    msg_blk_no;
                       time_t first_blk_time;
                       int    controller_sn;
                       int    profile_length;
                       int    termination_flag;
                       int    bottom_temperature;
                       int    bottom_salinity;
                       int    bottom_pressure;
                       int    battery_volts;
                       int    surface_pressure;
                       int    internal_vacuum;
                       int    msg_type_flag;
                       int    current_pressure;
                       int    battery_voltage;
                       int    float_status;
                       int    hours_since_start;
                       int    minutes_since_start;
                       int    seconds_since_start;
                       int    ARGOS_rep_counts;
                       int    up_time_intervals;
                       int    down_time_intervals;
                       int    trip_interval_time;
                       int    software_rev_month;
                       int    software_rev_day;
                       int    software_rev_year;
		       
		       /* added new fields here         */
		       int    deep_pump_duration;          //22       different from TS3
		       int    first_surface_pump;          //23       different from TS3
		       int    second_surface_pump;         //24       different from TS3
		       int    ascend_time;                 //25       different from TS3
		       
		       
                       float  latitude[max_positions];
                       float  longitude[max_positions];
                       char   date_time[max_positions][32];
                       int    n_xmits[max_positions];
                       char   sat_name[max_positions];
                       char   loc_class[max_positions];
                       double position_jd[max_positions];		       
                       int    press_sal_temp[15*frames_per_message][4]; //[0]=p,[1]=s,[2]=t,[3]=bin
                     };

char *strupr(char *input)          // changes characters to upper case
{
   int  i;

   for (i=0; input[i] != '\0'; i++)
     input[i] = toupper(input[i]);
   return input;
}

double dfr97 (const int yp, const int mp, const int dp,
     const int hr, const int minutes, const int secs, int *doy)
{
 int     yr, mo, dy, day_from_97;
 long    passed_jd, jd_xx0101, jd_970101;
 double  d_jd;

 yr = yp;
 mo = mp;
 dy = dp;

 if (yr <= 15)
  yr += 2000;
 else if (yr < 1900)
  yr += 1900;
 if ((yr<1997) || (yr>2015) || (mo<1) || (mo>12) || (dy<1) || (dy>31))
  return (0.0);

 d_jd = ymd2julday( yr, mo, dy );
 jd_xx0101 = ymd2julday( yr, mo, dy );
 *doy = (int) (d_jd - jd_xx0101 + 1);
 d_jd += (hr+(minutes + secs/60.0)/60.0)/24.0;

 return (d_jd);
}

int data_flag2(int bin_no, int *status_word)
{
 int i;
 int ranges[frames_per_message][2] = {0,2,3,7,8,12,13,17,18,22,23,27,28,32,33,37,38,42,43,47,48,52,53,57};

 for (i=0; i<frames_per_message; i++)
  if ((bin_no >= ranges[i][0]) && (bin_no <= ranges[i][1]))
	  return status_word[i];
 return BAD;
}

int write_to_disk (char *dir_name, struct type15_meta_struct *s, struct output_record *x)
{

/*  Writes output data to disk for T/P floats.  Profile 0 is
 transmitted when float is first turned on.  It has no 
 temperature or pressure data. */

 int  i,j,id;
 char *ptr_cnt_filename;
 struct tm *time_str;

 id = x->float_id;
 if ((x->status[0]) == NEVER_RECEIVED)    //Must have received first scan
 {
  if (trace_file != NULL)
     fprintf (trace_file,"First scan not received from float %05d - no data written\n",x->float_id);
  if (error_file != NULL)
     fprintf (error_file,"First scan not received from float %05d - no data written\n",x->float_id);
  return(1);
 }
 sprintf(buffer,"%s%04d_%05d_%03d.cnt",dir_name,s->AOML_number,s->ARGOS_id,x->profile_no);
 if (trace_file != NULL)
    fprintf (trace_file,"Opening output file %s\n",buffer);
 if ((out_file = fopen(buffer,"w+")) == NULL) // used to be fopen(buffer,"a+")
 {
  if (trace_file != NULL)
   fprintf (trace_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  if (error_file != NULL)
   fprintf (error_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  return 1;
 }
 if (trace_file != NULL)
  fprintf (trace_file," WRITE_TO_DISK: output file %s successfully opened\n",buffer);

// Write Output Profile Count File Name to List File
// -------------------------------------------------  
 if ( (ptr_cnt_filename = strrchr(buffer, '/')) == NULL)
 {
  if (trace_file != NULL)
   fprintf (trace_file,
            "%s : Can't get cnt file basename %s\n", PRGNAME, buffer);
   if (error_file != NULL)
    fprintf (error_file,
             "%s: Can't get cnt file basename %s\n", PRGNAME, buffer);
   return 1;
 } // endif: ptr_cnt_filename
 ptr_cnt_filename++;
 fprintf (list_file,"%s\n", ptr_cnt_filename);
 
 sprintf (buffer,
     "INTERNAL ID NUMBER                      %04d\n"
     "WMO ID NUMBER                           %-5d\n"
     "TRANSMISSION ID NUMBER                  %-5d\n"
     "PROFILE NUMBER                          %d\n",
      s->AOML_number,abs(s->WMO_id),s->ARGOS_id,x->profile_no);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "INSTRUMENT TYPE                         %s\n",s->float_type);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "WMO INSTRUMENT TYPE (TABLE 1770)        %d\n"
     "WMO RECORDER TYPE (TABLE 4770)          %d\n"
     "ARGOS PROGRAM NUMBER                    %d\n",
		s->wmo_instrument,s->wmo_recorder,s->exp_no);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "PI                                      %s\n",s->PI);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 x->first_blk_time -= (long) (x->msg_blk_no * s->ARGOS_rep_rate);

 sprintf (buffer, "START OF TRANSMISSION                   %s",
          julday2ymd((long)((float)x->first_blk_time/24./3600.)));
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 sprintf(buffer, " %s\n", timefrac2hms((double)x->first_blk_time/24./3600.));
 fwrite (strupr(buffer),1,strlen(buffer),out_file);


 if (x->profile_no > 0)
 {
  sprintf (buffer,
     "DRIFT TEMPERATURE (COUNTS)              %-5d\n"
     "DRIFT PRESSURE (COUNTS)                 %-5d\n"
     "DRIFT SALINITY (COUNTS)                 %-5d\n"
     "SURFACE PRESSURE (COUNTS)               %-5d\n"
     "BATTERY (COUNTS)                        %-5d\n",
      x->bottom_temperature,x->bottom_pressure,x->bottom_salinity,
      x->surface_pressure,x->battery_voltage);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,     
     "INTERNAL VACUUM                         %-3d\n",
     x->internal_vacuum);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
 sprintf (buffer,
     "PROFILE TERMINATION FLAG                %-2d\n",
		x->termination_flag);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);


 sprintf (buffer,
     "NUMBER OF COLUMNS                       4\n"
     "1. COLUMN                               PRESSURE (COUNTS)\n"
     "2. COLUMN                               TEMPERATURE (COUNTS)\n"
     "3. COLUMN                               SALINITY (COUNTS)\n"
     "4. COLUMN                               CHECKSUM FLAG\n"
     "==========================================================\n");
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 }
 else
 {
  sprintf (buffer,
     "CURRENT PRESSURE (COUNTS)               %-5d\n"
     "BATTERY (COUNTS)                        %-3d\n",
     x->current_pressure,x->battery_voltage);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
  sprintf (buffer,
	    "FLOAT STATUS                            %-3d\n"
     "TIME SINCE START (HR/MIN/SEC)           %02d:%02d:%02d\n"
     "TRANSMISSION REPETITION RATE (SEC)      %-3d\n",
     x->float_status,x->hours_since_start,x->minutes_since_start,
     x->seconds_since_start,x->ARGOS_rep_counts);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
  sprintf (buffer,
	    "UP TIME INTERVALS (HRS)                 %-3d\n"
     "DOWN TIME INTERVALS (HRS)               %-5d\n"
     "TRIP INTERVAL TIME                      %-3d\n",
     x->up_time_intervals,x->down_time_intervals,
     x->trip_interval_time);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  //add new fields here
  sprintf (buffer,
     "DEEP PUMP TIME DURATION (MINUTES)       %-5d\n"
     "FIRST SURFACE PUMP TIME (MINUTES)       %-5d\n"
     "SECOND SURFACE PUMP TIME (MINUTES)      %-5d\n"
     "ASCEND TIME (MINUTES)                   %-5d\n",
     x->deep_pump_duration,x->first_surface_pump,
     x->second_surface_pump,x->ascend_time);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

         
  sprintf (buffer,
     "SOFTWARE REVISION DATE (MON/DAY/YR)     %02d/%02d/%02d\n"
     "==========================================================\n",
     x->software_rev_month,x->software_rev_day,x->software_rev_year);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
 }

 i = 0;
 sprintf (buffer,"   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS\n");
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 while (x->position_jd[i] > 0.0)
 {
//                  "----LLL.LLL---LLLL.LLL-YYYY/MM/DD-HH:MM:SS---XXX---S-----C"
  sprintf (buffer,"    %+7.3f   %+8.3f %-19s   %03d   %1c     %1c\n",
    x->latitude[i],x->longitude[i],x->date_time[i],x->n_xmits[i],x->sat_name[i],
    x->loc_class[i]);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
  i++;
 }
 fprintf (out_file,"==========================================================\n");

/* Write depth, temperature, and salinity to disk */
   
 if (x->profile_no > 0)
 {
  i = 1;
  while (x->press_sal_temp[i][0] > -999)
  {
   if ((j = data_flag2(x->press_sal_temp[i][3],x->status)) == NEVER_RECEIVED)
    sprintf (buffer," %5d  %5d  %5d  %d\n",0,0,0,0);
   else
    sprintf (buffer," %5d  %5d  %5d  %d\n",
	                      x->press_sal_temp[i][0],x->press_sal_temp[i][2],
		              x->press_sal_temp[i][1],j);
   fwrite (strupr(buffer),1,strlen(buffer),out_file);
   if (++i > 5*frames_per_message)
	   break;
  }
 }

   if (out_file != NULL)
    fclose (out_file);   /* Close output file */

  return 0;
}

int most_likely_frame (char **buffer, int n_scans, int frame_num, int good_bad)
{
 int i,j,k,l,matches = 0;
 int num_matches,most_likely = -1;
 char tmp_buff[2][256];

//  Finds the most likely scan for the APEX floats
//  Returns number of most common scan
//  Returns -1 if error
//  good_bad=1 -> use checksum; good_bad=0->ignore checksum
/*
09335 1997 07 19 06 04 59 +00.000 +000.000 1 033 D 0 069515951 CA012000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFFFFFFFDFFFFFFFFFDF
*/

 for (i=0; i<n_scans; i++)
 {
  strcpy(tmp_buff[0],buffer[i]);
  if ((tmp_buff[0][good_bad_position] != '0') && good_bad)
   continue;       //Ignore if status byte = bad
  sscanf(&tmp_buff[0][start_of_hex_data+2],"%2X",&k);
  if (k != frame_num)
   continue;
  if (k == 1)
  {
   tmp_buff[0][4] = '0';
   tmp_buff[0][5] = '0'; //set message blk number = 0 because it's not constant
  }
  num_matches = 0;
  for (j=0; j<n_scans; j++)
  {
   strcpy(tmp_buff[1],buffer[j]);
   sscanf(&tmp_buff[1][start_of_hex_data+2],"%2X",&l);
   if (k != l)
    continue;   //If frames are different then skip
   if (l == 1)
   {
    tmp_buff[1][4] = '0';
    tmp_buff[1][5] = '0'; //set message blk number = 0 because it's not constant
   }
   if (strcmp(&tmp_buff[0][start_of_hex_data+6],&tmp_buff[1][start_of_hex_data+6]) == 0)
    num_matches++;  //Hex data the same => increment match count
  }
  if (num_matches > matches) //Is this the most matches?
  {
   matches = num_matches; //Yes - save match count
   most_likely = i;       //Save record number
  }
// TEST CS
  if ((num_matches==0) && (matches==0)) // Make sure to use messages only received once
  {
    most_likely=i;
  }

 }
 return most_likely;
}
 
int get_raw_data (struct output_record *y, char ** buffer, int n_scans)
{

  struct raw_data_record x;
  struct tm *time_str;
  int i,j,k,most_common[frames_per_message],roll_over=0;
  int id,iy,imo,ida,ih,im,is,g,ndoy,n_l,jd_old=0,n_pos=0;

  long int dummy;
  int l, frames;
  char ch;
  char frms[3]="";

  float lat,lon;
  unsigned char hex_data[80],sat,l_c;
  unsigned int z[32],old_count=0;
//09334 1997 08 24 15 24 14 +00.000 +000.000 0 041 J 0 069516551 F506FB11C711B11191115C11541153115211501145113010DF10B2108E103E0E

  for (i=0; i<frames_per_message; i++)
  {
   y->status[i] = NEVER_RECEIVED;
   for (j=0; j<30; j++)
    x.data.data_bytes[i][j] = 0;
   if ((j = most_likely_frame (buffer, n_scans, i+1, 1)) >= 0) //find most likely - good CRC only
    most_common[i] = j;
   else if ((j = most_likely_frame (buffer, n_scans, i+1, 0)) >= 0) //find most likely - any CRC
    most_common[i] = j;
   else 
    most_common[i] = -1;
  }
/*
  if (most_common[0] < 0)
  {
   if (trace_file != NULL) fprintf (trace_file," Can't decode message without frame 1\n");
   if (error_file != NULL) fprintf (error_file," Can't decode message without frame 1\n");
   return -1;	//Can't decode message without frame 1 
  }
*/
  for (j=1; j<max_positions; j++)
   x.jd[j] = -1.0;

   for (i=0; i<n_scans; i++)
   {
    if (sscanf(buffer[i],"%d %d %d %d %d %d %d %f %f %d %d %c %c %ld %s",
        &id,&iy,&imo,&ida,&ih,&im,&is,&lat,&lon,&g,&n_l,&sat,&l_c,&dummy,hex_data) != 15)
      continue;

   // Read third and fourth characters from hex_data, convert from hex to decimal and
   // skip if > frames_per_message.
   frames=0;
   for (l=0; l<2 ; l++)
   {
      ch=*(hex_data+l+2);
      switch (ch)
      {
        case 'A': strcpy(frms,"10"); strcat(frms, "\0"); break;
        case 'B': strcpy(frms,"11"); strcat(frms, "\0"); break;
        case 'C': strcpy(frms,"12"); strcat(frms, "\0"); break;
        case 'D': strcpy(frms,"13"); strcat(frms, "\0"); break;
        case 'E': strcpy(frms,"14"); strcat(frms, "\0"); break;
        case 'F': strcpy(frms,"15"); strcat(frms, "\0"); break;
        default : frms[0]=ch; frms[1]='\0'; break;
      }
      frames+=((1-l)*16*atoi(frms)+l*atoi(frms));
   }
   if (frames > frames_per_message)
   {
     continue;
   }

   x.jd[n_pos] = dfr97(iy,imo,ida,ih,im,is,&ndoy);
	
   x.latitude[n_pos] = lat;
   x.longitude[n_pos] = lon;

   sprintf(x.date_time[n_pos], "%4d/%02d/%02d%3d:%02d:%02d\0",iy,imo,ida,ih,im,is);
   x.loc_class[n_pos] = l_c;
   x.sat_name[n_pos] = sat;
   x.n_xmits[n_pos] = (int) floor((n_l-1)/8);
   if ((lat != (float) 0.0) || (lon != (float) 0.0))
   {
    n_pos = (n_pos < (max_positions-1)) ? n_pos + 1 : n_pos;
    continue;
   }
   x.latitude[n_pos] = (float) 99.999;
   x.longitude[n_pos] = (float) 999.999;
   x.n_xmits[n_pos] = 0;
   n_pos = (n_pos == 0) ? n_pos+1 : n_pos;

   for (j=0; j<32; j++)
    sscanf( (const char *) &hex_data[2*j],"%2x",&z[j]);

   if ( (z[1] - min_frame_no < 0) ||
        (z[1] - min_frame_no > frames_per_message - 1 + min_frame_no))
    continue;

   if (i != most_common[z[1] - min_frame_no] )
    continue;

   y->status[z[1]-1] = (g == 0) ? GOOD : BAD;
   if (z[1] == 1)
   {
	if (z[2] < old_count)
		roll_over++;
	old_count = z[2];
    y->msg_blk_no = z[2]-1 + roll_over*256; //count starts at 1
    y->first_blk_time = ((ymd2julday(iy,imo,ida)*24+ih)*60+im)*60+is;
   }
   
   //key part
   for (k=0; k<30; k++)
    x.data.data_bytes[z[1]-1][k] = (unsigned char) z[k+2];
  }

  x.jday = (int) floor(x.jd[0]);

  if (trace_file != NULL)
  {
    fprintf (trace_file,"T/P Float: profile number %d status = ",(int)x.data.d_fmt.profile_no);
    for (j=(frames_per_message-1); j>=0; j--)
       fprintf (trace_file,"%1d",y->status[j]);
    fprintf (trace_file," (0=NOT RCVD,1=OK,4=BAD)\n");
  }
  y->float_id = id;
  y->jd = x.jday;
  if (x.data.t_fmt.profile_no > 0)
  {
   y->controller_sn = 256*x.data.d_fmt.controller_sn[0]+x.data.d_fmt.controller_sn[1];
   y->profile_no = x.data.d_fmt.profile_no;
   y->profile_length = x.data.d_fmt.profile_length;
   y->termination_flag = x.data.d_fmt.termination_flag;
   y->bottom_temperature = (256*x.data.d_fmt.bottom_temperature[0] + x.data.d_fmt.bottom_temperature[1]);
   y->bottom_salinity = (256*x.data.d_fmt.bottom_salinity[0] + x.data.d_fmt.bottom_salinity[1]);
   y->bottom_pressure = (256*x.data.d_fmt.bottom_pressure[0] + x.data.d_fmt.bottom_pressure[1]);
   y->battery_voltage = x.data.d_fmt.battery_volts;
   y->surface_pressure = (256*x.data.d_fmt.surface_pressure[0] + x.data.d_fmt.surface_pressure[1]);
   y->internal_vacuum = x.data.d_fmt.internal_vacuum;
  }
  else if (x.data.t_fmt.profile_no == 0)
  {
   y->controller_sn = 256*x.data.t_fmt.controller_sn[0] + x.data.t_fmt.controller_sn[1];
   y->profile_no = x.data.t_fmt.profile_no;
   y->msg_type_flag = x.data.t_fmt.msg_type_flag;
   y->current_pressure = 256*x.data.t_fmt.current_pressure[0] + x.data.t_fmt.current_pressure[1];
   y->battery_voltage = x.data.t_fmt.battery_volts;
   y->internal_vacuum = x.data.t_fmt.internal_vacuum;
   y->float_status = x.data.t_fmt.float_status;
   y->hours_since_start = (x.data.t_fmt.hours_since_start/16)*10 + x.data.t_fmt.hours_since_start%16;
   y->minutes_since_start = (x.data.t_fmt.minutes_since_start/16)*10 + x.data.t_fmt.minutes_since_start%16;
   y->seconds_since_start = (x.data.t_fmt.seconds_since_start/16)*10 + x.data.t_fmt.seconds_since_start%16;
   y->ARGOS_rep_counts = x.data.t_fmt.ARGOS_rep_counts;
   y->up_time_intervals = x.data.t_fmt.up_time_intervals;
   y->down_time_intervals = 256*x.data.t_fmt.down_time_intervals[0] + x.data.t_fmt.down_time_intervals[1];
   y->trip_interval_time = x.data.t_fmt.trip_interval_time;

//added new fields
   y->deep_pump_duration = x.data.t_fmt.deep_pump_duration;
   y->first_surface_pump = x.data.t_fmt.first_surface_pump;
   y->second_surface_pump = x.data.t_fmt.second_surface_pump;
   y->ascend_time = x.data.t_fmt.ascend_time;
   
   y->software_rev_month = (x.data.t_fmt.software_rev_month/16)*10 + x.data.t_fmt.software_rev_month%16;
   y->software_rev_day = (x.data.t_fmt.software_rev_day/16)*10 + x.data.t_fmt.software_rev_day%16;
   y->software_rev_year = (x.data.t_fmt.software_rev_year/16)*10 + x.data.t_fmt.software_rev_year%16;
  }
  for (j=0; j<max_positions; j++)
  {
   y->latitude[j] = x.latitude[j];
   y->longitude[j] = x.longitude[j];
   y->position_jd[j] = x.jd[j];
   y->n_xmits[j] = x.n_xmits[j];
   y->sat_name[j] = x.sat_name[j];
   y->loc_class[j] = x.loc_class[j];
   strncpy(y->date_time[j],x.date_time[j],24);
  }

  actual_frames_per_message=ceil((( (float)(y->profile_length-2)*6)/30))+1;
  if (actual_frames_per_message > frames_per_message)
  {
    if (trace_file != NULL)
      fprintf (trace_file,"actual_frames_per_message exceeds frames_per_message.\n");
    if (error_file != NULL)
      fprintf (error_file,"actual_frames_per_message exceeds frames_per_message.\n");
    exit (1);
  }

  if (y->profile_no > 0)
  {
   y->press_sal_temp[0][0] = y->bottom_pressure;
   y->press_sal_temp[0][1] = y->bottom_salinity;
   y->press_sal_temp[0][2] = y->bottom_temperature;
   y->press_sal_temp[0][3] = 0;
   for (j=1; j<=y->profile_length; j++)
   { 
     
     //Temperature
     y->press_sal_temp[j][2] =(256*x.data.d_fmt.T_S_P[j-1][0] + x.data.d_fmt.T_S_P[j-1][1]);
      
     //Salinity
     y->press_sal_temp[j][1] =(256*x.data.d_fmt.T_S_P[j-1][2] + x.data.d_fmt.T_S_P[j-1][3]);
     
     //Pressure
     y->press_sal_temp[j][0] =(256*x.data.d_fmt.T_S_P[j-1][4] + x.data.d_fmt.T_S_P[j-1][5]);
     
     
     y->press_sal_temp[j][3] = j;	//Bin number
   }
   y->press_sal_temp[j][0] = -999;
   y->msg_blk_no *= actual_frames_per_message;
  }
  if (trace_file != NULL)
   fprintf(trace_file," GET_RAW_DATA: finished (profile number = %d,number of meas = %d)\n", y->profile_no,k);

  return(y->profile_no);
}

 
int get_type15_data (int np, struct type15_meta_struct *s, char *out_dir)
{
  struct output_record y;
  char **buff;
  int i,j;
  int iy,imo,ida,ih,im,is,ndoy;
  int jd_old=0,jd;
  float jd_flt;
  int write_to_disk_error;
  int corrected_prof_no;
   
/* Allocate needed memory */

  if ((buff = (void *) malloc(max_scans*(sizeof(void *)))) == NULL)
  {
   if (trace_file != NULL)
        fprintf (trace_file," GET_type15_DATA: Error allocating buffer pointers\n");
   if (error_file != NULL)
        fprintf (error_file," GET_type15_DATA: Error allocating buffer pointers\n");
   return 1;
  }
  for (i=0; i<max_scans; i++)
   if ((buff[i] = (char *) malloc(buff_size*(sizeof(char)))) == NULL)
   {
   if (trace_file != NULL)
        fprintf (trace_file," GET_type15_DATA: Error allocating data buffer arrays\n");
   if (error_file != NULL)
        fprintf (error_file," GET_type15_DATA: Error allocating data buffer arrays\n");
    return 1;
   }

  if (trace_file != NULL)
   fprintf (trace_file," GET_type15_DATA: Arrays successfully allocated\n");
/* Get the data */

  j=0;
  while (fgets(buff[j],buff_size,input_file) != NULL)
  {
   if(sscanf(buff[j],"%d %d %d %d %d %d %d",
       &i,&iy,&imo,&ida,&ih,&im,&is) != 7)
    continue;

   if ((jd_flt = (float) dfr97(iy,imo,ida,ih,im,is,&ndoy)) < s->start_time)
	   continue;

   jd = (int)jd_flt;
   //   jd = (int) dfr97(iy,imo,ida,0,0,0,&ndoy);
   if ((abs(jd - jd_old) > 2) && (j > 0))
   {
    if (np > 0)
    {
     i=get_raw_data (&y,buff,j);
     if (abs (y.jd - (int)(s->start_of_cycles + (y.profile_no - 1) * s->cycle_time)) >= 2)
     {
        //changed here
        corrected_prof_no=(int) (((float) y.jd - s->start_of_cycles)/s->cycle_time + 1.5);
        if(corrected_prof_no>0) y.profile_no=corrected_prof_no;
     
        fprintf (trace_file," GET_TYPE15_DATA: transmitted profile no. = %d",y.profile_no);
        fprintf (trace_file," computed = %d - using computed value\n",
        y.profile_no);
     }
     if (y.profile_no >= 0 && i!=-1)
      write_to_disk (out_dir,s,&y);
    }
    sprintf (buff[0],"%s",buff[j]);
    j = 0;
   }
   jd_old = jd;
   if (++j >= max_scans)
   {
    if (trace_file != NULL)
     fprintf (trace_file," Maximum scans for profile exceeded\n");
    if (error_file != NULL)
     fprintf (error_file," Maximum scans for profile exceeded\n");
    exit (1);
   }  
  }//end of while loop

if (j > 0) // are there any data lines to save to a file? (j is n_scans in get_raw_data)
{
  i=get_raw_data (&y,buff,j);

  if (abs (y.jd - (int)(s->start_of_cycles + (y.profile_no - 1) * s->cycle_time)) >= 2)
  {
   //changed here
   corrected_prof_no=(int) (((float) y.jd - s->start_of_cycles)/s->cycle_time + 1.5);
   if(corrected_prof_no>0) y.profile_no=corrected_prof_no;
  
   fprintf (trace_file," GET_TYPE15_DATA: transmitted profile no. = %d",y.profile_no);
   fprintf (trace_file," computed = %d - using computed value\n",
   y.profile_no);
  }

  if (y.profile_no >= 0 && i!=-1)
  {
    write_to_disk_error=write_to_disk (out_dir,s,&y);
  }
  else
  {
    write_to_disk_error=1;
  }
}else
{
    write_to_disk_error =1; // no data to store -> something may be wrong
}//end of if(j>0)

  free (buff);
  if (trace_file != NULL)
      fprintf (trace_file," GET_type15_DATA completed successfully\n");
  return (write_to_disk_error);
}


int get_meta_file_name (char *str_name)
{
// unsigned char *ptr;
 char *ptr;

 if ((ptr = strstr(str_name,".hex"))!= NULL)
  sprintf (ptr,".meta");
 else if ((ptr = strstr(str_name,".HEX"))!= NULL)
  sprintf (ptr,".META");
 else
  strcat(str_name,".meta");
 return 0;
}

int read_type15_meta_file (char *meta_name, struct type15_meta_struct *meta_struct)
{
 int i,j,syear,smonth,sday,shour,smin,x;
 float x1;
 char *ptr;
 char *cal_keys[] = {"TSLOPE=","TOFF=","PSLOPE=","POFF=","SSLOPE=","SOFF=","VSLOPE=","VOFF=","VACSLOPE=","VACOFF=","\0"};

 for (i=0; i<(sizeof(struct aoml4_cals)/sizeof(double)); i++)
  meta_struct->cals.c[i] = 0.0;
 meta_struct->cycle_time = (float) 0.0;
 meta_struct->start_time = (float) 0.0;
 meta_struct->start_of_cycles = (float) 0.0;
 if (trace_file != NULL)
  fprintf(trace_file,"Opening meta file %s...\n",meta_name);
 if ((meta_file = fopen(meta_name,"r")) == NULL)
 {
  if (error_file != NULL)
   fprintf (error_file," ERROR opening meta file %s\n",meta_name);
  if (trace_file != NULL)
   fprintf (trace_file," ERROR opening meta file %s\n",meta_name);
  return 1;
 }
 else
 if (trace_file != NULL)
 fprintf (trace_file," Opened meta-file succesfully\n");

 while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
 {
  if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
  {
   if(sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) != 1)
    meta_struct->AOML_number = 0;
   continue;
  }
  if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) != 1)
     meta_struct->WMO_id = 0;
   continue;
  }
  if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) != 1)
    meta_struct->ARGOS_id = 0;
   continue;
  }
  if (strstr(strupr(buffer),"TRANSMISSION REPETITION RATE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %f",&meta_struct->ARGOS_rep_rate) != 1)
    meta_struct->ARGOS_rep_rate = 0;
   continue;
  }
  if (strstr(strupr(buffer),"DELAY OF FIRST DOWN TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&x) != 1)
// CS !!!!!! this has to be changed quit if no value !!!!

    x = 0;
   continue;
  }

  if (strstr(strupr(buffer),"WMO INSTRUMENT TYPE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_instrument) != 1)
    meta_struct->wmo_instrument = 0;
   continue;
  }
  if (strstr(strupr(buffer),"WMO RECORDER TYPE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_recorder) != 1)
    meta_struct->wmo_recorder = 0;
   continue;
  }
  if (strstr(strupr(buffer),"PROGRAM NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->exp_no) != 1)
    meta_struct->exp_no = 0;
   continue;
  }
  if (strstr(strupr(buffer),"DOWN TIME [DAYS]") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&x) != 1)
    x = 0;
   meta_struct->cycle_time += x;
   meta_struct->start_of_cycles += x;
   continue;
  }
  if (strstr(strupr(buffer),"FIRST DOWN TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %f",&x1) != 1)
    x1 = 0;
   meta_struct->start_of_cycles += (float) (x1/24.0);
   continue;
  }
  if ((strstr(strupr(buffer),"INSTRUMENT TYPE") != NULL) && (strstr(strupr(buffer),"WMO") == NULL))
  {
	if (sscanf(buffer,"%*s %*s %s",meta_struct->float_type) == 0)
	{
		sprintf (meta_struct->float_type,"\0");
	    continue;
	}
	if ((ptr = strstr(&buffer[strlen("FLOAT TYPE")],meta_struct->float_type)) != NULL)
		sprintf (meta_struct->float_type,"%s",ptr);
    if ((ptr = strstr(meta_struct->float_type,"\n")) != NULL)
	   sprintf (ptr,"\0");
   continue;
  }
  if (strstr(strupr(buffer),"PI     ") != NULL)
  {
	if (sscanf(buffer,"%*s %s",meta_struct->PI) == 0)
	{
		sprintf (meta_struct->PI,"\0");
	    continue;
	}
   if ((ptr = strstr(&buffer[3],meta_struct->PI)) != NULL)
		sprintf (meta_struct->PI,"%s",ptr);
   if ((ptr = strstr(meta_struct->PI,"\n")) != NULL)
	   sprintf (ptr,"\0");
   continue;
  }
  if (strstr(strupr(buffer),"UP TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&x) != 1)
    x = 0;
   meta_struct->cycle_time += (float) (x/24.0);
   continue;
  }
  if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",&sday,&smonth,&syear,&shour,&smin) == 5)
    meta_struct->start_time += (float) dfr97(syear,smonth,sday,shour,smin,0,&x);
   else
    meta_struct->start_time = 0;
   meta_struct->start_of_cycles += meta_struct->start_time;
   continue;
  }
  i = 0;
  while ((j = strlen(cal_keys[i])) > 1)
  {
   if ((ptr = strstr(buffer,cal_keys[i])) != NULL)
    meta_struct->cals.c[i] = atof(ptr+j);
   i++;
  }
 }
// for (i=0; i<(sizeof(struct aoml4_cals)/sizeof(double)); i++)
//  printf ("\nCal %0d %12.5lf",i,meta_struct->c[i]);
 return 0;
}

int main (int argc, char *argv[])
{
 char     *ptr_path_meta, *ptr_path_hex, *ptr_path_cnt, 
          *ptr_path_log, *ptr_path_list,
          pathname_input_filename[161]="",
          pathname_trace[161]="", pathname_error[161]="";

 char     input_filename[80], output_directory[256], msg[11]="", title[257]="";
 enum     {LAST=0,ALL} get_which_profile;
 struct   type15_meta_struct s = {0,0,0,0,0,0,"UNKNOWN","UNKNOWN",0.0,0.0,0.0};
 unsigned long t;
 int      ret_val = 0;

// Program Name
// ------------
 printf("Entering Program: %s\n", PRGNAME);
 strcpy(title, "Program generates temperature, salinity and pressure counts\n");
 strcat(title, "for APEX_TS3 Floats");
 printf("%s\n", title);


// Input Paramenters 
// -----------------
 if ( argc < 7 )
 {
  printf("Format: %s hex_infile nprofile cnt_outdir trace_outile error_outfile\n",
          PRGNAME);
  printf("\n");
  printf("hex_infile:    hexadecimal input file name\n");
  printf("nprofile:      1=get all profiles (default); 0=get last profile\n");
  printf("cnt_outdir:    output directory for generated count data file(s)\n");
  printf("trace_outile:  output directory for generated trace file\n");
  printf("error_outfile: output directory for generated error file\n");
  printf("list_outfile:  output directory for generated list file\n");
  return 1;
 }

// Input Hex File name
// -------------------
 printf ("%s: Input Hex File Name - %s\n", PRGNAME, argv[1]);
 sprintf (input_filename,"%s",argv[1]);

// Number of profiles to process: 0, get last profile or 1, get all profiles
// -------------------------------------------------------------------------
 if (atoi(argv[2]) >= 1)
  get_which_profile = ALL;
 else
  get_which_profile = LAST;

// Output directory path for count file(s)
// ---------------------------------------
 if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
 {
  printf("ERROR (%s): CNTFILES_DIR <-- no such environment variable\n", PRGNAME);
  t = time(NULL);
  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  ret_val = 1;
  return ret_val;
 }

 sprintf (output_directory,"%s/",argv[3]);
 printf ("%s: Output Directory - %s\n", PRGNAME, output_directory);

// TRACE File
// ----------
 if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
 {
  printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
  t = time(NULL);
  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  ret_val = 1;
  return ret_val;
 }

 sprintf (buffer,"%s",argv[4]);
 printf ("%s: Opening trace file %s\n", PRGNAME, buffer);

 if ((trace_file = fopen(buffer,"a+")) == NULL)
 {
   printf ("ERROR (%s): Opening trace file %s\n", PRGNAME, buffer);
   t = time(NULL);
   printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
   ret_val = 1;
   return ret_val;
 }

 printf("%s: Trace file opened succesfully\n", PRGNAME);
 if (trace_file != NULL)
 {
  t = time(NULL);
  fprintf (trace_file, "=> %s: start %s", PRGNAME, ctime(&t));
 }

// ERROR File
// ----------
 sprintf (buffer,"%s",argv[5]);
 printf ("%s: Opening error file %s\n", PRGNAME, buffer);

 if ((error_file = fopen(buffer,"a+")) == NULL)
 {
   printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
   if (trace_file != NULL)
     fprintf (trace_file," ERROR (%s): error opening file %s\n", PRGNAME, buffer);
   t = time(NULL);
   if (trace_file != NULL)
     fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
   printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 1;
   return ret_val;
 }

 if (trace_file != NULL)
   fprintf (trace_file,"%s: Error file %s successfully opened\n", PRGNAME, buffer);

 if (error_file != NULL)
 {
   t = time(NULL);
   fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
 }

  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
   if (trace_file != NULL)
     fprintf(trace_file,"ERROR (%s): ARGOLIST_DIR <-- no such environment variable",
          PRGNAME);
   if (error_file != NULL)
     fprintf(error_file,"ERROR (%s): ARGOLIST_DIR <-- no such environment variable",
          PRGNAME);
   t = time(NULL);
   if (trace_file != NULL)
     fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
   if (error_file != NULL)
      fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 1;
   return ret_val;
  }

// LIST File
// ---------
  sprintf (buffer,"%s",argv[6]);
  if (trace_file != NULL)
     fprintf (trace_file,"%s: Opening list file %s\n", PRGNAME, buffer);

  if ((list_file = fopen(buffer,"w")) == NULL)
  {
    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);
    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);

    t = time(NULL);
    if (trace_file != NULL)
      fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
    if (error_file != NULL)
       fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val =1;
    return ret_val;
  }
  if (trace_file != NULL)
    fprintf (trace_file,"%s: List file %s successfully opened\n", PRGNAME, buffer);

// Open Input Hex File
// -------------------
  if ( (ptr_path_hex = getenv("HEXFILES_DIR")) == NULL)
  {
    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): HEXFILES_DIR <-- no such environment variable\n",
               PRGNAME);
    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): HEXFILES_DIR <-- no such environment variable\n",
               PRGNAME);

    if (trace_file != NULL)                  // Close Log files
      fclose (trace_file);
    if (error_file != NULL)
      fclose (error_file);
 
    t = time(NULL);
   if (trace_file != NULL)
     fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
   if (error_file != NULL)
      fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }
  
  sprintf(pathname_input_filename, "%s/%s", ptr_path_hex, input_filename);

  if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
  {
     if (trace_file != NULL)
       fprintf (trace_file,"ERROR (%s): Input file %s not opened\n",
                PRGNAME, pathname_input_filename);
     if (error_file != NULL)
       fprintf (error_file,"ERROR (%s): input file %s not opened\n",
                PRGNAME, pathname_input_filename);
     ret_val = 1;
  }
  else
  {
     if (trace_file != NULL)
       fprintf (trace_file,"%s: Input file %s successfully opened\n",
                PRGNAME, pathname_input_filename);

// META File
// ---------
     if (get_meta_file_name (input_filename) != 0)
     {                                             // Failure: did not get meta filename
      if (error_file != NULL)
        fprintf (error_file,
                 "ERROR (%s): Finding meta file name - returned name %s\n",
                 PRGNAME, input_filename);
      if (trace_file != NULL)
        fprintf (trace_file,
                 "ERROR (%s): Finding meta file name - returned name %s\n",
                 PRGNAME, input_filename);
        ret_val = 1;
     }
     else
     {                                            // Success: got meta filename
        if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
        {
           if (trace_file != NULL)
             fprintf (trace_file,
                      "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                      PRGNAME);
           if (error_file != NULL)
                fprintf (error_file,
                         "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                          PRGNAME);

           if (trace_file != NULL)                  // Close Log files
             fclose (trace_file);
           if (error_file != NULL)
             fclose (error_file);
 
           t = time(NULL);
           if (trace_file != NULL)
             fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
           if (error_file != NULL)
             fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
           ret_val = 1;
           return(ret_val);
        } // endif  ptr_path_meta 

        sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

        if (read_type15_meta_file (pathname_input_filename, &s) != 0)
        {                                         // Failure: couldn't read meta_file
           if (trace_file != NULL)
             fprintf (trace_file, "ERROR (%s): Reading meta file %s\n", 
                      PRGNAME, pathname_input_filename);
           if (error_file != NULL)
             fprintf (error_file,"ERROR (%s): Reading meta file %s\n",
                      PRGNAME, pathname_input_filename);
           ret_val = 1;
        }
        else                                      // Success: read meta_file
        {
           if (trace_file != NULL)
             fprintf (trace_file,"%s: Meta file %s successfully read\n",
                     PRGNAME, pathname_input_filename);

// Make profile(s) selection
// -------------------------
           if (get_which_profile == ALL)
           {
              if (get_type15_data(get_which_profile,&s,output_directory) !=0 )
              ret_val = 1;
           }
           else
           {
              if (get_type15_data(get_which_profile,&s,output_directory) !=0 )
              ret_val = 1;
           } // endif: get_which_profile

        } // endif: read_type15_meta_file 
     } // endif: get_meta_file_name
  } //endif: input_file

// Write finish message to trace & error files
// -------------------------------------------
  t = time(NULL);

  if ( ret_val == 0)
    strcpy(msg, "finished");
  else
    strcpy(msg, "terminated");

  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));

  if (trace_file != NULL)
    fprintf (trace_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

  if (error_file != NULL)
    fprintf (error_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

// Close all files 
// ---------------
 if (input_file != NULL)
  fclose (input_file);

 if (trace_file != NULL)
  fclose (trace_file);

 if (error_file != NULL)
  fclose (error_file);

 if (list_file != NULL)
  fclose (list_file);

 return ret_val;
}
