/************************************************************************
   Revision Date: 12-Dec-12 (CA: - start of transmission now considers floats
    				   no message number
				   -Case1: more than 2 messages, SOT does not
				   consider CRC for SOT calculation
				   -Case2: only 2 messages, messages need to have a 
				   good CRC
				   -Case3: only 3 messages, messages need to have a 
				   good CRC and the 2 estimates taken to be same
    		  22-MAY-12 (CS: - block_transimission_duration[s-m], index
                                   "s-m" replaced with correct value "index")
                  01-JUN-11 (ER: - Jean-Philipe Rannou algorithm modified
                                   so it can work with message_number > 1
                  17-Mar-11 (ER: - function should return '0' when there
                                   are more than 1 candidate blktime but all of 
                                   them has the same block number (blktime=0)
                  02-Jul-10 (ER: - Changes in layout; commenting
                                   of code in order to make it more
                                   readable.
                  22-Feb-10 (LL  - increase candidate_size to 500)

  start_of_transmission.c
  =======================
  Finds the most likely start of transmission for APEX floats by following
  the Jean-Philippe Rannou method

  Notes from Claudia (2012/10/31): Originally, we only used messages with good CRCs
  Rannou's method does not mention the use of the CRC.
  So, if we have more than 2 messages with a good or bad CRC, we can ignore the
  CRC and use all of them to estimate the candidate_blknum and candidate_blktime
  for all messages to find the best solution.
  If we only have 2 messages, we need to keep the requirement of a good CRC.
  For the case of 3 messages we only get 2 estimates. In this case the estimates
  have to be the same. If they are different, then the program has to exit with
  "could not find ...".
  if 2 messages with good CRC are available - use the estimate for the single
  canditate_blk_time.
  otherwise, if max_col is 0, do not use any of the estimates.

  Input
     buff:    Holds a profile's worth of a hex file. Each element of the vector is
              a raw line from  the hex file.  
     n_scans: Number of elements in buff.
     input_msg_nbr : Message number with wich the algorithm will try to work
     test_flag 	   : Contains value 0,1,2,3, If value is 1, then hex_data does
     		     not contain msessage number.

  Output
     first_blk_time:  most likely start of transmission

  Return value:
     0: it could not find the most likely start of transmission based on a given msg number
     1: it could find the most likely start of transmission based on a given msg number

 ***********************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "hex2cnt_tsall.h"

int max_element(int, int);
long ymd2julday(int, int, int);

#define data_size 64
#define buffer_size 256
#define candidate_size 500 //300

int start_of_transmission (char **buffer, int n_scans, long *first_blk_time, int input_msg_nbr,int test_flag)
{

    int debug=OFF;
//    int debug=ON;

  int msg_nbr;  // message number read from the hex line.
  int i,s,t,m;

  int aoml_id = 0;
  int year = 0;
  int month = 0;
  int day = 0;
  int hour = 0;
  int minute = 0;
  int second = 0;
  float latitude = 0.0;
  float longtitude = 0.0;
  int good_bad_position = 0;
  int nnn = 0;
  char satellite[1];
  int loc_class = 0;
  int file_data = 0;
  char hex_data[data_size];

  char tmp_buff[buffer_size];
  int blk_num = 0;
  long blk_time;
  int  candidate_count = 0;
  int  candidate_blknum[candidate_size];
  long candidate_blktime[candidate_size];
  int candidate_good_bad_position[candidate_size];// to save the good or bad CRC, 
  int most_likely = 0;
  int j = 0,good_crc_cnt=0;
  unsigned int roll_over=0,old_count=0;

  if (debug==ON) {
    printf( "IN GRD BEGIN start_of_transmission\n" );
  }

  // Initialize vectors
  for( t=0; t<candidate_size; t++ )
  {
    candidate_blknum[t] = 0;
    candidate_blktime[t] = 0;
  }

  // Loop throu the transmission lines block to get 'good' messages 
  // for the select message number.
  for( i=0; i<n_scans; i++ )
  {
    strcpy(hex_data,"");
    strcpy(tmp_buff,"");

    strcpy(tmp_buff,buffer[i]);
    sscanf(tmp_buff,"%6d %04d %02d %02d %02d %02d %02d +%06f +%07f %d %03d %c %d %d %s",&aoml_id,&year,&month,&day,
  		&hour,&minute,&second,&latitude,&longtitude,&good_bad_position,&nnn,&satellite,&loc_class,&file_data,&hex_data);
     	
    //Ignore if status byte = bad
  /*  if (good_bad_position != 0)
      continue;       
*/
    if (strcmp(hex_data,"")==0)
      strcpy(hex_data,"0000000000000000000000000000000000000000000000000000000000000000");
    
    if (strcmp(hex_data,"0000000000000000000000000000000000000000000000000000000000000000") == 0)
	continue;
   
    if(test_flag != 1)
    	sscanf(&hex_data[2],"%2X",&msg_nbr);
     else
        msg_nbr = 1;

    if( msg_nbr != input_msg_nbr )	//we want only msg 
   	  continue;
  
    if (good_bad_position == 0)
      good_crc_cnt++;

    if (debug==ON) {
        //printf( "Competing: j=%d   %d/%d/%d %d:%d:%d", j, year, month, day, hour, minute, second);
    }

    if(test_flag != 1)
     sscanf(&hex_data[4],"%2X",&blk_num);
    else
    {
    // printf("IN SOT 1 blk_num = %d ,old_count= %d ,roll_over = %d \n",blk_num,old_count,roll_over);
     sscanf(&hex_data[2],"%2X",&blk_num);
     if (blk_num < old_count) roll_over++;
     old_count = blk_num;
     blk_num = blk_num-1 + roll_over*256; //count starts at 1
     //printf("IN SOT 2 blk_num = %d ,old_count= %d ,roll_over = %d \n",blk_num,old_count,roll_over);


     }

    blk_time = ((ymd2julday(year,month,day)*24+hour)*60+minute)*60+second;
    candidate_blknum[j] = blk_num;
    candidate_blktime[j] = blk_time;
    candidate_good_bad_position[j] = good_bad_position; // save good or bad CRC

    j++;
  } 

  candidate_count = j;
  if (debug==ON) {
        printf( "candidate_count=%d\n", candidate_count);
    }
  

  if(candidate_count == 2 && good_crc_cnt < 2 )
  {
   return 0; //return 0 which is the default value, if the CRC for both the candidates is bad
  }
   
  if (debug==ON) {
        printf( "candidate_count=%d\n", candidate_count);
    }

  long block_transimission_duration[candidate_size];
  long candidate_first_blk_time[candidate_size];
 /* TEMP FLAG DELETE IT */
 int flag_good = -2;
  //most_likely = 0;
  if( candidate_count >= 2 )
  {
    int index = 0;

    for(s=1; s<candidate_count; s++)
    {
      for(m=1; m<=s; m++)
      {

       if(good_crc_cnt >=2)
       {
         flag_good = 1;
	//printf("IN GRD candidate_blknum[s] =%d , candidate_blknum[s-m] =%d ,candidate_good_bad_position[s] =%d ,candidate_good_bad_position[s-m] = %d \n",candidate_blknum[s],candidate_blknum[s-m],candidate_good_bad_position[s],candidate_good_bad_position[s-m]);
          //If the consecutive blknumbers are not same and have a good CRC then get the candidate first_blk_time 
        if( (candidate_blknum[s] != candidate_blknum[s-m]) && (candidate_good_bad_position[s] == 0 && candidate_good_bad_position[s-m]==0) )
        {
          block_transimission_duration[index] = (long)(candidate_blktime[s]-candidate_blktime[s-m])/(candidate_blknum[s]-candidate_blknum[s-m]);
//CS - wrong
//        candidate_first_blk_time[index] = (long)(candidate_blktime[s-m] - (candidate_blknum[s-m] - 1)*block_transimission_duration[s-m]);
          candidate_first_blk_time[index] = (long)(candidate_blktime[s-m] - (candidate_blknum[s-m] - 1)*block_transimission_duration[index]);
	  //printf("candidate_first_blk_time[index] =%d \n",candidate_first_blk_time[index]);
          index++;
          m = candidate_count;
	}
       }//end of if good_crc_cnt
       else if(good_crc_cnt < 2)
      {
         flag_good = 0;
	//printf("IN GRD candidate_blknum[s] =%d , candidate_blknum[s-m] =%d  \n",candidate_blknum[s],candidate_blknum[s-m],candidate_good_bad_position[s],candidate_good_bad_position[s-m]);
        if( (candidate_blknum[s] != candidate_blknum[s-m]))
        {
          block_transimission_duration[index] = (long)(candidate_blktime[s]-candidate_blktime[s-m])/(candidate_blknum[s]-candidate_blknum[s-m]);
//CS - wrong
//        candidate_first_blk_time[index] = (long)(candidate_blktime[s-m] - (candidate_blknum[s-m] - 1)*block_transimission_duration[s-m]);
          candidate_first_blk_time[index] = (long)(candidate_blktime[s-m] - (candidate_blknum[s-m] - 1)*block_transimission_duration[index]);
          index++;
          m = candidate_count;
	}
      }//end if else
     }//end for m
    }//end for s
     
    /* if(flag_good == 0)
	printf("IN GRD IN SOT index = %d from else if(good_crc_cnt < 2) \n",index);
     else if (flag_good == 1)
	printf("IN GRD IN SOT index = %d from else if(good_crc_cnt >= 2) \n",index);
*/
    int ss, tt;
    int flag_index[candidate_size];
    int max_col = 0;
    for(ss=0; ss<index; ss++)
    {
      flag_index[ss] = 0;
    }

    if (debug==ON) {
        printf( "A ==== max_col=%d, index=%d\n", max_col,index);
    }

     for(ss=0; ss<index; ss++)
     {
      for(tt=0; tt<index; tt++)
      {
       if (candidate_first_blk_time[ss] == candidate_first_blk_time[tt] && ss != tt)
       {
         if (debug==ON) {
          //   printf( "ss=%d, tt=%d, candidate_first_blk_time[ss]=%d, candidate_first_blk_time[tt]=%d\n",ss,tt,candidate_first_blk_time[ss],candidate_first_blk_time[tt] );
         }
             //printf( "ss=%d, tt=%d, candidate_first_blk_time[ss]=%d, candidate_first_blk_time[tt]=%d\n",ss,tt,candidate_first_blk_time[ss],candidate_first_blk_time[tt] );
         flag_index[ss]++;
	//printf("flag_index[ss] = %d",flag_index[ss]);
         if( flag_index[ss] > max_col )
         {
           max_col = flag_index[ss];
           if (debug==ON) {
              printf( "B ==== max_col=%d\n", max_col);
           }
         }
       }
     }
   }
    if (debug==ON) {
        printf( "c ==== max_col=%d\n", max_col);
    }

    
    if (good_crc_cnt != 2 && max_col == 0)
    {
     //printf("IN GRD IN SOT I am returning back coz good_crc_cnt != 2 && max_col == 0  \n");
     return 0;
    }

    for(ss=0; ss<index; ss++)
    {
      if(debug==ON){
        printf("flag_index[%d] = %d \n",flag_index[ss]);
       }
      //printf("IN GRD SOT flag_index[ss] = %d and max_col = %d\n",flag_index[ss],max_col);

      if( flag_index[ss] == max_col )
      {
        *first_blk_time = candidate_first_blk_time[ss];
        most_likely = 1;
        if (debug==ON) {
            //printf( "ss=%d, first_blk_time=%d, candidate_first_blk_time[ss]=%d\n",ss,*first_blk_time,candidate_first_blk_time[ss] );
        }
      }
    }
  }// if( candidate_count >= 2 )

  if (debug==ON) {
    printf("IN GRD RESULT SOT most_likely %d\n",most_likely);
   if(most_likely ==1){
    printf("IN GRD RESULT SOT first_blk_time: %d  \n",*first_blk_time);
    printf("IN GRD RESULT SOT  Date Converted = %s  \n",julday2ymd((long)((float)(*first_blk_time/24./3600.))));
    printf("IN GRD RESULT SOT       hh min ss = %s  \n",timefrac2hms((double)((*first_blk_time)/24./3600.))); 
   }
  }

  return most_likely;

}
