#!/bin/sh
# ====================================================================================
# Revision Date: 08-Apr-13 (RS - ghost_hexlist is now created in ARGOSTMP_DIR)
#                15-Dec-05 (RS - changed HEXFILES_DIR to HEXFILES_RECENT_DIR)
#                14-Dec-05 (RS - echo hex filename in 'trace' logfile)
#                13-Dec-05 (CS - added float 1647)
#                17-Dec-02 (RS - introduce a list of ghost file)
#                01-May-02 (CS & RS)
#
# remove_ghosts.sh
# ================
# Calls a C-Program (remove_ghosts.x) to remove ghost float data cycles from 
# HEX files.
#
# Usage
# -----
# (1) Script
#         remove_ghosts.sh hex_file remove_ghosts_emailmsg
#
# hex_infile        = filename of 'ghost' file to be checked (input)
# remove_ghosts_msg = pathname of file will contain a message when
#                     the exit status is an error (4/9). This file 
#                     is appended to he 'argos_process.sh' email message. 
# 
# (2) C-Program
#        remove_ghosts.x hex_infile hex_outfile      \
#                        latmin latmax lonmin lonmax \
#                        gap_satellites              \
#                        HEXFILES_RECENT_DIR trace error
#
# hex_infile          = filename of 'ghost' file to be checked (input)
# hex_outfile         = filename of 'ghost' file with cycle data removed (output)  
# latmin,latmax       = latitude range to be searched for a 'ghost' float  
# lonmin,lonmax       = longitude range to be searched for a 'ghost' float  
# gap_satellites      = time elpased between satellite passes in minutes (input) 
# HEXFILES_RECENT_DIR = hex files directory location (input) 
# trace               = pathname of trace logfile (output) - open with append
# error               = pathname of error logfile (output) - open with append
# 
# Exit status values
# ------------------
# 0, everything is OK and no data cycles were removed 
# 1, everything is OK and 'ghost' data cycles were remove
# 2, C-Program error occurred
# 3, hex file is not in the 'ghost' list 
# 9, loop counter exceeded (MAXN=20)
# ====================================================================================
#  --------------- for debugging -----------------
#TODAY=`date +'%Y.%m.%d_%H.%M'`
#HEXFILES_RECENT_DIR=/b1/ARGOTEST/argos/data/hex
#ARGOSLOG_DIR=/b1/ARGOTEST/argos/log
#ARGOSPRG_DIR=/b1/ARGOTEST/argos/prg
#echo "??? TODAY:        $TODAY"
#echo "??? HEXFILES_RECENT_DIR: $HEXFILES_RECENT_DIR"
#echo "??? ARGOSLOG_DIR: $ARGOSLOG_DIR"
#echo "??? ARGOSPRG_DIR: $ARGOSPRG_DIR"
# ------------------------------------------------
script_name=`basename $0`
root_script_name=`echo $script_name| awk -F"." '{print $1}`
echo "=> Enter $script_name"

hex_infile=$1
email_msg=$2

if [ -f $ARGOSTMP_DIR/ghost_hexlist ]
then
  rm -f $ARGOSTMP_DIR/ghost_hexlist
fi

cat > $ARGOSTMP_DIR/ghost_hexlist << EOF
0111_23273.hex -60 -40 -180 180
0334_21654.hex 32 52 -180 180
1647_23273.hex -60 -40 -180 180
0969_34292.hex 40 60 -180 180
EOF

echo "   HEX_INFILE: $hex_infile  HEXFILES_RECENT_DIR: $HEXFILES_RECENT_DIR"
record=`grep $hex_infile $ARGOSTMP_DIR/ghost_hexlist 2> /dev/null`
status="$?"
echo "   STATUS(grep): $status"
if [ $status -ne 0 ]                      # Is it in 'ghost list'
then                                      #   No, exit with 3
  ret_val=3
  echo "   >>> $hex_infile not in 'ghost list' <<<"
  echo "=> Exit $script_name with status=$ret_val" 
  exit $ret_val
fi

trace=$ARGOSLOG_DIR/$root_script_name.$TODAY"_trace"
error=$ARGOSLOG_DIR/$root_script_name.$TODAY"_error"

cp $HEXFILES_RECENT_DIR/$hex_infile $HEXFILES_RECENT_DIR/$hex_infile.sav 

hex_outfile=$hex_infile.new
ghost_file=`echo $record  | awk '{print $1}'`
latmin=`echo $record  | awk '{print $2}'`
latmax=`echo $record  | awk '{print $3}'`
lonmin=`echo $record  | awk '{print $4}'`
lonmax=`echo $record  | awk '{print $5}'`

gap_satellites=2          

# ------------------------------------------------------------------
#              Loop to remove bad data cycles
#              ------------------------------
#
# Return Values for 'remove_ghosts.x':
#   0, if everything is OK and no data cycles were removed
#   1, if everything is OK and data cycles were removed
#   4, if an error occurred
# ------------------------------------------------------------------
NMAX=300
n=0
echo "<<<<< $hex_infile >>>>>" >> $trace

while :
do
  n=`expr $n + 1`

  echo " ??? n: $n"

 $ARGOSPRG_DIR/remove_ghosts.x $hex_infile $hex_outfile        \
                               $latmin $latmax $lonmin $lonmax \
                               $gap_satellites                 \
                               $HEXFILES_RECENT_DIR $trace $error
  status="$?"
  echo "   STATUS(remove_ghosts.x): $status"
  if [ $status -ne 1 ]
  then
    break
  fi

  if [ $n -gt  $NMAX ]      # Loop counter exceeded?
  then                      #   Yes, exit(9)
    ret_val=9
    mv $HEXFILES_RECENT_DIR/$hex_infile.sav $HEXFILES_RECENT_DIR/$hex_infile  

    msg=">>> $script_name: Loop counter - $hex_infile > allowed maximum ($NMAX) - No cycles removed <<<"
    echo "$msg"
    echo "$msg" >> $email_msg

    if [ -f $trace ]        # Does it exist?
    then                    #   Yes
      if [ ! -s $trace ]    #   Is it empty?
      then                  #     Yes, remove it 
        rm $trace
      fi
    fi

    if [ -f $error ]        # Does it exist?
    then                    #   Yes
      if [ ! -s $error ]    #   Is it empty?
      then                  #     Yes, remove it
        rm $error
      fi
    fi

    echo "=> Exit $script_name with status=$ret_val" 
    exit $ret_val
  fi

  mv $HEXFILES_RECENT_DIR/$hex_outfile $HEXFILES_RECENT_DIR/$hex_infile
done

ret_val=$status

if [ $ret_val -ne 0 ]
then
  if [ $ret_val -eq 4 ]
  then
    ret_val=2
    msg="   >>> Error in remove_ghosts.x for $hex_infile - Status Value: 4 <<<"
    echo "$msg"
    echo "$msg" >> $email_msg
  else
    msg=">>> Error in remove_ghosts.x for $hex_infile - Status Value: $ret_val <<<"
    echo "$msg"
    echo "$msg" >> $email_msg
  fi
fi

if [ -f $trace ]        # Does it exist?
then                    #   Yes
  if [ ! -s $trace ]    #   Is it empty?
  then                  #     Yes, remove it 
    rm $trace
  fi
fi

if [ -f $error ]        # Does it exist?
then                    #   Yes
  if [ ! -s $error ]    # Is it empty?
  then                  #   Yes, remove it
    rm $error
  fi
fi

echo "=> Exit $script_name with status=$ret_val" 
exit $ret_val
