#!/bin/sh
# ===============================================================================
# Revision Date: 14-Sep-01 (RS - improve documentation)
#                27-Mar-01 (RS)
#
# emailprofiles_ftpgts 
# --------------------
# Scripts executes the following tasks:
#
# 1) Gets the 'filename' (e.g. to_gtsgateway.2001.03.26_09.58.gts+20010326+1050)
# of the oldest linked file in GTSFILES_TO_AOML_DIR.
#
# 2) Creates a file (email_body) by retrieving profile record(s) in the
# FTGTS4EMAIL_TABLE matching the 'filename' and appending two columns (ftp date
# and  ftp time, taken from linked file suffix) to each record. 
#
# 3) Sends Email to the AOML operator (AOML_OPERATOR).
#
# 4) Updates FTGTS4EMAIL_TABLE by removing records that matches the 'filename'.
# 
# 5) ???
#
# Exit Status Values
# ------------------
# 0, file transmitted successfully
# 1, no GTS-Formatted file to process 
# 2, Failure detected when sending Email to AOML_OPERATOR
# 3, GTS Formatted filename not found in FTPGTS4EMAIL_TABLE
#
# ===============================================================================
# -------------------------------------------------------------------------------
#  Revision Date: 04-Dec-00 (RS)
#
#  ExitCodeTable
#  =============
#  Creates a table with exit code, reason and action for each process
#
#  Usage: ExitCodeTable
# -------------------------------------------------------------------------------
ExitCodeTable()
{
  processexit_table=$FTPTMP_DIR/processexit.$TODAY"_"table
  cat > $processexit_table << EOF
Code              Reason                                    Action
 0  /Email to AOML_OPERATOR sent successfully               /Exit
 1  /No GTS-Formatted file to process                       /Exit, see process log-files
 2  /Failure detected when sending Email to AOML_OPERATOR   /Exit, see process log-files
 3  /GTS Formatted filename not found in FTPGTS4EMAIL_TABLE /Exit, see process log-files
EOF

 return 
}
# -------------------------------------------------------------------------------
#  Revision Date: 27-Mar-01 (RS)
#
#  DelExitCodeTable
#  =================
#  Deletes 'Exit Code Table'
#
#  Usage: DelExitCodeTable
# -------------------------------------------------------------------------------
DelExitCodeTable()
{
   rm -f $processexit_table
   return
}
# -------------------------------------------------------------------------------
#  Usage: MatchExitCode code
#  Revision Date: 27-Mar-01 (RS)
#
#  MatchExitCode
#  =============
#  Looks up the exit code. If code is matched, it prints the reason
#  and action else prints a no match message.
#
#  Usage: MatchExitCode code
# -------------------------------------------------------------------------------
MatchExitCode()
{
  code=$1
  grep "$code " $processexit_table > /dev/null     # 0, match; !- 0 otherwise
  if [ "$?" = "0" ]
  then
    reason=`grep "$code " $processexit_table | cut -d"/"  -f2`
    action=`grep "$code " $processexit_table | cut -d"/"  -f3`
    echo "ExitCode: $code => $reason"
    echo "Action:   $action"
  else
   echo "Exit Code not found in: $processexit_table"
  fi
  return
}
# -------------------------------------------------------------------------------
#  Revision Date: 23-Mar-91 (RS)
#
#  EmailProfileSent_Table
#  ======================
#  After the GTS-Formatted file has  been successfully transfered, the function 
#  does the following tasks: 
#
#  (1) Retrieves record(s) matching the name of the transferred file
#  (e.g. to_gtsgateway.2001.03.22_16.54.gts)
#  
#  (2) Using the retrived records, it removes the first field,  splits the profile_id
#  field into: aoml_id, argo_id and profile number, and appends the FTP date and time
#  to each record.  All records written to a file which is used as the body of the
#  email for data tracking (sent to SEAS).
#
#  (3) Sends Email to the AOML operator.
#
#  (4) Updates the FTPGTS4EMAIL_TABLE by removing matching records, if Email was
#  sent successfully.
#
#  FTPGTS4EMAIL_TABLE:
#     gts_file   - gts file containing GTS-Formatted profile(s) under the
#                  'profile_id' column.
#     profile_id - identification of the QC profile (0014_12275_085; aomid, argoid
#                  and profile number) converted to GTS Format.
#     wmo        - 'World Meteorological Organization' float identification number
#     lat        - profile latitude (sxx.xxx) taken from 1st position record
#     lon        - profile longitude (sxxx.xxx) taken from 1st position record
#     date       - profile date (yyyy/mm/dd) taken from 1st position record
#     time       - profile time (hh:mm) taken from 1st position record
# -------------------------------------------------------------------------------
EmailProfilesSent_FromTable()
{
  echo "=> Enter $root_script_name: EmailProfilesSent_FromTable"

  if [ ! -f $FTPGTS_EMAILS ]
  then
    echo "FTPGTS_EMAILS File Created"
    touch $FTPGTS_EMAILS
  fi

  echo "-----------------------\t\tFTPGTS4EMAIL_TABLE\t\t-----------------------"
  cat $FTPGTS4EMAIL_TABLE

# --------------------------------------------------------------------------
# Retrieve record(s) from FTPGTS4EMAIL_TABLE belonging to GTP-Formatted
# file (to_gtsgateway....) 
# --------------------------------------------------------------------------
  retrieved_records=$FTPTMP_DIR/retrieved_records.$TODAY_table
  grep $filename $FTPGTS4EMAIL_TABLE > $retrieved_records
  status="$?"
  if [ $status = "0" ]                   # Did it find the GTS-Formatted file?
  then                                   #   Yes, prepare list for email   
    echo "\nProfile(s) in GTS-Formatted file:"
    cat $retrieved_records

    rm_col=$FTPTMP_DIR/removed_column.$TODAY_table           # remove 1st field 
    add_ftpdatetime=$FTPTMP_DIR/add_ftpdatetime.$TODAY_table # append date & time
    email_body=$FTPTMP_DIR/email_profiles2gts.$TODAY

    cat $retrieved_records | awk '{print substr($0, length($1) + 2)}' > $rm_col # rm 1st field  
    cat $rm_col | awk -F"_" '{print $1, $2, $3}'   > $add_ftpdatetime # split profile_id  
    sed s%'$'%" $ftp_date $ftp_time"% $add_ftpdatetime > $email_body  # append date & time
    rm -f $rm_col $add_ftpdatetime

# --------------------------------------------------------------------------
# Send mail containing GTS Formatted profiles (FTP) to AOML_OPERATOR
# --------------------------------------------------------------------------
    subj="ARGO - PROFILES FTP2GTS"  # Email to list to AOML_OPERATOR  
    echo "\nTo: $AOML_OPERATOR  From: $EMAIL_OPERATOR"
    echo "Subj: $subj"
    echo "Body:"
    cat $email_body 
    echo "Sending Email ...\c"

    /usr/bin/mailx -s "$subj" $AOML_OPERATOR < $email_body
    status="$?"
    tmp="$up_root_script_name -"
    if [ $status = "0" ]                        # Email to AOML_OPERATOR OK?
    then                                        #   Yes, notify EMAIL_OPERATOR
      echo "OK"

      echo "\nTo:$EMAIL_OPERATOR  From:$EMAIL_OPERATOR"
      subj=$tmp`echo $subj | awk -F- '{print $2}'` 
      echo "Subj: $subj"
      msg="List of profile(s) sent to GTS was mailed to $AOML_OPERATOR"  
      echo "Boby: \c"
      echo $msg

      echo "Sending Email ...\c"
      echo $msg | /usr/bin/mailx -s "$subj" $EMAIL_OPERATOR
      if [ "$?" = "0" ]
      then
        echo "OK" 
      else
        echo "FAILURE" 
      fi

# --------------------------------------------------------------------------
#  Append Email body text to FTPGTS_EMAILS
# --------------------------------------------------------------------------
      cat $email_body >> $FTPGTS_EMAILS
      echo "\nEmail body text appended to $FTPGTS_EMAILS"

# --------------------------------------------------------------------------
# Update FTPGTS4EMAIL_TABLE - remove records with the GTS_Formatted filename 
# --------------------------------------------------------------------------
      updated_table=$FTPTMP_DIR/updated_table.$TODAY
      comm -23 $FTPGTS4EMAIL_TABLE $retrieved_records > $updated_table
      mv $updated_table $FTPGTS4EMAIL_TABLE 
      dash="-----------------------"
      echo "\n$dash\t   UPDATED FTPGTS4EMAIL_TABLE   \t$dash"
      if [ -s $FTPGTS4EMAIL_TABLE ]
      then
        cat $FTPGTS4EMAIL_TABLE
        echo 
      else
        echo "\t\t\t\tTable is now empty"
      fi
       
      rm -f $retrived_records
      rm -f $email_body
      rm -f $pathname                           #  rm link (../gts/data/to_aoml 

      ret_val=0

    else                                        #   No, notify EMAIL_OPERATOR
      echo "FAILURE"

      echo "\nTo:$EMAIL_OPERATOR  From:$EMAIL_OPERATOR"
      subj=$tmp"Failed to mail list of profiles put into GTS" 
      echo "Subj: $subj"
      msg="Failure detected while mailing the list of profile(s) "
      msg=$msg"put (FTP) into GTS\nto $AOML_OPERATOR. See logfile(s):\n\n"
      msg=$msg"\t$log_file\n" 

      if [ -s $error ]
      then
        msg=$msg"\t$error"
      else
        rm $error
      fi

      echo "Body: $msg" 
      echo "Sending Email ...\c"
      echo $msg | /usr/bin/mailx -s "$subj" $EMAIL_OPERATOR
      if [ "$?" = "0" ]
      then
        echo "OK" 
      else
        echo "FAILURE" 
      fi
      ret_val=2
    fi #endif - Mail to AOML_OPERATOR sent OK?

    rm -f $email_body

  else                                   #   No, GTS-Formatted file not in table.
                                         #   notify AOML_OPERATOR
    echo "\nTo:$EMAIL_OPERATOR   From:$EMAIL_OPERATOR"
    subj=$tmp"GTS Formatted filename is not in FTPGTS4EMAIL_TABLE"
    msg="The GTS Formatted filename ($filename)\nwas not found in "
    msg=$msg"FTPGTS4EMAIL_TABLE ($FTPGTS4EMAIL_TABLE). See logfile(s):\n\n"
    msg=$msg"\t$log_file\n"

    if [ -s $error ]
    then
      msg=$msg"\t$error"
    else
      rm -f $error
    fi

    echo "Body: $msg" 
    echo "Sending Email ...\c"
    echo $msg | /usr/bin/mailx -s "$subj" $EMAIL_OPERATOR
    if [ "$?" = "0" ]
    then
      echo "OK"
    else
      echo "FAILURE"
    fi
    ret_val=3
  fi #endif - Is filename in FTPGTS4EMAIL_TABLE? 

  echo "=> Exit EmailProfilesSent_FromTable"
  return $ret_val 
}
# ------------------------------ main ------------------------------

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}'`
up_root_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

now=`date +'%Y.%m.%d_%H.%M'`
echo "<===================== Start  $script_name: $now ===================>"

ExitCodeTable                   # Load Exit Status Code Table

error=$FTPLOG_DIR/$root_script_name.$TODAY"_"error

exec 2>> $error                 # append stderr to error
touch $error                    # it will exists

# -----------------------------------------------------------------------
# Check if there is any GTS-Formatted file available? 
#   Yes, continue process
#   No,  send mail and exit
# -----------------------------------------------------------------------
echo "Any GTS-Formatted data linked file available in GTSFILES_TO_AOML_DIR?"

pathname_list="$FTPTMP_DIR/pathname_list.$TODAY"
/usr/bin/ls -1tr $GTSFILES_TO_AOML_DIR/to_gts* > $pathname_list

echo "PATHNAME_LIST ($pathname_list):"
cat  $pathname_list
nentries=`wc -l $pathname_list | awk '{print $1}`

if [ $nentries -eq 0 ]
then
  ret_val=1
  subj="$up_root_script_name - No GTS-Formatted file available"
  text="No GTS-Formatted file available for processing."

  echo "To: $EMAIL_OPERATOR  FROM: $EMAIL_OPERATOR"
  echo "Subj: $subj"
  echo "Body: $text"

  echo "Sending Email ...\c"
  echo $text | /usr/bin/mailx -s "$subj" $EMAIL_OPERATOR
  echo "OK"

  rm -f $pathname_list $error 
  now=`date +'%Y.%m.%d_%H.%M'`
  MatchExitCode $ret_val
  DelExitCodeTable
  echo "<=================== Terminated $script_name: $now =================>"
  exit $ret_val                           # exit=1, no file to process 
fi

pathname=`cat $pathname_list | head -n 1` 
rm -f $pathname_list

filename_datetime=`basename $pathname`
filename=`echo $filename_datetime | awk -F"+" '{print $1}'`
ftp_date=`echo $filename_datetime | awk -F"+" '{print $2}' | \
          awk -F"+" '{print substr($0,1,4) "/" substr($0,5,2) "/" substr($0,7,2)}'`
ftp_time=`echo $filename_datetime | awk -F"+" '{print $3}'`

echo "GTS-Formatted Pathname: $pathname"
echo "GTS-Formatted File: $filename"
echo "GTS-Formatted File FTP Date: $ftp_date FTP Time: $ftp_time"

# -----------------------------------------------------------------------
# Create and send Email to AOML_OPERATOR if conditions are met
# -----------------------------------------------------------------------
EmailProfilesSent_FromTable

now=`date +'%Y.%m.%d_%H.%M'`
MatchExitCode $ret_val
DelExitCodeTable

if [ $ret_val = "0" ]
then
  rm -f $error 
  echo "<===================== Finish $script_name: $now ===================>"
else
  echo "<=================== Terminated $script_name: $now =================>"
fi

exit $ret_val                           # exit=1, no file to process
