#!/bin/sh
######################################################################################
# Revision Date: 21-Feb-13 (RS - Checks size by getting 'bytes sent' from the 'ftplog'
#                                file. If 'bytes sent' is not found, the process 
#                                continues and an appropriate comment will be added
#                                to the email. If size does not match, the packed file  
#                                IOPX02_KWBC_{timestamp}.bufr will be moved to
#                                INCOMPLETE_DATA_BUFR_DIR)
#                10-Dec-12 (RS - Bypass checking size, remote list not availabe.
#                                NWS moving file immediately afte transfer is finished) 
#                12-Aug-10 (RS)
#
# ftpgts_bufr_to_gateway.sh
# =========================
# 1) FTP buffer file created by 'nc_to_gtsbufr.sh' to the GTS NWS GateWay
# 2) Delete 'nc' profile(s) encoded in the buffer file from 'GTS_BUFR4FTP_TABLE'
# 3) Append the 'nc' profile(s) encoded in the buffer file to the BUFR_TRACK_TABLE 
#
# ALERT: FTP file transmission must be set to 'bin'
#
# Input Files
# -----------
# BUFR file IOPX02_KWBC_{timestamp}.bufr is located in 'DATA_BUFR_DIR'
#
# Exit Traps 
# ----------
# No buffer file found 
# Connection to GTS NWS GateWay Host failed
# FTP Session failed
# Local vs. Remote file size differ
# Error reported by 'comm' when removing record(s) from GTS_BUFR4FTP_TABLE
#
# Note
# ----
# ARGO_DIR: global variable defined in $HOME/.argorc_sh
# FTP_DIR: ARGO_DIR/ftp
# GTS_DIR: ARGO_DIR/gts
# DATA_BUFR_DIR: GTS_DIR/gts/data/bufr
# DONE_DATA_BUFR_DIR: DATA_BUFR_DIR/done 
# FAILURE_DATA_BUFR_DIR: DATA_BUFR_DIR/failure 
# INCOMPLETE_DATA_BUFR_DIR: DATA_BUFR_DIR/incomplete 
# LISTS_GTSBUFR_DIR: GTS_DIR/lists/bufr
# GTS_BUFR4FTP_TABLE: LISTS_GTSBUFR_DIR/GTS_BUFR4FTP_TABLE
# GTS_BUFR_TRACK_TABLE: LISTS_GTSBUFR_DIR/GTS_BUFR_TRACK_TABLE
#######################################################################################
# ----------------------------- Define Global Variables ------------------------------- 
. $HOME/.argorc_sh
#EMAIL_OPERATOR=Reyna.Sabina@noaa.gov
#--------------------------------------------------------------------------------------
# FTPBufrFile 
# ===========
# Transfers buffer file first record of form 'IOPX02_KWBC_{timestamp}
#--------------------------------------------------------------------------------------
FTPBufrDataFile()
{
   funcname=FTPBufrDataFile
   putbufr=$1
   echo "=> Enter $funcname $putbufr "

   ARGO_INIT_DIR=$ARGO_DIR/init_scripts
   initfile=$ARGO_INIT_DIR/argo.init
   GTS_NWSGATEWAY_HOST=`grep "^GTS_NWSGATEWAY_HOST" $initfile | awk '{print $2}' | sed -g s/"'"//`  # GTSNWS IP address
   GTS_NWSGATEWAY_DATA_DIR=`grep "^GTS_NWSGATEWAY_DATA_DIR" $initfile | \
                            awk '{print $2}' | sed -g s/"'"//`                                      # bufr data dir
   GTS_NWSGATEWAY_USER=`grep "^GTS_NWSGATEWAY_USER" $initfile | awk '{print $2}'`                   # user
   GTS_NWSGATEWAY_PW=`grep "^GTS_NWSGATEWAY_PW" $initfile | awk '{print $2}' | sed -g s/"'"//`      # pw

   echo "   GTS_NWSGATEWAY_HOST: $GTS_NWSGATEWAY_HOST"
   echo "   GTS_NWSGATEWAY_DATA_DIR: $GTS_NWSGATEWAY_DATA_DIR"
   echo "   DATA_BUFR_DIR: $DATA_BUFR_DIR"

   user=$GTS_NWSGATEWAY_USER
   pw=$GTS_NWSGATEWAY_PW

   transfer_mode=bin
   toggle="prompt off"
   echo "   user: $user"
#? echo "   pwd: $pw"
   echo "   transfer_mode: $transfer_mode"
   echo "   toggle: $toggle"

   able2connect_log=$LOG_DIR/$root_script_name"_"able2connet.$TODAY"_"log
   export able2connect_log
   echo "   able2connect_log: $able2connect_log"

   printf "   <- Connection to '$GTS_NWSGATEWAY_HOST' - "
   $SCRIPTS_DIR/able2connect.pl $GTS_NWSGATEWAY_HOST 1> /dev/null
   status_connect=$?

   if [ $status_connect -ne 0 ]
   then
     echo "FAILED ->"
     type=WARNING
     ref="Connection to GTS NWS GateWay Failed"
     msg="See logfiles:\n  $logfile\n  $able2connect_log"
     SendEmail $type "$ref" "$msg"
     echo "<---------------------------- Job Terminated at `date` ------------------------------->"
     exit
   fi
   echo "OK ->"

   remote_bufrlist=$TMP_DIR/remote_bufrlist.$TODAY
   ftplog=$LOG_DIR/$root_script_name"_"$TODAY"_"ftplog

   printf "   <- FTP Session to transmit buffer file - "
   ftp -n -v << endftp >> $ftplog
     open $GTS_NWSGATEWAY_HOST
     user $user $pw
     $transfer_mode
     $toggle
     cd  $GTS_NWSGATEWAY_DATA_DIR
     lcd $DATA_BUFR_DIR
     mput $putbufr
     dir  $putbufr $remote_bufrlist
endftp
   status_ftp=$?
   echo " FAILED with status=$status_ftp ->"
   if [ $status_ftp -ne 0 ]
   then
     outcome=FAILURE
     ref="FTP Session Failed" 
     msg="FTP session returned with status=$status_ftp."
     msg="$msg\nSee logfiles:\n  $logfile\n  $ftplog"
     SendEmail $outcome "$ref" "$msg"
     echo "<---------------------------- Job Terminated at `date` ------------------------------->"
     exit
   fi

   echo "=> Exit $funcname"
   return
}
#--------------------------------------------------------------------------------------
# CheckSizes
# ==========
# Compares the size of the remote bufer file with local buffer file to check whether
# or not the tranfer was succesfull. If successfull then the buffer file is moved to 
# the DONE_DATA_BUFR_DIR. If bufr file was not tranfer successfully then an email 
# is sent and the run terminated.  
#
# Note
# ----
# DATA_BUFR_DIR: ARGO_DIR/data/bufr
# DONE_DATA_BUFR_DIR: DATA_BUFR_DIR/done
#
#--------------------------------------------------------------------------------------
CheckSizes()
{
   funcname=CheckSizes
   echo "=> Enter $funcname ($1)"
   bufrfile=$1  

   bytes_sent=`grep "bytes sent " $ftplog | awk '{print $1}'`
   if [ "$bytes_sent" = "" ]          # Not found - assume complete transfer
   then
     type=ALERT
     msg="   ALERT -> Bytes Sent not found in 'ftplog."
     msg="$msg Size check could not be done, continue with process."
     echo "$msg"
     printf "   Move DATA_BUFR_DIR/$bufrfile ->  DONE_DATA_BUFR_DIR - "
     mv $DATA_BUFR_DIR/$bufrfile $DONE_DATA_BUFR_DIR
     echo "OK"
     foremail="Bytes sent string not found in 'ftplog' when '$bufrfile'" # for email (ret_val=0)
     foremail="$foremail\nwas transmitted and therefore the size check test could not be done."
     foremail="$foremail\nSee logfiles:\n  $logfile\n  $ftplog"
     echo "   => Return $funcname"
     return
   fi

   aoml_size=`ls -l $DATA_BUFR_DIR/$bufrfile | awk '{print $5}'`
   if [ $bytes_sent -eq $aoml_size ]  # Are the sizes equal?
   then                               #  Yes, transfer OK
     echo "      Transfer complete (remote size $bytes_sent = local size $aoml_size)"
     printf "   Move DATA_BUFR_DIR/$bufrfile ->  DONE_DATA_BUFR_DIR - "
     mv $DATA_BUFR_DIR/$bufrfile $DONE_DATA_BUFR_DIR
     echo "OK"
   else                               #  No, incomplete transfer
     outcome=FAILURE
     ref="Transfer Incomplete"
     msg="Transfer incomplete (remote size $bytes_sent, local size $aoml_size)"
     echo "   $msg"
     printf "   Move DATA_BUFR_DIR/$bufrfile ->  INCOMPLETE_DATA_BUFR_DIR - "
     mv $DATA_BUFR_DIR/$bufrfile $INCOMPLETE_DATA_BUFR_DIR
     echo "OK"
     msg="$msg\n$bufrfile moved to $INCOMPLETE_DATA_BUFR_DIR"
     msg="$msg\nSee lofiles:\n  $logfile\n  $ftplog"   
     SendEmail $outcome "$ref" "$msg"
     echo "<---------------------------- Job Terminated at `date` ------------------------------->"
     exit
   fi

   echo "=> Return $funcname"
   return
}
#--------------------------------------------------------------------------------------
# UpdateBufr4FTPTable 
# ===================
# Removes record(s) from 'GTS_BUFR4FTP_TABLE' which correspond to the buffer
# file (IOPX02_KWBC_{timestamp).bufr) that was successfully transmitted to the Gateway.
# If an error is detected then an email is sent and the run is terminated.   
#
# Table 'GTS_BUFR4FTP_TABLE' consists of the following fields: 
#   . Name of the output buffer file
#   . Name of the encoded real-time NetCDF file
#   . WMO number
#   . Latitude: profile latitude  
#   . Longitude: profile longitude
#   . Date (yyyy-mm-dd): profile date 
#   . Time (hh:mm): profile time
#--------------------------------------------------------------------------------------
UpdateBufr4FTPTable()
{
  funcname=UpdateBufr4FTPTable
  echo "=> Enter $funcname $1"
  filter=$1 

  recbufr2remove=$TMP_DIR/$root_script_name"_"$TODAY.recbufr2remove
  printf "   Create lists with record(s) to be removed from GTS_BUFR4FTP_TABLE - "
  grep $filter $GTS_BUFR4FTP_TABLE >> $recbufr2remove 2> /dev/null
  if [ ! -s $recbufr2remove ]
  then
    echo "ALERT, no record to be removed found"
    echo "ALERT"
    outcome=FAILURE
    ref="Record(s) to be removed from 'GTS_BUFR4FTP_TABLE' not found"
    msg="UpdateBuf4FTPTable - Records to be removed from:\n   $GTS_BUFR4FTP_TABLE\n"
    msg="were not found.\nSee logfiles:\n   $logfile"  
    SendEmail $outcome "$ref" "$msg"
    echo "<---------------------------- Job Terminated at `date` ------------------------------->"
    exit
  fi

  echo "OK"
  echo "   <<< Records to be removed from 'GTS_BUFR4FTP_TABLE' >>>"
  awk '{print "       " $0}' $recbufr2remove

  printf "   <- Remove records from GTS_BUFR4FTP_TABLE - "
  updated_gts_bufr4ftp_table=$TMP_DIR/updated_gts_bufr4ftp_table.$TODAY
  comm -23 $GTS_BUFR4FTP_TABLE $recbufr2remove > $updated_gts_bufr4ftp_table
  status_comm=$?
  if [ $status_comm -eq 0 ]
  then
    printf "OK => move to GTS_BUFR4FTP_TABLE"
    mv $updated_gts_bufr4ftp_table $GTS_BUFR4FTP_TABLE
    echo "OK ->"
    echo "   <<< Updated GTS_BUFR4FTP_TABLE: $GTS_BUFR4FTP_TABLE >>>"
    awk '{print "       " $0}' $GTS_BUFR4FTP_TABLE
  else
    echo "FAILURE"
    outcome=FAILURE
    ref="Error reported by 'comm' in 'UpdateBuf4FTPTable'"
    msg="Error reported by 'comm' when removing record(s) of FTPed  buffer files.\n"
    msg=$msg"from: $GTS_BUFR4FTP_TABLE\nSee logfiles:\n"
    msg=$msg"   $logfile"  
    SendEmail $outcome "$ref" "$msg"
    echo "<---------------------------- Job Terminated at `date` ------------------------------->"
    exit 
  fi

  echo "=> Exit $funcname"
  return
}
#--------------------------------------------------------------------------------------
# AppendBufrTrackTable
# ====================
# Append to the 'GTS_BUFR_TRACK_TABLE' the record(s) which correspond to the 
# buffer file(s) that were successfully transfmitted to the Gateway. 
# Two additional colunms are added to indicate the date (yyyy-mm-dd) and 
# time (hh:mm) that transmission took effect.
#
# Table 'GTS_BUFR_TRACK_TABLE' consists of the following fields:
#   . Name of the output buffer file
#   . Name of the encoded real-time NetCDF file
#   . WMO number
#   . Latitude: profile latitude  
#   . Longitude: profile longitude
#   . Date (yyyy-mm-dd): profile date 
#   . Time (hh:mm): profile time
#   . Date (yyyy-mm-dd): date profile was transmitted  
#   . Time (hh:mm): time profile was transmitted 
#--------------------------------------------------------------------------------------
AppendBufrTrackTable()
{
  funcname=AppendBufrTrackTable
  echo "=> Enter $funcname"

  GTS_BUFR_TRACK_TABLE=$LISTS_GTSBUFR_DIR/GTS_BUFR_TRACK_TABLE
  echo "   LISTS_GTSBUFR_DIR: $LISTS_GTSBUFR_DIR"
  echo "   GTS_BUFR_TRACK_TABLE: $GTS_BUFR_TRACK_TABLE"
  
  yyyymmdd=`echo $TODAY | awk -F'_' '{print $1}' | sed -g s/'\.'/-/`
  hhmm=`echo $TODAY | awk -F'_' '{print $2}' | sed -g s/'\.'/:/`
  
  echo "   <<< Update GTS_BUFR_TRACK_TABLE: $GTS_BUFR_TRACK_TABLE >>>"
  cat $recbufr2remove | 
  while read record
  do
    echo "       $record $yyyymmdd $hhmm"
    echo "$record $yyyymmdd $hhmm" >> $GTS_BUFR_TRACK_TABLE
  done 
  echo "=> Exit $funcname"
  return
}
#--------------------------------------------------------------------------------------
# SendEmail
#--------------------------------------------------------------------------------------
SendEmail()
{
   funcname=SendEmail
   type=$1
   title=$2
   body=$3

   echo "=> Enter $funcname ($type)"

   case $type in
     'ALERT')   subj="$up_root_script_name - $type, $title"
                and_but=and;;
     'FAILURE') subj="$up_root_script_name - $type, $title" 
                and_but=and;;
     'OK')      subj="$up_root_script_name - $type, $title"
                add_but=but;; 
     'WARNING') subj="$up_root_script_name - $type, $title" 
                and_but=and;;
   esac 
  
   if [ -s $error ]
   then
     subj=$subj" $and_but stderror reported" 
     if [ $type = "OK" ]
     then
       body="$body\nSee logfiles:\n  $logfile\n  $error"
     else
       body="$body\n  $error"
     fi
   else
     rm -f $error
   fi
 
   echo "   [Email Begin]"
   echo "     To: $EMAIL_OPERATOR"
   echo "     Subj: $subj"
   echo "     Text: "
   echo "$body" | awk '{print "     " $0}'
   printf "     Sending Email ... "
   printf "$body" | mailx -s"$subj" $EMAIL_OPERATOR
   echo "OK"
   echo "   [Email End]"
   echo "=> Exit $funcname ($type)"

   return
}
#------------------------------------- Main -------------------------------------------
TODAY=`date +'%Y.%m.%d_%H.%M.%S'`
script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}'`
up_root_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

FTP_DIR=$ARGO_DIR/ftp
LOG_DIR=$FTP_DIR/log
SCRIPTS_DIR=$FTP_DIR/scripts
TMP_DIR=$FTP_DIR/tmp

GTS_DIR=$ARGO_DIR/gts
DATA_DIR=$GTS_DIR/data
DATA_BUFR_DIR=$DATA_DIR/bufr
DONE_DATA_BUFR_DIR=$DATA_BUFR_DIR/done
FAILURE_DATA_BUFR_DIR=$DATA_BUFR_DIR/failure
INCOMPLETE_DATA_BUFR_DIR=$DATA_BUFR_DIR/incomplete
LISTS_GTSBUFR_DIR=$GTS_DIR/lists/bufr
PRG_JAVA_DIR=$GTS_DIR/prg/java

GTS_BUFR4FTP_TABLE=$LISTS_GTSBUFR_DIR/GTS_BUFR4FTP_TABLE

logfile=$LOG_DIR/$root_script_name.$TODAY"_log"
logftp=$LOG_DIR/$root_script_name.$TODAY"_logftp"
error=$LOG_DIR/$root_script_name.$TODAY"_error"
exec 1> $logfile
exec 2> $error

echo "<----------------------------- Job Begin at `date` -------------------------------->"
echo "<< FTP Directory Definitions >>"
echo "   FTP_DIR: $FTP_DIR"
echo "   LOG_DIR: $LOG_DIR"
echo "   SCRIPTS_DIR: $SCRIPTS_DIR"
echo "   TMP_DIR: $TMP_DIR"

echo "<< GTS Directory Definitions >>"
echo "   GTS_DIR: $GTS_DIR"
echo "   DATA_DIR: $DATA_DIR"
echo "   DATA_BUFR_DIR: $DATA_BUFR_DIR"
echo "   DONE_DATA_BUFR_DIR: $DONE_DATA_BUFR_DIR"
echo "   FAILURE_DATA_BUFR_DIR: $FAILURE_DATA_BUFR_DIR"
echo "   INCOMPLETE_DATA_BUFR_DIR: $FAILURE_DATA_BUFR_DIR"
echo "   LISTS_GTSBUFR_DIR: $LISTS_GTSBUFR_DIR"
echo "   PRG_JAVA_DIR: $PRG_JAVA_DIR"

echo "<< Email Definitions >>"
echo "   EMAIL_OPERATOR: $EMAIL_OPERATOR"

echo "<< Logfiles (stdout & stderror >>"
echo "   logfile: $logfile"
echo "   logftp: $logftp"
echo "   error: $error"

now=`date +'%Y.%m.%d_%H.%M'`

# ----------------------------------------------------------------------------
# Check if there is any list of the BUFR encoded file(s) to process.
# The 'nc' files are encoded into one or more buffer file(s) of the form:
#      IOPX02_KWBC.{timestamp}.bufr01 ...  IOPX02_KWBC.{timestamp}.bufrn 
# ----------------------------------------------------------------------------
product=`head -1 $PRG_JAVA_DIR/gtsheader.txt`
sender=`tail -1 $PRG_JAVA_DIR/gtsheader.txt`
echo "<< GTS Identifiers Used >>>"
echo "   Product Identifier: $product"
echo "   Sender Identifier: $sender"

if [ ! -s $GTS_BUFR4FTP_TABLE ]
then
  outcome=ALERT
  ref=" No buffer entries for FTP found in table" 
  msg="Table 'GTS_BUFR4FTP_TABLE', with buffer files not packed such as:\n"
  msg="     IOPX02_KWBC_{timestamp}.bufrnn (nn=01, 02, ... )"
  msg="$msg\ncreated by 'nc_to_gtsbuffer.sh' has no entries (processed previously)."
  msg="$msg\nLocation: $GTS_BUFR4FTP_TABLE"
  SendEmail $outcome "$ref" "$msg"
  echo "<---------------------------- Job Terminated at `date` ------------------------------->"
  exit
fi

prefix_bufr=`head -1 $GTS_BUFR4FTP_TABLE | awk '{print $1}' | awk -F"." '{print $1}'`
putbufr="$prefix_bufr.bufr"
printf "   Does '$DATA_BUFR_DIR/$putbufr?' exist? - "
if [ ! -f $DATA_BUFR_DIR/$putbufr ]
then
  echo "NO"
  printf "   Does '$INCOMPLETE_DATA_BUFR_DIR/$putbufr' exist? - "
  if [ -f $INCOMPLETE_DATA_BUFR_DIR/$putbufr ]
  then
    echo "YES"
    printf "   Move -> $DATA_BUFR_DIR - "
    mv $INCOMPLETE_DATA_BUFR_DIR/$putbufr $DATA_BUFR_DIR
    echo "OK"
  else 
   outcome=ALERT
   echo "NO"
   ref="Buffer encoded files not available"
   msg="Buffer file ('$putbufr') was not found neither in:\n"
   msg=$msg"  $DATA_BUFR_DIR nor $INCOMPLETE_DATA_BUFR_DIR"
   SendEmail $outcome "$ref" "$msg"
   echo "<---------------------------- Job Terminated at `date` ------------------------------->"
   exit
  fi
else
  echo "Yes, FTP to GTS Gateway"
fi

#??
foremail=""

FTPBufrDataFile $putbufr

CheckSizes  $putbufr

UpdateBufr4FTPTable $putbufr

AppendBufrTrackTable

# For Email
# ---------
if [ "$foremail" = "" ]
then
  type=OK
  ref="Buffer file tramsmitted to NWS Gateway"
  body=""
else
  type=ALERT
  ref="Buffer file tramsmitted but CheckSize Test not Done"
  body=$foremail
fi

SendEmail $type "$ref" "$body"

echo "<----------------------------- Job Finished at `date` -------------------------------->"

exit
