#!/bin/sh
# =====================================================================
# Revision Date: 21-Feb-13 (RS - in accordance with EF's desire, the 
#                                email notification when there are no   
#                                GTS-Formatted files to FTP is suppressed.
#                                The 'mailx' line has been commented)
#                13-Jan-12 (RS - Checks size by getting 'bytes sent'
#                                from the 'ftplog' file. If the string
#                                is not found, the process continues 
#                                and an appropriate comment will be 
#                                added to the email)
#                10-Dec-12 (RS - Bypass checking size, remote list not 
#                                available. NWS is moving file immediately
#                                after transfer is finished)
#                30-Apr-02 (RS - add prefix to processexit_table)
#                28-Mar-02 (RS - add to email name of the file put in
#                           the GTS Gateway)
#                21-Mar-02 (RS - usr/pw implementation)
#                25-Jan-02 (RS - new GTS data filename format)
#                17-Dec-01 (JZ - Check existance of gateway_dir_listing)
#                29-Mar-01 (RS - link GTS-Formatted file for Email
#                           processing)
#                21-Mar-01 (RS - add size check test)
#                20-Mar-01 (RS - change ftp transmition type to 'bin')
#                02-Mar-01 (RS - don't email when there is no file
#                           to process)
#                08-Jan-01 (RS - move to ftp directory)
#                04-Dec-00 (RS - implement Exit Code table)
#                21-Sep-00 (RS - update able2connect.pl)
#                22-Aug-00 (JZ)
#
# ftpgts_to_gateway.sh
# --------------------
# Transfer a FTP GTS-Formated file (gts_file) from GTSFILES_DIR to the
# NWS GTS GateWay Host. If no error is detected then the 'gts_file' is
# moved to DONE_GTSFILES_DIR and a soft link pointing to the 'gts-file'
# is created in ARGO_DIR/gts/data/to_aoml.     
#
# Definitions
# -----------
# GTSFILES_DIR: ARGO/gts/data
# DONE_GTSFILES_DIR: GTSFILES_DIR/done
# GTSFILES_TO_AOML_DIR: GTSFILES_DIR/to_aoml
#
# NWS Gateway variables defined in ARGO/init_scripts/argo.init: 
# GTSGATEWAY_HOST   'ingest.nws.noaa.gov'  # NWS GTS GateWay IP address
# GTSGATEWAY_USER   ftpin004               # ftp user
# 
# Note
# ----
# Script: ARGO_OPERATIONS/track/ftp/scrits/ftp_gtsdis_info.sh
# Creates a list (GTS_FTP_LIST) of the files in GTSFILES_TO_AOML_DIR 
# for tracking.    
#
# Exit Status Values
# ------------------
# 0, file transmitted successfully
# 1, no file to FTP
# 2, no connection to GTS GateWay Host available
# 3, FTP data file transfer not complete
# 4, FTP session to transfer data file to GTSGATEWAY_HOST failed
#
# =====================================================================
# --------------------------------------------------------------------
#  Revision Date: 04-Dec-00 (RS)
#
#  ExitCodeTable
#  =============
#  Creates a table with exit code, reason and action for each process
#
#  Usage: ExitCodeTable
# --------------------------------------------------------------------
ExitCodeTable()
{
  echo "   >>> Enter ExitCodeTable <<<"
  processexit_table=$FTPTMP_DIR/processexit_ftpgts_to_gateway.$TODAY"_"table
  echo "       processexit_table: $processexit_table"

  cat > $processexit_table << EOF
Code              Reason                                                       Action
 0  /File transmitted successfully                                      /Exit 
 1  /No file to FTP                                                     /Exit, see process log-files
 2  /No connection to GTS GateWay Host available                        /Exit, see process log-files
 3  /FTPGTS_TO_GATEWAY terminated abnormally                            /Exit, see process log-files
 4  /FTP session to transfer data file to GTSGATEWAY_HOST failed        /Exit, see process log-files
 5  /Error moving GTSFILES_DIR/datafile to DONE_GTSFILES_DIR            /Exit, see process log-files
EOF

 echo "   >>> Return ExitCodeTable <<<"

 return
}
# --------------------------------------------------------------------
#  Revision Date: 29-Nov-00 (RS)
#
#  DelExitCodeTable
#  =================
#  Deletes 'Exit Code Table'
#
#  Usage: DelExitCodeTable
# --------------------------------------------------------------------
DelExitCodeTable()
{
   echo "   >>> Enter DelExitCodeTable <<<"
   printf "       Remove 'processexit_table' - "
   rm -f $processexit_table
   echo "OK"
   echo "   >>> Return DelExitCodeTable <<<"
   return
}
# --------------------------------------------------------------------
#  Revision Date: 29-Nov-00 (RS)
#
#  MatchExitCode
#  =============
#  Looks up the exit code. If code is matched, it prints the reason
#  and action else prints a no match message.
#
#  Usage: MatchExitCode code
# --------------------------------------------------------------------
MatchExitCode()
{
  echo "   >>> Enter MatchExitCode <<<" 
  code=$1
  printf "       Code: $code, "

  entry=`grep "$code " $processexit_table`
  if [ "$entry" != "" ]
  then
    reason=`echo $entry | awk -F'/' '{print $2}'`
    action=`echo $entry | awk -F'/' '{print $3}'`
    echo "$reason"
    echo "       Action: $action"
  else
   echo "Not Found"
  fi
  echo "   >>> Return MatchExitCode <<<" 
  return
}
# --------------------------------------------------------------------
#  Revision Date: 20-Mar-01 (RS)
#
#  CheckSize
#  =========
#  Checks local and remote size of file transferred via FTP. 
#
#  Usage: CheckSize
#    Returns sizes_equal=y  or sizes_equal=n
#
# -------------------------------------------------------------------
CheckSize()
{
  echo "   => Enter CheckSize"
  bytes_sent=`grep "bytes sent " $ftplog | awk '{print $1}'`
  if [ "$bytes_sent" = "" ]          # Not found - assume complete transfer
  then                               
    sizes_equal=y
    msg="      <<< WARNING >>>\n      Bytes Sent not found in 'ftplog'."
    msg="$msg Size check could not be done."
    echo "$msg"
    add2text="\n$msg"                    # to be appended to email (ret_val=0)
    echo "   => Return CheckSize"
    return 
  fi

  aoml_size=`ls -l $GTSFILES_DIR/$filename | awk '{print $5}'`
  if [ $bytes_sent -eq $aoml_size ]  # Are the sizes equal?
  then                               #  Yes, transfer OK
    sizes_equal=y
    echo "      Transfer complete (remote size $bytes_sent = local size $aoml_size)"
    add2text=""
  else                               #  No, incomplete transfer
    sizes_equal=n
    msg="Transfer incomplete (remote size $bytes_sent, local size $aoml_size)"
    echo "      $msg"
    add2text="\n$msg"                  # to be appended to email (ret_val=6)
  fi

  echo "   => Return CheckSize"
  return
}
IsTransferComplete()
{
   echo "=> Enter IsTransferComplete"
   echo "   FTP Session - OK -> CheckSize"

   CheckSize

   if [ $sizes_equal = "y" ]     # Is transfer complete?
   then                          #   Yes, complete - SUCCESS
     msg="   FTP transfer complete.\n   "
     msg=$msg"$filename Uploaded to GTSGATEWAY_HOST"
     msg=$msg$add2text
     echo "$msg"
   else                          #   No, incomplete. Send mail & exit
     ret_val=3
     subj="up_root_script_name - FTP Incomplete data transfer"
     text="Incomplete data file transfer."
     text="$text$add2text"
     text="$text\nSee logfiles:\n   $log_file\n   $ftplog"

     if [ -s $error ]
     then
       text="$text\n   $error"
     else
       rm -f $error
     fi
     echo "$text" > $mail_message

     echo "   [Email Begin]"
     echo "   Subj: $subj"
     echo "   To: $EMAIL_OPERATOR"
     echo "   Text:"
     awk '{print "   " $0}' $mail_message
     printf "   Send - "
     mailx -s "$subj" $EMAIL_OPERATOR < $mail_message
     echo "OK"
     echo "   [Email End]"

     rm -f $mail_message
     now=`date +'%Y.%m.%d_%H.%M'`
     MatchExitCode $ret_val
     DelExitCodeTable
     echo "<-------------- Stop:  $script_name $now -------------->"
     exit $ret_val                # exit=6, FTP data file transfer not complete
   fi  # endif: is transfer complete?

   echo "=> Return IsTransferComplete"
   return
}
SessionFailed()
{
  echo "=> Enter SessionFailed"

  subj="$up_root_script_name - FTP Session failed"
  text="FTP Session to GTSGATEWAY_HOST failed."
  text="$text\nSee logfiles:\n   $log_file\n   $ftplog"

  if [ -s $error ]
  then
    text="$text\n   $error"
  else
    rm -f $error
  fi

  echo "$text" > $mail_message

  echo "   [Email Begin]"
  echo "   Subj: $subj"
  echo "   To: $EMAIL_OPERATOR"
  echo "   Text:"
  awk '{print "   " $0}'  $mail_message
  printf "   Send - "
  mailx -s "$subj" $EMAIL_OPERATOR < $mail_message
  echo "OK"
  echo "   [Email End]"
  rm -f $mail_message

  ret_val=4
  MatchExitCode $ret_val
  DelExitCodeTable
  now=`date +'%Y.%m.%d_%H.%M'`
  echo "<-------------- Exit:  $script_name $now -------------->"
  exit $ret_val
}
# =============================  Main =======================================
script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}'`
up_root_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

type=bin
user=$GTSGATEWAY_USER
pw=$GTSGATEWAY_PW

gateway_dir_listing=$FTPTMP_DIR/gateway.$TODAY"_"dir
mail_message=$FTPTMP_DIR/$root_script_name.$TODAY"_"mail

now=`date +'%Y.%m.%d_%H.%M'`

echo "<===================== Start  $script_name: $now ===================>"

ExitCodeTable                   # Load Exit Status Code Table

error=$FTPLOG_DIR/$root_script_name.$TODAY"_"error
exec 2>> $error                 # append stderr to error
touch $error                    # it will exists

# -----------------------------------------------------------------------
# If no GTS_Formmated data file is found then:
#  .  Send email of error first and then remove error
#  .  Remove  error file
# -----------------------------------------------------------------------
ls -tr $GTSFILES_DIR/to_gtsgateway*.gts > $FTPTMP_DIR/anyfiles 2> /dev/null
status_ls=$?
if [ $status_ls -ne 0 ]   # Is list empty?
then                      #  Yes
  ret_val=1
  subj="$up_root_script_name - GTS-Formatted data file not found"
  text="GTS-Formatted data file not found in: $GTSFILES_DIR"
  echo $text > $mail_message

  echo "=> [Email Begin]"
  echo "   Subj: $subj"
  echo "   To: $EMAIL_OPERATOR"
  echo "   Text:" 
  awk '{print "     " $0}'  $mail_message
  printf "   Send - "
#?mailx -s "$subj" $EMAIL_OPERATOR <  $mail_message
  echo "OK"
  echo "=> [Email End]"

  rm -f $error $mail_message
  rm $FTPTMP_DIR/anyfiles
  now=`date +'%Y.%m.%d_%H.%M'`
  MatchExitCode $ret_val
  DelExitCodeTable
  echo "<-------------- Stop:  $script_name $now -------------->" 
  exit $ret_val                           # exit=1, no file to ftp 
fi
# ---------------------------------------------------------------------------
# Test to see if GTSGATEWAY_HOST is alive 
# ---------------------------------------------------------------------------
echo "=> Check Connection to GTSGATEWAY_HOST($GTSGATEWAY_HOST)" 
able2connect_log=$FTPLOG_DIR/ftp2gts_able2connect.$TODAY"_"log
touch $able2connect_log
export able2connect_log


$FTPSCRIPT_DIR/able2connect.pl $GTSGATEWAY_HOST
status="$?"
if [ $status -eq 0 ]
then
  echo "   Conection -> OK" 
else
  ret_val=2
  echo "   Conection -> Failed or not Available" 
  subj="$up_root_script_name - Connection to GTS GateWay Failed or not Available "
  text="Connection to GTS GateWay (GTSGATEWAY_HOST) failed or not available.\n"
  text=$text"See logfiles:\n"
  text=$text"   $log_file\n"
  text=$text"   $able2connect_log"

  if [ -s $error ]
  then
    text=$text"\n\t$error"
  fi

  echo $text > $mail_message

  echo "   [Email Begin]"
  echo "   Subj: $subj"
  echo "   To: $EMAIL_OPERATOR"
  echo "   Text:"
  awk '{print "    " $0}'  $mail_message
  printf "    Send -  "
  mailx -s "$subj" $EMAIL_OPERATOR <  $mail_message
  echo "OK"
  echo "   [Email End]"

  rm -f $error $mail_message
  now=`date +'%Y.%m.%d_%H.%M'`
  MatchExitCode $ret_val
  DelExitCodeTable
  echo "<-------------- Stop:  $script_name $now -------------->" 
  exit $ret_val                # exit = 2, no connection available
fi
# ---------------------------------------------------------------------------
# Get path_filename to FTP & File name 
# ---------------------------------------------------------------------------
echo "   List of GTS-Formatted file(s):"
awk '{print "     " $0}' $FTPTMP_DIR/anyfiles
path_filename=`head -1 $FTPTMP_DIR/anyfiles`
filename=`basename $path_filename`
rm -f $FTPTMP_DIR/anyfiles
# ----------------------------------------------------------------
# FTP data file to $GTSGATEWAY_HOST
# ----------------------------------------------------------------
yyyymmdd=`echo $TODAY | awk -F"_" '{print $1}' | sed -n -g s%'\.'%%p`
hhmm=`echo $TODAY | awk -F"_" '{print $2}' | sed -n -g s%'\.'%%p`
ss=`date +'%S`
GTSGATEWAY_DATAFILE="ARGO_"$yyyymmdd"_"$hhmm$ss.dat
ftplog=$FTPTMP_DIR/$root_script_name.ftplog 

echo "   GTSGATEWAY_DATAFILE: $GTSGATEWAY_DATAFILE"
echo "   GTS-Formatted File to be uploaded $path_filename" 
echo "   ftplog: $ftplog"

if [ -f $ftplog ]
then
  rm -f $ftplog
fi

printf "=> Begin FTP Session to upload file to GTSGATEWAY_HOST - "
ftp -n -v<< endftp >> $ftplog
  open $GTSGATEWAY_HOST
  user $user $pw
  $type
  cd  $GTSGATEWAY_DATA_DIR
  lcd $GTSFILES_DIR 
  put $filename $GTSGATEWAY_DATAFILE
endftp
status_ftp="$?"
echo "Status=$status_ftp"

echo "   +++++++++++++++++++++ FTP Session Output +++++++++++++++++++++"
awk '{print "   " $0}'  $ftplog
echo "   +++++++++++++++++++ FTP Session Output End +++++++++++++++++++"

if [ $status_ftp -eq 0 ]  # FTP Error?
then                      #  No
  IsTransferComplete      #    If not complete exit (internal)
else                      # Yes, Send Email & Exit 
  SessionFailed
fi  # endif: FTP Error? 

# ----------------------------------------------------------------
# (1) Move data file to DONE_GTSFILES_DIR 
# (2) Create link (filename+today's date) to uploaded file 
#     for Email processing at AOML
# ----------------------------------------------------------------
echo "   GTSFILES_TO_AOML_DIR: $GTSFILES_TO_AOML_DIR" 
printf "=> Move $filename to $DONE_GTSFILES_DIR) - "
mv $GTSFILES_DIR/$filename $DONE_GTSFILES_DIR
echo "OK"

ret_val=0
cdsave=`pwd`                                    # link GTS-Formatted file 
cd $GTSFILES_TO_AOML_DIR
ftpdate=+`date +'%Y%m%d+%H:%M'`
echo  "=> Create link"
echo "   From: DONE_GTSFILES_DIR/$filename" 
printf "   TO: $GTSFILES_TO_AOML_DIR/$filename$ftpdate - "
ln -s $DONE_GTSFILES_DIR/$filename  $filename$ftpdate
echo "OK"
cd $cdsave

subj="$up_root_script_name - FTP transfer to GTS GateWay OK"
text="FTP transfer to GTS GateWay was successful."
text="$text\nGTS data file was moved to: $DONE_GTSFILES_DIR" 
text=$text"\nLocal file name: $filename"
text=$text"\nRemote file name: $GTSGATEWAY_DATAFILE"
text=$text$add2text

echo $text > $mail_message

echo "=> [Email Begin]"
echo "   Subj: $subj"
echo "   To: $EMAIL_OPERATOR"
echo "   Text:"
awk '{print "    " $0}'  $mail_message
printf "    Send - "
mailx -s "$subj" $EMAIL_OPERATOR < $mail_message
echo "OK"
echo "=> [Email End]"

rm -f $mail_message

MatchExitCode $ret_val
DelExitCodeTable
now=`date +'%Y.%m.%d_%H.%M'`
echo "<-------------- Stop:  $script_name $now -------------->"
rm -f $error
exit $ret_val
