#!/bin/sh
# ======================================================================================
# Revision Date: 13-May-13 (RS - turn back on FTP to GODAE)
#                01-May-13 (RS - change echo "   Location: $ftp_netcdf_list"
#                                to echo "ftp_netcdf_list: $ftp_netcdf_list" >> $logfile)
#                18-Feb-13 (RS - FTP to GODAE no longer required, code is commented)
#                19-Nov-10 (ER - Make the profile files to be FTP'ed before the delay 
#                                mode files.
#                09-Sep-08 (RS - reduce output to 'ftpnc_to_gdacs.[timestamp]_log'
#                                and introduce new trap 'Not connected' for ftp session)
#                03-Jan-07 (RS - redirect stderr to /dev/null when getting
#                                the 'nc' list to be processed)
#                30-Jun-05 (RS - add I/O error to ftp failures &
#                           catch empty remote_list)
#                23-Jun-05 (RS - change ls -l to dir -l in ftp session)
#                07-Mar-05 (RS - change ftp transmission to pasive)
#                06-Jul-04 (RS - add Std-Error to email subject)
#                29-Mar-04 (RS - add keywords to search pattern to
#                           detect possible error in logfile from
#                           ftp session)
#                09-Jul-03 (RS - change 'dir' to 'ls ...'
#                27-Jan-03 (RS - timestamp for file transsmision)
#                03-Dec-02 (RS - check for remote dir listing without 
#                           group entry)
#                28-May-02 (RS)
#
# ftpnc_to_gdacs.sh
# -----------------
# Submits Argo NetCDF formatted files (profile, trajectory, technical
# and meta) to FTP site IFREMER. FTP to  GODAE is no loger required 
#
# Tasks involved for sending NectCDF files to each GDAC site sare: 
#
# 1) Creates a list of file(s) to be transfered from directories with
# linked files: FTPDATA_GDC_DIR/to_ifremer
#  
# 2) Establishes a FTP connection to GDAC FTP host.
#
# 3) Transfers file(s) in the list to given the GDAC FTP site.
#
# 4) Determines file transfer completion by comparing profiles sizes
# in remote-directory and local-directory.
#
# FTPDATA_GDC_DIR=ARGO_DIR/ftp/data/gdc
#
# Exit status values for each GDC FTP session 
# -------------------------------------------
# 0, success
# 1, No NetCDF file to be transferred
# 2, No Connection available to GDC FTP server 
# 3, FTP error occurred while transferring file(s)
# 4, Transfer incomplete for at least one file in
#    FTPDATA_TOGDC_DIR[/to_godae or /to_ifremer] 
#
# Exit status values 
# ------------------
# 0, succes - no error detected  
# 1, error detected - check each GDC FTP session output
# 
# Note - FTP file transfer type is set to 'binary'
# ======================================================================================


# --------------------------------------------------------------------
#  Revision Date: 14-Dec-00 (RS)
#
#  ExitCodeTable
#  =============
#  Creates a table with exit code, reason and action for each process
#
#  Usage: ExitCodeTable
# --------------------------------------------------------------------
ExitCodeTable()
{
  processexit_table=$FTPTMP_DIR"/processexit_"$up_script_name.$TODAY"_table"
  cat > $processexit_table << EOF
Code              Reason                                                Action
 0  /Process finished successfully                                  /Exit - OK
 1  /No NetCDDF file to be transferred                              /Exit, see log-files
 2  /No connection available to FTP_GDC serve                       /Exit, see log-files
 3  /FTP error occurred while transferring file(s)                  /Exit, see log-files
 4  /Transfer incomplete for at least one file in FTPDATA_TOGDC_DIR /Exit, see log-files
EOF
 return
}


# --------------------------------------------------------------------
#  Revision Date: 29-Nov-00 (RS)
#
#  DelExitCodeTable
#  =================
#  Deletes 'Exit Code Table'
#
#  Usage: DelExitCodeTable
# --------------------------------------------------------------------
DelExitCodeTable()
{
   rm -f $processexit_table
   return
}


# --------------------------------------------------------------------
#  Revision Date: 29-Nov-00 (RS)
#
#  MatchExitCode
#  =============
#  Looks up the exit code. If code is matched, it prints the reason
#  and action else prints a no match message.
#
#  Usage: MatchExitCode code
# --------------------------------------------------------------------
MatchExitCode()
{
  code=$1
  grep "$code " $processexit_table > /dev/null     # 0, match; !- 0 otherwise
  if [ "$?" = "0" ]
  then
    reason=`grep "$code " $processexit_table | cut -d"/"  -f2`
    action=`grep "$code " $processexit_table | cut -d"/"  -f3`
    echo "\tExitCode: $code => $reason" >> $logfile
    echo "\tAction: $action"            >> $logfile
  else
   echo "\tExit Code ($code) not found in: $processexit_table" >> $logfile
  fi
  return
}


# --------------------------------------------------------------------
#  Revision Date: 21-Nov-00 (RS - allow 2 email operators)
#                 15-Sep-00 (RS)
#
#  ReadSet_InitVar
#  ===============
#  Reads and sets ARGO initialization environmental global variables
#  which are used to set depedent environmental global variables.
#
#  Usage: ReadSet_InitVar
# --------------------------------------------------------------------
ReadSet_InitVar()
{
  echo "--- Global Variables Initialization with Parameters ---" >> $logfile
  echo "        --- from Input File (argo.init) ---"             >> $logfile
  exec < $ARGO_DIR/init_scripts/argo.init
  while read envname  envvalue doc
  do
    case $envname in
      SENSOR_TYPES)
          t_sensor=`echo $envvalue | awk -F"," '{print $1}'` 
          s_sensor=`echo $envvalue | awk -F"," '{print $2}'`
          value="$t_sensor $s_sensor"
          name=$envname
          eval $name=$value
          eval export $name;;
       EMAIL_OPERATOR)
          opr_1=`echo $envvalue | awk -F"," '{print $1}'`
          opr_2=`echo $envvalue | awk -F"," '{print $2}'`
          value="$opr_1 $opr_2"
          name=$envname
          eval $name=$value
          eval export $name;;
      *)
          name=$envname
          value=$envvalue
          eval $name=$value
          eval export $name
   esac
  done

  return
}


# ------------------------------------------------------------------------
# FTPCheckCompletion
# ==================
# Determines the outcome of the file transfer by comparing entries
# in 'local_namesize_table' against 'remote_namesize_table'. It uses 
# the outcome of each FTP file transfer to:
#  . Insert in list for either: complete, incomplete or not-transfer files.
#  . Remove file symbolic link, if FTP transfer is complete. 
#
# Usage: unix> FTPCheckCompletion
#
# ------------------------------------------------------------------------
FTPCheckCompletion()
{
   timestamp=`date +'%Y.%m.%d_%H.%M.%S'`
   echo "=> Enter Function ... FTPCheckCompletion $timestamp" >> $logfile
   printf ">>> FTPCheckCompletion - Start: $timestamp " >> $timestamp_info
   exec < $local_namesize_table
   while read local_profile local_size 
   do
     grep $local_profile $remote_namesize_table > /dev/null
     if [ "$?" = "0" ]                               # Was it transferred?
     then                                            #   Yes, check its size
       remote_size=`grep $local_profile $remote_namesize_table | awk '{print $2}'`
       if [  $local_size = $remote_size ]            #   Are sizes equal?
       then
         echo $local_profile >> $complete_transfer_list
         rm $FTPDATA_TOGDC_DIR/$local_profile        # Remove sysmbolic link
         echo "   Remove FTPed file: $FTPDATA_TOGDC_DIR/$local_profile" >> $logfile
       else
         echo $local_profile >> $incomplete_transfer_list
         echo $local_profile >> $failed_list
       fi
     else
       echo $local_profile >> $not_transfer_list
       echo $local_profile >> $failed_list
     fi
   done

   timestamp=`date +'%Y.%m.%d_%H.%M.%S'`
   echo "Finished: $timestamp <<<" >> $timestamp_info
   echo "=> Exit Function  ... FTPCheckCompletion $timestamp" >> $logfile

   return 
}


# ------------------------------------------------------------------
# FTP_to_GDC
# ===========
# FTP to GDC specifies in the parameter argument
#
# Usage: FTPToGDC "godae"
#
#   returns:
#     0  Process finished successfully
#     1  No NetCDDF file to be transferred
#     2  No connection available to FTP_GDC serve
#     3  FTP error occurred while transferring file(s)
#     4  Transfer incomplete for at least one file in FTPDATA_TOGDC_DIR
#
# ------------------------------------------------------------------
FTP_to_GDC()
{
  gdc=$1

  now=`date +'%h %d, %Y %T %p'`
  echo "=> Start  FTP_to_GDC ($gdc): $now" >> $logfile
  exec 2> $error

  email_msg=$FTPLOG_DIR/$root_script_name"_"$gdc.$TODAY"_"email

  # 
  #      Create list of NetCDF file(s) to be transferred with
  #  files in FTPDATA_TOGDC_DIR (symbolic links to NETCDF_DATANC_DIR) 
  ftp_netcdf_list=$FTPTMP_DIR/$gdc"_ftp_netcdf_list.$TODAY"

  ls $FTPDATA_TOGDC_DIR/*.$extension  2> /dev/null > $ftp_netcdf_list 
  echo "\n-> NetCDF file(s) to be put in the FTP GDAC ($gdc) directory:" >> $logfile
  echo "   ftp_netcdf_list: $ftp_netcdf_list" >> $logfile

  for pathname in `cat $ftp_netcdf_list`
  do
    basename $pathname >> $to_transfer_list
  done

  count=`wc -l $ftp_netcdf_list | awk '{print $1}'`
  echo "   Number of Files to FTP = $count" >> $logfile

  if [ "$count" = "0" ]
  then
    return 1
  fi

  # 
  #  Check FTP connection to GDC FTP Server (FTP_GDC)
  echo "-> Checking connection to $FTP_GDC" >> $logfile
  echo "-> Checking connection to $FTP_GDC" >> $trace

  $FTPSCRIPT_DIR/able2connect.pl $FTP_GDC  >> $logfile
  status=$?
  if [ $status -eq 0 ]          # Is connection to FTP Server available?
  then                            #   Yes, coninue
    text="   Connection to $FTP_GDC is OK"
    echo "$text" >> $logfile
    echo "$text" >> $trace
  else  
    text="   Connection to $FTP_GDC is NOT OK: "
    echo "$text" >> $logfile
    echo "$text" >> $trace
    cat $able2connect_log >> $logfile
    cat $able2connect_log >> $trace
    return 2
  fi  

  # 
  #  FTP NetCDF files to GDC FTP 'Submit' location 
  type=bin
  toggle='prompt off'

  remote_list=$FTPTMP_DIR/$gdc"_remote_list.$TODAY"
  touch $remote_list
  echo "   remote_list: $remote_list"                           >> $logfile

  timestamp=`date +'%Y.%m.%d_%H.%M.%S'`
  text="-> BEGIN ... FTP Session - put NetCDF file(s)  $timestamp\n"
  text=$text"\t from: $FTPDATA_TOGDC_DIR\n"
  text=$text"\t to:   $FTP_GDC_DIR"
  echo $text >> $logfile

  printf ">>> FTP to $FTP_GDC - Start: $timestamp " >> $timestamp_info

ftp -n -v -p << endftp  > $ftp_stdout
  open $FTP_GDC
  user $FTP_GDC_USER $FTP_GDC_PW
  $type
  $toggle
  cd  $FTP_GDC_DIR
  pwd
  lcd $FTPDATA_TOGDC_DIR
  mput *.$extension
  dir -l "| grep nc > $remote_list" 
  quit
endftp

  timestamp=`date +'%Y.%m.%d_%H.%M.%S'`
  echo "Finished: $timestamp <<<" >> $timestamp_info
  echo "-> END   ... FTP Session  $timestamp" >> $logfile

  echo "-> ftp_stdout: $ftp_stdout" >> $logfile

  # 
  # Check for FTP error such as:  incomplete transfer, transfer aborted
  # and time out.  Command return status:
  #     0, error detected 
  #    >0, no error  
  # -------------------------------------------------------------------
  ret_val=0
  pattern="incomplete|Timeout|time out|aborted|service not available|Cannot connect"
  pattern="$pattern|I/O error|Could not open|Not connected"
  egrep -i "$pattern" $ftp_stdout > /dev/null
  status="$?"
  echo "\n   FTP Error Status (egrep - 0, error; > 0, OK): $status" >> $logfile

  if [ -s $remote_list ]     # Is remote_list empty?
  then                       #   No
    empty_remotefile=0
    echo "   FTP remote_list is NOT EMPTY"    >> $logfile
  else                       #   Yes
    empty_remotefile=1
    echo "   FTP Error: remote_list is EMPTY" >> $logfile
  fi

  if [ $status -eq 0 -o $empty_remotefile -eq 1 ]  # Did an error occur?
  then                              #   Yes
    return 3
  fi  # endif: FTP error 

  # 
  #                      *** Special Case ***
  # Check for FTP error #425 Can't buil data connection:"
  # Command return status:
  #     0, error detected
  #   >0, no error
  pattern="425 Can't build data connection:"
  grep "$pattern" $ftp_stdout > /dev/null
  status=$?
  echo "   FTP Error 425 Status (grep - 0, error; > 0, OK): $status" >> $logfile
  if [ $status = "0" ]             # Did an error occur?
  then                              #   Yes
    ftperror_425="\nFTP Error 425 - Can't build data connection: "
    ftperror_425=$ftperror_425"Operation already in progress"
    echo "$ftperror_425" >> $logfile
  else
    ftperror_425=""
  fi

  # 
  # Create a file to contain size and file name of NetCDF files
  # (symbolic links to NETCDF_DATANC_DIR) residing in FTPDATA_TOGDC_DIR
  # to be trasmitted to Global Data Center FTP site (FTP_GDC). 
  local_namesize_table=$FTPTMP_DIR/$gdc"_local_namesize_table"
  if [ -f $local_namesize_table ]
  then
    rm $local_namesize_table
  fi

  wpwd=`pwd`
  cd $FTPDATA_TOGDC_DIR

  for pathname in `cat $ftp_netcdf_list`
  do
    profile=`basename $pathname`
    ls -lL $profile  | awk '{print $9 " " $5}' >>  $local_namesize_table
  done
  cd $wpwd

  sort -i $local_namesize_table -o $local_namesize_table
  echo "-> Local Name & Size Table - NetCDF file(s) to be transferred" >> $logfile
  echo "   Location: $local_namesize_table"           >> $logfile

  # 
  # Create table (remote_namesize_table) from the remote directory 
  # listing obtained via FTP.  
  # -------------------------------------------------------------------
  echo "-> REMOTE FTP ($gdc) directory listing: " >> $logfile
  echo "   Location: $remote_list"           >> $logfile

  remote_namesize_table=$FTPTMP_DIR/$gdc"_remote_namesize_table"
  nfields=`head -n 1 $remote_list | awk '{print NF}'`
  echo "   Number of fields in remote directory listing: $nfields" >> $logfile
  if [ $nfields -eq 8 ]                        # IFREMER - has only 8 fields/record
  then
    cat  $remote_list | grep $extension | awk '{print $8 " " $4}' > $remote_namesize_table
  else
    cat  $remote_list | grep $extension | awk '{print $9 " " $5}' > $remote_namesize_table
  fi

  sort -i $remote_namesize_table -o $remote_namesize_table

  echo "-> Remote Name & Size Table of NetCDF file(s) transferred)" >> $logfile
  echo "   Location: $remote_namesize_table" >> $logfile


  # 
  #       FTPCheckCompletion - check FTP File Transfer Status
  #           (complete, incomplete or not-transferred) 
  #          Remove symbolic link if transfer is complete
  FTPCheckCompletion 

  n_complete=`wc -l $complete_transfer_list | awk '{print $1}'`
  n_incomplete=`wc -l $incomplete_transfer_list | awk '{print $1}'`
  n_nottransfer=`wc -l $not_transfer_list | awk '{print $1}'` 
  n_failures=`expr $n_incomplete + $n_nottransfer`

  echo "-> Transmission Stats"                                   >> $logfile
  echo "   Number of complete transfered files: $n_complete"     >> $logfile
  echo "   Number of incomplete transfered files: $n_incomplete" >> $logfile
  echo "   Number of files not transfered: $n_nottransfer"       >> $logfile
  echo "   Number of failures (incomplete + not): $n_failures"   >> $logfile

  #
  # Display transfer list (complete, incomplete & not transfer)
  if [ $n_complete -gt 0 ]
  then
    echo "-> Complete transferred file(s):"   >> $logfile
    echo "   Location: $complete_transfer_list" >> $logfile
  fi

  if [ $n_incomplete -gt 0 ]
  then
    echo "-> Incomplete transferred file(s):"            >> $logfile
    echo "   Location: $incomplete_transfer_list"          >> $logfile
    cat $incomplete_transfer_list | awk '{print "   " $0}' >> $logfile
  fi

  if [ $n_nottransfer -gt 0 ]
  then
    echo "-> Not transferred file(s):"            >> $logfile
    echo "   Location: $not_transfer_list"          >> $logfile
    cat $not_transfer_list | awk '{print "   " $0}' >> $logfile
  fi

  cat  $ftp_stdout >> $ftp_stdout.all 

  # 
  #  Check whether there were failures
  if [ $n_failures -eq 0 ]         # Any failure? 
  then                             #  No, all files transferred OK
    text="-> All file transfers from $FTPDATA_TOGDC_DIR were OK"
    echo $text >> $logfile
    echo $text >> $trace
    rm $ftp_stdout
  else                             #  Yes, either not trasferred or incomplete
    ret_val=4
    text="-> At least one file transfer from $FTPDATA_TOGDC_DIR was NOT COMPLETE"
    echo $text >> $logfile                
    echo $text >> $trace

    cat $incomplete_transfer_list  | awk '{print "   " $0}' >> $logfile
    cat $incomplete_transfer_list  | awk '{print "   " $0}' >> $trace 
    echo $text >> $trace
  fi

  echo "-> Returned Value (ret_val): $ret_val" >> $logfile
  echo "-> Returned Value (ret_val): $ret_val" >> $trace

  return $ret_val

# --------------------------------------------------------------------------
#    Check file transfers by checking updated 
#                    prepare Email
# --------------------------------------------------------------------------
  case $ret_val in
    0) subj="$up_script_name ($gdc): Transfer OK - all NetCDF file(s) transferred" 
       text="All NetCDF file(s) were successfully transferred.\n"
       echo $text >> $trace;; 

    4) subj="$up_script_name ($gdc): Transfer Error"
       text="The following NetCDF file(s) were not"
       text=$text" successfully transferred:"
       text=$text$ftperror_425

       if [ $n_incomplete -gt 0 ]
       then
         for profile in `cat $incomplete_transfer_list`
         do
           text="$text\n\t"`echo $profile`
         done
       fi
       if [ $n_nottransfer -gt 0 ]
       then
         for profile in `cat $not_transfer_list`
         do
           text="$text\n\t"`echo $profile`
         done
       fi

       text=$text"\n\nSee logfiles:"
       text=$text"\n\t$logfile"
       text=$text"\n\t$ftp_stdout"
       text=$text"\n\t$trace";;
  esac

  if [ -s $error ]      # Is error not empty?
  then                                #   Yes, cat to text
    subj="$subj & Std-Error Reported"
    text=$text"\nUnix standard-error (stderr) detected, see:\n"
    text=$text"\n\t$error"
  else
    rm $error
  fi

#?rs rm $complete_transfer_list $incomplete_transfer_list  $not_transfer_list

# ------------------------------------------------------------------------
#                              Send Email
# ------------------------------------------------------------------------
  echo $text            > $email_msg
  cat $timestamp_info  >> $email_msg

  echo "=> [Email Start - $gdc]"     >> $logfile
  echo "   Subj: $subj"        >> $logfile
  echo "   To: $EMAIL_OPERATOR"      >> $logfile
  echo "   Text:"                    >> $logfile
  awk '{print "   " $0}'  $email_msg >> $logfile

  printf "   Sending Email ... "     >> $logfile
  mailx -s "$subj" $EMAIL_OPERATOR   < $email_msg
  echo " OK" >> $logfile
  echo "=> [Email End - $gdc]"       >> $logfile

  MatchExitCode $ret_val
#?DelExitCodeTable

  now=`date +'%h %d, %Y %T %p'`
  if [ $ret_val -ne 0 ]
  then
    echo "=> Terminated $script_name ($gdc): $now" >> $logfile
  else
    echo "=> Finish $script_name ($gdc): $now"     >> $logfile
  fi
 
  return $ret_val
}


# ------------------------------------------------------------------
# Process_GDC
# ===========
# Tasks:
#   (1) prepare the calls to FTP_to_GDC
#   (2) call FTP_to_GDC as many times as needed in order to FTP 
#       all files to GDC
#   (3) process results, and take actions such as sending emails 
#       accordingly
#    
#
# Usage: 
#    Process_GDC  'GDC'
#    where 'GDC' can be godae or ifremer
#
Process_GDC()
{
  gdc=$1

  error=$FTPLOG_DIR/$root_script_name"_"$gdc.$TODAY"_"error
  trace=$FTPLOG_DIR/$root_script_name"_"$gdc.$TODAY"_"trace
  able2connect_log=$FTPLOG_DIR/able2connect"_"$gdc.$TODAY"_"log
  timestamp_info=$FTPLOG_DIR/timestamp_info"_"$gdc.$TODAY
  touch $error
  touch $trace
  touch $able2connect_log
  export able2connect_log

  #
  #  Set up GDC parameters for FTP transfers
  if [ "$gdc" = "godae" ]
  then
    FTP_GDC=$FTP_GODAE
    FTP_GDC_DIR=$FTP_GODAE_DIR
    FTP_GDC_USER=$FTP_GODAE_USER
    FTP_GDC_PW=$FTP_GODAE_PW
  else
    FTP_GDC=$FTP_IFREMER
    FTP_GDC_DIR=$FTP_IFREMER_DIR
    FTP_GDC_USER=$FTP_IFREMER_USER
    FTP_GDC_PW=$FTP_IFREMER_PW
  fi

  #
  # Where the files to be FTP'd are.
  FTPDATA_TOGDC_DIR=$FTPDATA_GDC_DIR/to_$gdc    
  FTPDATA_TOGDC_DIR_ALL=$FTPDATA_TOGDC_DIR/all
  #
  # Move files from to_{gdc} to to{gdc}/all
  mv $FTPDATA_TOGDC_DIR/*.$extension  $FTPDATA_TOGDC_DIR_ALL/ 2>/dev/null

  #
  # tracin' & loggin'
  now=`date +'%h %d, %Y %T %p'`
  echo "$now" >> $logfile
  echo "   FTP_GDC (GDC FTP Server): $FTP_GDC" >> $logfile
  echo "   FTP_GDC_USER: $FTP_GDC_USER" >> $logfile      
  echo "   FTP_GDC_DIR (GDC 'Submit' directory location): $FTP_GDC_DIR" >> $logfile
  echo "   FTPDATA_TOGDC_DIR: $FTPDATA_TOGDC_DIR" >> $logfile
  echo "$now" >> $trace
  echo "FTP_GDC (GDC FTP Server): $FTP_GDC" >> $trace
  echo "FTP_GDC_USER: $FTP_GDC_USER"        >> $trace
  echo "FTP_GDC_DIR (GDC 'Submit' directory location): $FTP_GDC_DIR" >> $trace
  echo "FTPDATA_TOGDC_DIR: $FTPDATA_TOGDC_DIR" >> $trace

  #
  #  Initialize counters of return values from FTP_to_GDC
  cnt_OK=0
  cnt_noFileToFTP_errors=0
  cnt_connection_errors=0
  cnt_FTP_errors=0
  cnt_NonCompleteTransfers_errors=0

  #
  # I/O working files
  echo "-> I/O Working Files" >> $logfile
  ftp_stdout=$FTPLOG_DIR/ftp_stdout_$gdc.$TODAY
  complete_transfer_list=$FTPTMP_DIR/$gdc"_"complete_transfer_list.$TODAY
  incomplete_transfer_list=$FTPTMP_DIR/$gdc"_"incomplete_transfer_list.$TODAY
  not_transfer_list=$FTPTMP_DIR/$gdc"_"not_transfer_list.$TODAY
  touch $complete_transfer_list
  touch $incomplete_transfer_list
  touch $not_transfer_list
  echo "   ftp_stdout: $ftp_stdout" >> $logfile
  echo "   complete_transfer_list:   $complete_transfer_list"   >> $logfile
  echo "   incomplete_transfer_list: $incomplete_transfer_list" >> $logfile
  echo "   not_transfer_list: $not_transfer_list"               >> $logfile

  
  #
  # FTP files to GDC in batches of 'FILES_PER_FTP_BLOCK'
  no_file_to_ftp_error=0          # 0 if at least one file was 'waiting' to be FTP'd
  keep_ftping="y"                 # boolean: keep FTPing files?
  nbr_iterations=0                # how many times the program has (tried) to FTP files
  ret_main=0                      # 0: Ok,  1: Some error
  while [ "$keep_ftping" = "y" ]
  do

    #
    # Get how many files are there to be FTP'd
    n_files_togdc=`ls -l $FTPDATA_TOGDC_DIR/*.$extension 2>/dev/null | wc -l`
    n_files_all=`ls -l $FTPDATA_TOGDC_DIR_ALL/*.$extension 2>/dev/null | wc -l`

    #
    # Find out if there are no files to be FTP'd
    empty_dirs="n"
    if [ $n_files_togdc -eq 0 ]
    then
      if [ $n_files_all -eq 0 ]
      then
        empty_dirs="y"
        keep_ftping="n"
        #
        # If directories are empty and it is the first
        # iteration, then there was no file to FTP since the beginning
        if [ $nbr_iterations -eq 0 ]
        then
          no_file_to_ftp_error=1
          ret_main=1
        fi
      fi
    fi

    #
    # Do the actual FTP 
    if [ "$empty_dirs" = "n" ]
    then
      #
      # If there are less than 'FILES_PER_FTP_BLOCK in the to_{GDC} directory
      # then move some of them from to_{GDC}/all directory 
      if [ $n_files_togdc -lt $FILES_PER_FTP_BLOCK ]
      then
        diff_nfiles=`expr $FILES_PER_FTP_BLOCK - $n_files_togdc`
        ls -r $FTPDATA_TOGDC_DIR_ALL/*.$extension 2>/dev/null | head -$diff_nfiles > $FTPTMP_DIR/files_to_move
        for f in `cat $FTPTMP_DIR/files_to_move` 
        do
          mv $f $FTPDATA_TOGDC_DIR/
        done
      fi

      # Call to function that FTPs  
      FTP_to_GDC $gdc
      status=$?
      echo "=> Status (return from FTP_to_GDC $gdc): $status" >> $logfile
      case "$status" in
        0) cnt_OK=`expr $cnt_OK + 1` ;;
        1) cnt_noFileToFTP_errors=`expr $cnt_noFileToFTP_errors + 1` ;;
        2) cnt_connection_errors=`expr $cnt_connection_errors + 1` ;;
        3) cnt_FTP_errors=`expr $cnt_FTP_errors + 1` ;;
        4) cnt_NonCompleteTransfers_errors=`expr $cnt_NonCompleteTransfers_errors + 1` ;;
      esac

      nbr_iterations=`expr $nbr_iterations + 1`
      if [ $nbr_iterations -gt $MAX_NBR_ITERATIONS ]
      then
        keep_ftping="n"
      fi

    fi

  done


  #
  #  Analyzing results of FTP process.
  #
  #  a) no files to transfer.
  if [ $no_file_to_ftp_error -ne 0 ]     
  then
    subj="$up_script_name ($gdc): NO NetCDF file to be transferred"
    text="No NetCDF file to be transferred to local FTP server for\n"
    text=$text"Internet access, see log-files:\n"
    text=$text"\n\t$logfile"
    text=$text"\n\t$trace"

    if [ -s $error ]
    then
      subj="$subj & Std-Error Reported"
      text=$text"\n\t$error"
    else
      rm  $error
    fi
    echo "$text" > $email_msg
  
    echo "=> [Email Start - $gdc]"     >> $logfile
    echo "   Subj: $subj"              >> $logfile
    echo "   To: $EMAIL_OPERATOR"      >> $logfile
    echo "   Text:"                    >> $logfile
    awk '{print "   " $0}'  $email_msg >> $logfile

    printf "   Sending Email ... "     >> $logfile
    mailx -s "$subj" $EMAIL_OPERATOR    < $email_msg
    echo "OK" >> $logfile
    echo "=> [Email End - $gdc]"       >> $logfile

    MatchExitCode 1

    now=`date +'%h %d, %Y %T %p'`
    echo "=> Finish $script_name ($gdc): $now" >> $logfile
    return
  fi

  #
  #  b) Happy Path.
  if [ $cnt_OK = $nbr_iterations ]
  then
    subj="$up_script_name ($gdc): Transfer OK - all NetCDF file(s) transferred" 
    text="All NetCDF file(s) were successfully transferred.\n"
    echo $text >> $trace
    if [ -s $error ]      # Is error not empty?
    then                                #   Yes, cat to text
      subj="$subj & Std-Error Reported"
      text=$text"\nUnix standard-error (stderr) detected, see:\n"
      text=$text"\n\t$error"
    else
      rm $error
    fi
    echo $text            > $email_msg
    cat $timestamp_info  >> $email_msg

    echo "=> [Email Start - $gdc]"     >> $logfile
    echo "   Subj: $subj"        >> $logfile
    echo "   To: $EMAIL_OPERATOR"      >> $logfile
    echo "   Text:"                    >> $logfile
    awk '{print "   " $0}'  $email_msg >> $logfile

    printf "   Sending Email ... "     >> $logfile
    mailx -s "$subj" $EMAIL_OPERATOR   < $email_msg
    echo " OK" >> $logfile
    echo "=> [Email End - $gdc]"       >> $logfile

    MatchExitCode 0
  else
    #
    # Not so happy path
    subj="$up_script_name ($gdc): At least an error was reported" 
    text="\n\nNumber of FTP attempts: $nbr_iterations \n"
    text=$text"Process finished successfully: $cnt_OK \n"
    text=$text"No NetCDDF file to be transferred: $cnt_noFileToFTP_errors \n"
    text=$text"No connection available to FTP_GDC server: $cnt_connection_errors \n"
    text=$text"FTP error occurred while transferring file(s): $cnt_FTP_errors \n"
    text=$text"Transfer incomplete for at least one file in $FTPDATA_TOGDC_DIR: $cnt_NonCompleteTransfers_errors \n\n\n"

    if [ -s $error ]      # Is error not empty?
    then                                #   Yes, cat to text
      subj="$subj & Std-Error Reported"
      text=$text"\nUnix standard-error (stderr) detected, see:\n"
      text=$text"\n\t$error"
    else
      rm $error
    fi

    text=$text"\n\nLog file is $logfile\n"

    echo $text            > $email_msg
    
    cat $timestamp_info  >> $email_msg

    echo $text >> $trace

    echo "=> [Email Start - $gdc]"     >> $logfile
    echo "   Subj: $subj"        >> $logfile
    echo "   To: $EMAIL_OPERATOR"      >> $logfile
    echo "   Text:"                    >> $logfile
    awk '{print "   " $0}'  $email_msg >> $logfile

    printf "   Sending Email ... "     >> $logfile
    mailx -s "$subj" $EMAIL_OPERATOR   < $email_msg
    echo " OK" >> $logfile
    echo "=> [Email End - $gdc]"       >> $logfile
    
  fi

}


# ==========================  Main script  ===================================
#-------------------- Initialize Variables ----------------------
TODAY=`date +'%Y.%m.%d_%H.%M'`

script_name=`basename $0`
root_script_name=`echo $script_name| awk -F"." '{print $1}`
up_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

FTP_DIR=$ARGO_DIR/ftp
FTPSCRIPT_DIR=$FTP_DIR/scripts
FTPLOG_DIR=$FTP_DIR/log
FTPTMP_DIR=$FTP_DIR/tmp
FTPDATA_DIR=$FTP_DIR/data
FTPDATA_GDC_DIR=$FTPDATA_DIR/gdc

                                   # for moving '*.nc' files to  'done' directory
NETCDF_DIR=$ARGO_DIR/netcdf
NETCDF_DATA_DIR=$NETCDF_DIR/data
NETCDF_DATANC_DIR=$NETCDF_DATA_DIR/nc
NETCDF_DATADONE_DIR=$NETCDF_DATA_DIR/done

logfile=$FTPLOG_DIR/$root_script_name.$TODAY"_"log

now=`date +'%h %d, %Y %T %p'`
echo "<================ Start  $script_name: $now ==============>" >> $logfile
ReadSet_InitVar                    # from 'argo.init'
ExitCodeTable

# ---------------------------------------------------------------------
# to_transfer_list = list with all file(s) to be transfered
# failed_list      = list with all file(s) that failed to be transfered
# ---------------------------------------------------------------------
to_transfer_list=$FTPTMP_DIR/to_transfer.$TODAY"_"list
failed_list=$FTPTMP_DIR/failed.$TODAY"_"list
touch $to_transfer_list
touch $failed_list

FILES_PER_FTP_BLOCK=100  
MAX_NBR_ITERATIONS=15

# ---------------------------------------------------------------------
#                    FTP to GDC - GODAE  
# ---------------------------------------------------------------------
echo ">************************************************************************************<" >> $logfile
extension="nc"
gdc="godae"
Process_GDC $gdc 
echo ">************************************************************************************<" >> $logfile

# ---------------------------------------------------------------------
#                   FTP to GDC - IFREMER 
# ---------------------------------------------------------------------
echo ">************************************************************************************<" >> $logfile
extension="nc"
gdc="ifremer"
Process_GDC $gdc
echo ">************************************************************************************<" >> $logfile

# ------------------------------------------------------------------------
# (1) Sort and remove dups in 'to_transfer_list' and 'failed_list'
# (2) Create a list 'ok_to_both_sites_list' of files that did not failed
# ------------------------------------------------------------------------
echo "                ---------- Summary ----------" >> $logfile
sort -u $to_transfer_list -o $to_transfer_list
echo "-> To be transmitted: $to_transfer_list"  >> $logfile
echo "   Location: $to_transfer_list" >> $logfile
 #?rs cat $to_transfer_list | awk '{print "   " $0}'  >> $logfile

sort -u $failed_list -o $failed_list
echo "-> Not successfully transmitted : $failed_list" >> $logfile
echo "   Location: $failed_list" >> $logfile
#?rs cat $failed_list | awk '{print "   " $0}' >> $logfile

ok_to_both_sites_list=$FTPTMP_DIR/ok_to_both_sites.$TODAY"_list"
comm -23 $to_transfer_list $failed_list > $ok_to_both_sites_list

echo "-> Successfully transmitted to all sites: $ok_to_both_sites_list" >> $logfile
echo "   Location: $ok_to_both_sites_list" >> $logfile
#?rs cat $ok_to_both_sites_list | awk '{print "   " $0}' >> $logfile

DelExitCodeTable

#?rs rm $to_transfer_list $failed_list $ok_to_both_sites_list

now=`date +'%h %d, %Y %T %p'`
if [ $ret_main -ne 0 ]
then
  echo "<============== Terminated $script_name: $now ================>" >> $logfile
else
  echo "<================== Finish $script_name: $now ================>" >> $logfile
fi

exit $ret_main
