#!/bin/sh
# =======================================================================
# Revision Date: 08-Jan-14 (RS - changes file extension of download file
#                                from 'Z' to 'gz')
#                02-Jun-06 (RS - change IP address and path to data)
#                23-Jun-05 (RS - change ls -l to dir -l in ftp session)
#                06-May-04 (RS - fix mv to ARCHIVE_DIR)
#                05-May-04 (RS - change ftp site and dir)
#                09-Jul-03 (RS - dir -> ls in FTP session)
#                24-Jun-02 (RS - when more that 1 file is FTPed from NCEP,
#                           rename *.Z file(s) and move to old analysis
#                           archive dir)
#                30-Apr-02 (RS - add prefix to processexit_table)
#                16-Apr-02 (JZ - modified email subjects
#                20-Sep-01 (RS - update doc & copy new analysis file
#                           to $ARGO_DIR/ftp/data/ncep/old_analysis)
#                27-Jun-01 (RS)
#
# ftpncep_to_aoml.sh
# ------------------
# FTP weekly analysis data file to AOML. Data file is located
# in the 'pub/cmb/Global' directory. FTP file is unpacked and
# move to the ARGO 'clima_mask' operational directory. 
#
# Exit Status Values
# ------------------
# 0, file transmitted successfully                      
# 1, no connection to NCEP FTP Host available           
# 2, FTP or STDERR failure while getting NCEP dir list
# 3, NCEP Directory Listing is empty                    
# 4, no new file to be transferred from NCEP                
# 5, At least one file transfer from NCEP was not complete
# 6, FTP file transfer from FTPNCEP_HOST failed 
# 7, Error detected while unpacking file
# Note - FTP file transfer type is set to 'bin'
#  
# =======================================================================
# --------------------------------------------------------------------
#  Revision Date: 27-Jun-01 (RS)
#
#  ExitCodeTable
#  =============
#  Creates a table with exit code, reason and action for each process
#
#  Usage: ExitCodeTable
# --------------------------------------------------------------------
ExitCodeTable()
{
  processexit_table=$FTPTMP_DIR/processexit_ftpncep_to_aoml.$TODAY"_"table
  cat > $processexit_table << EOF
Code              Reason                                    Action
 0  /File transmitted successfully                          /Exit 
 1  /No connection to NCEP FTP Host available               /Exit, see process log-files
 2  /FTP or STDERR failure while getting NCEP dir list      /Exit, see process log-files
 3  /NCEP Directory Listing is empty                        /Exit, see process log-files
 4  /No new file to be transferred from NCEP                /Exit, see process log-files
 5  /At least one file transfer from  NCEP was not complete /Exit, see process log-files
 6  /FTP file transfer from FTPNCEP_HOST failed             /Exit, see process log-files
 7  /Error detected while unpacking file                    /Exit, see process log-files
EOF

  return
}
# --------------------------------------------------------------------
#  Revision Date: 29-Nov-00 (RS)
#
#  DelExitCodeTable
#  =================
#  Deletes 'Exit Code Table'
#
#  Usage: DelExitCodeTable
# --------------------------------------------------------------------
DelExitCodeTable()
{
   rm -f $processexit_table
   return
}
# --------------------------------------------------------------------
#  Revision Date: 29-Nov-00 (RS)
#
#  MatchExitCode
#  =============
#  Looks up the exit code. If code is matched, it prints the reason
#  and action else prints a no match message.
#
#  Usage: MatchExitCode code
# --------------------------------------------------------------------
MatchExitCode()
{
  code=$1
  grep "$code " $processexit_table > /dev/null     # 0, match; !- 0 otherwise
  if [ "$?" = "0" ]
  then
    reason=`grep "$code " $processexit_table | cut -d"/"  -f2`
    action=`grep "$code " $processexit_table | cut -d"/"  -f3`
    echo "ExitCode: $code => $reason"
    echo "Action:   $action"
  else
   echo "Exit Code not found in: $processexit_table"
  fi
  return
}
# --------------------------------------------------------------------
#  Revision Date: 20-Mar-01 (RS)
#
#  CheckSizes
#  ==========
#  Checks local and remote size of file transferred via FTP. If remote
#  directory is empty, the comparison test cannot be donea - transfer
#  complete is assumed.
#
#  Usage: CheckSizes
#    Returns sizes_equal=y  or sizes_equal=n
#
# -------------------------------------------------------------------
CheckSizes()
{
  echo "=> Enter CheckSizes"

  failure_list=$FTPTMP_DIR/failure_list
  if [ -f $failure_list ]
  then
    rm -f $failure_list
  fi
  
  addok=""
  addfail=""
  sizes_equal='y'

  for filename in `cat $new_files`
  do
    printf "Checking Size for file: $FTPDATA_NCEP_DIR/$filename ... " 
    grep $filename $remote_dir_listing > $FTPTMP_DIR/entry
    status="$?"
    echo "Status(grep): $status"

    ncep_size=`cat $FTPTMP_DIR/entry | awk '{print $5}'`
    ls -l $FTPDATA_NCEP_DIR/$filename
    aoml_size=`ls -l $FTPDATA_NCEP_DIR/$filename | awk '{print $5}'`
    echo "ncep_size: $ncep_size aoml_size= $aoml_size"

    if [ $ncep_size -eq $aoml_size ]     # Are the sizes equal?
    then                                 #  Yes, transfer OK
      msg="Transfer complete (remote size $ncep_size = local size $aoml_size)\n"
      echo $msg
      addok=$addok"$filename ($aoml_size Kbytes)\n"
    else                                 #  No, incomplete transfer
      sizes_equal='n'
      echo $filename >> $failure_list
      msg="\nTransfer incomplete (remote size $ncep_size = local size $aoml_size)\n"
      echo $msg
      addfail=$addfail"$filename ($aoml_size Kbytes)\n"
    fi
  done

  if [ "$sizes_equal" = "n" ]           # Any file size not equal (remote vs local) 
  then                                  #   Yes, at least one file not completely transferred
    echo "\nFailure List (incomplete transfer): $failure_list"
    cat $failure_list
    comm -23 $remote_list $failure_list > $local_list	
    rm -f $remote_list $failure_list
  else                                  #   No, all file transferred completely
    echo "Move: $remote_list to $local_list"
    mv $remote_list $local_list
  fi

  echo "\nLOCAL_LIST (updated): $local_list"
  cat $local_list

  rm -f $remote_dir_listing
  rm -f $FTPTMP_DIR/entry

  echo "=> Exit CheckSizes"
  return
}
# --------------------------------------------------------------------
#  Revision Date: 29-Jun-01 (RS)
#
#  UnzipNewestFile_Move2Opr
#  ========================
#  Pick newest weekly analysis file, unpack it move it to
#  'clima_mask' directory.  If a weekly analysis file exists
#   in 'clima_mask' then it is moved to FTPDATA_NCEPOLD_DIR. 
# --------------------------------------------------------------------
UnzipNewestFile_Move2Opr()
{
   echo "=> Enter UnzipNewestFile_Move2Opr"
   savedir=`pwd`
   cd $FTPDATA_NCEP_DIR                           # Change to ftp/data/ncep
   echo "   Changed to $FTPDATA_NCEP_DIR"

   new_files_sorted="$FTPTMP_DIR/new_files_sorted"
   sort $new_files -o $new_files_sorted           # alphabetical sort

   zipfile=`tail -1 $new_files_sorted`            # pick most recept
   unzipfile=`echo $zipfile | awk -F"." '{print $1 "." $2}'`
   echo "   File to unpack: $FTPDATA_NCEP_DIR//$zipfile"
   echo "   Unpacked file:  $FTPDATA_NCEP_DIR/$unzipfile"

   echo "   Unpacking (gunzip) ... \c"            # unpack
   gunzip $zipfile
   status="$?"

   addsubj_unzip=""
   addtext_unzip=""
   if [ "$status" -eq "0" ]
   then
     echo "OK"
     ls $CLIMA_MASK_DIR/analysis_weekly.* > $FTPTMP_DIR/pathname  # old analysis pathname 
     status="$?"
     if [ $status -eq 0 ]                     # Any analysis file?
     then                                     #   Yes, move it
       pathname_old=`cat $FTPTMP_DIR/pathname`
       mv $pathname_old $FTPDATA_NCEPOLD_DIR
       status="$?"
       echo "   Old Analysis:  $pathname_old <mv to> $FTPDATA_NCEPOLD_DIR \c"
       if [ $status -eq 0 ]
       then 
         echo "-> OK"
       else
         echo "-> FAILED"
       fi
       rm -f $FTPTMP_DIR/pathname
     fi 

     yyyymmdd=`echo $unzipfile | awk -F"." '{print $1}' | sed 's/ocn//'`
     oprfile="analysis_weekly.$yyyymmdd"                         

#?   cp -p $FTPDATA_NCEP_DIR/$oprfile $FTPDATA_NCEPOLD_DIR       # copy to old-analysis
     cp -p $unzipfile $FTPDATA_NCEPOLD_DIR/$oprfile              # copy to old-analysis
     status="$?$"
#?   echo "   New Analysis: $FTPDATA_NCEP_DIR/$oprfile <cp to> $FTPDATA_NCEPOLD_DIR \c"
     echo "   New Analysis: $FTPDATA_NCEP_DIR/$unzipfile <cp to> $FTPDATA_NCEPOLD_DIR/$oprfile \c"
     if [ $status -eq 0 ]
     then 
       echo "-> OK"
     else
       echo "-> FAILED"
     fi

     mv $unzipfile $CLIMA_MASK_DIR/$oprfile    # move to clima_mask 
     status="$?"
     echo "   New Analysis: $FTPDATA_NCEP_DIR/$oprfile <mv to> CLIMA_MASK_DIR \c"
     if [ $status -eq 0 ]
     then 
       echo "-> OK"
     else
       echo "-> FAILED"
     fi

     addtext_unzip="\nNew weekly analysis file: $CLIMA_MASK_DIR/$oprfile"
   else
     echo "FAILED"
     addsubj_unzip="; an unpacking error occurred " 
     addtext_unzip="\nAn error occurred while unpacking $zipfile" 
   fi

# ---------------------------------------------------------------------------------
#      NCEP analysis filename format: ocnyyyymmdd.nc.Z (e.g. ocn20010603.nc.Z)
# If more than one file was pulled then changed filenames and
# move then to 'old_analysis' except the most current one which was
# processed above (unpacked). 
# ---------------------------------------------------------------------------------
   no_files=`wc -l $new_files_sorted | awk '{print $1}'` # number of file(s) pulled
   echo "  Number of files pulled = $no_files"
   if [ $no_files -gt 1 ]
   then
     count=0
     for filename in `cat $new_files_sorted`
     do
       count=`expr $count + 1`
       if [ $count -lt $no_files ]                 #  less than last file?
       then                                        #    yes, skip last file (current) 
         yyyymmdd=`echo $filename | awk -F"." '{print $1}' | sed 's/ocn//'`
         oprfile=analysis_weekly."$yyyymmdd".gz    
         mv $filename $FTPDATA_NCEPOLD_DIR/$oprfile
         echo "  File: $filename <mv to> $FTPDATA_NCEPOLD_DIR/$oprfile"
       fi
     done
   fi

   rm $new_files
   cd $savedir
   echo "   Changed to $savedir"
   echo "=> Exit UnzipNewestFile_Move2Opr"
   return
}
# ==========================  Set Variables ===============================
FTP_DIR=$ARGO_DIR/ftp
FTPSCRIPT_DIR=$FTP_DIR/scripts
FTPLOG_DIR=$FTP_DIR/log
FTPTMP_DIR=$FTP_DIR/tmp
FTPDATA_DIR=$FTP_DIR/data
FTPDATA_NCEP_DIR=$FTPDATA_DIR/ncep                  # ARGO_DIR/ftp/data 
# old FTPDATA_NCEPOLD_DIR=$FTPDATA_DIR/ncep/old_analysis
FTPDATA_NCEPOLD_DIR=$ARCHIVE_DIR

# old FTPNCEP_HOST='ftp.ncep.noaa.gov'
# old FTPNCEP_DATA_DIR='pub/cmb/Global'

#rs:02jun06 FTPNCEP_HOST='ftpprd.ncep.noaa.gov'
#rs:02jun06 FTPNCEP_DATA_DIR='pub/cmb/Products/G3R2TSo'
FTPNCEP_HOST='ftp.emc.ncep.noaa.gov'
FTPNCEP_DATA_DIR='cmb/Products/G3R2TSo'

#dbg FTPNCEP_HOST='db.aoml.noaa.gov'
#dbg FTPNCEP_DATA_DIR='JUNK'

CLIMA_MASK_DIR=$ARGO_DIR/qc/clima_mask

remote_list=$FTPDATA_NCEP_DIR/remote_list
local_list=$FTPDATA_NCEP_DIR/local_list
# ==========================  Main script  ===================================
TODAY=`date +'%Y.%m.%d_%H.%M'`

script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}'`
up_root_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

type=bin
user=anonymous
pw=sabina

remote_dir_listing=$FTPTMP_DIR/remote.$TODAY"_"dir
trace=$FTPLOG_DIR/$root_script_name.$TODAY"_"trace

now=`date +'%Y.%m.%d_%H.%M'`

echo "<===================== Start  $script_name: $now ===================>"
echo "<===================== Start  $script_name: $now ===================>" >> $trace

ExitCodeTable                   # Load Exit Status Code Table

error=$FTPLOG_DIR/$root_script_name.$TODAY"_"error
exec 2>> $error                 # append stderr to error
touch $error                    # it will exists

mail_message=$FTPTMP_DIR/$root_script_name.$TODAY"_"mail

# ---------------------------------------------------------------------------
# Test to see if FTPNCEP_HOST is alive 
# ---------------------------------------------------------------------------
able2connect_log=$FTPLOG_DIR/ftpncep_able2connect.$TODAY"_"log
touch $able2connect_log
export able2connect_log

$FTPSCRIPT_DIR/able2connect.pl $FTPNCEP_HOST
status="$?"
if [ $status = "0" ]
then
  text="Connection to $FTPNCEP_HOST is OK" 
  echo $text
  echo $text >> $trace
else
  ret_val=1
  subj="$up_root_script_name - Connection to NCEP not AVAILABLE"
  text="Connection to NCEP ($FTPNCEP_HOST) is not AVAILABLE.\n"
  text=$text"See logfiles:\n"
  text=$text"\n\t$logfile"
  text=$text"\n\t$able2connect_log"

  if [ -s $error ]
  then
    text=$text"\n\t$error"
  fi

  echo $text > $mail_message

  echo "Email Subj: $subj"
  echo "To: $EMAIL_OPERATOR"
  echo "Text:"
  cat  $mail_message

  echo "Sending Email ...\c"
  /usr/bin/mailx -s "$subj" $EMAIL_OPERATOR <  $mail_message
  echo "OK"

  rm -f $trace $error $mail_message
  now=`date +'%Y.%m.%d_%H.%M'`
  MatchExitCode $ret_val
  DelExitCodeTable
  echo "<-------------- Stop:  $script_name $now -------------->" 
  exit $ret_val                # Exit=1, no connection available
fi

# ---------------------------------------------------------------------------
# FTP to get remote directory listing from FTPNCEP_DATA_DIR 
# ---------------------------------------------------------------------------
msg="Start FTP - to obtain remote $FTPNCEP_DATA_DIR list"
echo $msg
echo $msg >> $trace

ftp -n -i<< endftp >> $trace    # pull remote dir list 
  open $FTPNCEP_HOST
  user $user $pw
  cd   $FTPNCEP_DATA_DIR
  dir -l "| grep ocn | grep nc | grep gz > $remote_dir_listing"
endftp
status="$?"
if [ "$status" -ne "0" ]            # Is it OK (get remote directory)?
then                                #   No, send Email & exit
  ret_val=2
  subj="$up_root_script_name - FTP Failure occurred when getting the "
  subj=$subj"NCEP directory listing"
  text="FTP Failure occurred while trying to get the NCEP\n"
  text=$text"directory listing.'"
  echo $text >> $trace

  text=$text"\nSee logfiles:\n"
  text=$text"\n\t$logfile"
  text=$text"\n\t$trace"

  if [ -s $error ]
  then
   text=$text"\n\t$error"
  else
    rm -f $error
  fi

  echo $text > $mail_message

  echo "Email Subj: $subj"
  echo "To: $EMAIL_OPERATOR"
  echo "Text:"
  cat $mail_message

  echo "Sending Email ...\c"
  /usr/bin/mailx -s "$subj" $EMAIL_OPERATOR < $mail_message
  echo "OK"

  rm -f $mail_message
  if [ -f $remote_dir_listing ]
  then
    rm -f $remote_dir_listing
  fi
  now=`date +'%Y.%m.%d_%H.%M'`
  MatchExitCode $ret_val
  DelExitCodeTable
  echo "<-------------- Stop:  $script_name $now -------------->" 
  exit $ret_val                # exit=2, FTP failed while checking for the existance 
fi                             #      of GTSGATEWAY_DATAFILE

# ----------------------------------------------------------------
# Check for standart error(s) 
# ----------------------------------------------------------------
 msg="End FTP: $FTPNCEP_DATA_DIR directory listing copied to:\n"
 msg=$msg"\t$remote_dir_listing"
 echo $msg
 echo $msg >> $trace

 if [ -s $error ]              #   Is error file not empty?
 then                          #     Yes, Email & Exit
   ret_val=2
   subj="$up_root_script_name - Standard error when getting remote directory"
   text="Standard-Output-Error (stderror) occured while getting remote directory\n"
   text=$text"occurred.\n"

   text=$text"See logfiles:"
   text=$text"\n\t$logfile"
   text=$text"\n\t$trace"
   text=$text"\n\t$error"

   echo $text > $mail_message
   echo "Email Subj: $subj"
   echo "To: $EMAIL_OPERATOR"
   echo "Text:"
   cat $mail_message

   echo "Sending Email ...\c"
   /usr/bin/mailx -s "$subj" $EMAIL_OPERATOR < $mail_message
   echo "OK"

   rm -f $mail_message
   now=`date +'%Y.%m.%d_%H.%M'`
   MatchExitCode $ret_val
   DelExitCodeTable

   echo "<-------------- Stop:  $script_name $now -------------->" 
   exit $ret_val                # Exit=2, std-error occurred 
 fi
# ----------------------------------------------------------------
# Is remote directory listing empty? 
# ----------------------------------------------------------------
 if [ ! -s $remote_dir_listing ]     # Is it empty?
 then                                #   Yes, send email & exit
   ret_val=3

   grep 'Login failed' $trace > /dev/null
   status=$?
   grep 'Lost connection' $trace > /dev/null
   status=`expr $status \* $?`
   grep 'Not connected' $trace > /dev/null
   status=`expr $status \* $?`

   if [ $status -eq 0 ]
   then
     subj="$up_root_script_name - Error when getting remote directory"
     text="Login failed, connection lost, or not connected\nwhile trying to get NCEP directory listing\n"
   else
      subj="$up_root_script_name - NCEP Directory List is Empty"
      text="NCEP directory List is empty ($remote_dir_listing).\n"
   fi

   text=$text"See logfiles:"
   text=$text"\n\t$logfile"
   text=$text"\n\t$trace"

   echo $text > $mail_message
   echo "Email Subj: $subj"
   echo "To: $EMAIL_OPERATOR"
   echo "Text:"
   cat $mail_message                                                    

   echo "Sending Email ...\c"
   /usr/bin/mailx -s "$subj" $EMAIL_OPERATOR < $mail_message
   echo "OK"

   rm -f $mail_message
   now=`date +'%Y.%m.%d_%H.%M'`
   MatchExitCode $ret_val
   DelExitCodeTable

   echo "<-------------- Stop:  $script_name $now -------------->"
   exit $ret_val                # Exit=3, FTP failed whit getting NCEP dir
  fi                                                                 

# ----------------------------------------------------------------
# Are there any new file(s) to FTP from NCEP? 
# ----------------------------------------------------------------
if [ -f $remote_list ]
then
  rm -f $remote_list
fi

echo "\nREMOTE_DIR_LIST: $remote_dir_listing"
cat $remote_dir_listing

cat $remote_dir_listing |                       # list with only filenames 
  while read priv node owner group size month day time filename 
  do
    echo $filename >> $remote_list
  done

echo "\nREMOTE_LIST: $remote_list"
cat $remote_list

# ----------------------------------------------------------------
# Obtain list of new file(s) to FTP
# ----------------------------------------------------------------
new_files=$FTPTMP_DIR/new_files
if [ -f $local_list -a  -s $local_list ]          # Does local list exists and not empty?
then                                              #  Yes, get new files
  echo "\nLOCAL_LIST is NOT EMPTY: $local_list"
  comm -23 $remote_list $local_list > $new_files  
else                                              #  No, remote to local_list & new_files 
  echo "\nLOCAL_LIST is EMPTY => copy REMOTE_LIST to LOCAL_LIST & NEW_FILES"
  cp -p $remote_list $local_list
  cp -p $remote_list $new_files
fi

echo "\nLOCAL_LIST: $local_list"
cat $local_list

if [ ! -s $new_files ]                          # Is list empty?
then                                            #   Yes, send email and exit
  ret_val=4
  subj="$up_root_script_name - No new file"
  text="No new file to transfer from NCEP"
  echo $text > $mail_message

  echo "Email Subj: $subj"
  echo "To: $EMAIL_OPERATOR"
  echo "Text: $text"

  printf "Sending Email ... "
  /usr/bin/mailx -s "$subj" $EMAIL_OPERATOR < $mail_message
  echo "OK"
  rm -f $mail_message
  rm -f $remote_dir_listing $new_files $error
  now=`date +'%Y.%m.%d_%H.%M'`
  MatchExitCode $ret_val
  DelExitCodeTable
  echo "<-------------- Stop:  $script_name $now -------------->"
  exit $ret_val                           # Exit=4, no file to ftp
fi

echo "\nFiles to be tranferred from NCEP:"
cat $new_files

# ----------------------------------------------------------------
# FTP data file(s) from $FTPNCEP_HOST
# ----------------------------------------------------------------
ftpinfo=$FTPTMP_DIR/ftptmp 
msg="Start FTP session to transfer data file from $FTPNCEP_DATA_DIR"
echo $msg
echo $msg >> $trace

ftpcmd_file=$FTPTMP_DIR/ftpcmd_file.$TODAY
echo "open $FTPNCEP_HOST"      > $ftpcmd_file
echo "user $user $pw"         >> $ftpcmd_file
echo "$type"                  >> $ftpcmd_file
echo "cd  $FTPNCEP_DATA_DIR"  >> $ftpcmd_file
echo "lcd $FTPDATA_NCEP_DIR"  >> $ftpcmd_file 

for filename in `cat $new_files`
do
  echo "get $filename" >> $ftpcmd_file
done

echo
echo "FTP Session commands File:"
cat $ftpcmd_file

ftp -n -v < $ftpcmd_file > $ftpinfo        # FTP - pull analysis file(s) 
 
status="$?"

if [ "$status" = "0" ]          # Is FTP session OK?
then                            #   Yes
  msg="FTP Session Returned Status is OK." 
  msg=$msg"  Now, check for FTP transfer completion"
  echo $msg
  echo $msg >> $trace

  CheckSizes                    # Check file size (remote vs local)
  
  if [ $sizes_equal = "y" ]     # Is transfer complete?
  then                          #     Yes, complete - SUCCESS  
    ret_val=0
    subj="$up_root_script_name - File(s) transferred completly"
    text="FTP file(s) transferred completely:\n"
    text=$text$addok

    UnzipNewestFile_Move2Opr    # Unzip newest weekly analysis file
                                # move to 'clima_mask' directory 

    add="+++++++++++++++++++++ FTP Session Output +++++++++++++++++++++"
    echo $add
    cat $ftpinfo
    add="+++++++++++++++++++ FTP Session Output End +++++++++++++++++++\n"
    echo $add

    if [ "$addsubj_unzip" != "" ]  # Any error while unpacking?
    then                           #   Yes, expand email subject & text, and 
      ret_val=7                    #   set error return code
      subj=$subj$addsubj_unzip
    fi

    text=$text$addtext_unzip  
    echo $text > $mail_message
    echo
    echo "Email Subj: $subj"
    echo "To: $EMAIL_OPERATOR"
    echo "Text:"
    cat $mail_message

    echo "Sending Email ...\c"
    /usr/bin/mailx -s "$subj" $EMAIL_OPERATOR < $mail_message
    echo "OK"

    rm -f $ftpinfo $error $trace

    now=`date +'%Y.%m.%d_%H.%M'`
    MatchExitCode $ret_val
    DelExitCodeTable
    echo "<===================== Finish $script_name: $now ===================>"
    echo "<===================== Finish $script_name: $now ===================>" >> $trace
    exit $ret_val                # Exit=0, FTP data file transfer OK 
  else                           #     Not complete
    ret_val=5
    subj="$up_root_script_name - At least one file transfer was NOT complete"
    text="File(s) NOT completely transferred:\n"
    text=$text$addfail
    text=$text"\nFile(s) completely transferred:\n"
    text=$text"$addok"
   
    add="See logfiles:"
    add=$add"\n\t$logfile"
    add=$add"\n\t$trace"
    add=$add"\n\t$ftpinfo"

    if [ -s $error ]
    then
      add=$add"\n\t$error" 
    else
      rm -f $error
    fi 

    text=$text$add  

    UnzipNewestFile_Move2Opr       # Unzip newest weekly analysis file 
                                   # and move to 'clima_mask' directory

    if [ "$addsubj_unzip" != "" ]  # Any error while unpacking?
    then                           #   Yes, expand email subject & text, and 
      ret_val=7                    #   set error return code
      subj=$subj$addsubj_unzip
    fi
    text=$text$addtext_unzip 

    echo $text > $mail_message
    echo
    echo "Email Subj: $subj"
    echo "To: $EMAIL_OPERATOR"
    echo "Text:"
    cat $mail_message

    echo "Sending Email ... \c"
    /usr/bin/mailx -s "$subj" $EMAIL_OPERATOR < $mail_message
    echo "OK"

    rm -f $ftpinfo $mail_message
    now=`date +'%Y.%m.%d_%H.%M'`
    MatchExitCode $ret_val
    DelExitCodeTable
    echo "<===================== Finish $script_name: $now ===================>"
    echo "<===================== Finish $script_name: $now ===================>" >> $trace
    exit $ret_val                # Exit=5, FTP data file transfer not complete
  fi  # endif: is transfer complete?
else                                      # No, FTP session failed
  ret_val=6
  subj="$up_root_script_name - FTP Session to transfer file from NCEP failed"
  text="FTP Session to transfer file from NCEP ($FTPNCEP_HOST) failed."

  echo $text     > $mail_message
  cat  $ftpinfo >> $mail_message

  add=$add"\nSee logfiles:\n"
  add=$add"\n\t$logfile"
  add=$add"\n\t$trace" 

  if [ -s $error ]
  then
    add=$add"\n\t$error"
  else
    rm -f $error
  fi

  echo $add >> $mail_message
  cat $mail_message >> $trace
  cat $mail_message >> $error

  echo "\nEmail Subj: $subj"
  echo "To: $EMAIL_OPERATOR"
  echo "Text:"
  cat $mail_message

  echo "\nSending Email ...\c"
  /usr/bin/mailx -s "$subj" $EMAIL_OPERATOR < $mail_message
  echo "OK"

  rm -f $mail_message $ftpinfo
  if [ -f $seas_dir_listing ]
  then
    rm -f $seas_dir_listing
  fi
  now=`date +'%Y.%m.%d_%H.%M'`
  MatchExitCode $ret_val
  DelExitCodeTable
  echo "<-------------- Stop:  $script_name $now -------------->" 
  exit $ret_val               # Exit=6, FTP file transfer failed 
fi  # endif: is FTP session OK? 
