#!/bin/sh
# =====================================================================
# Revision Date: 24-Jun-05 (RS - change ls -l to dir -l in ftp session)
#                09-Jul-03 (RS - dir -> ls in ftp session)
#                30-Apr-02 (RS - add prefix to processexit_table)
#                01-Oct-01 (RS - change argo.init for new FTP site)  
#                18-Apr-01 (CS) changed preparation of email text
#                12-Dec-00 (RS)
#
# ftpqc_to_localpub.sh
# --------------------
# Transfer QC profiles to Local Host designated location for FTP public
# Internet access.
#
# Tasks involved are: 
#
# 1) Creates a list of the symbolic linked file(s) to be transferred
# from 'FTP_TO_LOCAL_DIR' to the local FTP host area (FTP_LOCALQC_DIR).
#  
# 2) Establishes a FTP connection to local FTP host (FTP_LOCALHOST).
#
# 3) Transfers file(s) in the FTP_TO_LOCAL (./ftp/data/qcpack/to_local
# directory, where . => ARGO root)
#
# 4) Determines file transfer completion by comparing profiles sizes
# in remote-directory and local-directory.
#
# Exit status values
# ------------------
# 0, success
# 1, No QC compressed file to be transferred
# 2, No Connection available to FTP_LOCALHOST server 
# 3, FTP error occurred while transferring compressed file(s)
# 4, Incomplete transfer for some file(s) in FTP_TO_LOCAL_DIR
#
# Note - FTP file transfer type is set to 'binary'
#        FTP_LOCALHOST = ftp.aoml.noaa.gov 
# =======================================================================
# --------------------------------------------------------------------
#  Revision Date: 14-Dec-00 (RS)
#
#  ExitCodeTable
#  =============
#  Creates a table with exit code, reason and action for each process
#
#  Usage: ExitCodeTable
# --------------------------------------------------------------------
ExitCodeTable()
{
  processexit_table=$QCTMP_DIR/processexit_ftppackqc_to_publocal.$TODAY"_"table
  cat > $processexit_table << EOF
Code              Reason                                               Action
 0  /Process finished successfully                             /Return to ARGO
 1  /No QC compressed file to be transferred                   /Return to ARGO, see log-files
 2  /No connection available to FTP_LOCALHOST server           /Return to ARGO, see log-files
 3  /FTP error occurred while transferring compressed file(s)  /Return to ARGO, see log-files
 4  /Incomplete transfer for some file(s) in FTP_TO_LOCAL_DIR  /Continue with process, see log-files
EOF
 return
}
# --------------------------------------------------------------------
#  Revision Date: 29-Nov-00 (RS)
#
#  DelExitCodeTable
#  =================
#  Deletes 'Exit Code Table'
#
#  Usage: DelExitCodeTable
# --------------------------------------------------------------------
DelExitCodeTable()
{
   rm -f $processexit_table
   return
}
# --------------------------------------------------------------------
#  Revision Date: 29-Nov-00 (RS)
#
#  MatchExitCode
#  =============
#  Looks up the exit code. If code is matched, it prints the reason
#  and action else prints a no match message.
#
#  Usage: MatchExitCode code
# --------------------------------------------------------------------
MatchExitCode()
{
  code=$1
  grep "$code " $processexit_table > /dev/null     # 0, match; !- 0 otherwise
  if [ "$?" = "0" ]
  then
    reason=`grep "$code " $processexit_table | cut -d"/"  -f2`
    action=`grep "$code " $processexit_table | cut -d"/"  -f3`
    echo "\tExitCode: $code => $reason"
    echo "\tAction: $action"
  else
   echo "Exit Code not found in: $processexit_table"
  fi
  return
}
# ------------------------------------------------------------------------
# FTPCheckCompletion
# ==================
# Determines the outcome of the file transfer by comparing entries
# in 'local_namesize_table' against 'remote_namesize_table'. It uses 
# the outcome of each FTP file transfer to:
#  . Insert in list for either: complete, incomplete or not-transfer files.
#  . Remove file symbolic link, if FTP transfer is complete. 
#
# Usage: unix> FTPCheckCompletion
#
# ------------------------------------------------------------------------
FTPCheckCompletion()
{
   echo "\nEntering Function ... FTPCheckCompletion"

   exec < $local_namesize_table
   while read local_profile local_size 
   do
     grep $local_profile $remote_namesize_table > /dev/null
     if [ "$?" = "0" ]                               # Was it transferred?
     then                                            #   Yes, check its size
       remote_size=`grep $local_profile $remote_namesize_table | awk '{print $2}'`
       if [  $local_size = $remote_size ]            #   Are sizes equal?
       then
         echo $local_profile >> $complete_transfer_list
         rm $FTP_TO_LOCAL_DIR/$local_profile
         echo "Remove FTPed profile: $FTP_TO_LOCAL_DIR/$local_profile"
       else
         echo $local_profile >> $incomplete_transfer_list
       fi
     else
       echo $local_profile >> $not_transfer_list
     fi
   done
   return 
}
# ==========================  Main Script  ===================================
script_name=`basename $0`
root_script_name=`echo $script_name| awk -F"." '{print $1}`
up_prgname=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

now=`date +'%h %d, %Y %T %p'`
echo "<===================== Start  $script_name: $now ===================>"

ExitCodeTable

error=$FTPLOG_DIR/$root_script_name.$TODAY"_"error
exec 2> $error

trace=$FTPLOG_DIR/$root_script_name.$TODAY"_"trace
able2connect_log=$FTPLOG_DIR/able2connect.$TODAY"_"log
email_msg=$FTPLOG_DIR/$root_script_name.$TODAY"_"email

touch $trace
touch $able2connect_log
export able2connect_log

# -------------------------------------------------------------------
#      Create list of compressed file(s) to be transferred
#               (all files in FTP_TO_LOCAL_DIR) 
# -------------------------------------------------------------------
filter="qc_"
ftp_compressed_list=$FTPTMP_DIR/ftp_compressed_list.$TODAY
ls $FTP_TO_LOCAL_DIR/$filter* > $ftp_compressed_list 
echo "QC Compressed file(s) to be transferred:"
cat $ftp_compressed_list

count=`wc -l $ftp_compressed_list | awk '{print $1}'`
echo "Number of Files to FTP = $count"

if [ "$count" = "0" ]
then
  ret_val=1
  subj="$up_prgname: NO QC compressed file to be transferred"
  text="No QC compressed file to be transferred to local FTP server for\n"
  text=$text"Internet access, see log-files:\n"
  text=$text"\n\t$log_file"
  text=$text"\n\t$trace"

  if [ -s $error ]
  then
    text=$text"\n\t$error"
  else
    rm  $error
  fi
  echo $text > $email_msg

  echo "Email Subj: $subj"
  echo "To: $EMAIL_OPERATOR"
  echo "Text:"
  cat $email_msg

  echo "Sending Email ...\c"
  /usr/bin/mailx -s "$subj" $EMAIL_OPERATOR < $email_msg
  echo "OK"

  MatchExitCode $ret_val
  DelExitCodeTable

  now=`date +'%h %d, %Y %T %p'`
  echo "<===================== Finish $script_name: $now ===================>"
  exit $ret_val 
fi

# -------------------------------------------------------------------
#  Check FTP connection to the local host (FTP_LOCALHOST)
# -------------------------------------------------------------------
echo "FTP_LOCALHOST: $FTP_LOCALHOST"
echo "FTP_LOCALQC_DIR: $FTP_LOCALQC_DIR"

echo "FTP_LOCALHOST: $FTP_LOCALHOST"     >> $trace
echo "FTP_LOCALQC_DIR: $FTP_LOCALQC_DIR" >> $trace

$FTPSCRIPT_DIR/able2connect.pl  $FTP_LOCALHOST
status="$?"
if [ "$status" = "0" ]          # Is connection to FTP Server available?
then                            #   Yes, coninue
  text="Connection to $FTP_LOCALHOST is OK"
  echo $text
  echo $text >> $trace
  rm $able2connect_log 
else                            #   No 
  ret_val=2 
  subj="$up_prgname: NO Connection available to $FTP_LOCALHOST"
  text="NO Connection available to $FTP_LOCALHOST."
  text=$text"  For more details see:\n"
  text=$text"\n\t$log_file"
  text=$text"\n\t$able2connect_log"
  echo $text> $email_msg

  echo "Email Subj: $subj"
  echo "To: $EMAIL_OPERATOR"
  echo "Text:"
  cat $email_msg

  echo "Sending Email ...\c"
  /usr/bin/mailx -s "$subj" $EMAIL_OPERATOR < $email_msg
  echo "OK"

  MatchExitCode $ret_val
  DelExitCodeTable

  now=`date +'%h %d, %Y %T %p'`
  echo "<===================== Finish $script_name: $now ===================>"
  exit $ret_val
fi  # endif: able to connect

# -------------------------------------------------------------------
#  FTP QC compressed file(s) to the local FTP area for public 
#  access via the Internet
# -------------------------------------------------------------------
ftp_stdout=$FTPLOG_DIR/ftp_stdout.$TODAY
echo "ftp_stdout: $ftp_stdout"

type=bin
toggle='prompt off'

complete_transfer_list=$FTPTMP_DIR/complete_transfer_list.$TODAY
incomplete_transfer_list=$FTPTMP_DIR/incomplete_transfer_list.$TODAY
not_transfer_list=$FTPTMP_DIR/not_transfer_list.$TODAY

touch $complete_transfer_list
touch $incomplete_transfer_list
touch $not_transfer_list

echo "complete_transfer_list:   $complete_transfer_list"
echo "incomplete_transfer_list: $incomplete_transfer_list"
echo "not_transfer_list: $not_transfer_list"

echo "\nCompressed file(s) to be put in the FTP local directory:"
cat $ftp_compressed_list

remote_list=$FTPTMP_DIR/remote_list

text="BEGIN ... FTP Session - put compressed file(s)\n"
text=$text"\t  from: $FTP_TO_LOCAL_DIR\n"
text=$text"\t  to:   $FTP_LOCALQC_DIR"
echo $text

echo "ls -lL of  FTP_TO_LOCAL_DIR: $FTP_TO_LOCAL_DIR"
ls -lL $FTP_TO_LOCAL_DIR/$filter*

ftp -n -v << endftp  > $ftp_stdout
  open $FTP_LOCALHOST
  user $FTPLOCAL_USER $FTPLOCAL_PW
  $type
  $toggle
  cd  $FTP_LOCALQC_DIR
  pwd
  lcd $FTP_TO_LOCAL_DIR
  mput $filter* 
  dir -l "| grep qc_ | grep tar.gz > $remote_list"
endftp

echo "END   ... FTP Session"
echo "ftp_stdout: $ftp_stdout"
cat $ftp_stdout

# -------------------------------------------------------------------
# Check for FTP error such as:  incomplete transfer, transfer aborted
# and time out  
# -------------------------------------------------------------------
ret_val=0
pattern="incomplete|Timeout|aborted"
egrep -i "$pattern" $ftp_stdout > /dev/null
status="$?"
echo "\nFTP Error Status (egrep - 0, error; > 0, OK): $status"
if [ $status = "0" ]             # Did an error occur? 
then                              #   Yes
  ret_val=3
  subj="$up_prgname: FTP Error - No file transfer occurred"  
  text="FTP error occurred while transferring compressed file(s), "
  text=$text"see log-files:\n"
  text=$text"\t$log_file\n\t$ftp_stdout\n\t$trace" 

  if [ -s $error ]
  then
    text=$text"\n\t$error"
  else
    rm $error
  fi 

  echo $text > $email_msg

  echo "Email Subj: $subj"
  echo "To: $EMAIL_OPERATOR"
  echo "Text:"
  cat  $email_msg
  
  echo "Sending Email ...\c"
  /usr/bin/mailx -s "$subj" $EMAIL_OPERATOR < $email_msg
  echo "OK"

  rm $complete_transfer_list $incomplete_transfer_list $not_transfer_list
  rm $error

  if [ -f $remote_list ]
  then
    rm $remote_list
  fi

  MatchExitCode $ret_val
  DelExitCodeTable

  now=`date +'%h %d, %Y %T %p'`
  echo "<====================Terminated $script_name: $now =================>"
  exit $ret_val
fi  # endif: FTP error 

# -------------------------------------------------------------------
# Create table (local_namesize_table) with size and profile name from 
# QC data directory (QCFILES_DIR) for transferred profile.
# -------------------------------------------------------------------
local_namesize_table=$FTPTMP_DIR/local_namesize_table
if [ -f $local_namesize_table ]
then
  rm $local_namesize_table
fi

wpwd=`pwd`
cd $FTP_TO_LOCAL_DIR

echo "Directory $FTP_TO_LOCAL_DIR:"
ls -lL $FTP_TO_LOCAL_DIR

for pathname in `cat $ftp_compressed_list`
do
  profile=`basename $pathname`
  ls -lL $profile  | awk '{print $9 " " $5}' >>  $local_namesize_table
done
cd $wpwd

sort -i $local_namesize_table -o $local_namesize_table
echo "\nLocal Name & Size Table - compressed file()s to transfer"
echo "Location: $local_namesize_table"
cat  $local_namesize_table

# -------------------------------------------------------------------
# Create table (remote_namesize_table) from the remote directory 
# listing obtained via FTP.  
# -------------------------------------------------------------------
echo "\nCompressed file(s) transferred to the remote FTP area:"
cat $remote_list
remote_namesize_table=$FTPTMP_DIR/remote_namesize_table
cat  $remote_list | grep $filter | awk '{print $9 " " $5}' > $remote_namesize_table
sort -i $remote_namesize_table -o $remote_namesize_table
echo "\nRemote Name & Size Table (profiles transferred)"
echo "Location: $remote_namesize_table"
cat $remote_namesize_table
rm $remote_list

# -------------------------------------------------------------------
#                Check FTP File Transfer Status
#           (complete, incomplete or not-transferred) 
#          Remove symbolic link if transfer is complete
# -------------------------------------------------------------------
FTPCheckCompletion 

n_complete=`wc -l $complete_transfer_list | awk '{print $1}'`
n_incomplete=`wc -l $incomplete_transfer_list | awk '{print $1}'`
n_nottransfer=`wc -l $not_transfer_list | awk '{print $1}'` 
n_failures=`expr $n_incomplete + $n_nottransfer`

echo "\n"
echo "Number of complete transfered files: $n_complete" 
echo "Number of incomplete transfered files: $n_incomplete" 
echo "Number of files not transfered: $n_nottransfer" 
echo "Number of failures (incomplete + not): $n_failures"

# Display transfer list (complete, incomplete & not transfer)
# ----------------------------------------------------------- 
if [ $n_complete -gt 0 ]
then
  echo "\nComplete transferred file(s):"
  cat $complete_transfer_list
fi

if [ $n_incomplete -gt 0 ]
then
  echo "\nIncomplete transferred file(s):"
  cat $incomplete_transfer_list
fi

if [ $n_nottransfer -gt 0 ]
then
  echo "\nNot transferred file(s):"
  cat $not_transfer_list
fi

# -------------------------------------------------------------------
#                        Prepare email text
# -------------------------------------------------------------------
if [ $n_failures -eq 0 ]       # Any failure? 
then                             #  No, all files transferred OK
  text="\nTransfer Complete for files in $FTP_TO_LOCAL_DIR\n"
  echo $text
  echo $text >> $trace
  rm $ftp_stdout
else                             #  Yes, either not trasferred or incomplete
  ret_val=4
  text="\nTransfer Incomplete for the file(s) in $FTP_TO_LOCAL_DIR:\n"
  echo $text                 
  echo $text                    >> $trace

  cat $incomplete_transfer_list 
  cat $incomplete_transfer_list >> $trace 
  echo $text >> $trace
fi

echo "Returned Value (ret_val): $ret_val"
echo "Returned Value (ret_val): $ret_val" >> $trace

# --------------------------------------------------------------------------
#    Check file transfers by checking updated 
#                    prepare Email
# --------------------------------------------------------------------------
case $ret_val in
  0) subj="$up_prgname: Transfer OK - all compressed file(s) transferred" 
     text="All compressed file(s) were successfully transferred.\n"
     echo $text >> $trace;; 

  4) subj="$up_prgname: Transfer Error"
     text="The following compressed file(s) were not"
     text=$text" successfully transferred:\n"

     if [ $n_incomplete -gt 0 ]
     then
       for profile in `cat $incomplete_transfer_list`
       do
         text="$text\n\n\t"`echo $profile`
       done
     fi
     if [ $n_nottransfer -gt 0 ]
     then
       for profile in `cat $not_transfer_list`
       do
         text="$text\n\t"`echo $profile`
       done
     fi

     text=$text"\nSee log-files:\n"
     text=$text"\n\t$log_file"
     text=$text"\n\t$ftp_stdout"
     text=$text"\n\t$trace"
     text=$text"\n\t$error"

     echo $text >> $trace;;
esac

if [ -s $error ]      # Is error not empty?
then                                #   Yes, cat to text
  text=$text"\n\nUnix standard-error (stderr) detected, see log-file:\n"
  text=$text"\n\t$error"
else
  rm $error
fi

rm $complete_transfer_list $incomplete_transfer_list  $not_transfer_list

# ------------------------------------------------------------------------
#                              Send Email
# ------------------------------------------------------------------------
echo $text > $email_msg

echo "Email Subj: $subj"
echo "To: $EMAIL_OPERATOR"
echo "Text:"
cat $email_msg

echo "Sending Email ... \c"
/usr/bin/mailx -s "$subj" $EMAIL_OPERATOR < $email_msg
echo " OK"

MatchExitCode $ret_val
DelExitCodeTable

now=`date +'%h %d, %Y %T %p'`
if [ $ret_val -ne 0 ]
then
  echo "<====================Terminated $script_name: $now =================>"
else
  echo "<===================== Finish $script_name: $now ===================>"
fi

exit  $ret_val
