#!/bin/sh
###########################################################################################
# Revision Date: 13-Aug-12 (RS - PATH_NCDUMP defined in HOME/.argorc_sh)
#                16-Jan-12 (RS - check for dangling links in NCLINKS_DIR)
#                04-Jan-12 (RS - function 'RemoveOldRnc', remove from NCLINKS_DIR the
#                                R*.nc files older than a given number of days)
#                04-Nov-10 (RS - skip links with entry 'send to GTS' set to n in metafile) 
#                NAVO 09-Oct-10 (RS - path to 'ncdump' not in 'cron'. Implicit path introduced) 
#                19-Aug-10 (RS - create table with informaion of  encoded files.   
#                                Table will be used to compute GTS transmission statistics) 
#                15-Nov-09 (RS,CC)
#
# nc_to_gtsbuffer.sh 
# ==================
# 1) Encodes 'R*.nc' files located in DATA_NCLINKS_DIR into BURF format. It creates
#    a list (pathname_ncencoded) of the 'R*.nc' files that were successfully encoded.
# 2) Creates a working table (nc2gtsbufr_encoded_info_{timestamp}_table) with spatial 
#    and temporal information of the encoded profiles. The working table  will be appended
#    to table (GTS_BUFR4FTP_TABLE) with the follwing fields:
#    . Name of the output buffer file
#    . Name of the encoded real-time NetCDF file
#    . WMO number
#    . Latitude
#    . Longitude
#    . Date  (yyyy-mm-dd)
#    . Time  (hh:mm)
# 3) Packs buffer output file(s) into a file (IOPX02_KWBC_{timestamp}.bufr) to
#    be pushed to NWS Gateway. 	
# 4) Remove dangling link from 'NC_LINKS_DIR' when the R*.nc file is missing
#    in ARGO_DIR/data/nc
# 
# Usage
# -----
#         unix>nc_to_gtsbuffer.sh
#
# Java Program
# ------------
# The java encoder was provided by Anh Tran (MEDS, Canada) and adapted to the
# AOML environment by Chao Chen
#
# CLASSPATH=$LIB_DIR/WriteArgoBufr_fat.jar:$LIB_DIR/netcdfAll-4.0.jar:$CLASSPATH
#
# java -classpath $CLASSPATH dfo.isdm.WriteArgoBufr.WriteArgoBufr 
#                 $PRG_DIR/input.properties 
#                 $pathname_bufr $pathname_ncencoded 
# 
# Parameters:
# 1) input.properties: pathname to a file that defines the following Java properties:
#    ArgoBufr.inputFolder - location of the input 'nc' files 
#    ArgoBufr.Template - pathname of the Argo buffer template approved by WMO (argotemplate_v1.txt)
#    ArgoBufr.gtsHeader - pathname of GTS header identifiers file. It consists of two records: 
#                         .  product identifier (IOPX02, Argo)  
#                         .  sender identifier (KWBC, AOML/SEAS)indentifiers (gtsheader.txt)
# 2) pathname_bufr: string (DATA_BUFR_DIR/IOPX02_KWBC_{timestamp}.bufr) to which a number starting 
#                   with '01' will be appended to indicate the pathname of the first output BUFR file 
#                   When the total size of a new encoded BUFR  file is more than 15000, a new file  
#                   is created (e.g. DATA_BUFR_DIR/IOPX02_KWBC.{timestamp}.bufr02).
# 3) pathname_ncencoded: location of a file that contains a list of the 'R*.nc' files 
#                        that were successfully encoded
#
# Functions Used
# --------------
# CreateTable -> WriteRecord
# PackBufrForFtp
# 
# Exits Traps 
# -----------
# NC files not found in DATA_NCLINKS_DIR
# Java Exit Status is not zero
# Possible standar-error(s) 
#
# Definitions
# -----------
# GTS_DIR: ARGO_DIR/gts
# DATA_DIR: GTS_DIR/data
# DATA_BUFR_DIR: DATA_DIR/bufr
# DATA_NCLINKS_DIR: DATE_BUFR_DIR/nc_links
# LISTS_BUFR_DIR: GTS_DIR/lists/bufr
# PRG_DIR: GTS_DIR/prg/java
#
##########################################################################################
##########################################################################################
#                            Define Global Variables
##########################################################################################
. $HOME/.argorc_sh
#-----------------------------------------------------------------------------------------
#                            RemoveOldRnc 
#-----------------------------------------------------------------------------------------
# Removes R*.nc link file(s) older than a given number of days (MAXDAY2GTS=30) 
# from NCLINKS_DIR
#-----------------------------------------------------------------------------------------
RemoveOldRnc()
{
  fncname="RemoveOldRnc"
  echo "=> Enter $fncname" 

  MAXDAY2GTS=30

  today=`date +'%Y-%m-%d'`
  yyyy=`echo $today | awk -F'-' '{print $1}'`
  mm=`echo $today | awk -F'-' '{print $2}'`
  dd=`echo $today | awk -F'-' '{print $3}'`
  todayjulday=`$UNIXCMD_DIR/ymd2julday.x $yyyy $mm $dd`
  echo "   today: $today todayjulday: $todayjulday"

  echo "   <<< Loop to Remove 'nc' files older than $MAXDAY2GTS days >>>"
  nc2gts=0
  touch $nc_dangling
  for ncfile in `ls $NCLINKS_DIR`
  do
    path_ncfile=$NCLINKS_DIR/$ncfile
    echo "       path_ncfile: $path_ncfile"

    if [ ! -f $NCDATA_DIR/$ncfile ] 
    then
      printf "       NCDATA_DIR/$ncfile - Missing -> Remove from 'NCLINKS_DIR' -"   
      rm -f $path_ncfile
      echo "OK"
      echo "$NCDATA_DIR/$ncfile" >> $nc_dangling
      continue 
    fi

    julday=`$PATH_NCDUMP/ncdump $path_ncfile | grep "JULD = " | awk '{print $3}'`
    ref_datetime=`$PATH_NCDUMP/ncdump $path_ncfile | \
                  grep "REFERENCE_DATE_TIME = " | awk '{print $3}' | sed -g s/'"'// `
    date_hhmmss=`$UNIXCMD_DIR/ncjuld2timestamp.x $julday $ref_datetime`
    yyyymmdd=`echo $date_hhmmss | awk '{print $1}'`
    ncyyyy=`echo $yyyymmdd | awk -F'-' '{print $1}'`
    ncmm=`echo $yyyymmdd | awk -F'-' '{print $2}'`
    ncdd=`echo $yyyymmdd | awk -F'-' '{print $3}'`
    ncjulday=`$UNIXCMD_DIR/ymd2julday.x $ncyyyy $ncmm $ncdd`
    printf "       ncdate: $yyyymmdd ncjulday: $ncjulday"

    elapsed_days=`expr $todayjulday - $ncjulday`
    printf " elapsed_days: $elapsed_days"

    if [ $elapsed_days -gt $MAXDAY2GTS ]
    then
      printf " -> Remove - "
      rm -f $path_ncfile 
      echo "OK"
    else
      echo " -> OK"
      nc2gts=`expr $nc2gts + 1`
    fi
  done 
  echo "   <<< Loop End >>>"

  echo "   Number of 'nc' file(s) to GTS: $nc2gts"
  if [ $nc2gts -eq 0 ]
  then
    type="ALERT"
    ref="All NC files removed"
    msg="All NC files in:\n    $NCLINKS_DIR"
    msg=$msg"\nremoved (older than $MAXDAY2GTS days)."
    SendEmail $type "$ref" "$msg"
    echo "=> Exit $fncname - Job Terminated" 
    exit
  fi
  echo "=> Exit $fncname $1" 
  return
}
#-----------------------------------------------------------------------------------------
#                                CreateTable
#-----------------------------------------------------------------------------------------
# Gathers information from the "nc2gtsbufr_{timestamp}.logjava" and 'nc' files
# to create a table with the following fields:
#   . Name of the output buffer file 
#   . Name of the encoded real-time NetCDF file  
#   . WMO number
#   . Latitude 
#   . Longitude
#   . Date  (yyyy-mm-dd)
#   . Time  (hh:mm)
#
# Concatenates the 'nc2gtsbufr_encoded_info' table to GTS_BUFR4FTP_TABLE
#
# Function Used
# -------------
# WriteRecord
#-----------------------------------------------------------------------------------------
CreateTable()
{
  funcname=CreateTable
  echo "=> Enter $funcname" 

  GTS_BUFR4FTP_TABLE=$LISTS_BUFR_DIR/GTS_BUFR4FTP_TABLE
  nc2gtsbufr_encoded_info=$TMP_DIR/nc2gtsbufr_encoded_info_$TODAY"_"table

  echo "   GTS_BUFR4FTP_TABLE: $GTS_BUFR4FTP_TABLE"
  echo "   nc2gtsbufr_encoded_info: $nc2gtsbufr_encoded_info"

  completed_encode_wrbufr=$TMP_DIR/completed_encode_wrbufr"_"$TODAY.list
  pattern="^Completed encoding Section|^Complete write BUFR"
  egrep -i "$pattern" $logjava | awk '{print $8}' >  $completed_encode_wrbufr 
  nrecords=`wc -l $completed_encode_wrbufr | awk '{print $1}'`

  completed_wrbufr=$TMP_DIR/completed_wrbufr"_"$TODAY.list
  pattern="^Complete write BUFR"
  egrep -i "$pattern" $logjava | awk '{print $8}'  | sed s/'.$'// > $completed_wrbufr

  indxrec=1
  iwrbufr=1
  rec_wrbufr=`sed -n "$iwrbufr"p $completed_wrbufr`
  bufrno=`echo $rec_wrbufr | awk -F'.' '{print $6}'`

  echo "   <<< Encoded records to append to GTS_BUFR4FTP_TABLE >>>" 
  cat $completed_encode_wrbufr |
  while read  record
  do
    file=`basename $record`
    if [ "`echo $file | grep 'nc'`" ]
    then
      if [ $indxrec -eq 1 ]
      then
        prev_ncfile=$file
      else
        WriteRecord $DATA_NCLINKS_DIR/$prev_ncfile
        prev_ncfile=$file
      fi
    else
      if [ $indxrec -eq $nrecords ]
      then
        WriteRecord $DATA_NCLINKS_DIR/$prev_ncfile 
      else
        iwrbufr=`expr $iwrbufr + 1`
        rec_wrbufr=`sed -n "$iwrbufr"p $completed_wrbufr`
        bufrno=`echo $rec_wrbufr | awk -F'.' '{print $6}'`
      fi
    fi
    indxrec=`expr $indxrec + 1`
  done

  printf "   <- Append to $GTS_BUFR4FTP_TABLE - "
  cat $nc2gtsbufr_encoded_info >> $GTS_BUFR4FTP_TABLE
  echo "OK ->"
 
  echo "=> Exit $funcname" 
  return
}
#-----------------------------------------------------------------------------------------
#                               WriteRecord
#-----------------------------------------------------------------------------------------
WriteRecord()
{
  path_ncfile=$1

  wmo=`basename $prev_ncfile | awk '{print substr($1,2)}' | awk -F'_' '{print $1}'`
  lat=`$PATH_NCDUMP/ncdump $path_ncfile | grep "LATITUDE = " | awk '{print $3}'`
  lon=`$PATH_NCDUMP/ncdump $path_ncfile | grep "LONGITUDE = " | awk '{print $3}'`
  julday=`$PATH_NCDUMP/ncdump $path_ncfile | grep "JULD = " | awk '{print $3}'`
  ref_datetime=`$PATH_NCDUMP/ncdump $path_ncfile | \
                grep "REFERENCE_DATE_TIME = " | awk '{print $3}' | sed -g s/'"'// `
  date_hhmmss=`$UNIXCMD_DIR/ncjuld2timestamp.x $julday $ref_datetime`
  timestamp=`echo $date_hhmmss|  awk -F':' '{print $1 ":" $2}'`
  printf '   %s %s %7s %7.3f %8.3f %s\n' \
         `basename $rec_wrbufr` $prev_ncfile $wmo $lat $lon "$timestamp"
  printf '%s %s %7s %7.3f %8.3f %s\n' \
         `basename $rec_wrbufr` $prev_ncfile $wmo $lat $lon "$timestamp" >> $nc2gtsbufr_encoded_info
  return
}
#-----------------------------------------------------------------------------------------
#                             PackBufrForFTP
# Packs buffer file(s) (IOPX02_KWB _{timestamp}.bufr01 ... dd) and prefix each one  with 
# a 20 byte prefix that should be as follows:
#                ####018nnnnnn####CRCRLF
#  . 4 pound signs
#  . the number 018
#  . nnnnnn - 6 digit number that is the length of the bulletin (minus the # 20 byte prefix)
#  . 4 pound signs
#  . CR CR LF  - carriage return, carriage return, line feed
# and postfix each with CRCRLF. The prefix and postfix are reaquired for ingesting 
# the files at the NWS Gateway
#  
# Each packed buffer file will be moved to DATA_BUFR_DIR/done
#-----------------------------------------------------------------------------------------
PackBufrForFtp()
{
  funcname=PackBufrForFTP
  echo "=> Enter $funcname"

  bufr2pack=$TMP_DIR/bufr2pack"_"$TODAY.list
  echo "   bufr2pack: $bufr2pack" 
  awk '{print $1}' $nc2gtsbufr_encoded_info | sort -u > $bufr2pack
  root_name=`head -1 $bufr2pack | awk -F'.' '{print $1}'`
  packedbufr_file=$DATA_BUFR_DIR/$root_name.bufr
  echo "   packedbufr_file: $packedbufr_file"

  nfiles=`wc -l $bufr2pack | awk  '{print $1}'`
  echo "   <<< Pack Buffer Files ($nfiles) >>>"
  for bufrfile in `cat $bufr2pack`
  do
     pathname_bufrfile=$DATA_BUFR_DIR/$bufrfile
     len=`ls -l $pathname_bufrfile | awk '{print $5}'`
     len=`expr $len + 3`                               # add poscript 3 bytes
     printf "       $bufrfile - "
     printf "####018%06d####\r\r\n" $len  >> $packedbufr_file
     cat $pathname_bufrfile >> $packedbufr_file
     printf "\r\r\n" >> $packedbufr_file
     printf "OK -> move to DATA_BUFR_DIR/done - "
     mv $pathname_bufrfile $DATA_BUFR_DIR/done
     echo "OK"
  done

  echo "=> Exit $funcname"
  return
}
#-----------------------------------------------------------------------------------------
#                                   SendEmail
#-----------------------------------------------------------------------------------------
SendEmail()
{
   echo "=> [Email Begin]"
   type=$1
   title=$2   
   body=$3

   case $type in
     'ALERT')   subj="$up_script_name - $type, $title"
                and_but=and;;
     'FAILURE') subj="$up_script_name - $type, $title"
                and_but=and;;
     'OK')      subj="$up_script_name - $type, $title"
                and_but=but;;
   esac

   if [ -s $error ]
   then
     subj=$subj" $and_but Stderror" 
     if [ $type = "OK" ]
     then
       body="See logfiles:\n   $logfile\n   $error"
     else
       body=$body"\n   $error"
     fi
   else
     rm -f $error
   fi

   echo "     To: $EMAIL_OPERATOR"
   echo "     Subj: $subj"
   echo "     Text:"
   printf "$body" | awk '{print "     " $0}'
   printf "     Sending email ..."
   printf "$body" | mailx -s "$subj" $EMAIL_OPERATOR
   echo "OK"
   echo "=> [Email End]"
   return 
}
#--------------------------------------- Main --------------------------------------------
#debug EMAIL_OPERATOR="Reyna.Sabina@noaa.gov"
TODAY=`date +'%Y.%m.%d_%H.%M'`
YYYYMMDDHHMMSS=`date +%Y%m%d%H%M%S`

script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}'`
up_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

GTS_DIR=$ARGO_DIR/gts
LISTS_BUFR_DIR=$GTS_DIR/lists/bufr
LOG_DIR=$GTS_DIR/log
NCDATA_DIR=$ARGO_DIR/data/nc
PRG_DIR=$GTS_DIR/prg/java
LIB_DIR=$PRG_DIR/lib
TMP_DIR=$GTS_DIR/tmp

DATA_DIR=$GTS_DIR/data
DATA_BUFR_DIR=$DATA_DIR/bufr
DATA_NCLINKS_DIR=$DATA_BUFR_DIR/nc_links

UNIXCMD_DIR=$ARGO_DIR/unixcmd

nc_dangling=$LOG_DIR/nc_missing"_$TODAY"

logfile=$LOG_DIR/$root_script_name"_"$TODAY"_"log
logjava=$LOG_DIR/$root_script_name"_"$TODAY"_"logjava
error=$LOG_DIR/$root_script_name"_"$TODAY"_"error

exec 1> $logfile
exec 2> $error

echo "<--------------------------- Job Begin at `date` ---------------------------->"
echo "<<< Working Ditectories >>>"
echo "    GTS_DIR: $GTS_DIR"
echo "    PRG_DIR: $PRG_DIR"
echo "    LIB_DIR: $LIB_DIR"
echo "    LOG_DIR: $LOG_DIR"
echo "    TMP_DIR: $TMP_DIR"

echo "<<< I/O Data Directories >>>"
echo "    DATA_DIR: $DATA_DIR"
echo "    DATA_BUFR_DIR: $DATA_BUFR_DIR"
echo "    DATA_NCLINKS_DIR: $DATA_NCLINKS_DIR"
echo "    LISTS_BUFR_DIR: $LISTS_BUFR_DIR"
echo "    NCDATA_DIR: $NCDATA_DIR"

echo "<<< Unix Commands Directory >>>"
echo "    UNIXCMD_DIR: $UNIXCMD_DIR"

echo "<<< Logfiles >>>"
echo "   logfile: $logfile"
echo "   logjava: $logjava"
echo "   error: $error"
echo "   nc_dangling: $nc_dangling"

gtsheader_file=$PRG_DIR/gtsheader.txt
input_property_file=$PRG_DIR/input.properties

echo "<<< Input GTS Header & Property Files >>>"
echo "    gtsheader_file: $gtsheader_file"
echo "    input_property_file: $input_property_file"

NCLINKS_DIR=`grep "ArgoBufr.inputFolder" $input_property_file 2> /dev/null | awk -F'=' '{print $2}'`
printf "<- Any files to encoded in $NCLINKS_DIR? "
if [ "`ls $NCLINKS_DIR 2> /dev/null | head -1`" = "" ]  # Empty?
then
  echo "NO  ->"
  rm -f $TMP_DIR/anyfile
  type="ALERT"
  ref="NC files not found"
  msg="NC files not found in\n   $NCLINKS_DIR"
  SendEmail $type "$ref" "$msg" 
  echo "<-------------------------- Job Terminated at `date` --------------------------->"
  exit
fi
echo "YES ->"

RemoveOldRnc                  # Remove R*.nc links older than MAXDAY2GTS 
                              # and dangling "R*.nc" in NCLINKS_DIR
product=`head -1 $gtsheader_file`
sender=`tail -1 $gtsheader_file`
pathname_bufr=$DATA_BUFR_DIR/$product"_"$sender"_"$YYYYMMDDHHMMSS.bufr
pathname_ncencoded=$TMP_DIR/ncencoded_$TODAY.list

echo "=> Location of output BUFR file and list of NC encoded file(s)"
echo "   pathname_bufr: $pathname_bufr"
echo "   pathname_ncencoded: $pathname_ncencoded"

CLASSPATH=$LIB_DIR/WriteArgoBufr_fat.jar:$LIB_DIR/netcdfAll-4.0.jar:$CLASSPATH
export CLASSPATH

echo "=> Create the BUFR file from the input 'nc' file(s)"
printf "   <- Execute WriteArgoBufr - "
java -classpath $CLASSPATH dfo.isdm.WriteArgoBufr.WriteArgoBufr $PRG_DIR/input.properties \
                $pathname_bufr $pathname_ncencoded > $logjava
status_java=$?
if [ $status_java -eq 0 ]
then
  echo "OK ->"
  echo "   <<< BUFR file(s) Created >>>" 
  ls $pathname_bufr* | awk '{print "        " $0}'
  echo "   <<< Encoded NC file(s) in '$pathname_ncencoded' >>>"
  awk '{print "        " $0}' $pathname_ncencoded

  CreateTable

  PackBufrForFtp

  printf "   Remove 'nc' links in DATA_NCLINKS_DIR - "
  for pathname_nc in `cat $pathname_ncencoded`
  do
    rm -f $pathname_nc 
  done
  echo "OK"

  type=OK
  if [ ! -s $nc_dangling ]
  then  
    ref="BUFR file created"
  else
    ref="BUFR file created but dangling 'R*.nc' in 'DATA_NCLINKS_DIR'"
    msg="Softlink(s) in: $NCLINKS_DIR\n"
    msg=$msg"missing from: $NCDATA_DIR"
    msg=$msg"\nDangling 'R*.nc' file(s) removed. See logfiles:\n"
    msg=$msg"   $nc_dangling\n   $logfile"
  fi
else
  echo "FAILURE"
  type=FAILURE
  ref="Java Program reported error(s)"
  msg="At least one error reported by Java Program (status_java=$status_java)\n" 
  msg=$msg"See logfiles:\n   $logfile\n   $logjava"
fi

SendEmail $type "$ref" "$msg" 

echo "<--------------------------- Job Finished at `date` ---------------------------->"
exit 
