#!/bin/sh
################################################################################
# Revision Date: 25-Feb-13 (RS - skip R*.nc check if it belongs to a packed file
#                                IOPX02_KWBC_{timestamp}.bufr that is found
#                                in the INCOMPLETE_DATA_BUFR_DIR) 
#                23-Feb-12 (RS - revised to avoid the sync problem)
#                16-Nov-11 (RS - change -mtime from -1 to -2 in 
#                                GetDailyBufferBulletins)
#                12-Sep-11 (RS - email when machine (phodnet/phoddat) is not
#                                up. Data for tracking is located in:
#                                NWS_BUFR_BUL_DIR (/WSmounts/phodnet/data/gtsbufr)  
#                12-Apr-11 (RS) 
#
# track_bufr_sent2gts.sh
# ======================
# Tracks the 'R*.nc' files packed into the Argo buffer format(bufr) sent to 
# GTS via NWS Gateway.  
#
# The following tasks are permormed:
#  . Checks if there is any NWS buffer bulletin (BUFR_BUL_dd-mm-yyyy-hhmmss) 
#    that was downloaded to NWS_BUFR_BUL_DIR in less than 1 day ago.
#    If found proceed with the following tasks otherwise the run is aborted. 
#    Note: (1) Daily NWS buffer bulletin(s) is/are pushed from SEAS to an 
#    AOML FTP site directory (/phod/incoming/xbt/seas/bufr), (2) a cron
#    run is executed daily on a Linux machine (user: jharris) to download
#    the daily buffer bulletin(s) from directory (/phod/incoming/xbt/seas/bufr)
#    to NWS_BUFR_BUL_DIR.
#  . Checks if the buffer bulletin (BUFR_BUL_dd-mm-yyyy-hhmmss) file was
#    previously processed by looking in IPOX02_KWBC_DONE_DIR (AOML) and
#    IPOX01_KWBC_DIR (Navy) directories. If the buffer bulletin is found
#    do not copy file to DATA_BUFRBUL_DIR otherwise copy it.
#  . Checks if the buffer bulletin (BUFR_BUL_dd-mm-yyyy-hhmmss) is a
#    Navy file (IOPX01_KWB). If so, move it to NWS_BUL_DIR/IOPX01_KWBC.
#  . Executes a C- program (extract_bufrbul.x) to unpack the buffer file(s)
#    for each buffer bulletin corresponding to the date of the last buffer
#    bulletin.  
#  . Creates a list (nws_rnc_from_bufrbul_rncnotsent2gts_list of the R*.nc file(s)
#    that is obtained from the unpacked buffer file(s) which were sent to the 
#    GTS Gateway from NWS. 
#  . Creates a list (aoml_rncsent2gts_list) of the R*.nc file(s) that match 
#    the date of the last buffer bulletin in table (GTS_BUFR_TRACK_TABLE) that 
#    contains the R*.nc files sent to NWS from AOML.  
#  . Inserts an entry to the list (nws_rncnotsent2gts_list) when an R*.nc entry
#    in 'aoml_rncsent2gts_list' is not found in 'nws_rnc_from_bufrbul'. Skip R*.nc
#    entry check if it belons to a packed file of the form IOPX02_KWBC_{timestamp}.bufr 
#    found in the INCOMPLETE_DATA_BUFR_DIR. The incomplete transmitted packed file
#    will be sent to GTS when 'ftpgts_bufr_to_gateway.sh' runs again.    
#  . Sends an email if any of the following exceptions is reported:
#      No buffer bulletin less than 1 day ago found
#      I/O error executing a C-Program  
#      nws_rnc_from_bufrbul.yyyymmdd_hhmmss list is empty
#      aoml_rncsent2gts_list.yyyymmdd_hhmmdd list is empty  
#      rncnotsent2gts.yyyymmdd_hhmmss is not empty 
#
# C-Programs
# ----------
# extract_bufrbul.x: unpacks the buffer file(s) in the NWS BUFR_BUL_{timestamp}
#  pushed to AOML. 
# argobufrcump.x: converts an Argo buffer file to ASCII. It is based on the
#  WMO Argo Template (BUFR_Argo_template.4_5_1). The code way provided by 
#  Kanno (ykanno@met.kishou.go.jp).
#
# Definitions
# -----------
# ARGO_DIR: /a1/ARGO  (Defined in HOME/.argorc_sh)
# GTS_DIR: ARGO_DIR/gts
# DATA_BUFR_DIR: GTS_DIR/data/bufr
# INCOMPLETE_DATA_BUFR_DIR: DATA_BUFR_DIR/incomplete
# DATA_BUFRBUL_DIR: ARGO/gts/data/bufr/daily_nws_bufrbul
# IPOX01_KWBC_DIR: DATA_BUFRBUL_DIR/IOPX01
# IPOX02_KWBC_DONE_DIR: DATA_BUFRBUL_DIR/done
# IPOX02_KWBC_FAILURE_DIR: DATA_BUFRBUL_DIR/failure
# NWS_BUFR_BUL_DIR: /WSmounts/phodnet/data/gtsbufr   (defined in HOME/.argorc_sh)
# GTS_BUFR_TRACK_TABLE: GTS_DIR/lists/bufr/GTS_BUFR_TRACK_TABLE
#
# Notes
# -----
# mm-dd-yyyy-hhmmss: month, day, year, hour, minutes and seconds when the
#  bulletin was sent to GTS from the NWS Gateway.
################################################################################
. $HOME/.argorc_sh
#EMAIL_OPERATOR="Reyna.Sabina@noaa.gov"
# ------------------------------------------------------------------------------
#                          GetDailyBufferBulletins 
# Gets buffer bulletins (BUFR_BUL_dd-mm-yyyy-hhmmss) from NWS_BUFR_BUL_DIR that 
# are less than 1 day old. If none is found then mail is sent and and run 
# aborted. Each buffer bulletin found that has not been tracked (Argo AOML; IOPX02)
# or bypassed (Argo Navy; IOPX01) is copied to DATA_BUFRBUL_DIR for further 
# processing.
# ------------------------------------------------------------------------------
GetDailyBufferBulletins()
{
   fncname="GetDailyBufferBulletins"  
   echo "   <--- EnterGetDailyBufferBulletins --->"

   path2ping=`/usr/ucb/whereis ping | awk '{print $2}'`
   $path2ping $MACHINE_NWS_BUFR_BUL 1> /dev/null 2> /dev/null 
   status_ping=$?

   if [ $status_ping -ne 0 ]
   then
     txt="Connection to '$MACHINE_NWS_BUFR_BUL' is not available"
     echo "   $txt"
     title="FAILURE, $txt"
     timestamp=""
     msg="$txt"
     SendAbortEmail "$title" "$msg" ""
     echo "<------------------------------- Job Aborted at  `date` ------------------------>"
     exit
   fi

   bufrbul_less1day_list=$TMP_DIR/bufrbul_less1day_list.$TODAY
   cd $NWS_BUFR_BUL_DIR
   find . ! -name . -prune -name "BUFR_BUL*" -mtime -2 | \
            awk -F'/' '{print $2}' > $bufrbul_less1day_list 2> /dev/null 
   if [ ! -s $bufrbul_less1day_list ]
   then
     txt="No Buffer Bulletin (BUFR_BUL_*) less than 1 day ago found"
     echo "   $txt"
     title="$txt"
     timestamp=""
     msg="$txt in: \n    $NWS_BUFR_BUL_DIR"
     SendAbortEmail "$title" "$msg" ""
     echo "<------------------------------- Job Aborted at  `date` ------------------------>"
     exit   
  fi

  for bufrbul in `cat $bufrbul_less1day_list`
  do
    printf "   $bufrbul - " 
    pathbufrbul_done=$IPOX02_KWBC_DONE_DIR/$bufrbul
    if [ -f  $pathbufrbul_done ]
    then
      echo "Bulletin from AOML previously tracked  -> Don't Copy"
      continue
    fi

    pathbufrbul_navy=$IPOX01_KWBC_DIR/$bufrbul
    if [ -f  $pathbufrbul_navy ]
    then
      echo "Bulletin from Navy previously bypassed -> Don't Copy"
      continue
    fi

    cp -p $bufrbul $DATA_BUFRBUL_DIR
    echo "Copied"
  done
 echo "   <--- Return $fncname --->"
}
# ------------------------------------------------------------------------------
#                           ExtractBufrFromBulletins
#
# It executes a C-program ('extract_bufrbul.x')  to extracts 'bufr' files 
# from the NWS daily buffer bulletins (IOPX02 KWBC) that are pushed via FTP from 
# SEAS to AOML. It calls the function 'CreateRncListFromBufrFiles ' to create a 
# list of 'Rnc' file(s) in the buffer file(s) extracted from a NW buffer bulletin. 
# ------------------------------------------------------------------------------
ExtractBufrFromBulletins()
{
  fncname_extract="ExtractBufrFromBulletins"
  echo "   <--- Enter $fncname_extract timestamp: $timestamp --->"
  pathname=$1

  ARGO_NAVO=IOPX01_KWBC                   # WMO Argo source & sender

  printf "   Remove files in '$TMPBUFR_DIR' - "
  rm -f $TMPBUFR_DIR/bufr*
  echo "OK"

  bufrbul=`basename $pathname`
  echo "   <<< $bufrbul >>>"

  hhmmss_extract=`echo $bufrbul | awk -F'_' '{print $3}' | awk -F'-' '{print $4}'`
  timestamp_extract=$yyyymmdd"_"$hhmmss_extract

  wmo_list=$TMP_DIR/wmo_list.$timestamp_extract
  cycle_list=$TMP_DIR/cycle_list.$timestamp_extract
  nws_rnc_from_bufrbul=$TMP_DIR/nws_rnc_from_bufrbul.$timestamp_extract
  touch $nws_rnc_from_bufrbul

  log_extract=$LOG_DIR/log_extract.$timestamp_extract

  echo "   wmo_list: $wmo_list"
  echo "   cycle_list: $cycle_list"
  echo "   log_extract: $log_extract"
  echo "   nws_rnc_from_bufrbul: $nws_rnc_from_bufrbul"

  printf "   Exec 'extract_bufrbul.x' - "
  $PRG_DIR/extract_bufrbul.x $pathname $TMPBUFR_DIR > $log_extract
  nbufr=$?
  nprof=0
  case $nbufr in
    0) txt="Not a US Argo DAC BUFR Bulletin"
       echo $txt
       product_sender=`grep "product_sender:" $log_extract |
                       awk '{print $2 "_" $3}' |
                       awk -F',' '{print $1}' |
                       head -1`
       printf "   product_sender: $product_sender - "
       if [ $product_sender = $ARGO_NAVO ]
       then
         printf "Sent by NAVO -> Move to IPOX01_KWBC_DIR - "
         mv  $pathname $IPOX01_KWBC_DIR 
         echo "OK"
       else
         printf "Sent by OTHER Institution ->  Delete It - "
         rm -f $pathname 
         echo "OK"
       fi
       echo "   <--- Return $fncname_extract - status=0 --->"
       return 0;;
   -1) txt="I/O Error reported by 'extract_bufrbul.x' (C-program)"
       echo $txt
       title="$txt"
       msg="$txt.\nMove $bulbufr to IPOX02_KWBC_FAILURE_DIR"
       printf "   Move $bulbufr to IPOX01_KWBC_DIR - "
       mv  $pathname $IPOX02_KWBC_FAILURE_DIR
       echo "OK"
       SendAbortEmail "$title" "$msg" "$log_extract"
       echo "   <--- Terminated  $fncname_extract --->"
       echo "<------------------------------- Job Aborted at `date` ------------------------->"
       exit;;
    *) txt=" A US Argo DAC Bulletin)"
       echo $txt
       product_sender="IOPX02_KWBC"
       echo "   product_sender: $product_sender - Create Profile List (R*.nc) in the $nbufr buffer files"
       CreateRncListFromBufrFiles
       printf "   Move $bufrbul to IPOX02_KWBC_DONE_DIR - " 
       mv $pathname $IPOX02_KWBC_DONE_DIR
       echo "OK"

       if [ -s $nws_rnc_from_bufrbul ]
       then
         ret_val=1  # not empty 
         echo "   <--- Return $fncname_extract with status=$ret_val --->"
         return $ret_val
       else 
         ret_val=0  # empty 
         txt="List (nws_rnc_from_bufrbul) is empty"
         title="$txt"
         msg="$txt.  Lines with the WMO and/or Cycle\n"
         msg=$msg"were not found in the 'bufr' file(s) extracted from the NWS BUFR_BUL\n"
         msg=$msg"file(s) that is/are pushed daily to AOML."
         SendAbortEmail "$title" "$msg" "$log_extract"
         echo "   <--- Return $fncname_extract with status=$ret_val --->"
         return $ret_val
       fi
  esac
}
# ------------------------------------------------------------------------------
#                          CreateRncListFromBufrFiles
#
# Get a list of the 'bufr' file(s) extracted from the NWS buffer bulletin
# (Pushed daily to AOML via FTP from SEAS). It uses the C-program provided
#
# ------------------------------------------------------------------------------
CreateRncListFromBufrFiles()
{
  fncname="CreateRncListFromBufrFiles"
  echo "     <--- Begin $fncname --->"

  thisdir=`pwd`
  cd $PRG_DIR

  for pathname_bufr in `ls $TMPBUFR_DIR/bufr*`
  do
    bufrbulletin_ascii=$TMP_DIR/bufrbulletin_ascii.$timestamp_extract
    bufrbulletin_ascii_dos2unix=$TMP_DIR/bufrbulletin_ascii_dos2unix.$timestamp_extract
    printf "     `basename $pathname_bufr` to ASCII - "
    $PRG_DIR/argobufrdump.x $pathname_bufr > $bufrbulletin_ascii
    status=$?
    if [ $status -eq 0 ]
    then
      printf "OK -> dos2unix - "
      dos2unix $bufrbulletin_ascii > $bufrbulletin_ascii_dos2unix 2> /dev/null
      if [ -s $bufrbulletin_ascii_dos2unix ]
      then
        nprof=`expr $nprof + 1`
        echo "OK Dir: $bufrbulletin_ascii_dos2unix"
        grep "WMO Marine observing platform" $bufrbulletin_ascii_dos2unix \
              | awk -F';' '{print NR " " $2}' > $wmo_list
        grep "Float cycle number" $bufrbulletin_ascii_dos2unix \
              | awk -F';' '{print NR " " $2}' > $cycle_list

        i=1
        n_rnc=`wc -l $wmo_list | awk '{print $1}'`
        echo "       n_rnc= $n_rnc"
        while [ $i -le $n_rnc ]
        do
          wmo=`grep "^$i " $wmo_list | awk '{print $2}'`
          cycle=`grep "^$i " $cycle_list | awk '{print $2}'`
          rnc_profile=`printf "R%04d_%03d.nc" $wmo $cycle`
          printf "       i: %3d %s\n" $i $rnc_profile
          echo $rnc_profile >> $nws_rnc_from_bufrbul
          i=`expr $i + 1`
        done
      else                                  # bufrbulletin_ascii_dos2unix 
       echo "FAILURE"
       txt="Error reported by 'dos2unix'"
       title="$txt"
       msg="$txt."
       SendAbortEmail "$title" "$msg" "$bufrbulletin_ascii"
       echo "   <--- Terminated  $fncname --->>"
       echo "<------------------------------- Job Aborted at `date` ------------------------->"
       exit
      fi # bufrbulletin_ascii_dos2unix 
    else                                    # status=0
      echo "FAILURE (status=$status)"
      txt="Error reported by 'argobufrdump.x' (C-program)"
      title="$txt"
      msg="$txt."
      SendAbortEmail "$title" "$msg" "$bufrbulletin_ascii"
      echo "   <--- Terminated  $fncname --->"
      echo "<------------------------------- Job Aborted at `date` ------------------------->"
      exit
    fi                                     # status=0
  done #pathname_bufr

  cd $thisdir
  echo "     <--- Return $fncname --->"
  return
}
# ------------------------------------------------------------------------------ 
#                          GetTableRnc_SentToGTS 
# Gets the last bulletins sent to AOML from the Gateway (NWS) to obtain the 
# 'date' (year, month & day) the bulletin(s) were sent to GTS from the Gateway. 
# It creates a file (lastbufrbul) with entries from 'GTS_BUFR_TRACK_TABLE' that
# correpond to the 'date'. Execution is aborted if 'lastbufrbul' is empty. 
# ------------------------------------------------------------------------------ 
GetTableRnc_SentToGTS()
{
   fncname=GetTableRnc_SentToGTS
   echo "   <--- Enter $fncname --->"

   gts_bufr_track_table=$1

   aoml_rncsent2gts_list=$TMP_DIR/aoml_rncsent2gts.$timestamp
   touch $aoml_rncsent2gts_list

   echo "   gts_bufr_track_table(input file): $gts_bufr_track_table"
   echo "   track_bufrbul: $track_bufrbul"
   echo "   aoml_rncsent2gts_list(output file): $aoml_rncsent2gts_list"
   echo "   nws_rnc_from_bufrbul: $nws_rnc_from_bufrbul"

   nwc_rncfile=`head -1 $nws_rnc_from_bufrbul`
   ftped_date=`grep "$nwc_rncfile" $gts_bufr_track_table | awk '{print $8}' | head -1`
   ftped_time=`grep "$nwc_rncfile" $gts_bufr_track_table | awk '{print $9}' | head -1`
   echo "   nwc_rncfile: $nwc_rncfile ftped_date: $ftped_date ftped_time: $ftped_time"
#  awk '$8 ~ /'$ftped_date'/ {print $0}' $gts_bufr_track_table > $aoml_rncsent2gts_list 
   awk '$8 ~ /'$ftped_date'/ &&  $9 ~ /'$ftped_time'/ {print $0}' $gts_bufr_track_table > $aoml_rncsent2gts_list 

   if [ -s $aoml_rncsent2gts_list ]
   then
     sort -u $aoml_rncsent2gts_list -o $aoml_rncsent2gts_list
   else
     rm -f $aoml_rncsent2gts_list
     title="No data found in GTS_BUFR_TRACK_TABLE"
     echo "   $title"
     msg="No data was found in the 'GTS_BUFR_TRACK_TABLE'\n"
     SendAbortEmail "$title" "$msg" ""
     echo "   <--- Terminated  $fncname --->"
     echo "<------------------------------- Job Aborted at `date` ------------------------->"
     exit
   fi

   echo "   <--- Return  $fncname --->"
   return 
}
# ------------------------------------------------------------------------------
#                                SendAbortEmail
# ------------------------------------------------------------------------------
SendEmail()
{
   fncname="SendEmail"
   echo "   [Email Begin: $fncname]"
   notfound=0 
   body=""

   if [ -f $nws_rncnotsent2gts_list -a -s $nws_rncnotsent2gts_list ]
   then
     notfound=1 
     subj="$up_script_name ($timestamp) - At least one Rnc profile not sent to GTS GATEWAY" 
     body="At least one Rnc file that is in the following table:\n"
     body=$body"   $GTS_BUFR_TRACK_TABLE\nbut not in NWS BUFR_BUL sent daily to GTS.\n"
     body=$body"See: $nws_rncnotsent2gts_list\nSee logfiles:\n   $logfile\n   $log_extract" 
   else
     subj="$up_script_name ($timestamp) - OK"
   fi

   if [ -s $error ]
   then
     if [ $notfound -eq 0 ]
     then
       subj=$subj" but Standard-Error reported"
       body="See logfiles:\n   $logfile"
     else
       subj=$subj" and Standard-Error reported"
     fi
     body=$body"\n   $error"
   else
     rm -f $error
   fi

   echo "     To: $EMAIL_OPERATOR"
   echo "     Subj: $subj"
   echo "     Text:"
   printf "$body" | awk '{print "     " $0}'
   printf "     Sending email ..."
   printf "$body" | mailx -s "$subj" $EMAIL_OPERATOR
   echo "OK"
   echo "   [Email End: $fncname]"

   return
}
# ------------------------------------------------------------------------------ 
#                                SendAbortEmail
# ------------------------------------------------------------------------------ 
SendAbortEmail()
{
   fncname="SendAbortEmail"
   echo "   [Email Begin: $fncname]"
   title=$1
   body=$2
   logfnc=$3

   subj="$up_script_name ($timestamp) - $title"
   if [ "`echo $title | grep -i Connection`" = "" ]
   then
     body="$body\nSee logfiles:\n   $logfile"
   fi 

   if [ "$logfnc" != "" ]
   then
     body="$body\n   $logfnc"
   fi

  if [ -s $error ]
   then
     subj=$subj" and Standard-Error reported"
     body=$body"\n   $error"
   else
     rm -f $error
   fi

   echo "     To: $EMAIL_OPERATOR"
   echo "     Subj: $subj"
   echo "     Text:"
   printf "$body" | awk '{print "     " $0}'
   printf "     Sending email ..."
   printf "$body" | mailx -s "$subj" $EMAIL_OPERATOR
   echo "OK"
   echo "   [Email End: $fncname]"

   return
}
# ------------------------------------ Debug -----------------------------------
#EMAIL_OPERATOR="Reyna.Sabina@noaa.gov"
#NWS_BUFR_BUL_DIR=$HOME/BUFR_BULLETINS/gtsbufr
# ------------------------------------ Main-------------------------------------
TODAY=`date +'%Y.%m.%d_%H.%M'`
script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}'`
up_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

GTS_DIR=$ARGO_DIR/gts
LISTS_BUFR_DIR=$GTS_DIR/lists/bufr
DATA_DIR=$GTS_DIR/data
DATA_BUFR_DIR=$DATA_DIR/bufr
INCOMPLETE_DATA_BUFR_DIR=$DATA_BUFRBUL_DIR/incomplete

DATA_BUFRBUL_DIR=$DATA_BUFR_DIR/daily_nws_bufrbul
IPOX01_KWBC_DIR=$DATA_BUFRBUL_DIR/IOPX01_KWBC
IPOX02_KWBC_DONE_DIR=$DATA_BUFRBUL_DIR/done
IPOX02_KWBC_FAILURE_DIR=$DATA_BUFRBUL_DIR/failure

LOG_DIR=$GTS_DIR/log
PRG_DIR=$GTS_DIR/prg/c_programs
TMP_DIR=$GTS_DIR/tmp
TMPBUFR_DIR=$DATA_DIR/bufr/tmp_unpack_bulletins

GTS_BUFR_TRACK_TABLE=$LISTS_BUFR_DIR/GTS_BUFR_TRACK_TABLE

logfile=$LOG_DIR/$root_script_name"_"$TODAY.logfile
error=$LOG_DIR/$root_script_name"_"$TODAY.error

exec 1> $logfile
exec 2> $error

echo "<------------------------------- Job Begin at `date` -------------------------->"

echo "=> Directories"
echo "   GTS_DIR: $GTS_DIR"
echo "   DATA_DIR: $DATA_DIR"
echo "   DATA_BUFR_DIR: $DATA_BUFR_DIR"
echo "   DATA_BUFRBUL_DIR: $DATA_BUFRBUL_DIR"
echo "   IPOX01_KWBC_DIR (Argo NAVO): $IPOX01_KWBC_DIR"
echo "   IPOX02_KWBC_DONE_DIR (Argo AOML): $IPOX02_KWBC_DONE_DIR"
echo "   IPOX02_KWBC_FAILURE_DIR (Argo AOML): $IPOX02_KWBC_FAILURE_DIR"
echo "   IPOX02_KWBC_INCOMPLETE_DIR (Argo AOML): $IPOX02_KWBC_INCOMPLETE_DIR"
echo "   LISTS_BUFR_DIR: $LISTS_BUFR_DIR"
echo "   LOG_DIR: $LOG_DIR"
echo "   NWS_BUFR_BUL_DIR: $NWS_BUFR_BUL_DIR"
echo "   PRG_DIR: $PRG_DIR"
echo "   TMP_DIR: $TMP_DIR"
echo "   TMPBUFR_DIR: $TMPBUFR_DIR"

echo "=> Logfiles"
echo "   logfile: $logfile"
echo "   error: $error"

echo "=> Table with R*.nc packed into 'bufr' files sent to NWS Gateway from AOML"  
echo "   GTS_BUFR_TRACK_TABLE: $GTS_BUFR_TRACK_TABLE"

echo "=> Get buffer bulletins (BUFR_BUL_dd_mm_yyyy) downloaded less that 1 day ago"
GetDailyBufferBulletins

#?debug echo "   ???? Debug (must coment the statement above (GetDailyBufferBulletins)???" 
# bufrbul="BUFR_BUL_02-20-2012-190010"
# cp -p /ARGOTEST/ARGO/gts/data/bufr/SAVE.NWS_BUFRBUL/$bufrbul $DATA_BUFRBUL_DIR
# ls -l $DATA_BUFRBUL_DIR

# ------------------------------------------------------------------------------
#                       Process Daily Buffer Bulletins
# ------------------------------------------------------------------------------
for pathname_bufrbul2track in `ls -tr $DATA_BUFRBUL_DIR/BUFR_BUL_*`
do
  track_bufrbul=`basename $pathname_bufrbul2track`
  echo "=> Track 'R*.nc' packed into buffer files sent to GTS via NWS Gateway"
  echo "   pathname_bufrbul2track: $pathname_bufrbul2track" 
  echo "   track_bufrbul: $track_bufrbul"

  yyyymmdd=`echo $track_bufrbul | awk -F'_' '{print $3}' | awk -F'-' '{print $3 $1 $2}'`
  hhmmss=`echo $track_bufrbul | awk -F'_' '{print $3}' | awk -F'-' '{print $4}'`
  timestamp=$yyyymmdd"_"$hhmmss
  echo "   yyyymmdd: $yyyymmdd hhmmss: $hhmmss"
  echo "   timestamp: $timestamp" 
 
  ExtractBufrFromBulletins $pathname_bufrbul2track
  status_extract=$?
  if [ $status_extract -eq 0 ]   # Argo buffer bulletin or empty 'nws_rnc_from_bufrbul'?
  then                           #   Yes -> Go to end of loop  
    continue
  fi 

  GetTableRnc_SentToGTS $GTS_BUFR_TRACK_TABLE 

  echo "   >>> Check if at least one Rnc file was not sent to GTS from NWS <<<" 
  nws_rncnotsent2gts_list=$TMP_DIR/nws_rncnotsent2gts.$timestamp
  touch $nws_rncnotsent2gts_list
  echo "   nws_rncnotsent2gts_list: $nws_rncnotsent2gts_list"

  cat $aoml_rncsent2gts_list |
  while read bufr rncfile rest  
  do
    packedbufr=`echo $bufr | awk -F'.' '{print $1}'`.bufr
    if [ -f $INCOMPLETE_DATA_BUFR_DIR/$packedbufr ]
    then
        echo "   $packedbufr - FOUND in INCOMPLETE_DATA_BUFR_DIR -> Skip"   
        continue
    fi

    if [ "`grep $rncfile $nws_rnc_from_bufrbul`" = "" ]
    then
      echo "   rncfile: $rncfile - NOT FOUND"
      echo $rncfile >> $nws_rncnotsent2gts_list 
    fi 
  done
done

SendEmail 

echo "<------------------------------- Job Finished at `date` ------------------------>"

exit 
