#!/bin/sh
########################################################################################
# Revision Date: removed reference to TO_DATABASE_DIR.
#                                 Since database server (IDA) is no longer
#                                 supported, there is no need to create softlinks
#                                 to *.qc files)
#                06-Sep-07 (RS - Export QCINDATA_DIR)
#                03-Apr-06 (RS - move ANALYSIS_FILE definition to where the qc_process.sh
#                                is called)
#                06-Dec-05 (RS - introduce HEXFILES_RECENT_DIR & HEXFILES_HISTORICAL_DIR
#                                global path used in the 'argos_process.sh' for reducing
#                                the 'prv2hex-C' programs processing time for sorting
#                                the float hex files)
#                10-May-05 (RS - update documentation)
#                11-May-04 (RS - modify print message for 'qc2gts_format' exit status
#                12-Feb-04 (RS - changes for phy2gts_format to qc2gts_format)
#                15-Aug-03 (RS - remove PACKQC2FTP_TABLE code & QC2MATLAB_TABLE which
#                                is defined in 'qc_process.sh' according to the 
#                                operations mode)
#                05-Aug-03 (RS - add ARGOSPRGJAVA_DIR for updated hex2cnt.sh)
#                17-Jul-03 (RS - revised comments)
#                31-Oct-02 (RS - create links for network_info)
#                30-Sep-02 (RS - redefined NetCDF variables used in QC process)
#                18-Jul-02 (RS - add NetCDF variables used in QC process)
#                30-Apr-02 (RS - add prefix to processexit_table)
#                20-Mar-02 (RS - add ARGOSCRIPTS_DIR)
#                25-Jan-02 (RS - run optional processes according to OPERATIONAL_SITE
#                                GTS_DISTRIBUTION_SITE)
#                20-Sep-01 (RS - add ftp/qcpack global variable)
#                17-Sep-01 (RS - add topography file global variable)
#                22-Aug-01 (RS - introduce QC2MATLAB_TABLE)
#                08-Jun-01 (RS - add ftpprv_to publocal)
#                28-Mar-01 (RS - add emailprofiles_ftpgts process)
#                23-Mar-01 (RS - add GTS2FTP table)
#                08-Jan-01 (RS - improve process messages)
#                15-Dec-00 (RS - remove PRV_DATA & PRV_TIME references)
#                12-Dec-00 (RS - change table name (QC2FTPAOML_TABLE)
#                07-Dec-00 (RS - Change process execution order)
#                29-Nov-00 (RS - implement Exit Code table)
#                21-Nov-00 (RS - allow two email operators in ReadSet_InitVariables)
#                17-Oct-00 (RS - incorporate 'packqc_for_ftp)
#                15-Sep-00 (RS)
#
# argo.sh
# =======
# Run the following ARGO processes depending on parameter passed (mode), 
# default will run all process indicated with 'all': 
#    
#   argos_process.sh          (argos or all - production at ARGOS; backup at AOML)
#   qc_process.sh             (qc or all - production at ARGOS; backup at AOML)
#   qc2gts_format.sh          (gts or all - production at ARGOS; backup at AOML)  
#   packqc_for_ftp.sh         (packqc or all - production only at AOML) -> deprecated
#
#   ftpgts_to_gateway.sh      (ftp2gts - production at ARGOS or AOML)
#   emailprofiles_ftpgts      (emailprofiles - production only at AOML)
#   ftpprv_to_publocal.sh     (ftpprv2local - production only at AOML)
#   ftppackqc_to_publocal.sh  (ftp2local - production only at AOML)
#
# Examples:
#   1) Run all processes except: ftpgts_to_gateway.sh & ftpqc_to_aoml.sh
#      which are scheduled to run independently
#        unix> cron_argo.sh       (default)
#      or
#        unix> cron_argo.sh all
#   2) Run qc & qc2gts_format
#        unix> cron_argo.sh qc
#   3) Run only gts
#        unix> cron_argo.sh ftp2gts
#   4) only ftp2gts
#      unix> cron_argo.sh ftp2gts
#
# WARNINGS: 
#   1) Indepedent of GTS_DISTRIBUTION_SITE,
#          QC2GTSFMT_TABLE must exists for executing 'qc_process.sh'
#          and 'qc2gts_format.sh'
#   2) If OPERATIONAL_SITE is not equal to "AOML" then 
#          PACKQC2FMT_TABLE is remove before executing 'qc_process.sh'
#   3) OPERATIONAL_SITE and GTS_DISTRIBUTION_SITE global variable must be set
#      in .argorc/.argorc_sh
########################################################################################
cd $ARGO_DIR/init_scripts

# --------------------------------------------------------------------
#  Revision Date: 21-Nov-00 (RS - allow 2 email operators)
#                 15-Sep-00 (RS)
#
#  ReadSet_InitVar
#  =============== 
#  Reads and sets ARGO initialization environmental global variables
#  which are used to set depedent environmental global variables.
#
#  Usage: ReadSet_InitVar
# --------------------------------------------------------------------
ReadSet_InitVar()
{
  echo "--- Global Variables Initialization with Parameters ---"
  echo "        --- from Input File (argo.init) ---"
  exec < argo.init
  while read envname  envvalue doc
  do
    case $envname in
      SENSOR_TYPES)
          t_sensor=`echo $envvalue | awk -F"," '{print $1}'`
          s_sensor=`echo $envvalue | awk -F"," '{print $2}'`
          value="$t_sensor $s_sensor"
          name=$envname
          eval $name=$value
          eval export $name;;
       EMAIL_OPERATOR)
          opr_1=`echo $envvalue | awk -F"," '{print $1}'`
          opr_2=`echo $envvalue | awk -F"," '{print $2}'`
          value="$opr_1 $opr_2"
          name=$envname
          eval $name=$value
          eval export $name;;
      *)
          name=$envname
          value=$envvalue
          eval $name=$value
#debug    eval echo $name"="$value
          eval export $name;;
   esac
  done

  return
}
#-------------------- init global var in argo.init --------------------
ReadSet_InitVar                           # independent env variables
#---------------------------- init argo global ------------------------
echo "=> Setting Environmental variables for: ARGO" 
ARGOLIST_DIR=$ARGO_DIR/lists              # ARGO list directory
ARGOSCRIPTS_DIR=$ARGO_DIR/scripts         # ARGO scripts directory (general use) 
ARGOUNIX_CMD_DIR=$ARGO_DIR/unixcmd        # ARGO Unix command directory
EMAILFILES_DIR=$ARGO_DIR/emails           # directory for email file(s)

INIT_SCRIPTS_DIR=$ARGO_DIR/init_scripts   # ARGO initialization scripts directory
METAFILES_DIR=$ARGO_DIR/meta              # directory for meta files

PHYFILES_DIR=$ARGO_DIR/data/phy             # physical file (*.phy) directory
DONE_PHYFILES_DIR=$PHYFILES_DIR/done        # QC succesfuly processed PHY file(s)
DONEOLD_PHYFILES_DIR=$PHYFILES_DIR/done_old # QC succesfuly processed PHY file(s)
FAILED_PHYFILES_DIR=$PHYFILES_DIR/failure   # QC unsuccesfuly processed PHY file(s)
export PHYFILES_DIR DONE_PHYFILES_DIR DONEOLD_PHYFILES_DIR FAILED_PHYFILES_DIR

export ARGOLIST_DIR
export ARGOSCRIPTS_DIR
export ARGOUNIX_CMD_DIR
export EMAILFILES_DIR
export INIT_SCRIPTS_DIR
export METAFILES_DIR

#---------------------------- init argos global ------------------------
echo "=> Setting Environmental variables for: argos" 
ARGOS_DIR=$ARGO_DIR/argos                         # ARGOS root directory
ARGOSCOLLECT_DIR=$ARGOS_DIR/collect               # ARGOS collect directory
ARGOSLOG_DIR=$ARGOS_DIR/log                       # ARGOS log directory
ARGOSTMP_DIR=$ARGOS_DIR/tmp                       # ARGOS temporary directory

ARGOSSCRIPT_DIR=$ARGOS_DIR/scripts                # ARGOS script directory
ARGOSPRG_DIR=$ARGOS_DIR/prg                       # ARGOS program directory
ARGOSPRGJAVA_DIR=$ARGOSPRG_DIR/java               # ARGOS Java program directory
ARGOSLIST_DIR=$ARGOS_DIR/lists                    # ARGOS list directory
ARGOSLIST_SUCCESS_DIR=$ARGOSLIST_DIR/success      # ARGOS success list directory
ARGOSLIST_FAILURE_DIR=$ARGOSLIST_DIR/failure      # ARGOS failure list directory
PRVFILES_DIR=$ARGOS_DIR/data/prv                  # ARGOS PRV files directory
HEXFILES_DIR=$ARGOS_DIR/data/hex                  # ARGOS HEX files directory
HEXFILES_HISTORICAL_DIR=$HEXFILES_DIR/historical  # ARGOS HEX historical files directory
HEXFILES_RECENT_DIR=$HEXFILES_DIR/recent          # ARGOS HEX recent files directory
CNTFILES_DIR=$ARGOS_DIR/data/cnt                  # ARGOS CNT files directory

DONE_PRVFILES_ARGO_DIR=$PRVFILES_DIR              # ARGOS PRV files done directory
DONE_CNTFILES_DIR=$CNTFILES_DIR/done              # ARGOS CNT files done directory
DONEOLD_CNTFILES_DIR=$CNTFILES_DIR/done_old       # ARGOS CNT files done_old directory
FAILURE_CNTFILES_DIR=$CNTFILES_DIR/failure        # ARGON CNT files that failed directory

FLOATIDFILES_DIR=$METAFILES_DIR                   # ARGOS AOML float id file directory name
FAILURE_LIST_DIR=$ARGOSLIST_FAILURE_DIR           # ARGOS failure list directory

export ARGOS_DIR
export ARGOSCOLLECT_DIR
export ARGOSLOG_DIR
export ARGOSTMP_DIR
export ARGOSSCRIPT_DIR
export ARGOSPRG_DIR ARGOSPRGJAVA_DIR
export ARGOSLIST_DIR
export ARGOSLIST_SUCCESS_DIR
export ARGOSLIST_FAILURE_DIR
export PRVFILES_DIR
export HEXFILES_DIR HEXFILES_HISTORICAL_DIR HEXFILES_RECENT_DIR
export CNTFILES_DIR
export DONE_PRVFILES_ARGO_DIR
export DONE_CNTFILES_DIR
export DONEOLD_CNTFILES_DIR
export FAILURE_CNTFILES_DIR
export FLOATIDFILES_DIR
export FAILURE_LIST_DIR
#---------------------------- init qc global ---------------------------
echo "=> Setting Environmental variables for: qc" 
QC_DIR=$ARGO_DIR/qc                               # QC root directory
QCFILES_DIR=$ARGO_DIR/data/qc                     # QC physical file(s) directory 
QCLOG_DIR=$QC_DIR/log                             # QC log files directory
PHY2GTSFMT_TABLE=$ARGOLIST_DIR/PHY2GTSFMT"_"TABLE # QC profiles -> FTP to GTS Host table 
QCSCRIPT_DIR=$QC_DIR/scripts                      # dir for QC scripts
QCPRG_DIR=$QC_DIR/prg                             # dir for QC programs
QCLIST_DIR=$QC_DIR/lists                          # dir for QC lists
QCLIST_SUCCESS_DIR=$QCLIST_DIR/success            # dir for QC success list
QCLIST_FAILURE_DIR=$QCLIST_DIR/failure            # dir for QC failure list
QCTMP_DIR=$QC_DIR/tmp                             # dir for QC temporary work

QCINDATA_DIR=$QC_DIR/clima_mask                   # dir for QC mask & clima data files 
MASK_T=$QCINDATA_DIR/$MASK_FILE                   # dir for global ocean t mask  data
TOPO=$QCINDATA_DIR/$TOPO_FILE                     # dir for global topography data

CLIMATOLOGY_T=$QCINDATA_DIR/$CLIMA_FILE_T         # dir for global climatology_t data
CLIMATOLOGY_S=$QCINDATA_DIR/$CLIMA_FILE_S         # dir for global climatology_t data

NCDATA_DIR=$ARGO_DIR/data/nc                      # dir for NetCDF data
NETCDF_QCLINKS_DIR=$ARGO_DIR/netcdf/data/qc_links # dir for links to QC files  

export QC_DIR QCINDATA_DIR
export QCFILES_DIR
export QCLOG_DIR
export PHY2GTSFMT_TABLE 
export QCSCRIPT_DIR
export QCPRG_DIR
export QCLIST_DIR
export QCLIST_SUCCESS_DIR
export QCLIST_FAILURE_DIR
export QCTMP_DIR
export MASK_T TOPO CLIMATOLOGY_T CLIMATOLOGY_S
export NETCDF_QCLINKS_DIR
#----------------------------  start  gts   ----------------------------
echo "=> Setting Environmental variables for: gts" 
GTS_DIR=$ARGO_DIR/gts                        # GTS root dir
GTSSCRIPT_DIR=$GTS_DIR/scripts               # scripts dir 
GTSLOG_DIR=$GTS_DIR/log                      # GTS logfiles dir
GTSFILES_DIR=$GTS_DIR/data                   # GTS-Formatted profile dir
GTSTMP_DIR=$GTS_DIR/tmp                      # GTS temporary work dir

GTSPRG_DIR=$GTS_DIR/prg                      # GTS programs dir
GTSLIST_DIR=$GTS_DIR/lists                   # GTS lists dir
GTSLIST_SUCCESS_DIR=$GTSLIST_DIR/success     # GTS success list dir
GTSLIST_FAILURE_DIR=$GTSLIST_DIR/failure     # GTS failure list dir
DONE_GTSFILES_DIR=$GTSFILES_DIR/done         # transmitted GTS-Formatted dir

# Convert to hours
PHY2GTSFMT_TIME2REMOVE=`expr $DAYS2REMOVE_FROM_PHY2GTSFMT_TABLE \* 24` 

export GTS_DIR
export GTS2FTP_TABLE
export GTSSCRIPT_DIR
export GTSLOG_DIR
export GTSFILES_DIR
export GTSTMP_DIR
export GTSPRG_DIR
export GTSLIST_DIR
export GTSLIST_SUCCESS_DIR
export GTSLIST_FAILURE_DIR
export GTSLOG_DIR
export DONE_GTSFILES_DIR
export PHY2GTSFMT_TIME2REMOVE
export FTP2GTS_TABLE
#----------------------------  start ftp2local  ----------------------------
echo "=> Setting Environmental variables for: ftp2local" 
FTP_DIR=$ARGO_DIR/ftp
FTPSCRIPT_DIR=$FTP_DIR/scripts
FTPLOG_DIR=$FTP_DIR/log
FTPTMP_DIR=$FTP_DIR/tmp
FTPDATA_DIR=$FTP_DIR/data
FTP_QCPACK_DIR=$FTPDATA_DIR/qcpack
FOR_FTP_DIR=$ARGOSCOLLECT_DIR/data/for_ftp

export FTP_DIR
export FTPSCRIPT_DIR
export FTPLOG_DIR
export FTPTMP_DIR
export FTPDATA_DIR
export FTP_QCPACK_DIR
export FOR_FTP_DIR
#----------------------------  start packqc ----------------------------
echo "=> Setting Environmental variables for: packqc" 
FTPDATA_QCPACK_DIR=$FTPDATA_DIR/qcpack            # QC-compressed files 
FTP_TO_LOCAL_DIR=$FTPDATA_QCPACK_DIR/to_local     # FTP->local pending files (links to qcpack) 
FTP_TO_PRGNUM_DIR=$FTPDATA_QCPACK_DIR/to_prgnum   # FTP->prgnum pending files (links to qcpack)

export FTPDATA_QCPACK_DIR
export FTP_TO_LOCAL_DIR
export FTP_TO_PRGNUM_DIR
#----------------------  start emailprofiles_ftsgts ----------------------
#GTS2FTP_TABLE=$ARGOLIST_DIR/GTS2FTP_TABLE    
echo "=> Setting Environmental variables for:emailprofiles_ftsgts" 
FTPGTS4EMAIL_TABLE=$ARGOLIST_DIR/FTPGTS4EMAIL_TABLE # GTS-Formatted filename (FTPed) + prof info for EMAIL
GTSFILES_TO_AOML_DIR=$GTSFILES_DIR/to_aoml          # Linked GTS-Formatted files FTPed to GTS dir
FTPGTS_EMAILS=$EMAILFILES_DIR/FTPGTS_EMAILS         # File with the body of emails sent to AOML_OPERATOR

export FTPGTS4EMAIL_TABLE
export GTSFILES_TO_AOML_DIR
export FTPGTS_EMAILS
#-------------------------------- Main end env   ----------------------------
TODAY=`date +'%Y.%m.%d_%H.%M'`

yymmdd=`echo $TODAY | awk -F"_" '{print $1}'`
hhmm=`echo $TODAY | awk -F"_" '{print $2}'`
TODAY_DATE=`echo $yymmdd | sed -n -g s%'\.'%/%p`
TODAY_TIME=`echo $hhmm | sed -n -g s%'\.'%:%p`

export TODAY
export TODAY_DATE
export TODAY_TIME

script_name=`basename $0`
now=$TODAY
echo "<******************** Start  $script_name: $now *******************>"
echo "$script_name: TODAY = $TODAY TODAY_DATE = $TODAY_DATE TODAY_TIME = $TODAY_TIME"

nparm="$#"         
if [ $nparm -eq 0 ]            # Any command parameter?  
then                           #   Yes, set default -  run all processes
 mode=all 
else
 mode=$1                       #   No, set mode [all, argos, qc, gts, ftpgts] 
fi                             

echo "Running MODE is: $mode"           
ret_val=0

# -----------------------------------------------------------------------
#                           Argos Process
# -----------------------------------------------------------------------
if [ "$mode" = "all" -o "$mode" = "argos" ]
then
  process_name='argos_process.sh'
  log_file=$ARGOSLOG_DIR/argos_process.$TODAY"_"log
  export log_file

  now=`date +'%Y.%m.%d_%H.%M'`
  echo "\n<----------------   Start  $process_name: $now   ------------------>"
  echo "STDOUT Logfile Location for $process_name is: $log_file"

  argosconnection_datetime_file=$ARGOSTMP_DIR/argosconnection_datetime.$TODAY
  touch $argosconnection_datetime_file
  export argosconnection_datetime_file

  $ARGOSSCRIPT_DIR/argos_process.sh $mode > $log_file
  argos_status="$?"

  now=`date +'%Y.%m.%d_%H.%M'`
  echo "<----------------   Finish $process_name: $now   ------------------>"
  printf "$script_name: Argos Process -> Exit Status = $argos_status"
  if [ "$argos_status" != "0" ]
  then
    ret_val=1
    echo " - Not OK"
    case $argos_status in
      1) reason="No PRV file found";;
      2) reason="No PRV file processed succesfully by prv2hex.x";; 
      3) reason="No HEX profile created by prv2hex.x";;
      4) reason="No CNT/PHY profile created by hex2cnt_type.x/cnt2phy_type";;
      5) reason="No New or Updated CNT profiles created";;
      *) reason="Unkown - value not predefined";;
    esac
    echo "Reason: $reason"
    echo "For more details, see logfiles"
    echo "<********************  Stop $script_name: $now  *******************>"
    exit $ret_val 
  else
    echo " - OK"
  fi
  rm -f $argosconnection_datetime_file
fi

# -----------------------------------------------------------------------
#                           QC Process
# -----------------------------------------------------------------------
if [ "$mode" = "all" -o "$mode" = "qc" ]
then
  process_name='qc_process.sh'
  log_file=$QCLOG_DIR/qc_process.$TODAY"_"log
  export log_file

  file=`ls $QCINDATA_DIR/$ANALYSIS_FILE_PREFIX.*`
  ANALYSIS_FILE=$file
  export ANALYSIS_FILE

  now=`date +'%Y.%m.%d_%H.%M'`
  echo "<----------------   Start  $process_name: $now   ------------------>"
  echo "STDOUT Logfile Location for $process_name is: $log_file"
  echo "OPERATIONAL_SITE: $OPERATIONAL_SITE"
  echo "GTS_DISTRIBUTION_SITE: $GTS_DISTRIBUTION_SITE"
  echo "ANALYSIS_FILE: $ANALYSIS_FILE"

  DEBUG=$QCDEBUG
  export DEBUG

  if [ $OPERATIONAL_SITE != $GTS_DISTRIBUTION_SITE ] # Is oper. site different than gts distr. site?
  then                                               #   Yes 
    echo "  Operation Site is Not the Same as the GTS Distribution Site"
    echo "    Does PHY2GTSFMT_TABLE exists? \c "
    if [ -f $PHY2GTSFMT_TABLE ]                      #   Does the table exists?
    then                                             #     Yes, remove it
      rm -f $PHY2GTSFMT_TABLE
      echo "Yes - Removed"
    else
      echo "No" 
    fi
  fi

  $QCSCRIPT_DIR/qc_process.sh > $log_file
  qc_status="$?"

  now=`date +'%Y.%m.%d_%H.%M'`
  echo "<----------------   Finish $process_name: $now   ------------------>"
  printf "$script_name: QC Process -> Exit Status = $qc_status"
  if [ "$qc_status" != "0" ]
  then
    ret_val=2
    echo " - Not OK, see logfiles" 
    echo "<********************  Stop $script_name: $now  *******************>"
    exit $ret_val 
  else
    echo " - OK" 
  fi
fi

# -----------------------------------------------------------------------
#                        PHY2GTS_FORMAT Process
#            - Only at GTS Distribution Site (ARGOS/AOML) - 
# -----------------------------------------------------------------------
#?GTS_DISTRIBUTION_SITE=AOML         #? Used ONLY at AOML for argotest
#?export GTS_DISTRIBUTION_SITE

if [ "$mode" = "all" -o "$mode" = "gts" ]
then
  process_name='qc2gts_format.sh'
  now=`date +'%Y.%m.%d_%H.%M'`

  if [ "$OPERATIONAL_SITE" = "$GTS_DISTRIBUTION_SITE" ]  # Is operational site = gts distr. site?
  then                                                   #   Yes, exec qc2gts_format.sh
    process_name='qc2gts_format.sh'
    log_file=$GTSLOG_DIR/qc2gts_format.$TODAY"_"log
    export log_file

    echo "<----------------   Start  $process_name: $now   ------------------>"
    echo "Operational Site is the SAME as the GTS Distribution Site"  
    echo "OPERATIONAL_SITE: $OPERATIONAL_SITE"
    echo "GTS_DISTRIBUTION_SITE: $GTS_DISTRIBUTION_SITE"

    echo "STDOUT Logfile Location for $process_name is: $log_file"

    DEBUG=$GTSDEBUG
    export DEBUG

    $GTSSCRIPT_DIR/qc2gts_format.sh > $log_file
    gts_status="$?"

    now=`date +'%Y.%m.%d_%H.%M'`
    echo "<----------------   Finish $process_name: $now   ------------------>"
    printf "$script_name: QC2GTS_FORMAT Process -> Exit Status = $gts_status"
    if [ "$gts_status" != "0" ]
    then
      ret_val=3
      echo " - ERROR" 
      reason="System/Fortran Error Reported"
      echo "Reason: $reason"
      echo "For more details, see logfiles"
      echo "<********************  Stop $script_name: $now  *******************>"
      exit $ret_val
    else
      echo " - OK" 
    fi
  else                             # Operational site is not GTS Distribution site 
    echo "<----------------  Skip  $process_name: $now   ------------------>"
    echo "Operational Site is NOT the SAME as the GTS Distribution Site"  
    echo "OPERATIONAL_SITE: $OPERATIONAL_SITE"
    echo "GTS_DISTRIBUTION_SITE: $GTS_DISTRIBUTION_SITE"  
    echo "<----------------   Skip  $process_name: $now   ------------------>"
  fi # endif - operational site = gts distribution site
fi # endif - qc2gts_format process 

# =======================================================================
#               Exit if Not Exclusive Mode Proccesses
# =======================================================================
if [ $mode != "ftp2gts"  -a $mode != "ftpprv2local" -a $mode != "ftp2local" ]
then
  echo "<******************** Finish $script_name: $now *******************>"
  exit $ret_val
fi

# =======================================================================
#                   ONLY for Exclusive Mode Proccesses
# =======================================================================
# -----------------------------------------------------------------------
#                        FTPPRV_TO_PUBLOCAL Process
#                       - Only at Data Center Site -
#                                   AOML
# -----------------------------------------------------------------------
if [ "$mode" = "ftpprv2local" ]
then
  process_name='ftpprv_to_publocal.sh'
  log_file=$FTPLOG_DIR/ftpprv_to_publocal.$TODAY"_"log
  export log_file

  now=`date +'%Y.%m.%d_%H.%M'`
  echo "<----------------   Start  $process_name: $now   ------------------>"
  echo "STDOUT Logfile Location for $process_name is: $log_file"

  $FTPSCRIPT_DIR/$process_name > $log_file
  ftpprv2local_status="$?"

  now=`date +'%Y.%m.%d_%H.%M'`
  echo "<----------------   Finish $process_name: $now   ------------------>"
  echo "  $script_name: FTPPRV_TO_PUBLOCAL Process -> Exit Status = $ftpprv2local_status"
  if [ "$ftpprv2local_status" != "0" ]
  then
    echo " - Not OK, see logfiles"
    echo "<********************  Stop $script_name: $now  *******************>"
    exit $ret_val
  else
    ret_val=4
    echo " - OK"
    echo "<******************** Finish $script_name: $now *******************>"
    exit $ret_val
  fi
fi

# -----------------------------------------------------------------------
#                      FTPGTS_TO_GATEWAY Process 
#                  - Only at GTS Distribution Site -
#                            ARGOS/AOML
# -----------------------------------------------------------------------
if [ "$mode" = "ftp2gts" ]
then
  process_name='ftpgts_to_gateway.sh'
  log_file=$FTPLOG_DIR/ftpgts_to_gateway.$TODAY"_"log
  export log_file 

  now=`date +'%Y.%m.%d_%H.%M'`
  echo "<----------------   Start  $process_name: $now   ------------------>"
  echo "STDOUT Logfile Location for $process_name is: $log_file"

  $FTPSCRIPT_DIR/$process_name > $log_file
  ftp2gts_status="$?"

  now=`date +'%Y.%m.%d_%H.%M'`
  echo "<----------------   Finish $process_name: $now   ------------------>"
  printf "$script_name: FTP_TO_GATEWAY Process -> Exit Status = $ftp2gts_status"
  if [ "$ftp2gts_status" != "0" ]
  then
    ret_val=5
    echo " - Not OK, see logfiles"
    echo "<********************  Stop $script_name: $now  *******************>"
    exit $ret_val
  else
    echo " - OK"
    echo "<******************** Finish $script_name: $now *******************>"
    exit $ret_val 
  fi
fi

# -----------------------------------------------------------------------
#                      EMAILPROFILES_FTPGTS Process
#                       - Only at Data Center Site -
#                            Not  Activated
# -----------------------------------------------------------------------
if [ "$mode" = "emailprofiles" ]
then
  process_name='emailprofiles_ftpgts.sh'
  log_file=$FTPLOG_DIR/emailprofiles_ftpgts.$TODAY"_"log
  export log_file

  now=`date +'%Y.%m.%d_%H.%M'`
  echo "\n<----------------   Start  $process_name: $now   ------------------>"
  echo "STDOUT Logfile Location for $process_name is: $log_file"

  $FTPSCRIPT_DIR/$process_name > $log_file
  emailprofiles_status="$?"

  now=`date +'%Y.%m.%d_%H.%M'`
  echo "<----------------   Finish $process_name: $now   ------------------>"
  printf "$script_name: EMAILPROFILES_FTPGTS Process -> Exit Status = $emailprofiles_status"
  if [ "$emailprofiles_status" != "0" ]
  then
    ret_val=6
    echo " - Not OK, see logfiles"
    echo "<********************  Stop $script_name: $now  *******************>"
    exit $ret_val
  else
    echo " - OK"
    echo "<******************** Finish $script_name: $now *******************>"
    exit $ret_val
  fi
fi
