/***************************************************************************** 
  Revision date: 13-Jun-2012 VMH change dO2Freq to ofreq, dO2FreqOffset to ofreqOffset
                 15-Nov-12  ER: Initial version (est.)

  2.java
  ======
  Modules supporting oxygen calculations.
*****************************************************************************/
import java.util.Arrays;
import java.lang.System;
public class O2_all {



  public static double atg(double p,double t,double s)
  {
    double ds,h1,h2,h3,h4,h5,out;
    ds=s-35.0;
    h1 = ((-2.1687e-13*t + 1.8676e-11)*t - 4.6206e-10)*p;
    h2 = (2.7759e-10*t - 1.1351e-8)*ds;
    h3 = ((-5.4481e-12*t + 8.733e-10)*t - 6.7795e-8)*t;
    h4 = (-4.2393e-7*t + 1.8932e-5)*ds;
    h5 = ((6.6228e-9*t - 6.836e-7)*t + 8.5258e-5)*t + 3.5803e-4;
    out=(h1 + (h2 + h3 + 1.8741e-6))*p + h4 + h5;
    return out;
  }

  public static double ptem(
        double p_bar, double t, double s, double pr)
  {
    double h1,h2,xk,theta,q,p;

    h1=pr-p_bar;
    h2=atg(p_bar,t,s);
    xk=h1*h2;
    theta=t+0.5*xk;
    q=xk;
    p=p_bar+0.5*h1;
    h2=atg(p,theta,s);
    xk=h1*h2;
    theta=theta+0.29289322*(xk-q);
    q=0.58578644*xk+0.121320344*q;
    h2=atg(p,theta,s);
    xk=h1*h2;
    theta=theta+1.707106781*(xk-q);
    q=3.414213562*xk-4.121320344*q;
    p=p+0.5*h1;
    h2=atg(p,theta,s);
    xk=h1*h2;
    theta=theta+(xk-2.0*q)/6.0;
    return theta;
  }    


  
  /*==========================================================================
     getDissolvedO2_ml_l

     Calculate Dissolved O2 in ml/l given:
       Soc
       Dissolved oxygen frequency
       Dissolved oxygen frequency Offset
       Coefficients a, b, c, pCorr
       Pressure (p) in dbar
       Temperature (t) in deg.C
       Salinity (s) in PSU
  ===========================================================================*/ 
     public static double getDissolvedO2_ml_l( 
                          double Soc, double ofreq, double ofreqOffset,
                          double a, double b, double c, double pCorr, 
                          double p, double t, double s ) {
       
       double t2 = t*t;
       double t3 = t2*t;

       return Soc * 
              (ofreq + ofreqOffset) *
              (1.0 + (a*t) + (b*t2) + (c*t3) ) *
              getOxsol( t, s ) *
              Math.exp( pCorr * p / (t + 273.15) );
     }
     


  /*==========================================================================
     getOxsol

     Calculate Oxygen saturation value in ml/l after Garcia and Gordon (1992)

     Input: 
       t: temperature in Celsius
       s: salinity in PSU

     Return:
       Oxygen saturation value after Garcia and Gordon (1992) in ml/l
       (Volume of oxygen gas at standard temperature and pressure conditions (STP) 
        absorbed from humidity-saturated air at a total pressure of one atmosphere, 
        per unit volume of the liquid at the temperature of measurement 

     -5 <= t <= 40C      0<= S <= 42C
  ===========================================================================*/ 
  public static double getOxsol(double t, double s) {
  
    final double A0 = 2.00907;
    final double A1 = 3.22014;
    final double A2 = 4.05010;
    final double A3 = 4.94457;
    final double A4 = -0.256847;
    final double A5 = 3.88767;
    final double B0 = -6.24523E-3;
    final double B1 = -7.37614E-3;
    final double B2 = -1.03410E-2;
    final double B3 = -8.17083E-3;
    final double C0 = -4.88682E-7;

    double ts = Math.log( (298.15-t) / (273.15+t) );
    double ts2 = ts*ts;
    double ts3 = ts2*ts;
    double ts4 = ts3*ts;
    double ts5 = ts4*ts;
 
    return Math.exp( A0 + A1*ts + A2*ts2 + A3*ts3 + A4*ts4 + A5*ts5 + 
                     s*( B0 + B1*ts + B2*ts2 + B3*ts3 ) + 
                     C0*s*s );
  }



/*========================================================================
     function to compute density of seawater with hydro values s,t,p      
  ======================================================================== 
   This function computes sigma(s,t,p){=rho(s,t,p)-1000.0 [MKS units]}.
   It uses the international equation of state for seawater given by
   UNESCO (1981).  Reference: Atmosphere-Ocean Interactions by Gill,
   Appendix 3 (Properties of Seawater), 1982, pp 599-601.

   Input:
      sal  = (float) salinity [PSU] of seawater parcel.
      temp = (float) temperature [C] of seawater parcel
                     at pressure (pres [dBar]).
      pres = (float) pressure [dBar] of seawater parcel.

   Output:
      eqstate = sigma(s,t,p) [g/(cu cm)]; density of seawater.

   Written by Dana Swift (March 1988)
=============================================================================*/
public static double eqstate( double s, double t, double p ) 
{
   double sig;
   double rho_0t0, rho_st0, rho_stp, sbm_0t0, sbm_st0, sbm_stp;

   double a1 = 999.842594,  a2 = 6.793952e-2,  a3 = -9.095290e-3,
          a4 = 1.001685e-4, a5 = -1.120083e-6, a6 = 6.536332e-9;

   double b1 = 0.824493,    b2 = -4.0899e-3,   b3 = 7.6438e-5,
          b4 = -8.2467e-7,  b5 = 5.3875e-9,    b6 = -5.72466e-3,
          b7 = 1.0227e-4,   b8 = -1.6546e-6,   b9 = 4.8314e-4;

   double c1 = 19652.21,    c2 = 148.4206,     c3 = -2.327105,
          c4 = 1.360477e-2, c5 = -5.155288e-5;

   double d1 = 54.6746,     d2 = -0.603459,    d3 = 1.09987e-2,
          d4 = -6.1670e-5,  d5 = 7.944e-2,     d6 = 1.6483e-2,
          d7 = -5.3009e-4;

   double e1 = 3.239908,    e2 = 1.43713e-3,   e3 = 1.16092e-4,
          e4 = -5.77905e-7, e5 = 2.2838e-3,    e6 = -1.0981e-5,
          e7 = -1.6078e-6,  e8 = 1.91075e-4,   e9 = 8.50935e-5,
          e10= -6.12293e-6, e11= 5.2787e-8,    e12= -9.9348e-7,
          e13= 2.0816e-8,   e14= 9.1697e-10;

   /* compute secant bulk modulus of pure water at one atmosphere */
   sbm_0t0 = c1 + t*(c2 +t*(c3 + t*(c4 + t*c5)));

   /* compute secant bulk modulus of sea water at one atmosphere */
   sbm_st0 = sbm_0t0 + s*(d1 + t*(d2 + t*(d3 + t*d4))) +
            Math.pow(s,1.5)*(d5 + t*(d6 + t*d7));

   /* compute secant bulk modulus of sea water at given pressure */
   sbm_stp = sbm_st0 + (p*(e1 + t*(e2 + t*(e3 + t*e4)))) +
         (p*s*(e5 + t*(e6 + t*e7))) + (p*Math.pow(s,1.5)*e8) +
         (p*p*(e9 + t*(e10 + t*e11))) +
         (p*p*s*(e12 + t*(e13 + t*e14)));

   /* compute density of pure water at one atmosphere */
   rho_0t0 = a1 + t*(a2 + t*(a3 + t*(a4 + t*(a5 + t*a6))));

   /* compute density of sea water at one atmosphere */
   rho_st0 = rho_0t0 + s*(b1 + t*(b2 + t*(b3 + t*(b4 + t*b5)))) +
       Math.pow(s,1.5)*(b6 + t*(b7 + t*b8)) + Math.pow(s,2.0)*b9;

   /* compute density of sea water at given pressure */
   rho_stp = rho_st0/(1.0 - (p/sbm_stp));

   /* convert from density to sigma units */
   sig = rho_stp - 1000.0;

   return(sig);
}



  /*========================================================================
      getSeaWaterDensity

      Compute density of seawater given hydro values s,t,p                 
  
      This method computes sigma(s,t,p){=rho(s,t,p)-1000.0 [MKS units]}.
      It uses the international equation of state for seawater given by
      UNESCO (1981).  Reference: Atmosphere-Ocean Interactions by Gill,
      Appendix 3 (Properties of Seawater), 1982, pp 599-601.

      Input:
        s = salinity [PSU] of seawater parcel.
        t = temperature [C] of seawater parcel at pressure (pres [dBar]).
        p = pressure [dBar] of seawater parcel.

      Output:
        sigma(s,t,p) [g/(cu cm)]; density of seawater.

      Algorithm taken from a similar function written in C 
      by Claudia Schmid
  ==========================================================================*/
  public static double getSeaWaterDensity( double s, double t, double p_bar ) {
    
    double c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15;
    double d1,d2,d3,d4,d5,d6,d7,d8,d9,d10,d11,d12,d13,d14,d15,d16,d17,d18,d19,d20,d21,d22,d23,d24,d25,d26;
    double t2,t3,t4,t5,s2,s3,sqrts3,p2,p3,rho_st,sigma_st,rk,h1,h2,alpha_stp,rho_stp,sigma_stp;
               
    c1=999.842594;
    c2=6.793952e-2;
    c3=-9.095290e-3;
    c4=1.001685e-4;
    c5=-1.120083e-6;
    c6=6.536332e-9;
    c7=8.24493e-1;
    c8=-4.0899e-3;
    c9=7.6438e-5;
    c10=-8.2467e-7;
    c11=5.3875e-9;
    c12=-5.72466e-3;
    c13=1.0227e-4;
    c14=-1.6546e-6;
    c15=4.8314e-4;

    d1=19652.21;
    d2=148.4206;
    d3=-2.327105;
    d4=1.360477e-2;
    d5=-5.155288e-5;
    d6=3.239908;
    d7=1.43713e-3;
    d8=1.16092e-4;
    d9=-5.77905e-7;
    d10=8.50935e-5;
    d11=-6.12293e-6;
    d12=5.2787e-8;
    d13=54.6746;
    d14=-0.603459;
    d15=1.09987e-2;
    d16=-6.167e-5;
    d17=7.944e-2;
    d18=1.6483e-2;
    d19=-5.3009e-4;
    d20=2.2838e-3;
    d21=-1.0981e-5;
    d22=-1.6078e-6;
    d23=1.91075e-4;
    d24=-9.9348e-7;
    d25=2.0816e-8;
    d26=9.1697e-10;

    t2 = t * t;
    t3 = t2 * t;
    t4 = t3 * t;
    t5 = t4 * t;
    s2 = s * s;
    s3 = s2 * s;
    sqrts3 = Math.sqrt(s3);
    p2 = p_bar * p_bar;
    p3 = p2 * p_bar;

    // Calculating RHO(S,T,0)

    rho_st = c1 + c2 * t + c3 * t2 + c4 * t3 + c5 * t4 + c6 * t5;
    rho_st = rho_st + c7 * s + c8 * t * s + c9 * t2 * s + c10 * t3 * s;
    rho_st = rho_st + c11 * t4 * s + c12 * sqrts3 + c13 * sqrts3 * t;
    rho_st = rho_st + c14 * sqrts3 * t2 + c15 * s2;
    sigma_st = rho_st - 1000;

    // Calculating K(S,T,p)

    rk = d1 + d2 * t + d3 * t2 + d4 * t3 + d5 * t4 + d6 * p_bar;
    rk = rk + d7 * t * p_bar + d8 * t2 * p_bar + d9 * t3 * p_bar;
    rk = rk + d10 * p2 + d11 * t * p2 + d12 * t2 * p2 + d13 * s;
    rk = rk + d14 * t * s + d15 * t2 * s + d16 * t3 * s + d17 * sqrts3;
    rk = rk + d18 * t * sqrts3 + d19 * t2 * sqrts3 + d20 * p_bar * s;
    rk = rk + d21 * t * p_bar * s + d22 * t2 * p_bar * s;
    rk = rk + d23 * p_bar * sqrts3 + d24 * p2 * s + d25 * t * p2 * s;
    rk = rk + d26 * t2 * p2 * s;
    
    // Calculating RHO(S,T,p)
    h1 = (1 - p_bar / rk);
    h2 = 1 / rho_st;
    alpha_stp = h2 * h1;
    h1 = (1 - p_bar / rk);
    rho_stp = rho_st / (1 - p_bar / rk);
    sigma_stp = rho_stp - 1000;
    return sigma_st;
  }    
   
  public static double getOxygen_SBE63(double p, double t, double s, double phase63, double t63,MetaFileInfo_all meta) 
   {
      double sigmaP = 1.0368519;
      //System.out.println("phaseShit0 = ");
      // 1.  Convert Phase to Voltage
     double voltage = phase63/39.457071;
      //System.out.println("voltage = "+voltage);

     //Step 2: Compute zero-reference phase-shift:
      double phaseShift0;
      phaseShift0 = (meta.oca[0]+meta.oca[1]*t63+meta.oca[2]*voltage*voltage);
      //System.out.println("phaseShit0 = "+phaseShift0);

      //Step 3: Compute luminescence phase shift:     
      double phaseShiftLum = (meta.ocb[0] +meta.ocb[1]*voltage);
      //System.out.println("phaseShitLum = "+phaseShiftLum);

      //Step 4: Compute Stern-Volmer constant:
      double ksv = meta.occ[0]+t63*(meta.occ[1]+t63*meta.occ[2]);      
      //System.out.println("ksv = "+ksv);
     
      //Step 5: Compute pressure correction:
      double pCorr =Math.exp(meta.oce*p/(t63+273.15));
      //System.out.println("pCorr = "+pCorr);

      //Step 6: Compute [O2] in fresh-water at zero pressure:
      double o2 = (((phaseShift0/phaseShiftLum)-1)/ksv);
      //System.out.println("o2 = "+o2);
      
      //Compute Ts Value
      double ts = Math.log((298.15-t63)/(273.15+t63));
      //System.out.println("ts = "+ts);

      //Step 8: Compute salinity compensation factor:	
      double sCorr = Math.exp((meta.ocd[0]+ts*(meta.ocd[1]+ts*(meta.ocd[2]+ts*meta.ocd[3])))*s+meta.ocd[4]*s*s);
      //System.out.println("sCorr = "+sCorr);
     

      //Step 9: Compute pressure & salinity compensated [O2]:
      //double o2Ml = o2*pCorr*sCorr;
      o2 = o2*pCorr*sCorr;
      //System.out.println("o2ML = "+o2);
      
      //double o2UmolesL = o2Ml*44.66;
      o2 = o2*44.66;
      //System.out.println("o2UmolesL = "+o2);

      o2 = o2/sigmaP; // cell Sigma-p or g2 in the spreadsheet


      //System.out.println("o2UmolesKG = "+o2);
	
	return o2;
   }

  //public double static  getOxygen_Optode4330( double p, double t, double s, double tphase, double topt, boolean hasNitrate, double no3, boolean hasPH, double ph, boolean hasOptical )
   public static double getOxygen_Optode4330( double p, double t, double s, double tphase, double topt,MetaFileInfo_all meta,String doxy )
  {
	
    double p0 =0.0;
    double p1 =0.0;
    double p2 =0.0;
    double p3 =0.0;
   
    double[][] foilMatrix =null;
    if(doxy.equals("DISSOLVED OXYGEN2B"))
    {
		p0 = meta.getDphase2P0();
		p1 = meta.getDphase2P1();
		p2 = meta.getDphase2P2();
		p3 = meta.getDphase2P3();
		foilMatrix = meta.foilMatrix2;
    }
    else {
    	if(doxy.equals("DISSOLVED OXYGEN2A") || doxy.equals("DISSOLVED OXYGEN2")){
		p0 = meta.getDphaseP0();
		p1 = meta.getDphaseP1();
		p2 = meta.getDphaseP2();
		p3 = meta.getDphaseP3();
		foilMatrix = meta.foilMatrix;
	}
    }

   /* if(doxy.equals("DISSOLVED OXYGEN2"))
    {
		p0 = meta.getDphase2P0();
		p1 = meta.getDphase2P1();
		p2 = meta.getDphase2P2();
		p3 = meta.getDphase2P3();
		foilMatrix = meta.foilMatrix2;
    }
    else {
    	if(doxy.equals("DISSOLVED OXYGEN")){
		p0 = meta.getDphaseP0();
		p1 = meta.getDphaseP1();
		p2 = meta.getDphaseP2();
		p3 = meta.getDphaseP3();
		foilMatrix = meta.foilMatrix;
	}
    }
  */


     // Get dphase
     double dphase = p0 + 
                     p1*( tphase + p2*( tphase  + p3*tphase));

     // Optode Calibration
     double o2 = Optode4330_Calibration(topt, dphase, foilMatrix);

     // Calculating oxsol
     double oxsol = O2_all.getOxsol(topt,s);
     oxsol= oxsol*Constants.uM;

     // compute the temperature in Kelvins
     double K=(topt+273.15);

     // compute the vapor pressure
     double Pv=Math.exp(52.57 - (6690.9/K) - 4.681*Math.log(K));

     // compute the air saturation 
     double airSat = o2/((Constants.NOM_AIR_P-Pv)*Constants.NOM_AIR_MIX);

     // compute the oxygen concentration
     o2 = oxsol*airSat;

     // correcting for pressure 
     o2 = o2 * (1+(0.032*p)/1000.0);       
     
     // correcting for density
     double theta = O2_all.ptem(p/10.0, t, s, 0);
     double density=(1000+O2_all.eqstate(s,theta,0))/1000.0;
     //System.out.println("o2 before applyying density = "+o2);
     //System.out.println("density = "+density);
     o2 = o2 / density;
    
    return o2;

  }
  
 
 
 public static double getOxygen_Optode3830(double p, double t, double s, double bphase, double topt,MetaFileInfo_all meta,String doxy)
  {

    double p0 =0.0;
    double p1 =0.0;
    double p2 =0.0;
    double p3 =0.0;
    double[][] foilMatrix =null;
    /*if(doxy.equals("DISSOLVED OXYGEN2"))
    {
        p0 = meta.getDphase2P0();
        p1 = meta.getDphase2P1();
        p2 = meta.getDphase2P2();
        p3 = meta.getDphase2P3();
	foilMatrix = meta.foilMatrix2;
    }
    else {

    if(doxy.equals("DISSOLVED OXYGEN")){
        p0 = meta.getDphaseP0();
        p1 = meta.getDphaseP1();
        p2 = meta.getDphaseP2();
        p3 = meta.getDphaseP3();
	foilMatrix = meta.foilMatrix;
     }
    }
    */
    if(doxy.equals("DISSOLVED OXYGEN2B"))
    {
        p0 = meta.getDphase2P0();
        p1 = meta.getDphase2P1();
        p2 = meta.getDphase2P2();
        p3 = meta.getDphase2P3();
	foilMatrix = meta.foilMatrix2;
    }
    else {

    if(doxy.equals("DISSOLVED OXYGEN2A") || doxy.equals("DISSOLVED OXYGEN2")){
        p0 = meta.getDphaseP0();
        p1 = meta.getDphaseP1();
        p2 = meta.getDphaseP2();
        p3 = meta.getDphaseP3();
	foilMatrix = meta.foilMatrix;
     }
    }



     // 1.  Get dphase
     double dphase =p0 + 
                    p1*( bphase + p2*( bphase  + p3*bphase));
     //System.out.println("dphase = "+dphase);

     // 2.  Optode Calibration
     double o2 = Optode3830_Calibration(topt, dphase, foilMatrix);
     //System.out.println("o2 = "+o2);

     // 3. Correcting for salinity 
     double ts=Math.log((298.15-topt)/(273.15+topt));
     double[] b = {-6.24097e-3, -6.93498e-3, -6.90358e-3, -4.29155e-3};
     double c0 = -3.11680e-7;
     o2 = o2 * Math.exp(s * dswiftpoly(ts,b,3) + c0*s*s );
     //System.out.println("o2 = "+o2);

     // 4. Correcting for pressure
     o2 = o2 * (1+0.032*p/1000.0);
     //System.out.println("o2 = "+o2);

     // 5. Correcting for density
     double theta = O2_all.ptem(p/10.0, t, s, 0);
     double density=(1000+O2_all.eqstate(s,theta,0))/1000.0;
     //System.out.println("theta = "+theta);
     //System.out.println("density = "+density);
     o2 = o2 / density;
     
     return o2;


  }

  public static double getOxygen_SBE43F(double p, double t, double s,double ofreq,MetaFileInfo_all meta)
  {


     double o2,rho,theta;

     if(ofreq == Constants.fillValueOFreq) {
        o2 = Constants.fillValueO2;
        rho   = Constants.fillValueO2;
        theta = Constants.fillValueO2;
     }
     else {
	//System.out.println("Entered getOxygen_SBE43F ");
        // 1. Convert O2 freq into O2 concentration in ml/l
        //    1.a Retrieve meta data coefficients
        double soc = meta.getSoc();
        double fOffset = meta.getFOffset();
        double aa = meta.getAA();
        double bb = meta.getBB();
        double cc = meta.getCC();
        double pCorr = meta.getPCorr();
    
        //    1.c Get O2 concentration  (ml/l)      
        o2 = O2_all.getDissolvedO2_ml_l( soc, (double)ofreq, fOffset, aa, bb, cc, pCorr, p, t, s ) ;
	//System.out.println("O2 in getOxygen_SBE43F ="+o2);

        // 3. Convert O2 (ml/l) to O2 (micromole/l)
        o2 = o2 * 44.6596;
	//System.out.println("O2 in getOxygen_SBE43F 2 ="+o2);

        // 4. Convert O2 (micromole/l) to O2 (micromole/kg)
        //    4.a Insitu density
        theta = O2_all.ptem(p/10.0, t, s, 0);
        rho = 1 + O2_all.eqstate(s, theta, 0)/1000;
        //    4.b conversion itself
        o2 = o2 / rho;
	//System.out.println("O2 in getOxygen_SBE43F 3 ="+o2);
     }



	return o2;

  }


   /***********************************************************************
    function to compute oxygen concentration from temperature and DPhase

    This function computes the oxygen concentration according to the matrix
    equation.

      \begin{verbatim}
                                          | C00 C01 C02 C03 C04 C05 |   | 1   |
                                          | C10 C11 C12 C13 C14 C15 |   | T   |
      [O] = | D^0 D^1 D^2 D^3 D^4 D^5 | * | C20 C21 C22 C23 C24 C25 | * | T^2 |
                                          | C30 C31 C32 C33 C34 C35 |   | T^3 |
                                          | C40 C41 C42 C43 C44 C45 |   | T^4 |
                                          | C50 C51 C52 C53 C54 C55 |   | T^5 |
      \end{verbatim} 

    where D is the compensated phase difference and T is the temperature (C).
    The foil-matrix, C, contains calibration coefficients supplied by
    Aanderaa.                                         
      
      \begin{verbatim}                                                      
      input:
         t............The water temperature (C) as reported by the optode.                     
         DPhase.......The compensated phase difference as reported by the
                        optode.
         foilmat......A 6x6 matrix containing the calibration coefficients                             
                        supplied by Aanderaa for each sensor.                                          
      
      output: 
         This function returns the oxygen concentration (uMolar).
      \end{verbatim}
   **************************************************************************/  
  public static double Optode4330_Calibration(double t, double dphase, double FoilMatrix[][])
   { 
     int i; 
     double o2;
     double[] coef;
     
     coef = new double[6];
     // compute the temperature-dependent coefficients
     for (i=0; i<6; i++) {
       coef[i] = dswiftpoly(t,FoilMatrix[i],5);
     }

     // compute the oxygen concentration
     o2 = dswiftpoly(dphase,coef,5);
     return o2;
   }

  public static double Optode3830_Calibration(double t, double dphase, double FoilMatrix[][])
   {
     int i;
     double o2;
     double[] coef;

     coef = new double[5];
    
/*     System.out.println("In Optode 3830 = ");
     System.out.println("t = "+t);
     System.out.println("FoilMatrix = "+FoilMatrix);
*/
     // compute the temperature-dependent coefficients
     for (i=0; i<5; i++) {
       coef[i] = dswiftpoly(t,FoilMatrix[i],3);
     }

     // compute the oxygen concentration
     o2 = dswiftpoly(dphase,coef,4);
     return o2;
   }

  /***************************************************************************
    function to evaluate an n(th) order polynimial

   This function evaluates an (n)th order polynomial at a specified
   point (x) using Horner's algorthm.  The polynomial has the form:

           y = A0 + x(A1 + x(A2 + x(A3 + ... + x(An))))

      Input:
         x.........The point where the polynomial is evaluated.
         coeff[]...An array of coefficients: coeff[]={A0,A1,A2,...,An}.
                   Beware: An (n)th order polynomial requires an array
                   with (n+1) elements.
         n.........The order of the polynomial to be evaluated.

   written by Dana Swift 3/10/88
   *************************************************************************/
   public static double dswiftpoly(double x, double coeff[], int n)
   {
     int i;
     double P=coeff[n];
     
    // evaluate the (n)th order polynomial using Horner's algorithm
     for (i=n; i>0; i--)
     {
       P *= x;
       P += coeff[i-1];
     }

     return P;
   }




}









