#!/bin/sh
##############################################################################################
# Revision Date: 06-Mar-14 (RS)
#
# mv_Iridium_AllMsg2aomlid.sh
# ===========================
# Moves incoming data files in 'msg' directory under ARGO_DIR/iridium/data
# to the subdirectory corresponding to its AOML identification number.
#
# It performs the following tasks:
#
#  . Checks for the profile 'msgid' is in the following Iridium Floats List
#            META_DIR/floatno_iridiumid_list
#    to get the 'aomlid'. If not found the 'msgid' is added to:
#            TMP_DIR/notfound_aomlid.[type]_list
#  . Creates the 'aomlid' directory if not found
#
# Usage
# -----
#           MSG2PHY_DIR/scripts/mv_Iridium_AllMsg2aomlid.sh
#
# Example:  MSG2PHY_DIR/scripts/mv_Iridium_AllMsg2aomlid.sh
#
# Definitions
# -----------
# ARGO_DIR: defined in ~/.argorc_sh
# META_DIR: ARGO/meta
# IRIDIUM_DIR: ARGO_DIR/iridium
# MSG2PHY_DIR: IRIDIUM_DIR/msg2phy
# DATA_DIR: IRIDIUM_DIR/data
# DATA_TYPE_DIR: DATA_DIR/msg
# DONE_DATAMSG_DIR: DATA_TYPE_DIR/done
# LOG_DIR: IRIDIUM_DIR/log
# TMP_DIR: IRIDIUM_DIR/tmp
##############################################################################################
. $HOME/.argorc_sh
#----------------------------------- Main ------------------------
TODAY=`date +'%Y.%m.%d_%H.%M'`
script_name=`basename $0`
root_script_name=`echo $script_name| awk -F"." '{print $1}`
up_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

type=msg

IRIDIUM_DIR=$ARGO_DIR/iridium
DATA_DIR=$IRIDIUM_DIR/data
DATA_TYPE_DIR=$DATA_DIR/$type
DONE_DATAMSG_DIR=$DATA_TYPE_DIR/done

MSG2PHY_DIR=$IRIDIUM_DIR/msg2phy
LOG_DIR=$MSG2PHY_DIR/log
TMP_DIR=$MSG2PHY_DIR/tmp

floatno_iridiumid_list=$ARGO_DIR/meta/floatno_iridiumid_list

msgid_list=$TMP_DIR/msgid.$type"_list"
aomlid_list=$TMP_DIR/aomlid.$type"_list"
notfound_aomlid_list=$TMP_DIR/notfound_aomlid"_$TODAY".$type"_list"

email_msg=$LOG_DIR/$root_script_name"_$TODAY.email"
logfile=$LOG_DIR/$root_script_name"_$TODAY.log"
error=$LOG_DIR/$root_script_name"_$TODAY.error"

exec 1> $logfile
exec 2> $error

echo "=> Data Directories"
echo "   ARGO_DIR: $ARGO_DIR"
echo "   IRIDIUM_DIR: $IRIDIUM_DIR"
echo "   MSG2PHY_DIR: $MSG2PHY_DIR"
echo "   DATA_DIR: $DATA_DIR"
echo "   DATA_TYPE_DIR: $DATA_TYPE_DIR"
echo "   DONE_DATAMSG_DIR: $DONE_DATAMSG_DIR"

echo "=> Working Directories"
echo "   LOG_DIR: $LOG_DIR"
echo "   TMP_DIR: $TMP_DIR"

echo "=> Logfiles"
echo "   email_msg: $email_msg"
echo "   logfile: $logfile"
echo "   error: $error"

echo "=> Lists"
echo "   floatno_iridiumid_list: $floatno_iridiumid_list"
echo "   msgid_list: $msgid_list"
echo "   aomlid_list: $aomlid_list"
echo "   notfound_aomlid_list: $notfound_aomlid_list"

printf "   Change Dir to '$DONE_DATAMSG_DIR' - "
cd $DONE_DATAMSG_DIR
echo "OK"

ls *.$type 2> /dev/null | awk -F'.' '{print $1}' | sort -u > $msgid_list

n_msg=`wc -l $msgid_list | awk '{print $1}'`
echo "   Number of msg: $n_msg"

if [ $n_msg -eq 0 ]
then
  echo "   <<<<< No Profiles to be moved found -> Exit >>>>>"
  exit
fi

exec < $msgid_list
while read msgid
do
  aomlid=`grep $msgid $floatno_iridiumid_list | awk '{print $1}'`
  if [ "$aomlid" = "" ]
  then
    echo $msgid  >> $notfound_aomlid_list
    continue
  fi

  pathname_aomlid=$DONE_DATAMSG_DIR/$aomlid

  printf "   Does '$pathname_aomlid' Exists? "
  if [ -d $pathname_aomlid ]
  then
    printf "Yes;"
  else
    printf "No, create it - "
    mkdir $pathname_aomlid
    printf "OK;"
  fi

  printf " Move '$msgid*.$type' -> $pathname_aomlid -  "
  mv $msgid*.$type  $pathname_aomlid
  echo "OK"
done

if [ -s $error ]
then
  exec 1> /dev/tty
  echo "##########################  WARNING ###############################"
  echo "                     Standard Error Found > /dev/tty"
  echo "  See files:"
  echo "     $error"
  echo "     $logfile"
  echo "###################################################################"
  exec 1>> $logfile
else
  rm -f $error
fi

echo "=> Finished"
exit

