#!/bin/sh
#####################################################################################
# Revision Date: 23-Oct-13 (RS - modified to accommodate more flexible 'Fill Values'
#                                used for the position of the SIO SOLO Iridium floats)
#                27-Aug-13 (RS - add SIO to processing & improve documentation)
#                23-May-22 (RS - modified to process only 'phy' files from WHOI.
#                                UW is sending 'msg' files which are proceess by 
#                               'msg2phy.sh' located in:
#                                    /a1/ARGO/iriridium/msg2phy/scripts)   
#                29-Jun-12 (RS - fix case when only one position is reported)
#                27-Mar-10 (ER - Test that lookup with transmission id 
#                                into floatno_iridium_list returns no more than 1
#                                aomlid.  This become a new warning.  )
#                09-Feb-09 (ER - directory for incoming phy files moved from 
#                                /ARGO_DIR/iridium/data to ARGO_DIR/iridium/data/phy.
#                                Its correspondent done and failure dirs also moved.)
#                15-Sep-09 (RS - lat & lon fill values (+99.999 & +999.99) in the
#                                second position line are considered valid)
#                07-Sep-09 (RS - check for invalid lat & lon values of the
#                                position line after the +99.999 +999.999 ... )
#                14-Apr-09 (RS - detect 'intrument type' mismatch  between 
#                                the meta file and 'phy' file)         
#                26-Feb-08 (RS - detect profiles with no the meta file)
#                19-Feb-08 (RS - detect corrupted 'phy' file)
#                24-Oct-07 (LL,RS - check WMO of incoming 'phy' file with meta)
#                04-Jun-07 (RS - fix bug with UW profiles)
#                29-May-07 (RS - check for bad 'phy' filename)
#                29-Jan-07 (RS - detect profile with wrong name (non-numeric
#                                profile number) 
#                23-Oct-06 (RS - introduce WHOI Iridium floats)
#                17-Apr-06 (RS - account for empty profiles)
#                06-Mar-06 (RS - revise email messages, missing metafile is
#                                now a warning)
#                13-Feb-06 (RS - bypass floats with no meta file)
#                19-Jan-06 (RS - remove leading zeros in the profile number)
#                06-Jan-06 (RS - stderror, error traps, email & documentation)
#                23-Dec-05 (XXD)
#
# complete_phy.sh
# ===============
# Process the incoming Iridium 'phy' files located in IRIDIUM_PHY_DIR sent from
# Scrippts (SIO) and Woods Hole (WHOI). Performs the following tasks for each profile:
# 
# (1) Checks if the profile is empty 
#     Yes, move to FAILURE_PHY_DIR and continue with next profile 
# (2) Checks if the profile is corrupted 
#     Yes, move to FAILURE_PHY_DIR and continue with next profile 
# (3) Checks if the meta file exists. 
#     No, move to FAILURE_PHY_DIR and continue with next profile
# (4) Checks the Instrument type 
#     Continues process  
# (5) Get acroname of the Institute acronym that provided the profile (source:{SIO, WHOI}) 
# (6) Checks the WMO ID exists in meta file
#     No, move to FAILURE_PHY_DIR and continue with next profile
# (7) Checks if DONE_PHY_DIR/done/aomlid exists
#     Yes, then copies profile to it
#     No, create it and copy profile to it
#     Move profile name to ARGO_PHY_DIR 
#     Add profile to a list (phy_list, with rofile name) to be QCed   
# (8) The 'phy_list' is copied to ARGO_LISTS_DIR (used by qc_process.sh) 
#
#   
# Definitions 
# ------------
# ARGO_DIR=/a1/ARGO
# ARGO_PHY_DIR=ARGO_DIR/data/phy
# ARGO_LISTS_DIR=ARGO_DIR/lists
# IRIDIUM_DIR=ARGO_DIR/iridium/
# IRIDIUM_PHY_DIR=IRIDIUM_DIR/data/phy
# DONE_PHY_DIR=IRIDIUM_DIR/data/phy/done
# FAILURE_PHY_DIR=IRIDIUM_DIR/data/phy/failure
#
# Notes
# -----
# Incoming files processed successfully are moved to DONE_PHY_DIR 
# Incoming files processed unsuccessfully are moved to FAILURE_PHY_DIR 
#####################################################################################
. $HOME/.argorc_sh

IsFillValue()
{
  slat=`echo $lat | awk -F'.' '{print $1}'`
  slon=`echo $lon | awk -F'.' '{print $1}'`

  if [ "`echo $slat | grep '+'`" = "" ]
  then
    ilat=$slat
  else
    ilat=`echo $slat | awk -F'+' '{print $2}'`
  fi

  if [ "`echo $slon | grep '+'`" = "" ]
  then
    ilon=$slon
  else
    ilon=`echo $slon | awk -F'+' '{print $2}'`
  fi

  if [ $ilat -ge 99 -a $ilon -ge 999 ]
  then
     return 1   # fill-values
   else
     return 0   # No fill-values
  fi
}
CheckInstType()
{
  meta=$ARGO_META_DIR/$metafile
  phy=$IRIDIUM_PHY_DIR/$incoming_phy
  inst_meta=`grep -i "^instrument type" $meta | awk '{print $3}'` 
  inst_phy=`grep -i "^instrument type" $phy | awk '{print $3}'` 

  if [ $inst_meta = $inst_phy ]
  then
    return 0
  else
    return 1
  fi 
}
CorruptedProf()
{
  grep -n "^=========================" $incoming_phy | awk -F':' '{print $1}' > $tmplist 
  nequal=`wc -l $tmplist | awk '{print $1}'`

  if [ $nequal -eq 0 ]
  then
    msg="\n        WARNING, corrupted (no '=== ... ===' line)"
    printf  "$msg"
    return 1  
  fi

  if [ $nequal -gt 2 ]
  then
    msg="\n        WARNING, corrupted (more that 1 '=== ... ===' line)"
    printf  "$msg"
    return 1 
  fi

  grep "LATITUDE" $incoming_phy | grep "LONGITUDE" | grep "POSITION-QC" > /dev/null
  status=$?
  if [ $status -ne 0 ]
  then
    msg="\n      WARNING, corrupted (no 'LATITUDE LONGITUDE ...' line)"
    printf  "$msg"
    return 1  
  fi

# lpos1: +99.999 +999.999 yyy/mm/dd hr:mm:sc ...
# ----------------------------------------------
  lposhead=`grep -n "LATITUDE" $incoming_phy | awk -F':' ' {print $1}'` 

  lpos1=`expr $lposhead + 1`
  posline=`sed  -n "$lpos1,$lpos1"p $incoming_phy` 
  echo "$posline" | grep $pattern1 | grep $pattern2 > /dev/null
  status=$?

  if [ $status -ne 0 ]
  then
    msg="\n      WARNING, corrupted (invalid date and/or time in '+99.999 +999.999 ...')"
    printf  "$msg"
    return 1 
  fi

# lpos2: lat lon yyy/mm/dd hr:mm:sc ...  
# --------------------------------------
  lpos2=`expr $lposhead + 2`
  posline2=`sed  -n "$lpos2,$lpos2"p $incoming_phy` 
  echo "$posline2" | grep $pattern1 | grep $pattern2 > /dev/null
  status_line2=$?
  case $status_line2 in
  0) lat=`echo $posline2 | awk '{print $1}'`
     lon=`echo $posline2 | awk '{print $2}'`;;
  *) echo $posline2 | grep "===================================="  > /dev/null
     status=$?
     if [ $status -eq 0 ]
     then 
       lat=`echo $posline | awk '{print $1}'`
       lon=`echo $posline | awk '{print $2}'`
     else
       msg="\n        WARNING, corrupted (invalid date and/or time in 1st 'lat lon date time ...')"
       printf  "$msg"
       return 1
     fi;;
  esac

  IsFillValue
  status_fillvalue=$?
  printf "fillvalue=$status_fillvalue "
  if [ $status_fillvalue -eq 1 ]
  then
     return 0
  fi

  if [ $lat -lt -90 -o $lat -gt 90 -o $lon -lt -180 -o $lon -gt 180 ]
  then
    msg="\n        WARNING, corrupted (invalid lat and/or lon in 1st 'lat lon date time' ...)"
    printf "$msg"
    return 1
  fi

  return 0
}
CheckWMOID()
{
   wmoid_phy=`grep -i 'WMO ID number' $incoming_phy | awk '{print $4}'`
   printf "wmoid:$wmoid_phy"
   if [ "$wmoid_phy" = "" ]
   then
     printf "WMOID number not found"
     return 1
   fi 

   wmoid_meta=`grep -i 'WMO ID number' $ARGO_META_DIR/$metafile | awk '{print $4}'`
   if [ "$wmoid_meta" = "" ] 
   then
     printf " WMOID not in metafile"
     return 1
   fi

   if [ $wmoid_phy -ne $wmoid_meta ]
   then
     printf " DIFFERS"
     return 1
   fi

   return 0
}
# ----------------------------------------------------------------------------------------
#  Revision Date: 23-May-13 (RS)
#
#  ComposeAndSendEmail
#  ===================
#  Compose and send mail. If failure is non-zero then checks the following lists: 
#
#  Lists  
#  -----
#  bad_phyname_list - OK
#  corruptedprof_list - OK     
#  emptyprofile_list - OK
#  failure_list - OK
#  insttype_mismatch - OK (list -> should change it to missing_metafile_list)
#  missing_metafile  - OK (list -> should change it to missing_metafile_list)
#
#  Flags
#  -----
#  corruptedprof - OK
#  emptyprofile - OK
#  failure - OK
#  warning - OK
# 
# Note
# ----
# The following lists were only used for UW 'phy' profiles not in the code:
#
#   aoml_list -  temporary file
#   missing_transid -  temporary file
#   too_many_aomlid_list - temporary file 
# ----------------------------------------------------------------------------------------
ComposeAndSendEmail()
{
   echo "=> ComposeAndSendEmail"
   echo "   failure=$failure warning=$warning "

   any2report=0
   touch $email
   text=""
  
   if [ $failure -ne 0 ]    # Is failure non-zero? 
   then                     # Yes, check failure lists
     any2report=1 
     subj="$up_script_name - FAILURE reported"

     if [ -s $failure_list ]
     then
       echo "Failure(s) reported while processing the following file(s):" > $email
       awk '{print "  " $0}' $failure_list >> $email
     else
       rm -f $failure_list
     fi # endif - failure_list

     if [ -s $emptyprofile_list ]
     then
       echo "Empty profile(s):" >> $email
       awk '{print "  " $0}'  $emptyprofile_list >> $email
     else
       rm -f $emptyprofile_list
     fi # endif - emptyprofile_list

     if [ -s $corruptedprof_list ]
     then
       echo "Corrupted profile(s):" >> $email
       awk '{print "  " $0}'  $corruptedprof_list >> $email
     else
       rm -f $corruptedprof_list
     fi # endif - corruptedprof_list

     if [ -s $missing_metafile ]
     then
       echo "Metafile(s) missing for:" >> $email
       awk '{print "  " $0}'  $missing_metafile >> $email
     else
       rm -f $missing_metafile
     fi # endif - missing_metafile

     if [ -s $insttype_mismatch ]
     then
       echo "Instrument mismatch:" >> $email
       awk '{print "  " $0}'  $insttype_mismatch >> $email
     else
       rm -f $insttype_mismatch
     fi # endif - insttype_mismatch

     text="See logfiles:\n  $logfile\n"

   elif [ $warning -ne 0 -o $emptyprofile -ne 0  -o $corruptedprof -ne 0 ]
   then                         # No - falure=0 but at least a non-zero flag found  
     any2report=1 
     subj=""

     if [ $warning -ne 0 ]
     then
       subj="$up_script_name - WARNING, see details in message body "
       if [ -s $missing_metafile ]
       then
         echo "Metafile(s) missing:" >> $email
         awk '{print "  " $0}'  $missing_metafile >> $email
       fi 

       if [ -s $insttype_mismatch ]
       then
         echo "Instrument type mismatch:" >> $email
         awk '{print "  " $0}'  $insttype_mismatch >> $email
       fi
     fi #endif - warning ne 0

     if [ $emptyprofile -ne 0 ]
     then
       if [ "$subj"  = "" ]
       then 
         subj="$up_script_name - WARNING, empty profile(s)"
       else
         subj=$subj" &  Empty profile(s)"
        fi
       echo "Empty profile(s):" >> $email
       awk '{print "  " $0}'  $emptyprofile_list >> $email
     fi #endif - emptyprofile ne 0

     if [ $corruptedprof -ne 0 ]
     then
       if [ "$subj"  = "" ]
       then
         subj="$up_script_name - WARNING, corrupted profile(s)"
       else
         subj=$subj" &  Corrupted profile(s)"
       fi
       echo "Corrupted profile(s):" >> $email
       awk '{print "  " $0}'  $corruptedprof_list>> $email
     fi #endif - corruptedprof ne 0

     text="See logfiles:\n  $logfile\n"
     rm -f $failure_list
  else                                    # failure=0 & all flags are equal to zero  
    subj="$up_script_name - OK"
    rm -f $corruptedprof_list $missing_metafile $failure_list
  fi # endif failure ne 0  


  if [ -s $error ]
  then
    if [ $any2report -eq 0 ]
    then
      subj="$up_script_name - FAILURE, STDERROR reported"
      text="See logfiles:\n  $logfile\n"
    else
      subj=$subj". And STDERROR reported"
    fi
    text=$text"  $error\n"
  else
    rm -f $error
  fi

  echo "=> [Email Begin]"
  echo "   Subject: $subj"
  echo "   To: $EMAIL_OPERATOR"
  echo "   Text:"
  printf "$text" >> $email
  awk '{print "   " $0}' $email
  printf "   Sending Email ... "
  mailx -s"$subj" $EMAIL_OPERATOR < $email
  echo "OK"
  rm -f $email
  echo "=> [Email End]"

  return
}
# --------------------------- Main -------------------------
IRIDIUM_DIR=$ARGO_DIR/iridium           
IRIDIUM_PHY_DIR=$IRIDIUM_DIR/data/phy         # Incoming 'phy' files 
DONE_PHY_DIR=$IRIDIUM_PHY_DIR/done
FAILURE_PHY_DIR=$IRIDIUM_PHY_DIR/failure

COMPLETE_PHY_DIR=$IRIDIUM_DIR/to_complete_phy
SCRIPTS_DIR=$COMPLETE_PHY_DIR/scripts
PRG_DIR=$COMPLETE_PHY_DIR/prg
LOG_DIR=$COMPLETE_PHY_DIR/log
TMP_DIR=$COMPLETE_PHY_DIR/tmp

ARGO_META_DIR=$ARGO_DIR/meta              
ARGO_PHY_DATA_DIR=$ARGO_DIR/data/phy
ARGO_LISTS_DIR=$ARGO_DIR/lists


TODAY=`date +'%Y.%m.%d_%H.%M'`
script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}`
up_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

logfile=$LOG_DIR/$root_script_name.$TODAY"_"log
error=$LOG_DIR/$root_script_name.$TODAY"_"stderror
email=$LOG_DIR/$root_script_name.$TODAY"_"email
missing_metafile=$TMP_DIR/$root_script_name.$TODAY"_"nometa
insttype_mismatch=$TMP_DIR/$root_script_name.$TODAY"_"intstype_mismath
emptyprofile_list=$TMP_DIR/$root_script_name.$TODAY"_"emptyprof
corruptedprof_list=$TMP_DIR/$root_script_name.$TODAY"_"corrupted
failure_list=$TMP_DIR/$root_script_name.$TODAY"_"failures
touch $emptyprofile_list $corruptedprof_list 
touch $missing_metafile $insttype_mismatch $failure_list 

phy_list=$TMP_DIR/cnt2phy.$TODAY"_"list     # For QC process
touch $phy_list

exec 1> $logfile
exec 2> $error 

echo "--------------------------------- Start Job ---------------------------------" 
echo "=> Directories"
echo "   ARGO_DIR: $ARGO_DIR"
echo "   ARGO_META_DIR: $ARGO_META_DIR"
echo "   IRIDIUM_DIR: $IRIDIUM_DIR"
echo "   COMPLETE_PHY_DIR: $COMPLETE_PHY_DIR"
echo "   ARGO_META_DIR: $ARGO_META_DIR"
echo "   SCRIPTS_DIR: $SCRIPTS_DIR"
echo "   PRG_DIR: $PRG_DIR"
echo "   LOG_DIR: $LOG_DIR"
echo "   TMP_DIR: $TMP_DIR"

echo "=> Data directories"
echo "   IRIDIUM_PHY_DIR: $IRIDIUM_PHY_DIR"
echo "   DONE_PHY_DIR: $DONE_PHY_DIR"
echo "   FAILURE_PHY_DIR: $FAILURE_PHY_DIR"
echo "   ARGO_LISTS_DIR: $ARGOS_LISTS_DIR"
echo "   ARGO_PHY_DATA_DIR: $ARGO_PHY_DATA_DIR"

echo "=> Log & miscellaneous files"
echo "   logfile: $logfile"
echo "   error: $error"
echo "   email: $email"
echo "   phy_list: $phy_list"
echo "   missing_metafile: $missing_metafile"
echo "   insttype_mismatch: $insttype_mismatch"
echo "   emptyprofile_list: $emptyprofile_list"
echo "   corruptedprof_list: $corruptedprof_list"
echo "   failure_list: $failure_list"

echo "=> Loop - Process incoming 'phy' files from UW and WHOI"
echo "   Change Directory to: $IRIDIUM_PHY_DIR"
cd $IRIDIUM_PHY_DIR

nphy=0
emptyprofile=0
corruptedprof=0
nonnumeric_profno=0    
failure=0
warning=0

tmplist=$TMP_DIR/$root_script_name.$TODAY"_"tmplist

pattern1="[0-2,9][0-9]\{3\}\/[0-1,9][0-9]\/[0-3,9][0-9]" # validate position (lat,lon)
pattern2="[0-9][0-9]:[0-9][0-9]:[0-9][0-9]"              # validate date-time

for incoming_phy in `ls *.phy 2> /dev/null`
do
  printf "   $incoming_phy "
  nphy=`expr $nphy + 1`
  rootname=`echo $incoming_phy | awk -F"." '{print $1}'`
  profno=`echo $rootname | awk -F"_" '{print $3}'`

  if [ ! -s $incoming_phy ]
  then
    emptyprofile=1
    echo $incoming_phy >> $emptyprofile_list
    printf "EMPTY, mv to failure - "
    mv $incoming_phy $FAILURE_PHY_DIR
    if [ $? -eq 0 ]
    then
      echo "OK"
    else
      echo "FAILURE"
    fi
    continue
  fi

  CorruptedProf                        # Is profile corrupted?
  status=$?
  printf "corrupt=$status "
  if [ $status -ne 0 ]
  then
    corruptedprof=1
    echo $incoming_phy >> $corruptedprof_list
    printf " -> mv to failure - "
    mv $incoming_phy $FAILURE_PHY_DIR
    if [ $? -eq 0 ]
    then
      echo "OK"
    else
      echo "FAILURE"
    fi
    continue
  fi

  aomlid=`echo $incoming_phy | awk -F'_' '{print $1}'`
  printf "aomlid:$aomlid "
  trans_id=`echo $incoming_phy | awk -F'.' '{print $1}' | awk -F'_' '{print $2}'`
  metafile=$aomlid"_"$trans_id.meta

  if [ -f $ARGO_META_DIR/$metafile ]
  then
    pi_address=`grep "principal investigator address" $ARGO_META_DIR/$metafile | awk '{print substr($0, 41)}'` 

    if [ "`echo $pi_address | egrep -c 'NIWA|SIO'`" -ne 0 ]
    then
      source="SIO" 
    else
      source="WHOI"
    fi
    printf "(%-4s): aomlid:%04d " $source $aomlid

    CheckInstType
    status_check=$?
    if [ $status_check -ne 0 ]
    then
      warning=1
      echo $incoming_phy >> $insttype_mismatch
#?    printf "WARNING, Instrument type mismatch -> mv to failure - "
#?    mv $incoming_phy $FAILURE_PHY_DIR
#?    echo "OK"
#?    continue
    fi 
 
    CheckWMOID
    status_check=$?
    if [ $status_check -ne 0 ]
    then
      failure=1
      echo $incoming_phy >> $failure_list
      printf " -> mv to failure - "
      mv $incoming_phy $FAILURE_PHY_DIR
      echo "OK"
      continue
    else  
      printf "; mkdir done/$aomlid? "
      if [ -d $DONE_PHY_DIR/$aomlid ]
      then
         printf "N, "  
      else
         printf "Y, "
         mkdir $DONE_PHY_DIR/$aomlid
      fi

      printf "cp to done/$aomlid - " 
      cp -p $incoming_phy $DONE_PHY_DIR/$aomlid 
      printf "OK; "

      printf "mv ARGO/data/phy - "
      mv $incoming_phy $ARGO_PHY_DATA_DIR 
      echo "OK"
      
      echo $incoming_phy >> $phy_list
      continue
    fi
  else
    warning=1
    echo $incoming_phy >> $missing_metafile
    printf "WARNING, No META File found -> mv to failure - "
    mv $incoming_phy $FAILURE_PHY_DIR
    echo "OK"
    continue
  fi
done

if [ $nphy -eq 0 ]
then
  echo "   No PHY incoming files to process"
  echo "=> [Email Begin]"
  subj="$up_script_name, No PHY incoming files to process"
  echo "   Subject: $subj"
  echo "   To: $EMAIL_OPERATOR"
  printf "   Sending Email ... "
  echo " " | mailx -s"$subj" $EMAIL_OPERATOR
  echo "OK"
  echo "=> [Email End]"
  rm -f $error
  echo "--------------------------------- End   Job ---------------------------------" 
  exit
fi

# If list is not empty copy to ARGO_LISTS_DIR for QC process
# ---------------------------------------------------------
if [ -s $phy_list ]
then
  printf "=> Copy $phy_list to ARGO_LISTS_DIR - "
  cp -p $phy_list $ARGO_LISTS_DIR
  status=$?
  if [ $status -eq 0 ]
  then
    echo "OK"
  else
    echo "FAILURE"
    echo "=> [Email Begin]"
    subj="$up_script_name - FAILURE, copy PHY_LIST to ARGO_LISTS_DIR" 
    text="Failure reported while executing a 'cp'\n"
    text=$text"  From: $phy_list\n  To:   $ARGO_LISTS_DIR\n"
    text=$text"See logfile(s):\n   $logfile\n"

    if [ -s $error ]
    then
      subj=$subj" & STDERROR" 
      text=$text"   $error"
    else
      rm -f $error
    fi
    echo "   Subject: $subj"
    echo "   To: $EMAIL_OPERATOR"
    echo "   Text:"
    printf "$text" > $email
    awk '{print "   " $0}' $email
    printf "   Sending Email ... "
    mailx -s"$subj" $EMAIL_OPERATOR < $email
    echo "OK"
    rm -f $email
    echo "=> [Email End]"
    exit
  fi
fi

# Compose Sent Email
# -------------------
ComposeAndSendEmail

echo "--------------------------------- End   Job ---------------------------------" 
exit
