#!/bin/sh 
# ========================================================================= 
# Revision Date: 20-Oct-03 (RS - modified email subjects)
#                08-Sep-03 (RS - check phy file in ARGO_DIR/data/phy/done)
#                29-Jul-03 (XX - add magnitude check for p,t,c,s)
#                10-Jul-03 (RS - check for no WMO in nc file)
#                12-May-03 (RS - add to exits from Java programm)
#                03-Dec-02 (RS - changes for production run) 
#                26-nov-02 (XX)
#
# nc2phy.sh
# ---------
# Creates AOML formatted 'phy' file(s) from ARGO formatted NetCDF 
# file(s) (e.g., WHOI floats that used Orbcomm telecomunication
# to transmit data). 
#
# Task involved are:
#
# 1) Obtain a list of NetCDF file(s) in ORBCOMM_DIR/data/nc
#
# 2) Get WMO number and find the metadata file which corresponds to
# this number. 
#   - If successful, proceed to create 'phy' file by executing 'NetcdfReader'.
#     NetCDF file is moved to ORBCOMM_DIR/data/done 
#   - If not, increment counter
#
# 3) Check to see if newly created 'phy' profile exists in n ARGO_DIR/data/phy/done (done) 
#    - If profile exists in 'done', check for differences.
#        If different, move new 'phy' profile to ARGO_DIR/data/phy/orbcomm_check
#        add profile to a list of profiles to be checked (phy_check_list)
#        If not different,  add profile name to a list of newly created profiles (phy_list)   
#    - If profile does not exists then add to 'phy_list' 
#
# 4) Create and send Email message 
#
# Conversion Exit Status 
# ----------------------
#   1, OK
#   2, WMO ID in 'nc' file is a 'fill' value (0)
#   3, WMO number not found in meta files
#   4, WMO ID in 'nc' file name differs from WMO ID inside the 'nc' file
#   5, WMO ID in 'nc' file can not be aphanumeric
#   6, WMO instrument type from meta file and 'nc' file do not match
#   7, WMO number from 'meta' file and 'nc' file do not match.
#   8, Number of columns is not 3 (p, t, c)
#   9, Profile direction (up/down) is inconsistant with the file name convention
#  10, Magnitude of data value(s) exceed format specifications
#  11, Not a NetCDF file
# >11, FAILURE -  Unexpected error reported
# 
# Exit Code 
# ---------
# 0, OK
# 1, no NetCDF file found
# 2, at least one of the following conditines was met
#    - some WMO number not found in metadata files   
#    - can not decode data if number of columns is not 3 (p, t, c) - java exit = 3
#    - FLOAT_WMO_NUMBER and data.PLATFORM_NUMBER differ - java exit = 4
#    - WMO_INST and WMO_INST_TYPE differ - java exit = 5
#    - Not a NetCDF file - java exit = 255 changed to 6 
# 
# Note:
#   ORBCOMM_DIR = $ARGO_DIR/orbcomm
# ====================================================================================
PHY_Exists_inDone()
{
   profile_name=$1
   phy_pathname=$PHY_DIR/$profile_name
   phydone_pathname=$PHYDONE_DIR/$profile_name

   printf  "     Profile: %-21s" $profile_name >> $logfile

   if [ -f $phydone_pathname ]
   then
     printf  " in %-s"  $PHYDONE_DIR  >> $logfile
     diff $phy_pathname $phydone_pathname > /dev/null
     status_diff=$?                                     #0, different; >0, same
     if [ $status_diff -eq 0 ]
     then
       echo " , Does Not Differ" >> $logfile 
       echo $profile_name >> $phy_list 
       return
     else
       mv $phy_pathname $PHYORBCOMM_CHECK_DIR
       echo " , Differs - Moved to '../orbcomm_check'"  >> $logfile
       echo $profile_name >> $phy_check_list 
       return
     fi
   fi

   echo  " Not in $PHYDONE_DIR"  >> $logfile
   echo $profile_name >> $phy_list 

   return 
}
PrintExitStatus()
{
  yymmdd=`echo $TODAY | awk -F"_" '{print $1}'`
  hhmm=`echo $TODAY | awk -F"_" '{print $2}'`
  TODAY_DATE=`echo $yymmdd | sed -n -g s%'\.'%/%p`
  TODAY_TIME=`echo $hhmm | sed -n -g s%'\.'%:%p`
  ZONE=`date +'%Z`

  equal="======================================================================="
  dash="-----------------------------------------------------------------------"
  echo $equal  > $SUMMARY_NC2PHY_EXIT_STATUS
  echo "        Conversion from NetCDf to 'PHY'  Exit Status Report" >> $SUMMARY_NC2PHY_EXIT_STATUS
  echo "                       $TODAY_DATE $TODAY_TIME $ZONE" >> $SUMMARY_NC2PHY_EXIT_STATUS
  echo "                       -----------------------" >> $SUMMARY_NC2PHY_EXIT_STATUS
  echo "   1, OK"  >> $SUMMARY_NC2PHY_EXIT_STATUS 
  echo "   2, WMO ID in 'nc' file is a 'fill' value (0)" >> $SUMMARY_NC2PHY_EXIT_STATUS
  echo "   3, WMO number not found in meta files" >> $SUMMARY_NC2PHY_EXIT_STATUS >> $SUMMARY_NC2PHY_EXIT_STATUS
  echo "   4, WMO ID in 'nc' file name differs from WMO ID inside the 'nc' file" >> $SUMMARY_NC2PHY_EXIT_STATUS
  echo "   5, WMO ID field in 'nc' file name can not be aphanumeric" >> $SUMMARY_NC2PHY_EXIT_STATUS
  echo "   6, WMO instrument type from meta file and 'nc' file do not match" >> $SUMMARY_NC2PHY_EXIT_STATUS
  echo "   7, WMO number from 'meta' file and 'nc' file do not match" >> $SUMMARY_NC2PHY_EXIT_STATUS
  echo "   8, Number of columns is not 3 (p, t, c)" >> $SUMMARY_NC2PHY_EXIT_STATUS
  echo "   9, Profile direction (up/down) is inconsistant with the file" >> $SUMMARY_NC2PHY_EXIT_STATUS
  echo "      naming convection" >> $SUMMARY_NC2PHY_EXIT_STATUS
  echo "  10, Magnitude of data value(s) exceed format specifications" >> $SUMMARY_NC2PHY_EXIT_STATUS
  echo "  11, Not a NetCDF file" >> $SUMMARY_NC2PHY_EXIT_STATUS
  echo " >11, FAILURE -  Unexpected error reported" >> $SUMMARY_NC2PHY_EXIT_STATUS
  echo $dash >> $SUMMARY_NC2PHY_EXIT_STATUS

  printf "   Exit Status      NetCDF File\n" >> $SUMMARY_NC2PHY_EXIT_STATUS

  cat $failed_list  | awk '{printf "      %-3d         %s\n", $1, $2}' \
       >> $SUMMARY_NC2PHY_EXIT_STATUS

  printf "\n"  >> $SUMMARY_NC2PHY_EXIT_STATUS
  printf "   Total conversions to 'PHY' (status = 1): %4d\n" $nphy  >> $SUMMARY_NC2PHY_EXIT_STATUS
  printf "   Total conversion failures (status > 1 ): %4d\n" $nophy >> $SUMMARY_NC2PHY_EXIT_STATUS
  echo "$equal" >> $SUMMARY_NC2PHY_EXIT_STATUS

  cat $SUMMARY_NC2PHY_EXIT_STATUS >> $logfile

  rm $exit_status 
  return
}
# ==========================  define directories  ================================
#? debug ARGO_DIR=/b1/ARGOTEST         # .argorc
#ARGO_DIR=/b1/ARGOTEST
ARGOLIST_DIR=$ARGO_DIR/lists
META_DIR=$ARGO_DIR/meta
ORBCOMM_DIR=$ARGO_DIR/orbcomm
NC_DIR=$ORBCOMM_DIR/data/nc
DONE_DIR=$NC_DIR/done
FAILURE_DIR=$NC_DIR/failure
NC2PHY_DIR=$ORBCOMM_DIR/nc2phy
SCRIPT_DIR=$NC2PHY_DIR/script
PRG_DIR=$NC2PHY_DIR/prg
LOG_DIR=$NC2PHY_DIR/log
TMP_DIR=$NC2PHY_DIR/tmp
PHY_DIR=$ARGO_DIR/data/phy
PHYDONE_DIR=$PHY_DIR/done
PHYORBCOMM_CHECK_DIR=$PHY_DIR/orbcomm_check

# ------------------------ Only For Debugging --------------------------------
#EMAIL_OPERATOR=Reyna.Sabina@noaa.gov
# ==========================  Main script  ===================================
#-------------------------- Initialize Variables -----------------------------
TODAY=`date +'%Y.%m.%d_%H.%M.%S'`
MAXNO_EXITSTATUS=11

script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}'`
up_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

# logfile, email message & stderror file
# --------------------------------------
logfile=$LOG_DIR/$root_script_name.$TODAY"_log"
logjava=$LOG_DIR/$root_script_name.$TODAY"_logjava"
email_message=$TMP_DIR/root_script_name.$TODAY"_email"

error=$LOG_DIR/$root_script_name.$TODAY"_stderror"     # standard error
touch $error
exec 2>>$error

now=`date +'%h %d, %Y %T %p'`
echo "<================ Start  $script_name: $now ==============>" > $logfile
echo "-> Local variables used by '$script_name'"      >> $logfile
echo "   ARGOLIST_DIR: $ARGOLIST_DIR"                 >> $logfile
echo "   NC2PHY_DIR: $NC2PHY_DIR"                     >> $logfile
echo "   PRG_DIR: $PRG_DIR"                           >> $logfile
echo "   LOG_DIR: $LOG_DIR"                           >> $logfile
echo "   TMP_DIR: $TMP_DIR"                           >> $logfile
echo "   META_DIR: $META_DIR"                         >> $logfile 
echo "   PHY_DIR: $PHY_DIR"                           >> $logfile 
echo "   PHYDONE_DIR: $PHYDONE_DIR"                   >> $logfile 
echo "   PHYORBCOMM_CHECK_DIR: $PHYORBCOMM_CHECK_DIR" >> $logfile 
echo "   NC_DIR: $NC_DIR"                             >> $logfile 
echo "   DONE_DIR: $DONE_DIR"                         >> $logfile 
echo "   FAILURE_DIR: $DONE_DIR"                      >> $logfile 
sep="................................"

# Working lists
# -------------
only_name=`echo $script_name | awk -F"." '{print $1}'`
nc_list=$TMP_DIR/$script_name.$TODAY"_nclist"
exit_status=$TMP_DIR/exit_status.$TODAY
failed_list=$TMP_DIR/failed.$TODAY"_nclist"

touch $failed_list

SUMMARY_NC2PHY_EXIT_STATUS=$TMP_DIR/SUMMARY_NC2PHY_EXIT_STATUS.$TODAY

phy_list=$TMP_DIR/cnt2phy.$TODAY"_"list                  # like in cnt2phy.sh
phy_check_list=$TMP_DIR/phy_check.$TODAY"_"list          # profiles in 'done' but different 
tmpfile=$TMP_DIR/$script_name.$TODAY"_tmpfile"           # phy profile name (output from Java) 

echo "-> Working lists & temporary file:"     >> $logfile
echo "   nc_list: $nc_list"                   >> $logfile 
echo "   phy_list: $phy_list"                 >> $logfile
echo "   phy_check_list: $phy_check_list"     >> $logfile
echo "   failed_list $failed_list"            >> $logfile
echo "   tmpfile : $tmpfile"                  >> $logfile

# -------------------------------------------------------------------
#      Create list of NetCDF file(s) to be converted 
# -------------------------------------------------------------------
ls $NC_DIR/*.nc > $nc_list 2> /dev/null

if [ ! -s $nc_list ]               # Is it empty?
then                               #   Yes, Email & exit
    ret_val=1
    subj="$up_script_name: WARNING, NO NetCDF file to be converted"
    text="There is no NetCDF file to be converted.\n"
    if [ -s $error ]
    then
      subj=`echo $subj | sed s/OK/FAILURE`
      text=$text"Standard-Error detected. See logfiles:\n"
      text=$text"\n\t$logfile\n\t$error"
    else
      rm  $error
    fi
    echo $text  > $email_message
    echo "-> Email"                                   >> $logfile
    echo "   Subj: $subj"                             >> $logfile
    echo "   To: $EMAIL_OPERATOR"                     >> $logfile
    echo "   Text:"                                   >> $logfile
    cat $email_message | awk '{print "        " $0}'  >> $logfile

    echo "   Sending Email ...\c"                     >> $logfile
    mailx -s "$subj" $EMAIL_OPERATOR < $email_message
    echo "OK"                                         >> $logfile

    now=`date +'%h %d, %Y %T %p'`
    echo "<================== Terminated $script_name: $now ================>" >> $logfile
    exit $ret_val
fi

# -------------------------------------------------------------------
#     Loop over NetCDF file(s) to be converted to 'phy' file(s)
# -------------------------------------------------------------------

echo "\n-> Convert NetCDF file(s) to AOML 'phy' format:" >> $logfile
nphy=0
nophy=0

for nc in `cat $nc_list`                   # nc: pathname to NetCDF file
do
  filename=`basename $nc`
  printf "   %-s " $filename >> $logfile;

  wmo=`echo "$filename"| awk -F"_" '{print $1}' | sed -e "s/^R//`;

  printf "WMO= %-7s " $wmo     >> $logfile

  echo $wmo | grep '[^0-9]' > /dev/null    # Is char a not a number? 
  chk=$?                                   #   status: 0, true; 1,false

  if [ $chk -eq 0 ]                        # Not Numeric?
  then                                     #   Yes
    status=5
    nophy=`expr $nophy + 1`
    printf "%-3d %s\n" $status $filename  >> $exit_status
    printf "%-3d %s\n" $status $filename  >> $failed_list
    mv $nc $FAILURE_DIR
    printf "- not numeric, status: %3d -> moved to 'failure'\n" $status >> $logfile
    continue
  fi

  c=`grep -i "WMO ID number" $META_DIR/*.meta | grep $wmo`;
  chk=$?

  if [ $chk -ne 0 ]              # Was it a match?   
  then
    status=3 
    nophy=`expr $nophy + 1`
    printf "%-3d %s\n" $status $filename  >> $exit_status
    printf "%-3d %s\n" $status $filename  >> $failed_list
    mv $nc $FAILURE_DIR
    printf "not in meta file, status: %3d -> moved to 'failure'\n" $status  >> $logfile
    continue
  fi

  meta=`echo "$c" | awk -F":" '{print $1}'`
  printf " - found," >> $logfile

  echo "   $sep $filename $sep"     >> $logjava  
  java -classpath .:$PRG_DIR/netcdf2.jar:$PRG_DIR -Dphy_dir=$PHY_DIR/ \
       NetcdfReader $nc $meta $tmpfile >> $logjava; 
  status=$?

  if [ $status -eq 255 ]
  then
    status=$MAXNO_EXITSTATUS
  fi

  printf "%-3d %s\n" $status $filename >> $exit_status

  printf " status: %2d" $status  >> $logfile
  echo "   $sep       end       $sep"     >> $logjava  

  if [ $status -eq 1 ]          # OK?
  then                          #   Yes, success
    nphy=`expr $nphy + 1`       #   OK counter
    mv $nc $DONE_DIR
    printf ", moved to 'done'\n"  >> $logfile

    if [ -f $tmpfile ]          #   tmpfile exists?
    then                        #     Yes
      if [ -s $tmpfile ]        #     Not empty?
      then                      #       Yes
        phy_filename=`cat $tmpfile`
#?      printf " -> %-s\n" $phy_filename >> $logfile
#?      echo $phy_filename >> $phy_list
        PHY_Exists_inDone $phy_filename
      else                      #       No, empty
        printf " -> tmpfile empty\n" >> $logfile
      fi # endif - tmpfile (not empty)
      rm $tmpfile
    else                         #     No, files does not exits 
      printf " -> no tmpfile\n"      >> $logfile 
    fi #endif - tmpfile exists?
  else                           #   No, failure reported
    nophy=`expr $nophy + 1`
    printf "%-3d %s\n" $status $filename >> $failed_list
    mv $nc $FAILURE_DIR
    printf ", moved to 'failure'\n"   >> $logfile

    if [ $status -gt $MAXNO_EXITSTATUS ]
    then
      printf " (not defined)" >> $logfile
    fi
  fi # endif - OK?
done 

# ----------------------------------------------------------------------
#       Move phy_list (cnt2phy.TODAY_list) to ARGO list
#       Print sucessfully converted profile files 
#       Print unsucessfully converted profile files 
#       Print profiles in PHYDONE_DIR but differ 
# ----------------------------------------------------------------------
if [ -f $phy_list ]
then
  printf "-> List of Converted Profile Files (Exit Status = 1)"  >> $logfile
  echo " - to be QCed"                                           >> $logfile  
  cat $phy_list | awk '{print "   " $0}' >> $logfile
  mv $phy_list $ARGOLIST_DIR
  echo "\n-> Move: $phy_list\n   To: $ARGOLIST_DIR"             >> $logfile
fi

if [ -f $failed_list ]
then
  echo "\n-> List of Profile Files that Failed to Convert to 'PHY' (Exit Status > 1)"  >> $logfile
  cat $failed_list | awk '{print "   " $0}' >> $logfile
fi

if [ -f $phy_check_list ]
then
  echo "-> List of Converted Profile Files (Exit Status = 1)"       >> $logfile
  echo "   in $PHYDONE_DIR but differ - moved to $PHYORBCOMM_CHECK" >> $logfile
  cat $phy_check_list | awk '{print "   " $0}'                      >> $logfile
  subj_check=" - Profiles in 'done' but Different"                  >> $logfile
  text_check="\nProfiles in $PHYORBCOMM_CHECK_DIR that differ\n"    >> $logfile
  text_check=$text_check"from the ones in $PHYDONE_DIR are:"        >> $logfile
else
  subj_check=""
  text_check=""
fi

# ------------------------------------------------------
#           Print Exit Status Summary Table 
# ------------------------------------------------------
PrintExitStatus

cat $SUMMARY_NC2PHY_EXIT_STATUS > $email_message

# Is error (stderr) file not empty?
# ---------------------------------
if [ -s $error ]
then
  stderror=1
else
  stderror=0
  rm -f $error
fi

if [ $nophy -eq 0 ]             # All files converted? 
then                            #  Yes
  ret_val=0
  subj="$up_script_name - OK, All 'NetCDF' Files Were Converted to 'PHY'"
  if [ $stderror -eq 1 ]
  then
    subj=$subj" & Stderror Reported"
    subj=`echo $subj | sed -s/OK/FAILURE`
    text="See logfiles:\n   $logfile\n   $error\n"
  else
    text="FYI, logfiles are:\n   $logfile\n   $logjava\n"
  fi
  subj=$subj"$subj_check"
  text=$text"$text_check"
  echo "$text" > $email_message
else                            # No, compile Email body
  ret_val=2
  subj="$up_script_name - FAILURE, At Least One 'NetCDF' File Was Not Converted to 'PHY'"
  cat $SUMMARY_NC2PHY_EXIT_STATUS > $email_message
  text="See logfiles:\n   $logfile\n   $logjava\n"
  if [ $stderror -eq 1 ]
  then
    subj=$subj" & Stderror Reported"
    text=$text"   $error\n"
  fi
  subj=$subj"$subj_check"
  text=$text"$text_check"
  echo "$text" >>  $email_message
fi

if [ -f $phy_check_list ]               # list of phy profiles in PHYORBCOMM_CHECK_DIR
then                                    # and PHYDONE_DIR that differ
  cat $phy_check_list >> $email_message
fi
 
# ------------------------------------------------------
#                     Send Email
# ------------------------------------------------------
echo "[EMAIL Begin]"             >> $logfile
echo "   Email Subj: $subj"      >> $logfile
echo "   To: $EMAIL_OPERATOR"    >> $logfile
echo "   Text:"                  >> $logfile
cat $email_message | awk '{print "        " $0}'  >> $logfile
printf "   Sending ..."          >> $logfile
mailx -s "$subj" $EMAIL_OPERATOR < $email_message
echo "   OK"                     >> $logfile
echo "[EMAIL End]"               >> $logfile

rm $email_message

echo "<================== Finish $script_name: $now ================>" >> $logfile

exit $ret_val
