#!/bin/sh
# =======================================================================
# Revision Date: 18-Mar-13 (RS - correct typo '4th positional' to 
#                                '5th positional' in documentation)
#                10-May-05 (RS - update documentation for status=5)
#                10-Sep-03 (RS - add documentation)
#                15-May-03 (RS - add parameter in call to progname.x )
#                19-Dec-00 (RS)
#
# qc_sensor.sh
# =============
# Runs a Fortran program (qc_t.x or qc_s.x) to perform quality
# control tests for the physical profile data for temperature or
# salinity depending of the command 5th positional parameter
# 'sensor_type'. The quality control tests are:
#
#   Gross
#   CheckSumm 
#   Speed 
#   Vertical Gradient
#   Analysis (NCEP)
#   Climatology 
#   
# Also, it appends the standard output file to a sensor summary file.
#
# Returned Unix status values are:
#
# 0, not valid
# 1, passed complete
# 2, passed incomplete
# 3, no data lines
# 4, failed 
# 5, no position and nodata
# 6, illegal position and/or date
# 7, position on land with data
# 8, no position with data
# 9, execution failed
# =======================================================================

infile_pathname=$1
meta_pathname=$2
outfile_pathname=$3
profile_id=$4
sensor_type=$5

script_name=`basename $0`
prgname="qc_"$sensor_type

now=`date +'%h %d, %Y %T %p'`
echo "<=============== Start  $script_name $profile_id: $now =============>"
echo "Entering Script: $0 to execute Fortran $prgname"

echo "infile_pathname:     $infile_pathname"
echo "meta_pathname:       $meta_pathname"
echo "outfile_pathname:    $outfile_pathname"
echo "sensor type:         $sensor_type"

error=$QCLOG_DIR/$prgname.$TODAY"_"error
if [ -f $error ]
then
  rm -f $error
fi                 
exec 2>> $error

# ------------------------------------------------------------------------
#                      QC-T Process for PHY profile 
# ------------------------------------------------------------------------
$QCPRG_DIR/$prgname.x $infile_pathname $meta_pathname $outfile_pathname 
status="$?"
echo "$script_name: $QCPRG_DIR/$prgname.x  status= $status"

ret_val=$status

if [ "$status" -lt "9" ]                          # QC exec terminated?
then                                              #  No, cat to summary
  echo "$prgname: Executed Successfully (Fortran)"
  if [ "$status" = "1" ]                          # Did profile pass QC?
  then                                            #   Yes
    echo "$prgname: $profile_id PASSED QC Test with Status=$status => PROFILE IS GOOD" 
  else
    text="$prgname: $profile_id DID NOT PASSED QC Test with Status=$status => PROFILE IS"
    if [ $status -eq 2 ]
    then
      text=$text" ALMOST GOOD" 
    else
      text=$text" BAD" 
    fi
    echo $text 
  fi
else
  echo "$prgname: Execution TERMINATED ANORMALLY (Fortran) with Status = $status"
fi

# Does script error file exists?
# ------------------------------
if [ -f $error ]                           # does it exist?
then
  if [ -s $error ]
  then                                     # delete if empty
    msg="<----------------------------------- "
    msg=$msg" $script_name: $profile_id "
    msg=$msg"----------------------------------->"
    echo "$msg"  >> $qc_process_stderror
    cat  $error  >> $qc_process_stderror
  fi
  rm -f $error
fi

now=`date +'%h %d, %Y %T %p'`
echo "<=============== Finish $script_name $profile_id: $now =============>"

exit $ret_val 
